@@ -29,15 +29,26 @@ int main(int argc, char* argv[])
   // Assemble matrix
   Poisson::BilinearForm a(V, V);
   Matrix A;
-  MPI::barrier();
+  dolfin::MPI::barrier();
   double t = time();
   assemble(A, a);
-  MPI::barrier();
+  dolfin::MPI::barrier();
   t = time() - t;
 
   // Report timing
-  if (MPI::process_number() == 0)
-    info("TIME: %.5g", t);
+  if (dolfin::MPI::process_number() == 0)
+    info("TIME (first assembly): %.5g", t);
+
+  // Re-assemble matrix
+  dolfin::MPI::barrier();
+  t = time();
+  assemble(A, a, false);
+  dolfin::MPI::barrier();
+  t = time() - t;
+
+  // Report timing
+  if (dolfin::MPI::process_number() == 0)
+    info("TIME (second assembly): %.5g", t);
 
   return 0;
 }
