@@ -6,6 +6,8 @@
 //
 // This demo program solves a hyperelastic problem
 
+// Begin demo
+
 #include <dolfin.h>
 #include "HyperElasticity.h"
 
@@ -54,9 +56,11 @@ public:
 
   void eval(Array<double>& values, const Array<double>& x) const
   {
+    const double scale = 0.5;
+
     // Center of rotation
-    double y0 = 0.5;
-    double z0 = 0.5;
+    const double y0 = 0.5;
+    const double z0 = 0.5;
 
     // Large angle of rotation (60 degrees)
     double theta = 1.04719755;
@@ -67,8 +71,8 @@ public:
 
     // Rotate at right end
     values[0] = 0.0;
-    values[1] = y - x[1];
-    values[2] = z - x[2];
+    values[1] = scale*(y - x[1]);
+    values[2] = scale*(z - x[2]);
   }
 
 };
@@ -76,44 +80,46 @@ public:
 int main()
 {
   // Create mesh and define function space
-  UnitCube mesh (8, 8, 8);
+  UnitCube mesh (16, 16, 16);
   HyperElasticity::FunctionSpace V(mesh);
 
-  // Define Dirichlet boundary condition
+  // Define Dirichlet boundaries
   Left left;
   Right right;
 
+  // Define Dirichlet boundary functions
   Clamp c;
   Rotation r;
 
+  // Create Dirichlet boundary conditions
   DirichletBC bcl(V, c, left);
   DirichletBC bcr(V, r, right);
-
   std::vector<const BoundaryCondition*> bcs;
-  bcs.push_back(&bcl);
-  bcs.push_back(&bcr);
+  bcs.push_back(&bcl); bcs.push_back(&bcr);
 
-  // Define source and solution functions
-  Constant B(0.0, 0.0, 0.0);
-  Constant T(0.0, 0.0, 0.0);
+  // Define source and boundary traction functions
+  Constant B(0.0, -0.5, 0.0);
+  Constant T(0.1,  0.0, 0.0);
+
+  // Define solution function
   Function u(V);
 
   // Set material parameters
-  double E  = 10.0;
-  double nu = 0.3;
-  Constant mu(E / (2*(1 + nu)));
-  Constant lambda(E*nu / ((1 + nu)*(1 - 2*nu)));
+  const double E  = 10.0;
+  const double nu = 0.3;
+  Constant mu(E/(2*(1 + nu)));
+  Constant lambda(E*nu/((1 + nu)*(1 - 2*nu)));
 
-  // Create forms
-  HyperElasticity::BilinearForm a(V, V);
-  a.mu = mu; a.lmbda = lambda;
-  a.u = u;
-  HyperElasticity::LinearForm L(V);
-  L.mu = mu; L.lmbda = lambda; L.B = B; L.T = T;
-  L.u = u;
+  // Create (linear) form defining (nonlinear) variational problem
+  HyperElasticity::LinearForm F(V);
+  F.mu = mu; F.lmbda = lambda; F.B = B; F.T = T; F.u = u;
 
-  // Solve nonlinear variational problem
-  VariationalProblem problem(a, L, bcs, true);
+  // Create jacobian dF = F' (for use in nonlinear solver).
+  HyperElasticity::BilinearForm dF(V, V);
+  dF.mu = mu; dF.lmbda = lambda; dF.u = u;
+
+  // Solve nonlinear variational problem (F(u; v) = 0)
+  VariationalProblem problem(F, dF, bcs);
   problem.solve(u);
 
   // Save solution in VTK format
