@@ -9,7 +9,7 @@ on the unit square with source f given by
 and boundary conditions given by
 
     u(x, y) = 0        for x = 0 or x = 1
-du/dn(x, y) = -sin(5*x) for y = 0 or y = 1
+du/dn(x, y) = sin(5*x) for y = 0 or y = 1
 """
 
 __author__ = "Anders Logg (logg@simula.no)"
@@ -17,11 +17,13 @@ __date__ = "2007-08-16 -- 2010-03-05"
 __copyright__ = "Copyright (C) 2007-2009 Anders Logg"
 __license__  = "GNU LGPL Version 2.1"
 
+# Begin demo
+
 from dolfin import *
 
 # Create mesh and define function space
 mesh = UnitSquare(32, 32)
-V = FunctionSpace(mesh, "CG", 1)
+V = FunctionSpace(mesh, "Lagrange", 1)
 
 # Define Dirichlet boundary (x = 0 or x = 1)
 def boundary(x):
@@ -32,12 +34,12 @@ u0 = Constant(0.0)
 bc = DirichletBC(V, u0, boundary)
 
 # Define variational problem
-v = TestFunction(V)
 u = TrialFunction(V)
+v = TestFunction(V)
 f = Expression("10*exp(-(pow(x[0] - 0.5, 2) + pow(x[1] - 0.5, 2)) / 0.02)")
-g = Expression("-sin(5*x[0])")
-a = inner(grad(v), grad(u))*dx
-L = v*f*dx + v*g*ds
+g = Expression("sin(5*x[0])")
+a = inner(grad(u), grad(v))*dx
+L = f*v*dx + g*v*ds
 
 # Compute solution
 problem = VariationalProblem(a, L, bc)
