@@ -5,25 +5,24 @@
 // Modified by Garth N. Wells, 2010
 //
 // First added:  2009-12-08
-// Last changed: 2010-05-22
+// Last changed: 2010-12-06
 //
 
 #include <vector>
-#include <boost/scoped_array.hpp>
 #include <armadillo>
 
 #include <dolfin/common/Array.h>
-#include <dolfin/log/log.h>
-#include <dolfin/la/GenericVector.h>
-#include <dolfin/mesh/Cell.h>
-#include <dolfin/mesh/BoundaryMesh.h>
-#include <dolfin/mesh/FacetCell.h>
+#include <dolfin/common/Timer.h>
 #include <dolfin/fem/BasisFunction.h>
 #include <dolfin/fem/DofMap.h>
 #include <dolfin/fem/DirichletBC.h>
 #include <dolfin/function/Function.h>
 #include <dolfin/function/FunctionSpace.h>
-#include <dolfin/common/Timer.h>
+#include <dolfin/la/GenericVector.h>
+#include <dolfin/log/log.h>
+#include <dolfin/mesh/BoundaryMesh.h>
+#include <dolfin/mesh/Cell.h>
+#include <dolfin/mesh/FacetCell.h>
 #include "Extrapolation.h"
 
 using namespace dolfin;
@@ -33,7 +32,11 @@ void Extrapolation::extrapolate(Function& w, const Function& v)
 {
   // Using set_local for simplicity here
   not_working_in_parallel("Extrapolation");
-  warning("Extrapolation not fully implemented yet.");
+
+  // Too verbose
+  //info("Extrapolating function: %s --> %s",
+  //     v.function_space().element().signature().c_str(),
+  //     w.function_space().element().signature().c_str());
 
   // Check that the meshes are the same
   if (&w.function_space().mesh() != &v.function_space().mesh())
@@ -56,7 +59,7 @@ void Extrapolation::extrapolate(Function& w, const Function& v)
   coefficients.resize(W.dim());
 
   // Local array for dof indices
-  boost::scoped_array<uint> dofs(new uint[W.dofmap().max_local_dimension()]);
+  std::vector<uint> dofs(W.dofmap().max_cell_dimension());
 
   // Iterate over cells in mesh
   for (CellIterator cell0(mesh); !cell0.end(); ++cell0)
@@ -65,7 +68,7 @@ void Extrapolation::extrapolate(Function& w, const Function& v)
     c0.update(*cell0);
 
     // Tabulate dofs for w on cell and store values
-    W.dofmap().tabulate_dofs(dofs.get(), c0, cell0->index());
+    W.dofmap().tabulate_dofs(&dofs[0], *cell0);
 
     // Compute coefficients on this cell
     uint offset = 0;
@@ -82,11 +85,11 @@ void Extrapolation::compute_coefficients(std::vector<std::vector<double> >& coef
                                          const FunctionSpace& W,
                                          const Cell& cell0,
                                          const ufc::cell& c0,
-                                         const boost::scoped_array<uint>& dofs,
+                                         const std::vector<uint>& dofs,
                                          uint& offset)
 {
   // Call recursively for mixed elements
-  uint num_sub_spaces = V.element().num_sub_elements();
+  const uint num_sub_spaces = V.element().num_sub_elements();
   if (num_sub_spaces > 0)
   {
     for (uint k = 0; k < num_sub_spaces; k++)
@@ -121,11 +124,11 @@ void Extrapolation::compute_coefficients(std::vector<std::vector<double> >& coef
   arma::Col<double> x = arma::solve(A, b);
 
   // Insert resulting coefficients into global coefficient vector
-  for (uint i = 0; i < W.dofmap().local_dimension(c0); ++i)
+  for (uint i = 0; i < W.dofmap().cell_dimension(cell0.index()); ++i)
     coefficients[dofs[i + offset]].push_back(x[i]);
 
   // Increase offset
-  offset += W.dofmap().local_dimension(c0);
+  offset += W.dofmap().cell_dimension(cell0.index());
 }
 //-----------------------------------------------------------------------------
 void Extrapolation::build_unique_dofs(std::set<uint>& unique_dofs,
@@ -161,14 +164,14 @@ void Extrapolation::add_cell_equations(arma::Mat<double>& A,
                                        std::map<uint, uint>& dof2row)
 {
   // Extract coefficents for v on patch cell
-  boost::scoped_array<double> dof_values(new double[V.element().space_dimension()]);
-  v.restrict(dof_values.get(), V.element(), cell1, c1, -1);
+  std::vector<double> dof_values(V.element().space_dimension());
+  v.restrict(&dof_values[0], V.element(), cell1, c1);
 
   // Iterate over given local dofs for V on patch cell
   for (std::map<uint, uint>::iterator it = dof2row.begin(); it!= dof2row.end(); it++)
   {
-    uint i = it->first;
-    uint row = it->second;
+    const uint i = it->first;
+    const uint row = it->second;
 
     // Iterate over basis functions for W on center cell
     for (uint j = 0; j < W.element().space_dimension(); ++j)
@@ -195,13 +198,13 @@ Extrapolation::compute_unique_dofs(const Cell& cell, const ufc::cell& c,
                                    uint& row,
                                    std::set<uint>& unique_dofs)
 {
-  boost::scoped_array<uint> dofs(new uint[V.dofmap().local_dimension(c)]);
-  V.dofmap().tabulate_dofs(dofs.get(), c, cell.index());
+  std::vector<uint> dofs(V.dofmap().cell_dimension(cell.index()));
+  V.dofmap().tabulate_dofs(&dofs[0], cell);
 
   // Data structure for current cell
   std::map<uint, uint> dof2row;
 
-  for (uint i = 0; i < V.dofmap().local_dimension(c); ++i)
+  for (uint i = 0; i < V.dofmap().cell_dimension(cell.index()); ++i)
   {
     // Ignore if this degree of freedom is already considered
     if (unique_dofs.find(dofs[i]) != unique_dofs.end())
@@ -221,7 +224,7 @@ Extrapolation::compute_unique_dofs(const Cell& cell, const ufc::cell& c,
 }
 //-----------------------------------------------------------------------------
 void Extrapolation::average_coefficients(Function& w,
-                                         std::vector<std::vector<double> >& coefficients)
+                               std::vector<std::vector<double> >& coefficients)
 {
   const FunctionSpace& W = w.function_space();
   Array<double> dof_values(W.dim());
