@@ -1,28 +1,39 @@
 // Copyright (C) 2009 Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
-// First added:  2009-12-08
-// Last changed: 2010-04-29
-//
 // Modified by Marie E. Rognes (meg@simula.no) 2010.
+// Modified by Garth N. Wells 2010.
+//
+// First added:  2009-12-08
+// Last changed: 2010-12-28
 
 #ifndef __EXTRAPOLATION_H
 #define __EXTRAPOLATION_H
 
+#include <map>
+#include <set>
 #include <vector>
 
+#include <dolfin/common/types.h>
+
 namespace arma
 {
   template <class T> class Mat;
   template <class T> class Col;
 }
 
+namespace ufc
+{
+  class cell;
+}
+
 namespace dolfin
 {
 
-  class Function;
   class Cell;
   class DirichletBC;
+  class Function;
+  class FunctionSpace;
 
   /// This class implements an algorithm for extrapolating a function
   /// on a given function space from an approximation of that function
@@ -62,7 +73,7 @@ namespace dolfin
                                      const Function&v, const FunctionSpace& V,
                                      const FunctionSpace& W, const Cell& cell0,
                                      const ufc::cell& c0,
-                                     const boost::scoped_array<uint>& dofs,
+                                     const std::vector<uint>& dofs,
                                      uint& offset);
 
     // Add equations for current cell
