@@ -2,14 +2,14 @@
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2009-11-11
-// Last changed: 2010-03-04
+// Last changed: 2011-02-16
 
 #ifndef __TIME_SERIES_H
 #define __TIME_SERIES_H
 
 #include <string>
-#include <dolfin/common/Variable.h>
 #include <dolfin/common/Array.h>
+#include <dolfin/common/Variable.h>
 
 namespace dolfin
 {
@@ -61,7 +61,8 @@ namespace dolfin
     void clear();
 
     /// Return filename for data
-    static std::string filename_data(std::string series_name, std::string type_name, uint index);
+    static std::string filename_data(std::string series_name,
+                                     std::string type_name, uint index);
 
     /// Return filename for times
     static std::string filename_times(std::string series_name, std::string type_name);
@@ -71,6 +72,9 @@ namespace dolfin
 
   private:
 
+    // Check if values are strictly increasing
+    static bool increasing(const std::vector<double>& times);
+
     // Name of series
     std::string _name;
 
@@ -81,6 +85,9 @@ namespace dolfin
     // True if series has been cleared
     bool _cleared;
 
+    // True if series is reversed
+    bool _reversed;
+
   };
 
 }
