@@ -4,7 +4,7 @@
 // Modified by Anders Logg, 2010.
 //
 // First added:  2009-12-06
-// Last changed: 2010-05-18
+// Last changed: 2010-08-11
 
 #ifndef __DOLFIN_ARRAY_H
 #define __DOLFIN_ARRAY_H
@@ -44,7 +44,7 @@ namespace dolfin
     Array(uint N, boost::shared_array<T> x) : _size(N), x(x) {}
 
     /// Construct array from a pointer. Array will not take ownership.
-    Array(uint N, T* x) : _size(N), x(boost::shared_array<T>(x, NoDeleter<T>())) {}
+    Array(uint N, T* x) : _size(N), x(boost::shared_array<T>(x, NoDeleter())) {}
 
     /// Destructor
     ~Array() {}
@@ -61,16 +61,29 @@ namespace dolfin
     void update(uint N, T* _x)
     {
       _size = N;
-      x.reset(_x, NoDeleter<T>());
+      x.reset(_x, NoDeleter());
     }
 
-    /// Return informal string representation (pretty-print)
+    /// Return informal string representation (pretty-print).
+    /// Note that the Array class is not a subclass of Variable (for
+    /// efficiency) which means that one needs to call str() directly
+    /// instead of using the info() function on Array objects.
     std::string str(bool verbose) const
     {
       std::stringstream s;
+
       if (verbose)
-        warning("Verbose output for Array<Y> not implemented.");
-      s << "<Array<T> of size " << size() << ">";
+      {
+        s << str(false) << std::endl << std::endl;
+
+        for (uint i = 0; i < size(); i++)
+          s << i << ": " << (*this)[i] << std::endl;
+      }
+      else
+      {
+        s << "<Array<T> of size " << size() << ">";
+      }
+
       return s.str();
     }
 
@@ -119,6 +132,14 @@ namespace dolfin
     T& operator[] (uint i)
     { assert(i < _size); return x[i]; }
 
+    /// Assign value to all entries
+    const Array<T>& operator= (T& x)
+    {
+      for (uint i = 0; i < _size; ++i)
+        this->x[i] = x;
+      return *this;
+    }
+
     /// Return pointer to data (const version)
     const boost::shared_array<T> data() const
     { return x; }
