@@ -1,21 +1,23 @@
-// Copyright (C) 2009 Garth N. Wellls.
+// Copyright (C) 2009-2011 Garth N. Wellls.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2009-08-09
-// Last changed:
+// Last changed: 2011-01-05
 
 #ifndef __DOLFIN_SET_H
 #define __DOLFIN_SET_H
 
 #include <algorithm>
 #include <vector>
+
 #include "dolfin/common/types.h"
 
 namespace dolfin
 {
 
-  /// This is a std::set like data structure. It is not ordered and it is based
-  /// a std::vector. It can be faster than a std::set for some cases.
+  /// This is a set-like data structure. It is not ordered and it is based
+  /// a std::vector. It uses linear search, and can be faster than std::set
+  // and boost::unordered_set in some cases.
 
   template<class T>
   class Set
@@ -58,6 +60,17 @@ namespace dolfin
         return false;
     }
 
+    /// Insert entries
+    template <class InputIt>
+    void insert(const InputIt first, const InputIt last)
+    {
+      for (InputIt position = first; position != last; ++position)
+      {
+        if (std::find(_x.begin(), _x.end(), *position) == _x.end())
+          _x.push_back(*position);
+      }
+    }
+
     const_iterator begin() const
     { return _x.begin(); }
 
@@ -84,10 +97,6 @@ namespace dolfin
     void clear()
     { _x.clear(); }
 
-    /// Resize set
-    void resize(uint n)
-    { _x.resize(n); }
-
     /// Index the nth entry in the set
     T operator[](uint n) const
     { return _x[n]; }
