@@ -2,7 +2,7 @@
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2009-08-09
-// Last changed: 2009-11-10
+// Last changed: 2010-11-18
 
 #include <boost/functional/hash.hpp>
 #include <cstdlib>
@@ -27,6 +27,13 @@ std::string dolfin::indent(std::string block)
   return s.str();
 }
 //-----------------------------------------------------------------------------
+std::string dolfin::to_string(int n)
+{
+  std::stringstream s;
+  s << n;
+  return s.str();
+}
+//-----------------------------------------------------------------------------
 std::string dolfin::to_string(const double* x, uint n)
 {
   std::stringstream s;
