@@ -1,9 +1,8 @@
-
-// Copyright (C) 2009 Anders Logg.
+// Copyright (C) 2009-2010 Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2009-08-09
-// Last changed: 2009-11-10
+// Last changed: 2010-11-18
 
 #ifndef __UTILS_H
 #define __UTILS_H
@@ -17,6 +16,9 @@ namespace dolfin
   /// Indent string block
   std::string indent(std::string block);
 
+  /// Return string representation of int
+  std::string to_string(int n);
+
   /// Return string representation of given array
   std::string to_string(const double* x, uint n);
 
