@@ -1,4 +1,4 @@
-// Copyright (C) 2007-2009 Anders Logg.
+// Copyright (C) 2007-2010 Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Garth N. Wells, 2007-2009
@@ -6,10 +6,11 @@
 // Modified by Kent-Andre Mardal, 2008
 //
 // First added:  2007-01-17
-// Last changed: 2010-02-27
+// Last changed: 2011-02-21
 
 #include <dolfin/log/dolfin_log.h>
 #include <dolfin/common/Timer.h>
+#include <dolfin/parameter/GlobalParameters.h>
 #include <dolfin/la/GenericTensor.h>
 #include <dolfin/mesh/Mesh.h>
 #include <dolfin/mesh/Cell.h>
@@ -22,9 +23,10 @@
 #include "GenericDofMap.h"
 #include "Form.h"
 #include "UFC.h"
+#include "FiniteElement.h"
+#include "OpenMpAssembler.h"
 #include "AssemblerTools.h"
 #include "Assembler.h"
-#include "FiniteElement.h"
 
 using namespace dolfin;
 
@@ -53,7 +55,7 @@ void Assembler::assemble(GenericTensor& A,
 
   // Extract cell domains
   MeshFunction<uint>* cell_domains = 0;
-  if (a.ufc_form().num_cell_integrals() > 0)
+  if (a.ufc_form().num_cell_domains() > 0)
   {
     cell_domains = new MeshFunction<uint>(mesh, mesh.topology().dim(), 1);
     sub_domain.mark(*cell_domains, 0);
@@ -61,8 +63,8 @@ void Assembler::assemble(GenericTensor& A,
 
   // Extract facet domains
   MeshFunction<uint>* facet_domains = 0;
-  if (a.ufc_form().num_exterior_facet_integrals() > 0 ||
-      a.ufc_form().num_interior_facet_integrals() > 0)
+  if (a.ufc_form().num_exterior_facet_domains() > 0 ||
+      a.ufc_form().num_interior_facet_domains() > 0)
   {
     facet_domains = new MeshFunction<uint>(mesh, mesh.topology().dim() - 1, 1);
     sub_domain.mark(*facet_domains, 0);
@@ -84,8 +86,31 @@ void Assembler::assemble(GenericTensor& A,
                          bool reset_sparsity,
                          bool add_values)
 {
-  // Note the importance of treating empty mesh functions as null pointers
-  // for the PyDOLFIN interface.
+  // All assembler functions above end up calling this function, which
+  // in turn calls the assembler functions below to assemble over
+  // cells, exterior and interior facets. Note the importance of
+  // treating empty mesh functions as null pointers for the PyDOLFIN
+  // interface.
+
+  // Check whether we should call the multi-core assembler
+  #ifdef HAS_OPENMP
+  const uint num_threads = parameters["num_threads"];
+  if (num_threads > 0)
+  {
+    //MulticoreAssembler::assemble(A, a,
+    //                             cell_domains,
+    //                             exterior_facet_domains,
+    //                             interior_facet_domains,
+    //                             reset_sparsity, add_values, num_threads);
+
+    OpenMpAssembler::assemble(A, a,
+                              cell_domains,
+                              exterior_facet_domains,
+                              interior_facet_domains,
+                              reset_sparsity, add_values);
+    return;
+  }
+  #endif
 
   // Check form
   AssemblerTools::check(a);
@@ -94,12 +119,12 @@ void Assembler::assemble(GenericTensor& A,
   UFC ufc(a);
 
   // Gather off-process coefficients
-  const std::vector<const GenericFunction*> coefficients = a.coefficients();
+  const std::vector<boost::shared_ptr<const GenericFunction> > coefficients = a.coefficients();
   for (uint i = 0; i < coefficients.size(); ++i)
     coefficients[i]->gather();
 
   // Initialize global tensor
-  AssemblerTools::init_global_tensor(A, a, ufc, reset_sparsity, add_values);
+  AssemblerTools::init_global_tensor(A, a, reset_sparsity, add_values);
 
   // Assemble over cells
   assemble_cells(A, a, ufc, cell_domains, 0);
@@ -110,7 +135,7 @@ void Assembler::assemble(GenericTensor& A,
   // Assemble over interior facets
   assemble_interior_facets(A, a, ufc, interior_facet_domains, 0);
 
-  // Finalise assembly of global tensor
+  // Finalize assembly of global tensor
   A.apply("add");
 }
 //-----------------------------------------------------------------------------
@@ -121,15 +146,26 @@ void Assembler::assemble_cells(GenericTensor& A,
                                std::vector<double>* values)
 {
   // Skip assembly if there are no cell integrals
-  if (ufc.form.num_cell_integrals() == 0)
+  if (ufc.form.num_cell_domains() == 0)
     return;
   Timer timer("Assemble cells");
 
   // Extract mesh
   const Mesh& mesh = a.mesh();
 
+  // Form rank
+  const uint form_rank = ufc.form.rank();
+
+  // Collect pointers to dof maps
+  std::vector<const GenericDofMap*> dofmaps;
+  for (uint i = 0; i < form_rank; ++i)
+    dofmaps.push_back(&a.function_space(i)->dofmap());
+
+  // Vector to hold dof map for a cell
+  std::vector<const std::vector<uint>* > dofs(form_rank);
+
   // Cell integral
-  ufc::cell_integral* integral = ufc.cell_integrals[0];
+  ufc::cell_integral* integral = ufc.cell_integrals[0].get();
 
   // Assemble over cells
   Progress p(AssemblerTools::progress_message(A.rank(), "cells"), mesh.num_cells());
@@ -139,35 +175,33 @@ void Assembler::assemble_cells(GenericTensor& A,
     if (domains && domains->size() > 0)
     {
       const uint domain = (*domains)[*cell];
-      if (domain < ufc.form.num_cell_integrals())
-        integral = ufc.cell_integrals[domain];
+      if (domain < ufc.form.num_cell_domains())
+        integral = ufc.cell_integrals[domain].get();
       else
         continue;
     }
 
     // Skip integral if zero
-    if (!integral) continue;
+    if (!integral)
+      continue;
 
     // Update to current cell
     ufc.update(*cell);
 
-    // Tabulate dofs for each dimension
-    for (uint i = 0; i < ufc.form.rank(); i++)
-      a.function_space(i)->dofmap().tabulate_dofs(ufc.dofs[i], ufc.cell, cell->index());
+    // Get local-to-global dof maps for cell
+    for (uint i = 0; i < form_rank; ++i)
+      dofs[i] = &(dofmaps[i]->cell_dofs(cell->index()));
 
     // Tabulate cell tensor
     integral->tabulate_tensor(ufc.A.get(), ufc.w, ufc.cell);
 
-    // Add entries to global tensor
+    // Add entries to global tensor. Either store values cell-by-cell
+    // (currently only available for functionals)
     if (values && ufc.form.rank() == 0)
       (*values)[cell->index()] = ufc.A[0];
     else
-    {
-      // Get local dimensions
-      for (uint i = 0; i < ufc.form.rank(); i++)
-        ufc.local_dimensions[i] = a.function_space(i)->dofmap().local_dimension(ufc.cell);
-      A.add(ufc.A.get(), ufc.local_dimensions.get(), ufc.dofs);
-    }
+      A.add(ufc.A.get(), dofs);
+
     p++;
   }
 }
@@ -179,15 +213,26 @@ void Assembler::assemble_exterior_facets(GenericTensor& A,
                                          std::vector<double>* values)
 {
   // Skip assembly if there are no exterior facet integrals
-  if (ufc.form.num_exterior_facet_integrals() == 0)
+  if (ufc.form.num_exterior_facet_domains() == 0)
     return;
   Timer timer("Assemble exterior facets");
 
   // Extract mesh
   const Mesh& mesh = a.mesh();
 
+  // Form rank
+  const uint form_rank = ufc.form.rank();
+
+  // Collect pointers to dof maps
+  std::vector<const GenericDofMap*> dofmaps;
+  for (uint i = 0; i < form_rank; ++i)
+    dofmaps.push_back(&a.function_space(i)->dofmap());
+
+  // Vector to hold dof map for a cell
+  std::vector<const std::vector<uint>* > dofs(form_rank);
+
   // Exterior facet integral
-  ufc::exterior_facet_integral* integral = ufc.exterior_facet_integrals[0];
+  const ufc::exterior_facet_integral* integral = ufc.exterior_facet_integrals[0].get();
 
   // Compute facets and facet - cell connectivity if not already computed
   const uint D = mesh.topology().dim();
@@ -196,7 +241,7 @@ void Assembler::assemble_exterior_facets(GenericTensor& A,
   assert(mesh.ordered());
 
   // Extract exterior (non shared) facets markers
-  MeshFunction<uint>* exterior_facets = mesh.data().mesh_function("exterior facets");
+  const MeshFunction<uint>* exterior_facets = mesh.data().mesh_function("exterior facets");
 
   // Assemble over exterior facets (the cells of the boundary)
   Progress p(AssemblerTools::progress_message(A.rank(), "exterior facets"), mesh.num_facets());
@@ -213,18 +258,19 @@ void Assembler::assemble_exterior_facets(GenericTensor& A,
     if (domains && domains->size() > 0)
     {
       const uint domain = (*domains)[*facet];
-      if (domain < ufc.form.num_exterior_facet_integrals())
-        integral = ufc.exterior_facet_integrals[domain];
+      if (domain < ufc.form.num_exterior_facet_domains())
+        integral = ufc.exterior_facet_integrals[domain].get();
       else
         continue;
     }
 
     // Skip integral if zero
-    if (!integral) continue;
+    if (!integral)
+      continue;
 
     // Get mesh cell to which mesh facet belongs (pick first, there is only one)
-    assert(facet->num_entities(mesh.topology().dim()) == 1);
-    Cell mesh_cell(mesh, facet->entities(mesh.topology().dim())[0]);
+    assert(facet->num_entities(D) == 1);
+    Cell mesh_cell(mesh, facet->entities(D)[0]);
 
     // Get local index of facet with respect to the cell
     const uint local_facet = mesh_cell.index(*facet);
@@ -232,19 +278,15 @@ void Assembler::assemble_exterior_facets(GenericTensor& A,
     // Update to current cell
     ufc.update(mesh_cell, local_facet);
 
-    // Tabulate dofs for each dimension
-    for (uint i = 0; i < ufc.form.rank(); i++)
-      a.function_space(i)->dofmap().tabulate_dofs(ufc.dofs[i], ufc.cell, mesh_cell.index());
+    // Get local-to-global dof maps for cell
+    for (uint i = 0; i < form_rank; ++i)
+      dofs[i] = &(dofmaps[i]->cell_dofs(mesh_cell.index()));
 
     // Tabulate exterior facet tensor
     integral->tabulate_tensor(ufc.A.get(), ufc.w, ufc.cell, local_facet);
 
-    // Get local dimensions
-    for (uint i = 0; i < ufc.form.rank(); i++)
-      ufc.local_dimensions[i] = a.function_space(i)->dofmap().local_dimension(ufc.cell);
-
     // Add entries to global tensor
-    A.add(ufc.A.get(), ufc.local_dimensions.get(), ufc.dofs);
+    A.add(ufc.A.get(), dofs);
 
     p++;
   }
@@ -257,7 +299,7 @@ void Assembler::assemble_interior_facets(GenericTensor& A,
                                          std::vector<double>* values)
 {
   // Skip assembly if there are no interior facet integrals
-  if (ufc.form.num_interior_facet_integrals() == 0)
+  if (ufc.form.num_interior_facet_domains() == 0)
     return;
 
   Timer timer("Assemble interior facets");
@@ -265,19 +307,33 @@ void Assembler::assemble_interior_facets(GenericTensor& A,
   // Extract mesh and coefficients
   const Mesh& mesh = a.mesh();
 
+  // Form rank
+  const uint form_rank = ufc.form.rank();
+
+  // Collect pointers to dof maps
+  std::vector<const GenericDofMap*> dofmaps;
+  for (uint i = 0; i < form_rank; ++i)
+    dofmaps.push_back(&a.function_space(i)->dofmap());
+
+  // Vector to hold dofs for cells
+  std::vector<std::vector<uint> > macro_dofs(form_rank);
+
   // Interior facet integral
-  ufc::interior_facet_integral* integral = ufc.interior_facet_integrals[0];
+  const ufc::interior_facet_integral* integral = ufc.interior_facet_integrals[0].get();
 
   // Compute facets and facet - cell connectivity if not already computed
-  mesh.init(mesh.topology().dim() - 1);
-  mesh.init(mesh.topology().dim() - 1, mesh.topology().dim());
+  const uint D = mesh.topology().dim();
+  mesh.init(D - 1);
+  mesh.init(D - 1, D);
   assert(mesh.ordered());
 
   // Get interior facet directions (if any)
   MeshFunction<uint>* facet_orientation = mesh.data().mesh_function("facet orientation");
-  if (facet_orientation && facet_orientation->dim() != mesh.topology().dim() - 1)
+  if (facet_orientation && facet_orientation->dim() != D - 1)
+  {
     error("Expecting facet orientation to be defined on facets (not dimension %d).",
           facet_orientation);
+  }
 
   // Assemble over interior facets (the facets of the mesh)
   Progress p(AssemblerTools::progress_message(A.rank(), "interior facets"), mesh.num_facets());
@@ -294,14 +350,15 @@ void Assembler::assemble_interior_facets(GenericTensor& A,
     if (domains && domains->size() > 0)
     {
       const uint domain = (*domains)[*facet];
-      if (domain < ufc.form.num_interior_facet_integrals())
-        integral = ufc.interior_facet_integrals[domain];
+      if (domain < ufc.form.num_interior_facet_domains())
+        integral = ufc.interior_facet_integrals[domain].get();
       else
         continue;
     }
 
     // Skip integral if zero
-    if (!integral) continue;
+    if (!integral)
+      continue;
 
     // Get cells incident with facet
     std::pair<const Cell, const Cell> cells = facet->adjacent_cells(facet_orientation);
@@ -316,24 +373,26 @@ void Assembler::assemble_interior_facets(GenericTensor& A,
     ufc.update(cell0, local_facet0, cell1, local_facet1);
 
     // Tabulate dofs for each dimension on macro element
-    for (uint i = 0; i < ufc.form.rank(); i++)
+    for (uint i = 0; i < form_rank; i++)
     {
-      const uint offset = a.function_space(i)->dofmap().local_dimension(ufc.cell0);
-      a.function_space(i)->dofmap().tabulate_dofs(ufc.macro_dofs[i],          ufc.cell0, cell0.index());
-      a.function_space(i)->dofmap().tabulate_dofs(ufc.macro_dofs[i] + offset, ufc.cell1, cell1.index());
+      // Get dofs for each cell
+      const std::vector<uint>& cell_dofs0 = dofmaps[i]->cell_dofs(cell0.index());
+      const std::vector<uint>& cell_dofs1 = dofmaps[i]->cell_dofs(cell1.index());
+
+      // Create space in macro dof vector
+      macro_dofs[i].resize(cell_dofs0.size() + cell_dofs1.size());
+
+      // Copy cell dofs into macro dof vector
+      std::copy(cell_dofs0.begin(), cell_dofs0.end(), macro_dofs[i].begin());
+      std::copy(cell_dofs1.begin(), cell_dofs1.end(), macro_dofs[i].begin() + cell_dofs0.size());
     }
 
     // Tabulate exterior interior facet tensor on macro element
     integral->tabulate_tensor(ufc.macro_A.get(), ufc.macro_w, ufc.cell0, ufc.cell1,
                               local_facet0, local_facet1);
 
-    // Get local dimensions
-    for (uint i = 0; i < ufc.form.rank(); i++)
-      ufc.macro_local_dimensions[i] = a.function_space(i)->dofmap().local_dimension(ufc.cell0)
-                                    + a.function_space(i)->dofmap().local_dimension(ufc.cell1);
-
     // Add entries to global tensor
-    A.add(ufc.macro_A.get(), ufc.macro_local_dimensions.get(), ufc.macro_dofs);
+    A.add(ufc.macro_A.get(), macro_dofs);
 
     p++;
   }
