@@ -6,25 +6,25 @@
 // Modified by Kent-Andre Mardal, 2008
 //
 // First added:  2007-01-17
-// Last changed: 2010-02-13
+// Last changed: 2011-02-21
 
 #include <boost/scoped_ptr.hpp>
-#include <dolfin/log/dolfin_log.h>
 #include <dolfin/common/Timer.h>
+#include <dolfin/function/FunctionSpace.h>
+#include <dolfin/function/GenericFunction.h>
 #include <dolfin/la/GenericTensor.h>
 #include <dolfin/la/SparsityPattern.h>
 #include <dolfin/la/LinearAlgebraFactory.h>
+#include <dolfin/log/dolfin_log.h>
+#include <dolfin/common/MPI.h>
 #include <dolfin/mesh/Mesh.h>
 #include <dolfin/mesh/Cell.h>
-#include <dolfin/function/FunctionSpace.h>
-#include <dolfin/function/GenericFunction.h>
-#include "GenericDofMap.h"
-#include "Form.h"
-#include "UFC.h"
-#include "AssemblerTools.h"
-#include "SparsityPatternBuilder.h"
-#include "DirichletBC.h"
+
 #include "FiniteElement.h"
+#include "Form.h"
+#include "GenericDofMap.h"
+#include "SparsityPatternBuilder.h"
+#include "AssemblerTools.h"
 
 using namespace dolfin;
 
@@ -36,7 +36,7 @@ void AssemblerTools::check(const Form& a)
 
   // Extract mesh and coefficients
   const Mesh& mesh = a.mesh();
-  const std::vector<const GenericFunction*> coefficients = a.coefficients();
+  const std::vector<boost::shared_ptr<const GenericFunction> > coefficients = a.coefficients();
 
   // Check that we get the correct number of coefficients
   if (coefficients.size() != a.ufc_form().num_coefficients())
@@ -48,7 +48,7 @@ void AssemblerTools::check(const Form& a)
   // Check that all coefficients have valid value dimensions
   for (uint i = 0; i < coefficients.size(); ++i)
   {
-    if(!coefficients[i])
+    if (!coefficients[i])
       error("Got NULL Function as coefficient %d.", i);
 
     // auto_ptr deletes its object when it exits its scope
@@ -93,16 +93,18 @@ Did you forget to specify the value dimension correctly in an Expression sub cla
     error("Unable to assemble, mesh is not correctly ordered (consider calling mesh.order()).");
 }
 //-----------------------------------------------------------------------------
-void AssemblerTools::init_global_tensor(GenericTensor& A,
-                                        const Form& a,
-                                        UFC& ufc,
-                                        bool reset_sparsity,
-                                        bool add_values)
+void AssemblerTools::init_global_tensor(GenericTensor& A, const Form& a,
+                                        bool reset_sparsity, bool add_values)
 {
   // Check that we should not add values
   if (reset_sparsity && add_values)
     error("Can not add values when the sparsity pattern is reset");
 
+  // Get dof maps
+  std::vector<const GenericDofMap*> dofmaps;
+  for (uint i = 0; i < a.rank(); ++i)
+    dofmaps.push_back(&(a.function_space(i)->dofmap()));
+
   if (reset_sparsity)
   {
     // Build sparsity pattern
@@ -110,15 +112,11 @@ void AssemblerTools::init_global_tensor(GenericTensor& A,
     boost::scoped_ptr<GenericSparsityPattern> sparsity_pattern(A.factory().create_pattern());
     if (sparsity_pattern)
     {
-      // Get dof maps
-      std::vector<const GenericDofMap*> dof_maps;
-      for (uint i = 0; i < ufc.form.rank(); ++i)
-        dof_maps.push_back(&(a.function_space(i)->dofmap()));
 
       // Build sparsity pattern
-      SparsityPatternBuilder::build(*sparsity_pattern, a.mesh(), dof_maps,
-                                    a.ufc_form().num_cell_integrals(),
-                                    a.ufc_form().num_interior_facet_integrals());
+      SparsityPatternBuilder::build(*sparsity_pattern, a.mesh(), dofmaps,
+                                    a.ufc_form().num_cell_domains(),
+                                    a.ufc_form().num_interior_facet_domains());
     }
     t0.stop();
 
@@ -128,7 +126,22 @@ void AssemblerTools::init_global_tensor(GenericTensor& A,
       A.init(*sparsity_pattern);
     else
     {
-      A.resize(ufc.form.rank(), ufc.global_dimensions.get());
+      // Build data structure for intialising sparsity pattern
+      std::vector<uint> global_dimensions(a.rank());
+      std::vector<std::pair<uint, uint> > local_range(a.rank());
+      std::vector<const boost::unordered_map<uint, uint>* > off_process_owner(a.rank());
+      for (uint i = 0; i < a.rank(); i++)
+      {
+        assert(dofmaps[i]);
+        global_dimensions[i] = dofmaps[i]->global_dimension();
+        local_range[i]       = dofmaps[i]->ownership_range();
+        off_process_owner[i] = &(dofmaps[i]->off_process_owner());
+      }
+
+      // Create and build sparsity pattern
+      SparsityPattern _sparsity_pattern;
+      _sparsity_pattern.init(global_dimensions, local_range, off_process_owner);
+      A.init(_sparsity_pattern);
       A.zero();
     }
     t1.stop();
@@ -137,6 +150,15 @@ void AssemblerTools::init_global_tensor(GenericTensor& A,
     Timer t2("Delete sparsity");
     t2.stop();
   }
+  else
+  {
+    // If tensor is not reset, check that dimensions are correct
+    for (uint i = 0; i < a.rank(); ++i)
+    {
+      if (A.size(i) != dofmaps[i]-> global_dimension())
+        error("Reset of tensor in assembly not requested, but dim %d of tesnor does not match form.", i);
+    }
+  }
 
   if (!add_values)
     A.zero();
