@@ -6,9 +6,8 @@
 // Modified by Johan Hake, 2009
 //
 // First added:  2007-04-10
-// Last changed: 2010-05-14
+// Last changed: 2011-02-17
 
-#include <boost/scoped_ptr.hpp>
 #include <boost/assign/list_of.hpp>
 
 #include <dolfin/common/constants.h>
@@ -16,6 +15,7 @@
 #include <dolfin/common/NoDeleter.h>
 #include <dolfin/function/GenericFunction.h>
 #include <dolfin/function/FunctionSpace.h>
+#include <dolfin/function/Constant.h>
 #include <dolfin/log/log.h>
 #include <dolfin/mesh/Mesh.h>
 #include <dolfin/mesh/MeshData.h>
@@ -36,16 +36,16 @@
 
 using namespace dolfin;
 
-const std::set<std::string> DirichletBC::methods = boost::assign::list_of("topological")("geometric")("pointwise");
+const std::set<std::string> DirichletBC::methods
+            = boost::assign::list_of("topological")("geometric")("pointwise");
 
 //-----------------------------------------------------------------------------
-DirichletBC::DirichletBC(const FunctionSpace& V,
-                         const GenericFunction& g,
-                         const SubDomain& sub_domain,
-                         std::string method)
+DirichletBC::DirichletBC(const FunctionSpace& V, const GenericFunction& g,
+                         const SubDomain& sub_domain, std::string method)
   : BoundaryCondition(V),
+    Hierarchical<DirichletBC>(*this),
     g(reference_to_no_delete_pointer(g)),
-    method(method), user_sub_domain(reference_to_no_delete_pointer(sub_domain))
+    _method(method), user_sub_domain(reference_to_no_delete_pointer(sub_domain))
 {
   check();
   parameters = default_parameters();
@@ -57,22 +57,21 @@ DirichletBC::DirichletBC(boost::shared_ptr<const FunctionSpace> V,
                          boost::shared_ptr<const SubDomain> sub_domain,
                          std::string method)
   : BoundaryCondition(V),
-    g(g),
-    method(method), user_sub_domain(sub_domain)
+    Hierarchical<DirichletBC>(*this),
+    g(g), _method(method), user_sub_domain(sub_domain)
 {
   check();
   parameters = default_parameters();
   init_from_sub_domain(user_sub_domain);
 }
 //-----------------------------------------------------------------------------
-DirichletBC::DirichletBC(const FunctionSpace& V,
-                         const GenericFunction& g,
+DirichletBC::DirichletBC(const FunctionSpace& V, const GenericFunction& g,
                          const MeshFunction<uint>& sub_domains,
-                         uint sub_domain,
-                         std::string method)
+                         uint sub_domain, std::string method)
   : BoundaryCondition(V),
+    Hierarchical<DirichletBC>(*this),
     g(reference_to_no_delete_pointer(g)),
-    method(method), user_sub_domain(static_cast<SubDomain*>(0))
+    _method(method)
 {
   check();
   parameters = default_parameters();
@@ -85,21 +84,19 @@ DirichletBC::DirichletBC(boost::shared_ptr<const FunctionSpace> V,
                          uint sub_domain,
                          std::string method)
   : BoundaryCondition(V),
-    g(g),
-    method(method), user_sub_domain(static_cast<SubDomain*>(0))
+    Hierarchical<DirichletBC>(*this),
+    g(g), _method(method)
 {
   check();
   parameters = default_parameters();
   init_from_mesh_function(sub_domains, sub_domain);
 }
 //-----------------------------------------------------------------------------
-DirichletBC::DirichletBC(const FunctionSpace& V,
-                         const GenericFunction& g,
-                         uint sub_domain,
-                         std::string method)
+DirichletBC::DirichletBC(const FunctionSpace& V, const GenericFunction& g,
+                         uint sub_domain, std::string method)
   : BoundaryCondition(V),
-    g(reference_to_no_delete_pointer(g)),
-    method(method), user_sub_domain(static_cast<SubDomain*>(0))
+    Hierarchical<DirichletBC>(*this),
+    g(reference_to_no_delete_pointer(g)), _method(method)
 {
   check();
   parameters = default_parameters();
@@ -108,24 +105,22 @@ DirichletBC::DirichletBC(const FunctionSpace& V,
 //-----------------------------------------------------------------------------
 DirichletBC::DirichletBC(boost::shared_ptr<const FunctionSpace> V,
                          boost::shared_ptr<const GenericFunction> g,
-                         uint sub_domain,
-                         std::string method)
+                         uint sub_domain, std::string method)
   : BoundaryCondition(V),
-    g(g),
-    method(method), user_sub_domain(static_cast<SubDomain*>(0))
+    Hierarchical<DirichletBC>(*this),
+    g(g), _method(method)
 {
   check();
   parameters = default_parameters();
   init_from_mesh(sub_domain);
 }
 //-----------------------------------------------------------------------------
-DirichletBC::DirichletBC(const FunctionSpace& V,
-                         const GenericFunction& g,
+DirichletBC::DirichletBC(const FunctionSpace& V, const GenericFunction& g,
                          const std::vector<std::pair<uint, uint> >& markers,
                          std::string method)
   : BoundaryCondition(V),
-    g(reference_to_no_delete_pointer(g)),
-    method(method), user_sub_domain(static_cast<SubDomain*>(0)),
+    Hierarchical<DirichletBC>(*this),
+    g(reference_to_no_delete_pointer(g)), _method(method),
     facets(markers)
 {
   check();
@@ -137,16 +132,16 @@ DirichletBC::DirichletBC(boost::shared_ptr<const FunctionSpace> V,
                          const std::vector<std::pair<uint, uint> >& markers,
                          std::string method)
   : BoundaryCondition(V),
-    g(g),
-    method(method), user_sub_domain(static_cast<SubDomain*>(0)),
-    facets(markers)
+    Hierarchical<DirichletBC>(*this),
+    g(g), _method(method), facets(markers)
 {
   check();
   parameters = default_parameters();
 }
 //-----------------------------------------------------------------------------
 DirichletBC::DirichletBC(const DirichletBC& bc)
-  : BoundaryCondition(bc._function_space)
+  : BoundaryCondition(bc._function_space),
+    Hierarchical<DirichletBC>(*this)
 {
   // Set default parameters
   parameters = default_parameters();
@@ -163,10 +158,13 @@ const DirichletBC& DirichletBC::operator= (const DirichletBC& bc)
 {
   _function_space = bc._function_space;
   g = bc.g;
-  method = bc.method;
+  _method = bc._method;
   user_sub_domain = bc.user_sub_domain;
   facets = bc.facets;
 
+  // Call assignment operator for base class
+  Hierarchical<DirichletBC>::operator=(bc);
+
   return *this;
 }
 //-----------------------------------------------------------------------------
@@ -197,32 +195,115 @@ void DirichletBC::apply(GenericMatrix& A,
   apply(&A, &b, &x);
 }
 //-----------------------------------------------------------------------------
+void DirichletBC::get_boundary_values(Map& boundary_values,
+                         std::string method) const
+{
+  // Create local data
+  BoundaryCondition::LocalData data(*_function_space);
+
+  // Compute dofs and values
+  compute_bc(boundary_values, data, method);
+}
+//-----------------------------------------------------------------------------
 void DirichletBC::zero(GenericMatrix& A) const
 {
   // A map to hold the mapping from boundary dofs to boundary values
-  std::map<uint, double> boundary_values;
+  Map boundary_values;
 
   // Create local data for application of boundary conditions
   BoundaryCondition::LocalData data(*_function_space);
 
   // Compute dofs and values
-  compute_bc(boundary_values, data);
+  compute_bc(boundary_values, data, _method);
 
   // Copy boundary value data to arrays
-  uint* dofs = new uint[boundary_values.size()];
-  std::map<uint, double>::const_iterator boundary_value;
+  std::vector<uint> dofs(boundary_values.size());
+  Map::const_iterator bv;
   uint i = 0;
-  for (boundary_value = boundary_values.begin(); boundary_value != boundary_values.end(); ++boundary_value)
-    dofs[i++] = boundary_value->first;
+  for (bv = boundary_values.begin(); bv != boundary_values.end(); ++bv)
+    dofs[i++] = bv->first;
 
   // Modify linear system (A_ii = 1)
-  A.zero(boundary_values.size(), dofs);
+  A.zero(boundary_values.size(), &dofs[0]);
 
   // Finalise changes to A
   A.apply("insert");
+}
+//-----------------------------------------------------------------------------
+void DirichletBC::zero_columns(GenericMatrix& A, GenericVector& b, double diag_val) const
+{
+  Map bv_map;
+  get_boundary_values(bv_map, _method);
 
-  // Clear temporary arrays
-  delete [] dofs;
+  // Create lookup table of dofs
+  //const uint nrows = A.size(0); // should be equal to b.size()
+  const uint ncols = A.size(1); // should be equal to max possible dof+1
+
+  std::pair<uint,uint> rows = A.local_range(0);
+  //std::pair<uint,uint> cols = A.local_range(1);
+
+  std::vector<char> is_bc_dof(ncols);
+  std::vector<double> bc_dof_val(ncols);
+  for (Map::const_iterator bv = bv_map.begin();  bv != bv_map.end();  ++bv)
+  {
+    is_bc_dof[bv->first] = 1;
+    bc_dof_val[bv->first] = bv->second;
+  }
+
+  // Scan through all columns of all rows, setting to zero if is_bc_dof[column]
+  // At the same time, we collect corrections to the RHS
+
+  std::vector<uint>   cols;
+  std::vector<double> vals;
+  std::vector<double> b_vals;
+  std::vector<uint>   b_rows;
+
+  for (uint row=rows.first; row<rows.second; row++)
+  {
+    // If diag_val is nonzero, the matrix is a diagonal block (nrows==ncols),
+    // and we can set the whole BC row
+    if (diag_val != 0.0 && is_bc_dof[row])
+    {
+      A.getrow(row, cols, vals);
+      for (uint j=0; j<cols.size(); j++)
+        vals[j] = (cols[j]==row) * diag_val;
+      A.setrow(row, cols, vals);
+      A.apply("insert");
+      b.setitem(row, bc_dof_val[row]*diag_val);
+    }
+    else // Otherwise, we scan the row for BC columns
+    {
+      A.getrow(row, cols, vals);
+      bool row_changed=false;
+      for (uint j=0; j<cols.size(); j++)
+      {
+        const uint col = cols[j];
+
+        // Skip columns that aren't BC, and entries that are zero
+        if (!is_bc_dof[col] || vals[j] == 0.0)
+          continue;
+
+        // We're going to change the row, so make room for it
+        if (!row_changed)
+        {
+          row_changed = true;
+          b_rows.push_back(row);
+          b_vals.push_back(0.0);
+        }
+
+        b_vals.back() -= bc_dof_val[col]*vals[j];
+        vals[j] = 0.0;
+      }
+      if (row_changed)
+      {
+        A.setrow(row, cols, vals);
+        A.apply("insert");
+      }
+    }
+  }
+
+  b.add(&b_vals.front(), b_rows.size(), &b_rows.front());
+  b.apply("add");
 }
 //-----------------------------------------------------------------------------
 const std::vector<std::pair<dolfin::uint, dolfin::uint> >& DirichletBC::markers()
@@ -230,16 +311,23 @@ const std::vector<std::pair<dolfin::uint, dolfin::uint> >& DirichletBC::markers(
   return facets;
 }
 //-----------------------------------------------------------------------------
-const GenericFunction& DirichletBC::value()
+const GenericFunction& DirichletBC::value() const
 {
   return *g;
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<const GenericFunction> DirichletBC::value_ptr()
+boost::shared_ptr<const GenericFunction> DirichletBC::value_ptr() const
 {
   return g;
 }
 //-----------------------------------------------------------------------------
+boost::shared_ptr<const SubDomain> DirichletBC::user_sub_domain_ptr() const
+{
+  if (!user_sub_domain)
+    error("No user subdomain defined");
+  return user_sub_domain;
+}
+//-----------------------------------------------------------------------------
 bool DirichletBC::is_compatible(GenericFunction& v) const
 {
   // This function only checks the values at vertices when it should
@@ -310,11 +398,34 @@ void DirichletBC::set_value(const GenericFunction& g)
   this->g = reference_to_no_delete_pointer(g);
 }
 //-----------------------------------------------------------------------------
+void DirichletBC::homogenize()
+{
+  const uint value_rank = g->value_rank();
+  if (!value_rank)
+  {
+    boost::shared_ptr<Constant> zero(new Constant(0.0));
+    set_value(zero);
+    return;
+  }
+
+  const uint value_dim = g->value_dimension(0);
+  std::vector<double> values;
+  for (uint i = 0; i < value_dim; i ++)
+    values.push_back(0.0);
+  boost::shared_ptr<Constant> zero(new Constant(values));
+  set_value(zero);
+}
+//-----------------------------------------------------------------------------
 void DirichletBC::set_value(boost::shared_ptr<const GenericFunction> g)
 {
   this->g = g;
 }
 //-----------------------------------------------------------------------------
+std::string DirichletBC::method() const
+{
+  return _method;
+}
+//-----------------------------------------------------------------------------
 void DirichletBC::apply(GenericMatrix* A,
                         GenericVector* b,
                         const GenericVector* x) const
@@ -323,32 +434,33 @@ void DirichletBC::apply(GenericMatrix* A,
   check_arguments(A, b, x);
 
   // A map to hold the mapping from boundary dofs to boundary values
-  std::map<uint, double> boundary_values;
+  Map boundary_values;
 
   // Create local data for application of boundary conditions
   BoundaryCondition::LocalData data(*_function_space);
 
   // Compute dofs and values
-  compute_bc(boundary_values, data);
+  compute_bc(boundary_values, data, _method);
 
   // Copy boundary value data to arrays
   const uint size = boundary_values.size();
   std::vector<uint> dofs(size);
   std::vector<double> values(size);
-  std::map<uint, double>::const_iterator boundary_value;
+  Map::const_iterator bv;
   uint i = 0;
-  for (boundary_value = boundary_values.begin(); boundary_value != boundary_values.end(); ++boundary_value)
+  for (bv = boundary_values.begin(); bv != boundary_values.end(); ++bv)
   {
-    dofs[i]     = boundary_value->first;
-    values[i++] = boundary_value->second;
+    dofs[i]     = bv->first;
+    values[i++] = bv->second;
   }
 
   // Modify boundary values for nonlinear problems
   if (x)
   {
-    // Gather values
+    // Get values (these must reside in local portion (including ghost
+    // values) of the vector
     std::vector<double> x_values(size);
-    x->get(&x_values[0], dofs.size(), &dofs[0]);
+    x->get_local(&x_values[0], dofs.size(), &dofs[0]);
 
     // Modify RHS entries
     for (uint i = 0; i < size; i++)
@@ -369,9 +481,7 @@ void DirichletBC::apply(GenericMatrix* A,
   {
     const bool use_ident = parameters["use_ident"];
     if (use_ident)
-    {
       A->ident(size, &dofs[0]);
-    }
     else
     {
       for (uint i = 0; i < size; i++)
@@ -380,12 +490,20 @@ void DirichletBC::apply(GenericMatrix* A,
         A->setitem(ij, 1.0);
       }
     }
-    A->apply("insert");
+
+    // Apply changes
+    A->apply("add");
   }
 }
 //-----------------------------------------------------------------------------
 void DirichletBC::check() const
 {
+  // Check for common errors, message below might be cryptic
+  if (g->value_rank() == 0 && _function_space->element().value_rank() == 1)
+    error("Unable to create boundary condition. Reason: Expecting a vector-valued boundary value but given function is scalar.");
+  if (g->value_rank() == 1 && _function_space->element().value_rank() == 0)
+    error("Unable to create boundary condition. Reason: Expecting a scalar boundary value but given function is vector-valued.");
+
   // Check that value shape of boundary value
   check_equal(g->value_rank(), _function_space->element().value_rank(),
               "create boundary condition", "value rank");
@@ -394,7 +512,7 @@ void DirichletBC::check() const
                 "create boundary condition", "value dimension");
 
   // Check that boundary condition method is known
-  if (methods.count(method) == 0)
+  if (methods.count(_method) == 0)
     error("Unable to create boundary condition, unknown method specified.");
 
   // Check that the mesh is ordered
@@ -446,7 +564,7 @@ void DirichletBC::init_from_mesh_function(const MeshFunction<uint>& sub_domains,
       continue;
 
     // Get cell to which facet belongs (there may be two, but pick first)
-    Cell cell(_function_space->mesh(), facet->entities(dim)[0]);
+    const Cell cell(_function_space->mesh(), facet->entities(dim)[0]);
 
     // Get local index of facet with respect to the cell
     const uint facet_number = cell.index(*facet);
@@ -466,8 +584,6 @@ void DirichletBC::init_from_mesh(uint sub_domain)
   // or it won't do anything good (since the markers are wrong anyway).
   // In conclusion: we don't need to order the mesh here.
 
-  cout << "Creating sub domain markers for boundary condition." << endl;
-
   // Get data
   const std::vector<uint>* facet_cells   = _function_space->mesh().data().array("boundary facet cells");
   const std::vector<uint>* facet_numbers = _function_space->mesh().data().array("boundary facet numbers");
@@ -479,11 +595,13 @@ void DirichletBC::init_from_mesh(uint sub_domain)
     info(_function_space->mesh().data());
     error("Mesh data \"boundary facet cells\" not available.");
   }
+
   if (!facet_numbers)
   {
     info(_function_space->mesh().data());
     error("Mesh data \"boundary facet numbers\" not available.");
   }
+
   if (!indicators)
   {
     info(_function_space->mesh().data());
@@ -496,7 +614,7 @@ void DirichletBC::init_from_mesh(uint sub_domain)
   assert(size == indicators->size());
 
   // Build set of boundary facets
-  for (uint i = 0; i < size; i++)
+  for (uint i = 0; i < size; ++i)
   {
     // Skip facets not on this boundary
     if ((*indicators)[i] != sub_domain)
@@ -507,9 +625,14 @@ void DirichletBC::init_from_mesh(uint sub_domain)
   }
 }
 //-----------------------------------------------------------------------------
-void DirichletBC::compute_bc(std::map<uint, double>& boundary_values,
-                             BoundaryCondition::LocalData& data) const
+void DirichletBC::compute_bc(Map& boundary_values,
+                             BoundaryCondition::LocalData& data,
+                             std::string method) const
 {
+  // Set method if dafault
+  if (method == "default")
+    method = _method;
+
   // Choose strategy
   if (method == "topological")
     compute_bc_topological(boundary_values, data);
@@ -521,7 +644,7 @@ void DirichletBC::compute_bc(std::map<uint, double>& boundary_values,
     error("Unknown method for application of boundary conditions.");
 }
 //-----------------------------------------------------------------------------
-void DirichletBC::compute_bc_topological(std::map<uint, double>& boundary_values,
+void DirichletBC::compute_bc_topological(Map& boundary_values,
                                          BoundaryCondition::LocalData& data) const
 {
   assert(_function_space);
@@ -538,48 +661,44 @@ void DirichletBC::compute_bc_topological(std::map<uint, double>& boundary_values
   const Mesh& mesh = _function_space->mesh();
   const GenericDofMap& dofmap = _function_space->dofmap();
 
+  // Create UFC cell object
+  UFCCell ufc_cell(mesh);
+
   // Iterate over facets
   Progress p("Computing Dirichlet boundary values, topological search", facets.size());
-  for (uint f = 0; f < facets.size(); f++)
+  for (uint f = 0; f < facets.size(); ++f)
   {
     // Get cell number and local facet number
-    uint cell_number = facets[f].first;
-    uint facet_number = facets[f].second;
+    const uint cell_number  = facets[f].first;
+    const uint facet_number = facets[f].second;
 
     // Create cell
     Cell cell(mesh, cell_number);
-    UFCCell ufc_cell(cell);
+    ufc_cell.update(cell, facet_number);
 
-    /// Restrict coefficient to cell
-    g->restrict(data.w, _function_space->element(), cell, ufc_cell, facet_number);
+    // Restrict coefficient to cell
+    g->restrict(data.w, _function_space->element(), cell, ufc_cell);
 
     // Tabulate dofs on cell
-    dofmap.tabulate_dofs(data.cell_dofs, ufc_cell, cell_number);
+    dofmap.tabulate_dofs(data.cell_dofs, cell);
 
     // Tabulate which dofs are on the facet
     dofmap.tabulate_facet_dofs(data.facet_dofs, facet_number);
 
-    // Debugging print:
-    /*
-       cout << endl << "Handling BC's for:" << endl;
-       cout << "Cell:  " << facet.entities(facet.dim() + 1)[0] << endl;
-       cout << "Facet: " << local_facet << endl;
-    */
-
     // Pick values for facet
     for (uint i = 0; i < dofmap.num_facet_dofs(); i++)
     {
-      const uint dof = data.cell_dofs[data.facet_dofs[i]];
+      const uint global_dof = data.cell_dofs[data.facet_dofs[i]];
       const double value = data.w[data.facet_dofs[i]];
-      boundary_values[dof] = value;
+      boundary_values[global_dof] = value;
     }
 
     p++;
   }
 }
 //-----------------------------------------------------------------------------
-void DirichletBC::compute_bc_geometric(std::map<uint, double>& boundary_values,
-                                       BoundaryCondition::LocalData& data) const
+void DirichletBC::compute_bc_geometric(Map& boundary_values,
+                                      BoundaryCondition::LocalData& data) const
 {
   assert(_function_space);
   assert(g);
@@ -601,23 +720,26 @@ void DirichletBC::compute_bc_geometric(std::map<uint, double>& boundary_values,
 
   // Iterate over facets
   Progress p("Computing Dirichlet boundary values, geometric search", facets.size());
-  for (uint f = 0; f < facets.size(); f++)
+  for (uint f = 0; f < facets.size(); ++f)
   {
     // Get cell number and local facet number
-    uint cell_number = facets[f].first;
-    uint facet_number = facets[f].second;
+    const uint cell_number = facets[f].first;
+    const uint facet_number = facets[f].second;
 
     // Create facet
     Cell cell(mesh, cell_number);
     Facet facet(mesh, cell.entities(mesh.topology().dim() - 1)[facet_number]);
 
+    // Create UFC cell object
+    UFCCell ufc_cell(mesh);
+
     // Loop the vertices associated with the facet
     for (VertexIterator vertex(facet); !vertex.end(); ++vertex)
     {
       // Loop the cells associated with the vertex
       for (CellIterator c(*vertex); !c.end(); ++c)
       {
-        UFCCell ufc_cell(*c);
+        ufc_cell.update(*c, facet_number);
 
         bool interpolated = false;
 
@@ -625,7 +747,7 @@ void DirichletBC::compute_bc_geometric(std::map<uint, double>& boundary_values,
         dofmap.tabulate_coordinates(data.coordinates, ufc_cell);
 
         // Loop over all dofs on cell
-        for (uint i = 0; i < dofmap.local_dimension(ufc_cell); ++i)
+        for (uint i = 0; i < dofmap.cell_dimension(c->index()); ++i)
         {
           // Check if the coordinates are on current facet and thus on boundary
           if (!on_facet(data.coordinates[i], facet))
@@ -634,24 +756,24 @@ void DirichletBC::compute_bc_geometric(std::map<uint, double>& boundary_values,
           if (!interpolated)
           {
             // Tabulate dofs on cell
-            dofmap.tabulate_dofs(data.cell_dofs, ufc_cell, c->index());
+            dofmap.tabulate_dofs(data.cell_dofs, *c);
 
-            /// Restrict coefficient to cell
+            // Restrict coefficient to cell
             g->restrict(data.w, _function_space->element(), cell, ufc_cell);
           }
 
           // Set boundary value
-          const uint dof = data.cell_dofs[i];
+          const uint global_dof = data.cell_dofs[i];
           const double value = data.w[i];
-          boundary_values[dof] = value;
+          boundary_values[global_dof] = value;
         }
       }
     }
   }
 }
 //-----------------------------------------------------------------------------
-void DirichletBC::compute_bc_pointwise(std::map<uint, double>& boundary_values,
-                                       BoundaryCondition::LocalData& data) const
+void DirichletBC::compute_bc_pointwise(Map& boundary_values,
+                                      BoundaryCondition::LocalData& data) const
 {
   assert(_function_space);
   assert(g);
@@ -661,40 +783,44 @@ void DirichletBC::compute_bc_pointwise(std::map<uint, double>& boundary_values,
   const Mesh& mesh = _function_space->mesh();
   const GenericDofMap& dofmap = _function_space->dofmap();
 
+  // Create UFC cell object
+  UFCCell ufc_cell(mesh);
+
   // Iterate over cells
   Progress p("Computing Dirichlet boundary values, pointwise search", mesh.num_cells());
   for (CellIterator cell(mesh); !cell.end(); ++cell)
   {
-    UFCCell ufc_cell(*cell);
+    // Update UFC cell
+    ufc_cell.update(*cell);
 
     // Tabulate coordinates of dofs on cell
     dofmap.tabulate_coordinates(data.coordinates, ufc_cell);
 
     // Interpolate function only once and only on cells where necessary
-    bool interpolated = false;
+    bool already_interpolated = false;
 
     // Loop all dofs on cell
-    for (uint i = 0; i < dofmap.local_dimension(ufc_cell); ++i)
+    for (uint i = 0; i < dofmap.cell_dimension(cell->index()); ++i)
     {
-      // Check if the coordinates are part of the sub domain
+      // Check if the coordinates are part of the sub domain (calls user-defined 'indside' function)
       if (!user_sub_domain->inside(data.array_coordinates[i], false))
         continue;
 
-      if (!interpolated)
+      if (!already_interpolated)
       {
-        interpolated = true;
+        already_interpolated = true;
 
         // Tabulate dofs on cell
-        dofmap.tabulate_dofs(data.cell_dofs, ufc_cell, cell->index());
+        dofmap.tabulate_dofs(data.cell_dofs, *cell);
 
-        /// Restrict coefficient to cell
+        // Restrict coefficient to cell
         g->restrict(data.w, _function_space->element(), *cell, ufc_cell);
       }
 
       // Set boundary value
-      const uint dof = data.cell_dofs[i];
+      const uint global_dof = data.cell_dofs[i];
       const double value = data.w[i];
-      boundary_values[dof] = value;
+      boundary_values[global_dof] = value;
     }
 
     p++;
@@ -704,17 +830,17 @@ void DirichletBC::compute_bc_pointwise(std::map<uint, double>& boundary_values,
 bool DirichletBC::on_facet(double* coordinates, Facet& facet) const
 {
   // Check if the coordinates are on the same line as the line segment
-  if ( facet.dim() == 1 )
+  if (facet.dim() == 1)
   {
     // Create points
     Point p(coordinates[0], coordinates[1]);
-    Point v0 = Vertex(facet.mesh(), facet.entities(0)[0]).point();
-    Point v1 = Vertex(facet.mesh(), facet.entities(0)[1]).point();
+    const Point v0 = Vertex(facet.mesh(), facet.entities(0)[0]).point();
+    const Point v1 = Vertex(facet.mesh(), facet.entities(0)[1]).point();
 
     // Create vectors
-    Point v01 = v1 - v0;
-    Point vp0 = v0 - p;
-    Point vp1 = v1 - p;
+    const Point v01 = v1 - v0;
+    const Point vp0 = v0 - p;
+    const Point vp1 = v1 - p;
 
     // Check if the length of the sum of the two line segments vp0 and vp1 is
     // equal to the total length of the facet
@@ -724,25 +850,25 @@ bool DirichletBC::on_facet(double* coordinates, Facet& facet) const
       return false;
   }
   // Check if the coordinates are in the same plane as the triangular facet
-  else if ( facet.dim() == 2 )
+  else if (facet.dim() == 2)
   {
     // Create points
-    Point p(coordinates[0], coordinates[1], coordinates[2]);
-    Point v0 = Vertex(facet.mesh(), facet.entities(0)[0]).point();
-    Point v1 = Vertex(facet.mesh(), facet.entities(0)[1]).point();
-    Point v2 = Vertex(facet.mesh(), facet.entities(0)[2]).point();
+    const Point p(coordinates[0], coordinates[1], coordinates[2]);
+    const Point v0 = Vertex(facet.mesh(), facet.entities(0)[0]).point();
+    const Point v1 = Vertex(facet.mesh(), facet.entities(0)[1]).point();
+    const Point v2 = Vertex(facet.mesh(), facet.entities(0)[2]).point();
 
     // Create vectors
-    Point v01 = v1 - v0;
-    Point v02 = v2 - v0;
-    Point vp0 = v0 - p;
-    Point vp1 = v1 - p;
-    Point vp2 = v2 - p;
+    const Point v01 = v1 - v0;
+    const Point v02 = v2 - v0;
+    const Point vp0 = v0 - p;
+    const Point vp1 = v1 - p;
+    const Point vp2 = v2 - p;
 
     // Check if the sum of the area of the sub triangles is equal to the total
     // area of the facet
-    if ( std::abs(v01.cross(v02).norm() - vp0.cross(vp1).norm() - vp1.cross(vp2).norm()
-        - vp2.cross(vp0).norm()) < DOLFIN_EPS )
+    if (std::abs(v01.cross(v02).norm() - vp0.cross(vp1).norm() - vp1.cross(vp2).norm()
+        - vp2.cross(vp0).norm()) < DOLFIN_EPS)
       return true;
     else
       return false;
@@ -753,24 +879,3 @@ bool DirichletBC::on_facet(double* coordinates, Facet& facet) const
   return false;
 }
 //-----------------------------------------------------------------------------
-void DirichletBC::get_bc(uint* indicators, double* values) const
-{
-  // A map to hold the mapping from boundary dofs to boundary values
-  std::map<uint, double> boundary_values;
-
-  // Create local data for application of boundary conditions
-  BoundaryCondition::LocalData data(*_function_space);
-
-  // Compute dofs and values
-  compute_bc(boundary_values, data);
-
-  std::map<uint, double>::const_iterator boundary_value;
-  uint i = 0;
-  for (boundary_value = boundary_values.begin(); boundary_value != boundary_values.end(); ++boundary_value)
-  {
-    i = boundary_value->first;
-    indicators[i] = 1;
-    values[i] = boundary_value->second;
-  }
-}
-//-----------------------------------------------------------------------------
