@@ -1,11 +1,11 @@
-// Copyright (C) 2007-2008 Anders Logg and Garth N. Wells.
+// Copyright (C) 2007-2010 Anders Logg and Garth N. Wells.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Kristian Oelgaard, 2007
 // Modified by Johan Hake, 2009
 //
 // First added:  2007-04-10
-// Last changed: 2010-04-22
+// Last changed: 2011-02-17
 //
 // FIXME: This class needs some cleanup, in particular collecting
 // FIXME: all data from different representations into a common
@@ -19,7 +19,10 @@
 #include <string>
 #include <vector>
 #include <boost/shared_ptr.hpp>
+#include <boost/unordered_map.hpp>
+
 #include <dolfin/common/types.h>
+#include <dolfin/common/Hierarchical.h>
 #include "BoundaryCondition.h"
 
 namespace dolfin
@@ -76,10 +79,14 @@ namespace dolfin
 
   /// This class specifies the interface for setting (strong)
 
-  class DirichletBC : public BoundaryCondition
+  class DirichletBC : public BoundaryCondition, public Hierarchical<DirichletBC>
   {
+
   public:
 
+    typedef boost::unordered_map<uint, double> Map;
+    //typedef std::map<uint, double> Map;
+
     /// Create boundary condition for subdomain
     DirichletBC(const FunctionSpace& V,
                 const GenericFunction& g,
@@ -116,13 +123,15 @@ namespace dolfin
                 uint sub_domain,
                 std::string method="topological");
 
-    // Create boundary condition for subdomain by boundary markers (cells, local facet numbers)
+    /// Create boundary condition for subdomain by boundary markers
+    /// (cells, local facet numbers)
     DirichletBC(const FunctionSpace& V,
                 const GenericFunction& g,
                 const std::vector<std::pair<uint, uint> >& markers,
                 std::string method="topological");
 
-    // Create boundary condition for subdomain by boundary markers (cells, local facet numbers)
+    /// Create boundary condition for subdomain by boundary markers
+    /// (cells, local facet numbers)
     DirichletBC(boost::shared_ptr<const FunctionSpace> V,
                 boost::shared_ptr<const GenericFunction> g,
                 const std::vector<std::pair<uint, uint> >& markers,
@@ -150,24 +159,38 @@ namespace dolfin
     void apply(GenericVector& b, const GenericVector& x) const;
 
     /// Apply boundary condition to a linear system for a nonlinear problem
-    void apply(GenericMatrix& A, GenericVector& b, const GenericVector& x) const;
+    void apply(GenericMatrix& A, GenericVector& b,
+               const GenericVector& x) const;
 
-    /// Make row associated with boundary conditions zero, useful for non-diagonal matrices in a block matrix.
+    /// Get Dirichlet dofs and values
+    void get_boundary_values(Map& boundary_values,
+                             std::string method="default") const;
+
+    /// Make row associated with boundary conditions zero, useful for
+    /// non-diagonal matrices in a block matrix.
     void zero(GenericMatrix& A) const;
 
-    /// Return boundary markers (facets stored as pairs of cells and local facet numbers)
+    /// Make columns associated with boundary conditions zero, and
+    /// update the RHS to reflect the changes. Useful for non-diagonals.
+    /// The diag_val parameter would normally be -1, 0 or 1.
+    void zero_columns(GenericMatrix& A, GenericVector& b, double diag_val=0) const;
+
+    /// Return boundary markers (facets stored as pairs of cells and local
+    /// facet numbers)
     const std::vector<std::pair<uint, uint> >& markers();
 
     /// Return boundary value g
-    const GenericFunction& value();
+    const GenericFunction& value() const;
 
     /// Return shared pointer to boundary value g
-    boost::shared_ptr<const GenericFunction> value_ptr();
+    /// Testing multiline comment
+    boost::shared_ptr<const GenericFunction> value_ptr() const;
 
-    /// Get Dirichlet values and indicators
-    void get_bc(uint* indicators, double* values) const;
+    //// Return shared pointer to sub-domain
+    boost::shared_ptr<const SubDomain> user_sub_domain_ptr() const;
 
-    /// Check if given function is compatible with boundary condition (checking only vertex values)
+    /// Check if given function is compatible with boundary condition
+    /// (checking only vertex values)
     bool is_compatible(GenericFunction& v) const;
 
     /// Set value g for boundary condition, domain remains unchanged
@@ -176,6 +199,13 @@ namespace dolfin
     /// Set value g for boundary condition, domain remains unchanged
     void set_value(boost::shared_ptr<const GenericFunction> g);
 
+    /// Set value to 0.0
+    void homogenize();
+
+    /// Return method used for computing Dirichet dofs ("topological",
+    /// "geometric" or "pointwise")
+    std::string method() const;
+
     /// Default parameter values
     static Parameters default_parameters()
     {
@@ -186,9 +216,6 @@ namespace dolfin
 
   private:
 
-    // Friends
-    friend class AdaptiveObjects;
-
     // FIXME: Make this function pure virtual in BoundaryCondition and reuse code
     // for different apply methods
 
@@ -202,25 +229,27 @@ namespace dolfin
     void init_from_sub_domain(boost::shared_ptr<const SubDomain> sub_domain);
 
     // Initialize sub domain markers from MeshFunction
-    void init_from_mesh_function(const MeshFunction<uint>& sub_domains, uint sub_domain);
+    void init_from_mesh_function(const MeshFunction<uint>& sub_domains,
+                                 uint sub_domain);
 
     // Initialize sub domain markers from mesh
     void init_from_mesh(uint sub_domain);
 
-    // Compute dofs and values for application of boundary conditions
-    void compute_bc(std::map<uint, double>& boundary_values,
-                    BoundaryCondition::LocalData& data) const;
+    // Compute dofs and values for application of boundary conditions using
+    // given method
+    void compute_bc(Map& boundary_values,
+                    BoundaryCondition::LocalData& data, std::string method) const;
 
     // Compute boundary values for facet (topological approach)
-    void compute_bc_topological(std::map<uint, double>& boundary_values,
+    void compute_bc_topological(Map& boundary_values,
                                 BoundaryCondition::LocalData& data) const;
 
     // Compute boundary values for facet (geometrical approach)
-    void compute_bc_geometric(std::map<uint, double>& boundary_values,
+    void compute_bc_geometric(Map& boundary_values,
                               BoundaryCondition::LocalData& data) const;
 
     // Compute boundary values for facet (pointwise approach)
-    void compute_bc_pointwise(std::map<uint, double>& boundary_values,
+    void compute_bc_pointwise(Map& boundary_values,
                               BoundaryCondition::LocalData& data) const;
 
     // Check if the point is in the same plane as the given facet
@@ -230,7 +259,7 @@ namespace dolfin
     boost::shared_ptr<const GenericFunction> g;
 
     // Search method
-    std::string method;
+    std::string _method;
 
     // Possible search methods
     static const std::set<std::string> methods;
