@@ -1,4 +1,4 @@
-// Copyright (C) 2007-2010 Anders Logg and Garth N. Wells.
+// Copyright (C) 2007-2011 Anders Logg and Garth N. Wells.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Martin Alnes, 2008
@@ -7,30 +7,28 @@
 // Modified by Niclas Jansson, 2009
 //
 // First added:  2007-03-01
-// Last changed: 2010-06-01
+// Last changed: 2011-02-23
 
+#include <dolfin/common/NoDeleter.h>
 #include <dolfin/common/Set.h>
 #include <dolfin/common/Timer.h>
-#include <dolfin/log/LogStream.h>
-#include <dolfin/main/MPI.h>
-#include <dolfin/mesh/MeshPartitioning.h>
-#include <dolfin/common/NoDeleter.h>
 #include <dolfin/common/types.h>
-#include <dolfin/mesh/Cell.h>
-#include <dolfin/mesh/MeshData.h>
-#include "UFC.h"
-#include "UFCCell.h"
-#include "DofMapBuilder.h"
-#include "DofMap.h"
-
+#include <dolfin/log/LogStream.h>
+#include <dolfin/common/MPI.h>
 #include <dolfin/mesh/BoundaryMesh.h>
+#include <dolfin/mesh/MeshData.h>
+#include <dolfin/mesh/MeshPartitioning.h>
+#include "DofMapBuilder.h"
+#include "UFCCell.h"
+#include "UFCMesh.h"
+#include "DofMap.h"
 
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
-DofMap::DofMap(boost::shared_ptr<ufc::dof_map> ufc_dofmap, Mesh& dolfin_mesh)
-             : _ufc_dofmap(ufc_dofmap), ufc_offset(0),
-               _parallel(MPI::num_processes() > 1)
+DofMap::DofMap(boost::shared_ptr<ufc::dofmap> ufc_dofmap, Mesh& dolfin_mesh)
+             : _ufc_dofmap(ufc_dofmap), ufc_offset(0), _is_view(false),
+               _distributed(MPI::num_processes() > 1)
 {
   assert(_ufc_dofmap);
 
@@ -42,10 +40,10 @@ DofMap::DofMap(boost::shared_ptr<ufc::dof_map> ufc_dofmap, Mesh& dolfin_mesh)
   for (uint d = 1; d <= dolfin_mesh.topology().dim(); ++d)
   {
     if (_ufc_dofmap->needs_mesh_entities(d) ||
-    	(_parallel && d == (dolfin_mesh.topology().dim() - 1)))
+    	(_distributed && d == (dolfin_mesh.topology().dim() - 1)))
     {
       dolfin_mesh.init(d);
-      if (_parallel)
+      if (_distributed)
         MeshPartitioning::number_entities(dolfin_mesh, d);
     }
   }
@@ -60,10 +58,10 @@ DofMap::DofMap(boost::shared_ptr<ufc::dof_map> ufc_dofmap, Mesh& dolfin_mesh)
   build(dolfin_mesh, ufc_mesh);
 }
 //-----------------------------------------------------------------------------
-DofMap::DofMap(boost::shared_ptr<ufc::dof_map> ufc_dofmap,
+DofMap::DofMap(boost::shared_ptr<ufc::dofmap> ufc_dofmap,
                const Mesh& dolfin_mesh)
-             : _ufc_dofmap(ufc_dofmap), ufc_offset(0),
-               _parallel(MPI::num_processes() > 1)
+             : _ufc_dofmap(ufc_dofmap), ufc_offset(0), _is_view(false),
+               _distributed(MPI::num_processes() > 1)
 {
   assert(_ufc_dofmap);
 
@@ -81,10 +79,10 @@ DofMap::DofMap(boost::shared_ptr<ufc::dof_map> ufc_dofmap,
   build(dolfin_mesh, ufc_mesh);
 }
 //-----------------------------------------------------------------------------
-DofMap::DofMap(boost::shared_ptr<ufc::dof_map> ufc_dofmap,
+DofMap::DofMap(boost::shared_ptr<ufc::dofmap> ufc_dofmap,
                const UFCMesh& ufc_mesh)
-             : _ufc_dofmap(ufc_dofmap), ufc_offset(0),
-               _parallel(MPI::num_processes() > 1)
+             : _ufc_dofmap(ufc_dofmap), ufc_offset(0), _is_view(false),
+               _distributed(MPI::num_processes() > 1)
 
 {
   assert(_ufc_dofmap);
@@ -95,18 +93,54 @@ DofMap::~DofMap()
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-void DofMap::tabulate_dofs(uint* dofs, const ufc::cell& ufc_cell,
-                           uint cell_index) const
+bool DofMap::needs_mesh_entities(unsigned int d) const
 {
-  // FIXME: Add assertion to test that this process has the dof.
-  std::copy(dofmap[cell_index].begin(), dofmap[cell_index].end(), dofs);
+  assert(_ufc_dofmap);
+  return _ufc_dofmap->needs_mesh_entities(d);
 }
 //-----------------------------------------------------------------------------
-void DofMap::tabulate_dofs(uint* dofs, const Cell& cell) const
+unsigned int DofMap::global_dimension() const
 {
-  // FIXME: Add assertion to test that this process has the dof.
-  const uint cell_index = cell.index();
-  std::copy(dofmap[cell_index].begin(), dofmap[cell_index].end(), dofs);
+  assert(_ufc_dofmap);
+  assert(_ufc_dofmap->global_dimension() > 0);
+  return _ufc_dofmap->global_dimension();
+}
+//-----------------------------------------------------------------------------
+unsigned int DofMap::cell_dimension(uint cell_index) const
+{
+  assert(cell_index < dofmap.size());
+  return dofmap[cell_index].size();
+}
+//-----------------------------------------------------------------------------
+unsigned int DofMap::max_cell_dimension() const
+{
+  assert( _ufc_dofmap);
+  return _ufc_dofmap->max_local_dimension();
+}
+//-----------------------------------------------------------------------------
+unsigned int DofMap::geometric_dimension() const
+{
+  assert(_ufc_dofmap);
+  return _ufc_dofmap->geometric_dimension();
+}
+//-----------------------------------------------------------------------------
+unsigned int DofMap::num_facet_dofs() const
+{
+  assert(_ufc_dofmap);
+  return _ufc_dofmap->num_facet_dofs();
+}
+//-----------------------------------------------------------------------------
+std::pair<unsigned int, unsigned int> DofMap::ownership_range() const
+{
+  if (is_view())
+    error("Cannot determine ownership range for sub dofmaps.");
+
+  return _ownership_range;
+}
+//-----------------------------------------------------------------------------
+const boost::unordered_map<unsigned int, unsigned int>& DofMap::off_process_owner() const
+{
+  return _off_process_owner;
 }
 //-----------------------------------------------------------------------------
 void DofMap::tabulate_facet_dofs(uint* dofs, uint local_facet) const
@@ -127,22 +161,22 @@ DofMap* DofMap::extract_sub_dofmap(const std::vector<uint>& component,
   assert(_ufc_dofmap);
 
   // Create UFC mesh
-  UFCMesh ufc_mesh(dolfin_mesh);
+  const UFCMesh ufc_mesh(dolfin_mesh);
 
   // Initialise offset
   uint offset = ufc_offset;
 
   // Recursively extract UFC sub-dofmap
-  boost::shared_ptr<ufc::dof_map>
-    ufc_sub_dof_map(extract_sub_dofmap(*_ufc_dofmap, offset, component, ufc_mesh, dolfin_mesh));
+  boost::shared_ptr<ufc::dofmap>
+    ufc_sub_dofmap(extract_sub_dofmap(*_ufc_dofmap, offset, component, ufc_mesh, dolfin_mesh));
 
   // Initialise ufc sub-dofmap
-  init_ufc_dofmap(*ufc_sub_dof_map, ufc_mesh, dolfin_mesh);
+  init_ufc_dofmap(*ufc_sub_dofmap, ufc_mesh, dolfin_mesh);
 
   // Create new dof map
-  DofMap* sub_dofmap = new DofMap(ufc_sub_dof_map, ufc_mesh);
+  DofMap* sub_dofmap = new DofMap(ufc_sub_dofmap, ufc_mesh);
 
-   // Create new dof map
+   // Create sub-map data structure
   std::vector<std::vector<uint> >& sub_map = sub_dofmap->dofmap;
   sub_map.resize(dolfin_mesh.num_cells());
 
@@ -156,10 +190,10 @@ DofMap* DofMap::extract_sub_dofmap(const std::vector<uint>& component,
     ufc_cell.update(*cell);
 
     // Resize for list for cell
-    sub_map[index].resize( ufc_sub_dof_map->local_dimension(ufc_cell) );
+    sub_map[index].resize( ufc_sub_dofmap->local_dimension(ufc_cell) );
 
     // Tabulate sub-dofs on cell (UFC map)
-    ufc_sub_dof_map->tabulate_dofs(&sub_map[index][0], ufc_mesh, ufc_cell);
+    ufc_sub_dofmap->tabulate_dofs(&sub_map[index][0], ufc_mesh, ufc_cell);
 
     // Add UFC offset
     for (uint i = 0; i < sub_map[index].size(); ++i)
@@ -186,6 +220,12 @@ DofMap* DofMap::extract_sub_dofmap(const std::vector<uint>& component,
     sub_dofmap->ufc_map_to_dofmap = ufc_map_to_dofmap;
   }
 
+  // Mark subdof map as a view
+  sub_dofmap->_is_view = true;
+
+  // Set local ownership range
+  sub_dofmap->_ownership_range = std::make_pair(0, 0);
+
   return sub_dofmap;
 }
 //-----------------------------------------------------------------------------
@@ -194,13 +234,13 @@ DofMap* DofMap::collapse(std::map<uint, uint>& collapsed_map,
 {
   assert(_ufc_dofmap);
 
-  // Create new dof map (this set ufc_offset = 0 and it will renumber the map
+  // Create new dof map (this sets ufc_offset = 0 and it will renumber the map
   // if runnning in parallel)
-  DofMap* collapsed_dof_map = new DofMap(_ufc_dofmap, dolfin_mesh);
+  DofMap* collapsed_dofmap = new DofMap(_ufc_dofmap, dolfin_mesh);
 
   // Dimension checks
-  assert(collapsed_dof_map->global_dimension() == global_dimension());
-  assert(collapsed_dof_map->dofmap.size() == dolfin_mesh.num_cells());
+  assert(collapsed_dofmap->global_dimension() == global_dimension());
+  assert(collapsed_dofmap->dofmap.size() == dolfin_mesh.num_cells());
   assert(dofmap.size() == dolfin_mesh.num_cells());
 
   // FIXME: Could we use a std::vector instead of std::map if the collapsed
@@ -211,7 +251,7 @@ DofMap* DofMap::collapse(std::map<uint, uint>& collapsed_map,
   for (uint i = 0; i < dolfin_mesh.num_cells(); ++i)
   {
     const std::vector<uint> dofs = this->dofmap[i];
-    const std::vector<uint> collapsed_dofs = collapsed_dof_map->dofmap[i];
+    const std::vector<uint> collapsed_dofs = collapsed_dofmap->dofmap[i];
     assert(dofs.size() == collapsed_dofs.size());
 
     for (uint j = 0; j < dofs.size(); ++j)
@@ -223,34 +263,40 @@ DofMap* DofMap::collapse(std::map<uint, uint>& collapsed_map,
   UFCCell ufc_cell(dolfin_mesh);
 
   // Build UFC-to-actual-dofs map
-  std::vector<uint> ufc_dofs(collapsed_dof_map->max_local_dimension());
+  std::vector<uint> ufc_dofs(collapsed_dofmap->max_cell_dimension());
   for (CellIterator cell(dolfin_mesh); !cell.end(); ++cell)
   {
     ufc_cell.update(*cell);
 
     // Tabulate UFC dofs (UFC map)
-    collapsed_dof_map->_ufc_dofmap->tabulate_dofs(&ufc_dofs[0], ufc_mesh, ufc_cell);
+    collapsed_dofmap->_ufc_dofmap->tabulate_dofs(&ufc_dofs[0], ufc_mesh, ufc_cell);
 
     // Build UFC-to-actual-dofs map
-    std::vector<uint>& collapsed_dofs = collapsed_dof_map->dofmap[cell->index()];
+    std::vector<uint>& collapsed_dofs = collapsed_dofmap->dofmap[cell->index()];
     for (uint j = 0; j < collapsed_dofs.size(); ++j)
-      collapsed_dof_map->ufc_map_to_dofmap[ufc_dofs[j]] = collapsed_dofs[j];
+      collapsed_dofmap->ufc_map_to_dofmap[ufc_dofs[j]] = collapsed_dofs[j];
   }
 
   // Reset offset of collapsed map
-  collapsed_dof_map->ufc_offset = 0;
+  collapsed_dofmap->ufc_offset = 0;
 
-  return collapsed_dof_map;
+  // Set local ownership range
+
+  // Update off-process owner
+
+  cout << "Leaving collapse dof map" << endl;
+
+  return collapsed_dofmap;
 }
 //-----------------------------------------------------------------------------
-ufc::dof_map* DofMap::extract_sub_dofmap(const ufc::dof_map& ufc_dofmap,
-                                         uint& offset,
-                                         const std::vector<uint>& component,
-                                         const ufc::mesh ufc_mesh,
-                                         const Mesh& dolfin_mesh)
+ufc::dofmap* DofMap::extract_sub_dofmap(const ufc::dofmap& ufc_dofmap,
+                                        uint& offset,
+                                        const std::vector<uint>& component,
+                                        const ufc::mesh ufc_mesh,
+                                        const Mesh& dolfin_mesh)
 {
   // Check if there are any sub systems
-  if (ufc_dofmap.num_sub_dof_maps() == 0)
+  if (ufc_dofmap.num_sub_dofmaps() == 0)
     error("Unable to extract sub system (there are no sub systems).");
 
   // Check that a sub system has been specified
@@ -258,15 +304,15 @@ ufc::dof_map* DofMap::extract_sub_dofmap(const ufc::dof_map& ufc_dofmap,
     error("Unable to extract sub system (no sub system specified).");
 
   // Check the number of available sub systems
-  if (component[0] >= ufc_dofmap.num_sub_dof_maps())
+  if (component[0] >= ufc_dofmap.num_sub_dofmaps())
     error("Unable to extract sub system %d (only %d sub systems defined).",
-                  component[0], ufc_dofmap.num_sub_dof_maps());
+                  component[0], ufc_dofmap.num_sub_dofmaps());
 
   // Add to offset if necessary
   for (uint i = 0; i < component[0]; i++)
   {
     // Extract sub dofmap
-    boost::scoped_ptr<ufc::dof_map> ufc_tmp_dofmap(ufc_dofmap.create_sub_dof_map(i));
+    boost::scoped_ptr<ufc::dofmap> ufc_tmp_dofmap(ufc_dofmap.create_sub_dofmap(i));
     assert(ufc_tmp_dofmap);
 
     // Initialise
@@ -277,23 +323,24 @@ ufc::dof_map* DofMap::extract_sub_dofmap(const ufc::dof_map& ufc_dofmap,
   }
 
   // Create UFC sub-system
-  ufc::dof_map* sub_dof_map = ufc_dofmap.create_sub_dof_map(component[0]);
+  ufc::dofmap* sub_dofmap = ufc_dofmap.create_sub_dofmap(component[0]);
+  assert(sub_dofmap);
 
   // Return sub-system if sub-sub-system should not be extracted, otherwise
   // recursively extract the sub sub system
   if (component.size() == 1)
-    return sub_dof_map;
+    return sub_dofmap;
   else
   {
     std::vector<uint> sub_component;
     for (uint i = 1; i < component.size(); i++)
       sub_component.push_back(component[i]);
 
-    ufc::dof_map* sub_sub_dof_map = extract_sub_dofmap(*sub_dof_map, offset,
-                                                       sub_component, ufc_mesh,
-                                                       dolfin_mesh);
-    delete sub_dof_map;
-    return sub_sub_dof_map;
+    ufc::dofmap* sub_sub_dofmap = extract_sub_dofmap(*sub_dofmap, offset,
+                                                     sub_component, ufc_mesh,
+                                                     dolfin_mesh);
+    delete sub_dofmap;
+    return sub_sub_dofmap;
   }
 }
 //-----------------------------------------------------------------------------
@@ -319,11 +366,13 @@ void DofMap::build(const Mesh& dolfin_mesh, const UFCMesh& ufc_mesh)
   }
 
   // Build (renumber) dofmap when running in parallel
-  if (_parallel)
+  if (_distributed)
     DofMapBuilder::parallel_build(*this, dolfin_mesh);
+  else
+    _ownership_range = std::make_pair(0, global_dimension());
 }
 //-----------------------------------------------------------------------------
-void DofMap::init_ufc_dofmap(ufc::dof_map& dofmap,
+void DofMap::init_ufc_dofmap(ufc::dofmap& dofmap,
                              const ufc::mesh ufc_mesh,
                              const Mesh& dolfin_mesh)
 {
@@ -348,26 +397,66 @@ void DofMap::init_ufc_dofmap(ufc::dof_map& dofmap,
   }
 }
 //-----------------------------------------------------------------------------
-dolfin::Set<dolfin::uint> DofMap::dofs(const Mesh& mesh, bool sort) const
+boost::unordered_set<dolfin::uint> DofMap::dofs() const
 {
-  dolfin::Set<uint> dof_list;
+  // Build set of dofs
+  boost::unordered_set<dolfin::uint> dof_list;
   for (uint i = 0; i < dofmap.size(); ++i)
     for (uint j = 0; j < dofmap[i].size(); ++j)
       dof_list.insert(dofmap[i][j]);
 
-  if(sort)
-    dof_list.sort();
-
   return dof_list;
 }
 //-----------------------------------------------------------------------------
+void DofMap::renumber(const std::vector<uint>& renumbering_map)
+{
+  assert(global_dimension() == renumbering_map.size());
+
+  // Update or build ufc-to-dofmap
+  if (ufc_map_to_dofmap.size() == 0)
+  {
+    for (uint i = 0; i < dofmap.size(); ++i)
+      ufc_map_to_dofmap[i] = renumbering_map[ i ];
+  }
+  else
+  {
+    std::map<dolfin::uint, uint>::iterator index_pair;
+    for (index_pair = ufc_map_to_dofmap.begin(); index_pair != ufc_map_to_dofmap.end(); ++index_pair)
+      index_pair->second = renumbering_map[ index_pair->second ];
+  }
+
+  // Re-number dofs for cell
+  for (uint i = 0; i < dofmap.size(); ++i)
+  {
+    for (uint j = 0; j < dofmap[i].size(); ++j)
+      dofmap[i][j] = renumbering_map[ dofmap[i][j] ];
+  }
+}
+//-----------------------------------------------------------------------------
 std::string DofMap::str(bool verbose) const
 {
-  // TODO: Display information on renumbering
   // TODO: Display information on parallel stuff
 
+  // Prefix with process number if running in parallel
+  std::stringstream prefix;
+  if (MPI::num_processes() > 1)
+    prefix << "Process " << MPI::process_number() << ": ";
+
   std::stringstream s;
-  s << "<DofMap of global dimension " << global_dimension() << ">";
+  s << prefix.str() << "<DofMap of global dimension " << global_dimension() << ">" << std::endl;
+  if (verbose)
+  {
+    // Cell loop
+    for (uint i = 0; i < dofmap.size(); ++i)
+    {
+      s << prefix.str() << "Local cell index, cell dofmap dimension: " << i << ", " << dofmap[i].size() << std::endl;
+
+      // Local dof loop
+      for (uint j = 0; j < dofmap[i].size(); ++j)
+        s << prefix.str() <<  "  " << "Local, global dof indices: " << j << ", " << dofmap[i][j] << std::endl;
+    }
+  }
+
   return s.str();
 }
 //-----------------------------------------------------------------------------
