@@ -10,7 +10,7 @@
 #define __DOF_MAP_BUILDER_H
 
 #include <set>
-#include <tr1/unordered_set>
+#include <boost/unordered_set.hpp>
 #include <dolfin/common/Set.h>
 
 namespace dolfin
@@ -30,10 +30,10 @@ namespace dolfin
     //typedef std::set<dolfin::uint>::const_iterator set_iterator;
     //typedef Set<dolfin::uint> set;
     //typedef Set<dolfin::uint>::const_iterator set_iterator;
-    typedef std::tr1::unordered_set<dolfin::uint> set;
-    typedef std::tr1::unordered_set<dolfin::uint>::const_iterator set_iterator;
-    //typedef boost::unordered_set<dolfin::uint> set;
-    //typedef boost::unordered_set<dolfin::uint>::const_iterator set_iterator;
+    //typedef std::tr1::unordered_set<dolfin::uint> set;
+    //typedef std::tr1::unordered_set<dolfin::uint>::const_iterator set_iterator;
+    typedef boost::unordered_set<dolfin::uint> set;
+    typedef boost::unordered_set<dolfin::uint>::const_iterator set_iterator;
 
     typedef std::vector<dolfin::uint>::const_iterator vector_it;
 
@@ -44,13 +44,14 @@ namespace dolfin
 
   private:
 
-    static void compute_ownership(set& owned_dofs, set& shared_dofs,
-                                  set& forbidden_dofs, const DofMap& dofmap,
+    static void compute_ownership(set& owned_dofs, set& shared_owned_dofs,
+                                  set& shared_unowned_dofs,
+                                  const DofMap& dofmap,
                                   const Mesh& mesh);
 
-    static void parallel_renumber(const set& owned_dofs, const set& shared_dofs,
-                                  const set& forbidden_dofs, DofMap& dofmap,
-                                  const Mesh& mesh);
+    static void parallel_renumber(const set& owned_dofs, const set& shared_owned_dofs,
+                                  const set& shared_unowned_dofs,
+                                  DofMap& dofmap, const Mesh& mesh);
 
 
   };
@@ -58,5 +59,3 @@ namespace dolfin
 }
 
 #endif
-
-
