@@ -2,15 +2,14 @@
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2008-09-11
-// Last changed: 2009-12-11
+// Last changed: 2011-02-13
 
 #ifndef __FINITE_ELEMENT_H
 #define __FINITE_ELEMENT_H
 
-#include <boost/shared_ptr.hpp>
 #include <vector>
+#include <boost/shared_ptr.hpp>
 #include <dolfin/common/types.h>
-#include "UFC.h"
 #include "UFCCell.h"
 
 namespace dolfin
@@ -81,18 +80,15 @@ namespace dolfin
       return element;
     }
 
+    /// Extract sub finite element for component
+    boost::shared_ptr<const FiniteElement> extract_sub_element(const std::vector<uint>& component) const;
+
     /// Return ufc::finite_element
     boost::shared_ptr<const ufc::finite_element> ufc_element() const
     { return _ufc_element; }
 
-    /// Extract sub finite element for component
-    boost::shared_ptr<const FiniteElement> extract_sub_element(const std::vector<uint>& component) const;
-
   private:
 
-    // Friends
-    friend class AdaptiveObjects;
-
     // Recursively extract sub finite element
     static boost::shared_ptr<const FiniteElement> extract_sub_element(const FiniteElement& finite_element,
                                               const std::vector<uint>& component);
