@@ -1,36 +1,53 @@
 // Copyright (C) 2007-2008 Garth N. Wells.
 // Licensed under the GNU LGPL Version 2.1.
 //
-// Modified by Anders Logg, 2008.
+// Modified by Anders Logg, 2008-2011.
 // Modified by Martin Alnes, 2008.
 //
 // First added:  2007-12-10
-// Last changed: 2010-02-01
+// Last changed: 2011-02-03
 
 #include <string>
 #include <dolfin/common/NoDeleter.h>
-#include <dolfin/log/log.h>
-#include <dolfin/log/LogStream.h>
 #include <dolfin/fem/FiniteElement.h>
+#include <dolfin/function/Function.h>
 #include <dolfin/function/FunctionSpace.h>
 #include <dolfin/function/GenericFunction.h>
-#include "UFC.h"
+#include <dolfin/log/log.h>
+#include <dolfin/log/LogStream.h>
+#include <dolfin/mesh/Mesh.h>
+#include <dolfin/mesh/MeshData.h>
+#include <dolfin/mesh/MeshEntity.h>
+#include <dolfin/mesh/MeshFunction.h>
 #include "Form.h"
 
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
 Form::Form(uint rank, uint num_coefficients)
-  : _function_spaces(rank),
+  : Hierarchical<Form>(*this),
+    _function_spaces(rank),
     _coefficients(num_coefficients)
 {
   // Do nothing
 }
 //-----------------------------------------------------------------------------
+Form::Form(boost::shared_ptr<const ufc::form> ufc_form,
+           std::vector<boost::shared_ptr<const FunctionSpace> > function_spaces,
+           std::vector<boost::shared_ptr<const GenericFunction> > coefficients)
+  : Hierarchical<Form>(*this),
+    _ufc_form(ufc_form),
+    _function_spaces(function_spaces),
+    _coefficients(coefficients)
+{
+  // Do nothing
+}
+//-----------------------------------------------------------------------------
 Form::Form(const ufc::form& ufc_form,
            const std::vector<const FunctionSpace*>& function_spaces,
            const std::vector<const GenericFunction*>& coefficients)
-  : _function_spaces(function_spaces.size()),
+  : Hierarchical<Form>(*this),
+    _function_spaces(function_spaces.size()),
     _coefficients(coefficients.size())
 {
   for (uint i = 0; i < function_spaces.size(); i++)
@@ -75,7 +92,7 @@ const Mesh& Form::mesh() const
   // a functional) the (generated) subclass must set the mesh directly
   // by calling set_mesh().
 
-  // Extract all meshes
+  // Extract meshes from function spaces
   std::vector<const Mesh*> meshes;
   for (uint i = 0; i < _function_spaces.size(); i++)
     if (_function_spaces[i])
@@ -85,6 +102,19 @@ const Mesh& Form::mesh() const
   if (_mesh)
     meshes.push_back(&*_mesh);
 
+  // Extract meshes from coefficients. Note that this is only done
+  // when we don't already have a mesh sine it may otherwise conflict
+  // with existing meshes (if coefficient is defined on another mesh).
+  if (meshes.size() == 0)
+  {
+    for (uint i = 0; i < _coefficients.size(); i++)
+    {
+      const Function* function = dynamic_cast<const Function*>(&*_coefficients[i]);
+      if (function)
+        meshes.push_back(&function->function_space().mesh());
+    }
+  }
+
   // Check that we have at least one mesh
   if (meshes.size() == 0)
   {
@@ -101,7 +131,12 @@ const Mesh& Form::mesh() const
   return *meshes[0];
 }
 //-----------------------------------------------------------------------------
-const boost::shared_ptr<const FunctionSpace> Form::function_space(uint i) const
+boost::shared_ptr<const dolfin::Mesh> Form::mesh_shared_ptr() const
+{
+  return _mesh;
+}
+//-----------------------------------------------------------------------------
+boost::shared_ptr<const FunctionSpace> Form::function_space(uint i) const
 {
   assert(i < _function_spaces.size());
   return _function_spaces[i];
@@ -162,13 +197,14 @@ const GenericFunction& Form::coefficient(std::string name) const
   return coefficient(coefficient_number(name));
 }
 //-----------------------------------------------------------------------------
-std::vector<const GenericFunction*> Form::coefficients() const
+std::vector<boost::shared_ptr<const GenericFunction> > Form::coefficients() const
 {
-  std::vector<const GenericFunction*> V;
-  for (uint i = 0; i < _coefficients.size(); ++i)
-    V.push_back(_coefficients[i].get());
+  //std::vector<const GenericFunction*> V;
+  //for (uint i = 0; i < _coefficients.size(); ++i)
+  //V.push_back(_coefficients[i].get());
+  //return V;
 
-  return V;
+  return _coefficients;
 }
 //-----------------------------------------------------------------------------
 dolfin::uint Form::coefficient_number(const std::string & name) const
@@ -192,6 +228,11 @@ const ufc::form& Form::ufc_form() const
   return *_ufc_form;
 }
 //-----------------------------------------------------------------------------
+boost::shared_ptr<const ufc::form> Form::ufc_form_shared_ptr() const
+{
+  return _ufc_form;
+}
+//-----------------------------------------------------------------------------
 void Form::check() const
 {
   // Check that the number of argument function spaces is correct
