@@ -1,136 +1,134 @@
-// Copyright (C) 2007-2009 Garth N. Wells.
+// Copyright (C) 2007-2010 Garth N. Wells.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Ola Skavhaug, 2007.
 // Modified by Anders Logg, 2008-2009.
 //
 // First added:  2007-05-24
-// Last changed: 2010-04-21
+// Last changed: 2011-02-21
 
-#include <boost/scoped_array.hpp>
+#include <dolfin/common/timing.h>
+
+
+#include <dolfin/la/GenericSparsityPattern.h>
+#include <dolfin/common/MPI.h>
 #include <dolfin/mesh/Cell.h>
 #include <dolfin/mesh/Facet.h>
 #include <dolfin/mesh/Mesh.h>
-#include <dolfin/la/GenericSparsityPattern.h>
-#include <dolfin/function/FunctionSpace.h>
-#include "SparsityPatternBuilder.h"
 #include "GenericDofMap.h"
-#include "UFC.h"
-#include "Form.h"
+#include "SparsityPatternBuilder.h"
 
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
 void SparsityPatternBuilder::build(GenericSparsityPattern& sparsity_pattern,
                                    const Mesh& mesh,
-                                   std::vector<const GenericDofMap*>& dof_maps,
+                                   std::vector<const GenericDofMap*>& dofmaps,
                                    bool cells, bool interior_facets)
 {
-  const uint rank = dof_maps.size();
+  const uint rank = dofmaps.size();
+
+  // Get global dimensions and local range
+  std::vector<uint> global_dimensions(rank);
+  std::vector<std::pair<uint, uint> > local_range(rank);
+  std::vector<const boost::unordered_map<uint, uint>* > off_process_owner(rank);
 
-  // Get global dimensions
-  boost::scoped_array<uint> global_dimensions(new uint[rank]);
   for (uint i = 0; i < rank; ++i)
-    global_dimensions[i] = dof_maps[i]->global_dimension();
+  {
+    global_dimensions[i] = dofmaps[i]->global_dimension();
+    local_range[i]       = dofmaps[i]->ownership_range();
+    off_process_owner[i] = &(dofmaps[i]->off_process_owner());
+  }
 
   // Initialise sparsity pattern
-  sparsity_pattern.init(rank, global_dimensions.get());
+  sparsity_pattern.init(global_dimensions, local_range, off_process_owner);
 
   // Only build for rank >= 2 (matrices and higher order tensors)
   if (rank < 2)
     return;
 
-  // Allocate some more space
-  boost::scoped_array<uint> local_dimensions(new uint[rank]);
-  boost::scoped_array<uint> macro_local_dimensions(new uint[rank]);
-  uint** dofs = new uint*[rank];
-  uint** macro_dofs = new uint*[rank];
-  for (uint i = 0; i < rank; ++i)
-  {
-    local_dimensions[i] = dof_maps[i]->max_local_dimension();
-    macro_local_dimensions[i] = 2*dof_maps[i]->max_local_dimension();
+  // Create vector to point to dofs
+  std::vector<const std::vector<uint>* > dofs(rank);
 
-    dofs[i] = new uint[local_dimensions[i]];
-    macro_dofs[i] = new uint[macro_local_dimensions[i]];
-  }
+  std::vector<std::vector<uint> > _dofs(rank);
+
+  double t = time();
 
   // Build sparsity pattern for cell integrals
   if (cells)
   {
-    UFCCell ufc_cell(mesh);
-
+    Progress p("Building sparsity pattern over cells", mesh.num_cells());
     for (CellIterator cell(mesh); !cell.end(); ++cell)
     {
-      // Update to current cell
-      ufc_cell.update(*cell);
-
       // Tabulate dofs for each dimension and get local dimensions
       for (uint i = 0; i < rank; ++i)
-      {
-        dof_maps[i]->tabulate_dofs(dofs[i], ufc_cell, cell->index());
-        local_dimensions[i] = dof_maps[i]->local_dimension(ufc_cell);
-      }
+        dofs[i] = &dofmaps[i]->cell_dofs(cell->index());
 
-      // Fill sparsity pattern.
-      sparsity_pattern.insert(local_dimensions.get(), dofs);
+      // Insert non-zeroes in sparsity pattern
+      sparsity_pattern.insert(dofs);
+      p++;
     }
   }
+  t = time() -t;
+  //cout << "Sparsity time: " << t << endl;
 
   // FIXME: The below note is not true when there are no cell integrals,
   //        e.g. finite volume method
   // Note: no need to iterate over exterior facets since those dofs
   // are included when tabulating dofs on all cells
 
+  // Vector to store macro-dofs
+  std::vector<std::vector<uint> > macro_dofs(rank);
+
   // Build sparsity pattern for interior facet integrals
   if (interior_facets)
   {
-    UFCCell ufc_cell0(mesh);
-    UFCCell ufc_cell1(mesh);
-
     // Compute facets and facet - cell connectivity if not already computed
     mesh.init(mesh.topology().dim() - 1);
     mesh.init(mesh.topology().dim() - 1, mesh.topology().dim());
     if (!mesh.ordered())
       error("Mesh has not been ordered. Cannot compute sparsity pattern. Consider calling Mesh::order().");
 
+    Progress p("Building sparsity pattern over interior facets", mesh.num_facets());
     for (FacetIterator facet(mesh); !facet.end(); ++facet)
     {
       // Check if we have an interior facet
       if (facet->num_entities(mesh.topology().dim()) != 2)
+      {
+        p++;
         continue;
+      }
 
       // Get cells incident with facet
       Cell cell0(mesh, facet->entities(mesh.topology().dim())[0]);
       Cell cell1(mesh, facet->entities(mesh.topology().dim())[1]);
 
-      // Update to current cell
-      ufc_cell0.update(cell0);
-      ufc_cell1.update(cell1);
-
       // Tabulate dofs for each dimension on macro element
-      for (uint i = 0; i < rank; ++i)
+      for (uint i = 0; i < rank; i++)
       {
-        const uint offset = dof_maps[i]->local_dimension(ufc_cell0);
-        macro_local_dimensions[i] = offset + dof_maps[i]->local_dimension(ufc_cell1);
-        dof_maps[i]->tabulate_dofs(macro_dofs[i], ufc_cell0, cell0.index());
-        dof_maps[i]->tabulate_dofs(macro_dofs[i] + offset, ufc_cell1, cell1.index());
+        // Get dofs for each cell
+        const std::vector<uint>& cell_dofs0 = dofmaps[i]->cell_dofs(cell0.index());
+        const std::vector<uint>& cell_dofs1 = dofmaps[i]->cell_dofs(cell1.index());
+
+        // Create space in macro dof vector
+        macro_dofs[i].resize(cell_dofs0.size() + cell_dofs1.size());
+
+        // Copy cell dofs into macro dof vector
+        std::copy(cell_dofs0.begin(), cell_dofs0.end(), macro_dofs[i].begin());
+        std::copy(cell_dofs1.begin(), cell_dofs1.end(), macro_dofs[i].begin() + cell_dofs0.size());
+
+        // Store pointer to macro dofs
+        dofs[i] = &macro_dofs[i];
       }
 
-      // Fill sparsity pattern.
-      sparsity_pattern.insert(macro_local_dimensions.get(), macro_dofs);
+      // Vector to store macro-dofs
+      sparsity_pattern.insert(dofs);
+
+      p++;
     }
   }
 
   // Finalize sparsity pattern
   sparsity_pattern.apply();
-
-  // Clean up
-  for (uint i = 0; i < rank; i++)
-  {
-    delete [] dofs[i];
-    delete [] macro_dofs[i];
-  }
-  delete [] dofs;
-  delete [] macro_dofs;
 }
 //-----------------------------------------------------------------------------
