@@ -4,19 +4,20 @@
 // Modified by Anders Logg, 2008-2009.
 //
 // First added:  2009-06-22
-// Last changed: 2009-10-06
+// Last changed: 2011-01-25
 
 #ifndef __SYSTEM_ASSEMBLER_H
 #define __SYSTEM_ASSEMBLER_H
 
+#include <map>
 #include <vector>
 #include <dolfin/common/types.h>
+#include "DirichletBC.h"
 
 namespace dolfin
 {
 
   // Forward declarations
-  class DirichletBC;
   class GenericMatrix;
   class GenericTensor;
   class GenericVector;
@@ -92,32 +93,37 @@ namespace dolfin
     static void cell_wise_assembly(GenericMatrix& A, GenericVector& b,
                                    const Form& a, const Form& L,
                                    UFC& A_ufc, UFC& b_ufc, Scratch& data,
+                                   const DirichletBC::Map& boundary_values,
                                    const MeshFunction<uint>* cell_domains,
                                    const MeshFunction<uint>* exterior_facet_domains);
 
     static void facet_wise_assembly(GenericMatrix& A, GenericVector& b,
                                     const Form& a, const Form& L,
                                     UFC& A_ufc, UFC& b_ufc, Scratch& data,
+                                    const DirichletBC::Map& boundary_values,
                                     const MeshFunction<uint>* cell_domains,
                                     const MeshFunction<uint>* exterior_facet_domains,
                                     const MeshFunction<uint>* interior_facet_domains);
 
     static void assemble_interior_facet(GenericMatrix& A, GenericVector& b,
                                         UFC& A_ufc, UFC& b_ufc,
-                                        const Form& a,
-                                        const Form& L,
-                                        const Cell& cell0, const Cell& cell1, const Facet& facet,
-                                        const Scratch& data);
+                                        const Form& a, const Form& L,
+                                        const Cell& cell0, const Cell& cell1,
+                                        const Facet& facet,
+                                        Scratch& data,
+                                        const DirichletBC::Map& boundary_values);
 
     static void assemble_exterior_facet(GenericMatrix& A, GenericVector& b,
                                         UFC& A_ufc, UFC& b_ufc,
                                         const Form& a,
                                         const Form& L,
                                         const Cell& cell, const Facet& facet,
-                                        const Scratch& data);
+                                        Scratch& data,
+                                        const DirichletBC::Map& boundary_values);
 
-    static void apply_bc(double* A, double* b, const uint* indicators,
-                         const double* g, uint** global_dofs, const uint* dims);
+    static void apply_bc(double* A, double* b,
+                         const DirichletBC::Map& boundary_values,
+                         const std::vector<const std::vector<uint>* >& global_dofs);
 
     // Class to hold temporary data
     class Scratch
@@ -125,16 +131,14 @@ namespace dolfin
     public:
 
       Scratch(const Form& a, const Form& L);
+
       ~Scratch();
 
       void zero_cell();
 
-      uint A_num_entries, b_num_entries;
+      std::vector<double> Ae;
+      std::vector<double> be;
 
-      double* Ae;
-      double* be;
-      uint* indicators;
-      double* g;
     };
 
   };
