@@ -14,7 +14,10 @@
 #include <dolfin/fem/assemble.h>
 #include <dolfin/fem/Form.h>
 #include <dolfin/fem/Assembler.h>
+#include <dolfin/fem/SparsityPatternBuilder.h>
 #include <dolfin/fem/SystemAssembler.h>
 #include <dolfin/fem/VariationalProblem.h>
 
+#include <dolfin/fem/OpenMpAssembler.h>
+
 #endif
