@@ -2,7 +2,7 @@
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2009-09-28
-// Last changed: 2010-01-26
+// Last changed: 2011-01-21
 //
 // Modified by Johan Hake, 2009.
 
@@ -12,7 +12,6 @@
 #include <dolfin/mesh/Cell.h>
 #include <dolfin/mesh/Vertex.h>
 #include <dolfin/fem/UFCCell.h>
-#include "Data.h"
 #include "Expression.h"
 
 using namespace dolfin;
@@ -53,6 +52,18 @@ Expression::~Expression()
   // Do nothing
 }
 //-----------------------------------------------------------------------------
+void Expression::eval(Array<double>& values, const Array<double>& x,
+		      const ufc::cell& cell) const
+{
+  // Redirect to simple eval
+  eval(values, x);
+}
+//-----------------------------------------------------------------------------
+void Expression::eval(Array<double>& values, const Array<double>& x) const
+{
+  error("Missing eval() function (must be overloaded).");
+}
+//-----------------------------------------------------------------------------
 dolfin::uint Expression::value_rank() const
 {
   return value_shape.size();
@@ -66,20 +77,13 @@ dolfin::uint Expression::value_dimension(uint i) const
   return value_shape[i];
 }
 //-----------------------------------------------------------------------------
-void Expression::eval(Array<double>& values, const Data& data) const
-{
-  // Redirect to simple eval
-  eval(values, data.x);
-}
-//-----------------------------------------------------------------------------
 void Expression::restrict(double* w,
                           const FiniteElement& element,
                           const Cell& dolfin_cell,
-                          const ufc::cell& ufc_cell,
-                          int local_facet) const
+                          const ufc::cell& ufc_cell) const
 {
   // Restrict as UFC function (by calling eval)
-  restrict_as_ufc_function(w, element, dolfin_cell, ufc_cell, local_facet);
+  restrict_as_ufc_function(w, element, dolfin_cell, ufc_cell);
 }
 //-----------------------------------------------------------------------------
 void Expression::compute_vertex_values(Array<double>& vertex_values,
@@ -88,7 +92,6 @@ void Expression::compute_vertex_values(Array<double>& vertex_values,
   // Local data for vertex values
   const uint size = value_size();
   Array<double> local_vertex_values(size);
-  Data data;
 
   // Iterate over cells, overwriting values when repeatedly visiting vertices
   UFCCell ufc_cell(mesh);
@@ -96,16 +99,15 @@ void Expression::compute_vertex_values(Array<double>& vertex_values,
   {
     // Update cell data
     ufc_cell.update(*cell);
-    data.set(*cell, ufc_cell, -1);
 
     // Iterate over cell vertices
     for (VertexIterator vertex(*cell); !vertex.end(); ++vertex)
     {
-      // Update coordinate data
-      data.set(mesh.geometry().dim(), vertex->x());
+      // Wrap coordinate data
+      const Array<double> x(mesh.geometry().dim(), const_cast<double*>(vertex->x()));
 
       // Evaluate at vertex
-      eval(local_vertex_values, data);
+      eval(local_vertex_values, x, ufc_cell);
 
       // Copy to array
       for (uint i = 0; i < size; i++)
@@ -117,9 +119,3 @@ void Expression::compute_vertex_values(Array<double>& vertex_values,
   }
 }
 //-----------------------------------------------------------------------------
-void Expression::eval(Array<double>& values, const Array<double>& x) const
-{
-  error("Missing eval() for Expression (must be overloaded).");
-}
-//-----------------------------------------------------------------------------
-
