@@ -2,19 +2,19 @@
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2009-09-28
-// Last changed: 2010-01-26
+// Last changed: 2011-01-21
 
 #ifndef __EXPRESSION_H
 #define __EXPRESSION_H
 
 #include <vector>
+#include <ufc.h>
 #include <dolfin/common/Array.h>
 #include "GenericFunction.h"
 
 namespace dolfin
 {
 
-  class Data;
   class Mesh;
 
   /// This class represents a user-defined expression. Expressions can
@@ -54,6 +54,14 @@ namespace dolfin
     virtual ~Expression();
 
     //--- Implementation of GenericFunction interface ---
+    /// Note: The reimplementation of eval is needed for the Python interface
+
+    /// Evaluate at given point in given cell 
+    virtual void eval(Array<double>& values, const Array<double>& x,
+                      const ufc::cell& cell) const;
+
+    /// Evaluate at given point
+    virtual void eval(Array<double>& values, const Array<double>& x) const;
 
     /// Return value rank
     virtual uint value_rank() const;
@@ -61,25 +69,16 @@ namespace dolfin
     /// Return value dimension for given axis
     virtual uint value_dimension(uint i) const;
 
-    /// Evaluate function for given data
-    virtual void eval(Array<double>& values, const Data& data) const;
-
     /// Restrict function to local cell (compute expansion coefficients w)
     virtual void restrict(double* w,
                           const FiniteElement& element,
                           const Cell& dolfin_cell,
-                          const ufc::cell& ufc_cell,
-                          int local_facet) const;
+                          const ufc::cell& ufc_cell) const;
 
     /// Compute values at all mesh vertices
     virtual void compute_vertex_values(Array<double>& vertex_values,
                                        const Mesh& mesh) const;
 
-    //--- User-supplied callback for expression evaluation ---
-
-    /// Evaluate expression, must be overloaded by user (simple version)
-    virtual void eval(Array<double>& values, const Array<double>& x) const;
-
   protected:
 
     // Value shape
