@@ -1,31 +1,32 @@
 // Copyright (C) 2003-2009 Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
-// Modified by Garth N. Wells, 2005-2009.
+// Modified by Garth N. Wells, 2005-2010.
 // Modified by Martin Sandve Alnes, 2008.
 // Modified by Andre Massing, 2009.
 //
 // First added:  2003-11-28
-// Last changed: 2010-04-29
+// Last changed: 2011-02-21
 
 #include <algorithm>
+#include <map>
+#include <utility>
 #include <boost/assign/list_of.hpp>
-#include <boost/scoped_array.hpp>
-#include <dolfin/log/log.h>
+
+#include <dolfin/adaptivity/Extrapolation.h>
 #include <dolfin/common/utils.h>
 #include <dolfin/common/Array.h>
 #include <dolfin/common/NoDeleter.h>
-#include <dolfin/parameter/GlobalParameters.h>
-#include <dolfin/io/File.h>
-#include <dolfin/la/GenericVector.h>
-#include <dolfin/la/DefaultFactory.h>
 #include <dolfin/fem/FiniteElement.h>
 #include <dolfin/fem/GenericDofMap.h>
 #include <dolfin/fem/DirichletBC.h>
 #include <dolfin/fem/UFC.h>
+#include <dolfin/io/File.h>
+#include <dolfin/la/GenericVector.h>
+#include <dolfin/la/DefaultFactory.h>
+#include <dolfin/log/log.h>
 #include <dolfin/mesh/Vertex.h>
-#include <dolfin/adaptivity/Extrapolation.h>
-#include "Data.h"
+#include <dolfin/parameter/GlobalParameters.h>
 #include "Expression.h"
 #include "FunctionSpace.h"
 #include "Function.h"
@@ -34,28 +35,28 @@ using namespace dolfin;
 
 //-----------------------------------------------------------------------------
 Function::Function(const FunctionSpace& V)
-  : _function_space(reference_to_no_delete_pointer(V)),
-    allow_extrapolation(dolfin::parameters["allow_extrapolation"]),
-    local_scratch(V.element())
+  : Hierarchical<Function>(*this),
+    _function_space(reference_to_no_delete_pointer(V)),
+    allow_extrapolation(dolfin::parameters["allow_extrapolation"])
 {
   // Initialize vector
   init_vector();
 }
 //-----------------------------------------------------------------------------
 Function::Function(boost::shared_ptr<const FunctionSpace> V)
-  : _function_space(V),
-    allow_extrapolation(dolfin::parameters["allow_extrapolation"]),
-    local_scratch(V->element())
+  : Hierarchical<Function>(*this),
+    _function_space(V),
+    allow_extrapolation(dolfin::parameters["allow_extrapolation"])
 {
   // Initialize vector
   init_vector();
 }
 //-----------------------------------------------------------------------------
 Function::Function(const FunctionSpace& V, GenericVector& x)
-  : _function_space(reference_to_no_delete_pointer(V)),
+  : Hierarchical<Function>(*this),
+    _function_space(reference_to_no_delete_pointer(V)),
     _vector(reference_to_no_delete_pointer(x)),
-    allow_extrapolation(dolfin::parameters["allow_extrapolation"]),
-    local_scratch(V.element())
+    allow_extrapolation(dolfin::parameters["allow_extrapolation"])
 {
   // Assertion uses '<=' to deal with sub-functions
   assert(V.dofmap().global_dimension() <= x.size());
@@ -63,10 +64,10 @@ Function::Function(const FunctionSpace& V, GenericVector& x)
 //-----------------------------------------------------------------------------
 Function::Function(boost::shared_ptr<const FunctionSpace> V,
                    boost::shared_ptr<GenericVector> x)
-  : _function_space(V),
+  : Hierarchical<Function>(*this),
+    _function_space(V),
     _vector(x),
-    allow_extrapolation(dolfin::parameters["allow_extrapolation"]),
-    local_scratch(V->element())
+    allow_extrapolation(dolfin::parameters["allow_extrapolation"])
 {
   // Assertion uses '<=' to deal with sub-functions
   assert(V->dofmap().global_dimension() <= x->size());
@@ -74,19 +75,19 @@ Function::Function(boost::shared_ptr<const FunctionSpace> V,
 //-----------------------------------------------------------------------------
 Function::Function(boost::shared_ptr<const FunctionSpace> V,
                    GenericVector& x)
-  : _function_space(V),
+  : Hierarchical<Function>(*this),
+    _function_space(V),
     _vector(reference_to_no_delete_pointer(x)),
-    allow_extrapolation(dolfin::parameters["allow_extrapolation"]),
-    local_scratch(V->element())
+    allow_extrapolation(dolfin::parameters["allow_extrapolation"])
 {
   // Assertion uses '<=' to deal with sub-functions
   assert(V->dofmap().global_dimension() <= x.size());
 }
 //-----------------------------------------------------------------------------
 Function::Function(const FunctionSpace& V, std::string filename)
-  : _function_space(reference_to_no_delete_pointer(V)),
-    allow_extrapolation(dolfin::parameters["allow_extrapolation"]),
-    local_scratch(V.element())
+  : Hierarchical<Function>(*this),
+    _function_space(reference_to_no_delete_pointer(V)),
+    allow_extrapolation(dolfin::parameters["allow_extrapolation"])
 {
   // Initialize vector
   init_vector();
@@ -102,9 +103,9 @@ Function::Function(const FunctionSpace& V, std::string filename)
 //-----------------------------------------------------------------------------
 Function::Function(boost::shared_ptr<const FunctionSpace> V,
                    std::string filename)
-  : _function_space(V),
-    allow_extrapolation(dolfin::parameters["allow_extrapolation"]),
-    local_scratch(V->element())
+  : Hierarchical<Function>(*this),
+    _function_space(V),
+    allow_extrapolation(dolfin::parameters["allow_extrapolation"])
 {
   // Create vector
   DefaultFactory factory;
@@ -123,15 +124,16 @@ Function::Function(boost::shared_ptr<const FunctionSpace> V,
 }
 //-----------------------------------------------------------------------------
 Function::Function(const Function& v)
-  : allow_extrapolation(dolfin::parameters["allow_extrapolation"])
+  : Hierarchical<Function>(*this),
+    allow_extrapolation(dolfin::parameters["allow_extrapolation"])
 {
   // Assign data
   *this = v;
 }
 //-----------------------------------------------------------------------------
 Function::Function(const Function& v, uint i)
-  : allow_extrapolation(dolfin::parameters["allow_extrapolation"]),
-    local_scratch(v[i]._function_space->element())
+  : Hierarchical<Function>(*this),
+    allow_extrapolation(dolfin::parameters["allow_extrapolation"])
 {
   // Copy function space pointer
   this->_function_space = v[i]._function_space;
@@ -149,6 +151,7 @@ const Function& Function::operator= (const Function& v)
 {
   assert(v._vector);
 
+
   // Make a copy of all the data, or if v is a sub-function, then we collapse
   // the dof map and copy only the relevant entries from the vector of v.
   if (v._vector->size() == v._function_space->dim())
@@ -161,24 +164,23 @@ const Function& Function::operator= (const Function& v)
   }
   else
   {
+
     // Create collapsed dof map
+    const GenericDofMap& v_dofmap = v._function_space->dofmap();
     std::map<uint, uint> collapsed_map;
-    boost::shared_ptr<GenericDofMap> collapsed_dof_map(v._function_space->dofmap().collapse(collapsed_map, v._function_space->mesh()));
+    boost::shared_ptr<GenericDofMap> collapsed_dofmap(v_dofmap.collapse(collapsed_map, v._function_space->mesh()));
 
     // Create new FunctionsSpapce
-    _function_space = v._function_space->collapse_sub_space(collapsed_dof_map);
+    _function_space = v._function_space->collapse_sub_space(collapsed_dofmap);
 
     // FIXME: This assertion doesn't work in parallel
     //assert(collapsed_map.size() == _function_space->dofmap().global_dimension());
-
-    // Create new vector (global)
-    _vector.reset(v.vector().factory().create_vector());
-    _vector->resize( collapsed_dof_map->global_dimension() );
+    //assert(collapsed_map.size() == _function_space->dofmap().local_dimension());
 
     // Get row indices of original and new vectors
     std::map<uint, uint>::const_iterator entry;
     std::vector<uint> new_rows(collapsed_map.size());
-    std::vector<uint> old_rows(collapsed_map.size());
+    Array<uint> old_rows(collapsed_map.size());
     uint i = 0;
     for (entry = collapsed_map.begin(); entry != collapsed_map.end(); ++entry)
     {
@@ -186,15 +188,21 @@ const Function& Function::operator= (const Function& v)
       old_rows[i++] = entry->second;
     }
 
-    // Get old values and set new values
-    std::vector<double> values(collapsed_map.size());
-    //v.gather();
-    //v.get(&values[0], collapsed_map.size(), &old_rows[0]);
-    v.vector().get(&values[0], collapsed_map.size(), &old_rows[0]);
-    this->_vector->set(&values[0], collapsed_map.size(), &new_rows[0]);
+    // Gather values into an Array
+    Array<double> gathered_values;
+    v.vector().gather(gathered_values, old_rows);
+
+    // Initial new vector (global)
+    init_vector();
+    assert(_vector->size() == collapsed_dofmap->global_dimension());
+
+    // Set values in vector
+    this->_vector->set(&gathered_values[0], collapsed_map.size(), &new_rows[0]);
     this->_vector->apply("insert");
   }
-  local_scratch.init(this->_function_space->element());
+
+  // Call assignment operator for base class
+  Hierarchical<Function>::operator=(v);
 
   return *this;
 }
@@ -215,10 +223,10 @@ Function& Function::operator[] (uint i) const
   {
     // Extract function subspace
     std::vector<uint> component = boost::assign::list_of(i);
-    boost::shared_ptr<const FunctionSpace> sub_space(this->function_space().extract_sub_space(component));
+    boost::shared_ptr<const FunctionSpace> sub_space(_function_space->extract_sub_space(component));
 
     // Insert sub-Function into map and return reference
-    sub_functions.insert(i, new Function(sub_space, this->_vector));
+    sub_functions.insert(i, new Function(sub_space, _vector));
     return *(sub_functions.find(i)->second);
   }
 }
@@ -271,7 +279,7 @@ void Function::eval(Array<double>& values, const Array<double>& x) const
 
   // Find the cell that contains x
   const double* _x = x.data().get();
-  Point point(_function_space->mesh().geometry().dim(), _x);
+  const Point point(_function_space->mesh().geometry().dim(), _x);
   int id = _function_space->mesh().any_intersected_entity(point);
 
   // If not found, use the closest cell
@@ -289,11 +297,11 @@ void Function::eval(Array<double>& values, const Array<double>& x) const
     }
   }
 
+  // Create cell that contains point
+  const Cell cell(_function_space->mesh(), id);
+  const UFCCell ufc_cell(cell);
 
-  Cell cell(_function_space->mesh(), id);
-  UFCCell ufc_cell(cell);
-
-  // Evaluate function
+  // Call evaluate function
   eval(values, x, cell, ufc_cell);
 }
 //-----------------------------------------------------------------------------
@@ -302,17 +310,40 @@ void Function::eval(Array<double>& values,
                     const Cell& dolfin_cell,
                     const ufc::cell& ufc_cell) const
 {
+  // Developer note: work arrays/vectors are re-created each time this function
+  //                 is called for thread-safety
+
+  const FiniteElement& element = _function_space->element();
+
+  // FIXME: Rather than computing num_tensor_entries, we could just use
+  //        values.size()
+
+  // Compute in tensor (one for scalar function, . . .)
+  uint num_tensor_entries = 1;
+  for (uint i = 0; i < element.value_rank(); i++)
+    num_tensor_entries *= element.value_dimension(i);
+
+  assert(values.size() == num_tensor_entries);
+
+  // Create work vector for expansion coefficients
+  std::vector<double> coefficients(element.space_dimension());
+
   // Restrict function to cell
-  restrict(local_scratch.coefficients, _function_space->element(), dolfin_cell, ufc_cell, -1);
+  restrict(&coefficients[0], element, dolfin_cell, ufc_cell);
+
+  // Create work vector for basis
+  std::vector<double> basis(num_tensor_entries);
+
+  // Initialise values
+  for (uint j = 0; j < num_tensor_entries; ++j)
+    values[j] = 0.0;
 
   // Compute linear combination
-  for (uint j = 0; j < local_scratch.size; j++)
-    values[j] = 0.0;
-  for (uint i = 0; i < _function_space->element().space_dimension(); i++)
+  for (uint i = 0; i < element.space_dimension(); ++i)
   {
-    _function_space->element().evaluate_basis(i, local_scratch.values, &x[0], ufc_cell);
-    for (uint j = 0; j < local_scratch.size; j++)
-      values[j] += (local_scratch.coefficients[i])*(local_scratch.values[j]);
+    element.evaluate_basis(i, &basis[0], &x[0], ufc_cell);
+    for (uint j = 0; j < num_tensor_entries; ++j)
+      values[j] += coefficients[i]*basis[j];
   }
 }
 //-----------------------------------------------------------------------------
@@ -321,7 +352,10 @@ void Function::interpolate(const GenericFunction& v)
   // Gather off-process dofs
   v.gather();
 
+  // Initialise vector
   init_vector();
+
+  // Interpolate
   function_space().interpolate(*_vector, v);
 }
 //-----------------------------------------------------------------------------
@@ -340,30 +374,25 @@ dolfin::uint Function::value_dimension(uint i) const
   return _function_space->element().value_dimension(i);
 }
 //-----------------------------------------------------------------------------
-void Function::eval(Array<double>& values, const Data& data) const
+void Function::eval(Array<double>& values, const Array<double>& x,
+                    const ufc::cell& ufc_cell) const
 {
-  //assert(values);
   assert(_function_space);
 
-  // Check if UFC cell if available and cell matches
-  if (data._dolfin_cell && _function_space->has_cell(*data._dolfin_cell))
+  // Check if UFC cell comes from mesh, otherwise redirect to point-based evaluation
+  if (ufc_cell.mesh_identifier == (int) _function_space->mesh().id())
   {
-    // Efficient evaluation on given cell
-    assert(data._ufc_cell);
-    eval(values, data.x, *data._dolfin_cell, *data._ufc_cell);
+    const Cell cell(_function_space->mesh(), ufc_cell.index);
+    eval(values, x, cell, ufc_cell);
   }
   else
-  {
-    // Redirect to point-based evaluation
-    eval(values, data.x);
-  }
+    eval(values, x);
 }
 //-----------------------------------------------------------------------------
 void Function::restrict(double* w,
                         const FiniteElement& element,
                         const Cell& dolfin_cell,
-                        const ufc::cell& ufc_cell,
-                        int local_facet) const
+                        const ufc::cell& ufc_cell) const
 {
   assert(w);
   assert(_function_space);
@@ -371,19 +400,17 @@ void Function::restrict(double* w,
   // Check if we are restricting to an element of this function space
   if (_function_space->has_element(element) && _function_space->has_cell(dolfin_cell))
   {
-    // Get dofmap
+    // Get dofmap for cell
     const GenericDofMap& dofmap = _function_space->dofmap();
-
-    // Tabulate dofs
-    dofmap.tabulate_dofs(local_scratch.dofs, ufc_cell, dolfin_cell.index());
+    const std::vector<uint>& dofs = dofmap.cell_dofs(dolfin_cell.index());
 
     // Pick values from vector(s)
-    get(w, dofmap.local_dimension(ufc_cell), local_scratch.dofs);
+    _vector->get_local(w, dofs.size(), &dofs[0]);
   }
   else
   {
     // Restrict as UFC function (by calling eval)
-    restrict_as_ufc_function(w, element, dolfin_cell, ufc_cell, local_facet);
+    restrict_as_ufc_function(w, element, dolfin_cell, ufc_cell);
   }
 }
 //-----------------------------------------------------------------------------
@@ -392,9 +419,7 @@ void Function::compute_vertex_values(Array<double>& vertex_values,
 {
   assert(&mesh == &_function_space->mesh());
 
-  //vertex_values.resize(_function_space->dim());
-
-  // Gather off-process dofs
+  // Gather ghosts dofs
   gather();
 
   // Get finite element
@@ -402,7 +427,20 @@ void Function::compute_vertex_values(Array<double>& vertex_values,
 
   // Local data for interpolation on each cell
   const uint num_cell_vertices = mesh.type().num_vertices(mesh.topology().dim());
-  boost::scoped_array<double> local_vertex_values(new double[local_scratch.size*num_cell_vertices]);
+
+  // Compute in tensor (one for scalar function, . . .)
+  uint num_tensor_entries = 1;
+  for (uint i = 0; i < element.value_rank(); i++)
+    num_tensor_entries *= element.value_dimension(i);
+
+  // Resize Array for holding vertex values
+  vertex_values.resize(num_tensor_entries*(_function_space->mesh().num_vertices()));
+
+  // Create vector to hold cell vertex values
+  std::vector<double> cell_vertex_values(num_tensor_entries*num_cell_vertices);
+
+  // Create vector for expansion coefficients
+  std::vector<double> coefficients(element.space_dimension());
 
   // Interpolate vertex values on each cell (using last computed value if not
   // continuous, e.g. discontinuous Galerkin methods)
@@ -413,19 +451,20 @@ void Function::compute_vertex_values(Array<double>& vertex_values,
     ufc_cell.update(*cell);
 
     // Pick values from global vector
-    restrict(local_scratch.coefficients, element, *cell, ufc_cell, -1);
+    restrict(&coefficients[0], element, *cell, ufc_cell);
 
     // Interpolate values at the vertices
-    element.interpolate_vertex_values(local_vertex_values.get(), local_scratch.coefficients, ufc_cell);
+    element.interpolate_vertex_values(&cell_vertex_values[0],
+                                      &coefficients[0], ufc_cell);
 
     // Copy values to array of vertex values
     for (VertexIterator vertex(*cell); !vertex.end(); ++vertex)
     {
-      for (uint i = 0; i < local_scratch.size; ++i)
+      for (uint i = 0; i < num_tensor_entries; ++i)
       {
-        const uint local_index  = vertex.pos()*local_scratch.size + i;
+        const uint local_index  = vertex.pos()*num_tensor_entries + i;
         const uint global_index = i*mesh.num_vertices() + vertex->index();
-        vertex_values[global_index] = local_vertex_values[local_index];
+        vertex_values[global_index] = cell_vertex_values[local_index];
       }
     }
   }
@@ -433,178 +472,77 @@ void Function::compute_vertex_values(Array<double>& vertex_values,
 //-----------------------------------------------------------------------------
 void Function::gather() const
 {
-  // Gather off-process coefficients if running in parallel and function has a vector
   if (MPI::num_processes() > 1)
-  {
-    assert(_function_space);
-
-    // Initialise scratch space
-    gather_scratch.init(_function_space->dofmap().max_local_dimension());
-
-    // Compute lists of off-process dofs
-    compute_off_process_dofs();
-
-    // Create off process vector if it doesn't exist
-    if (!_off_process_vector.get())
-      _off_process_vector.reset(_vector->factory().create_local_vector());
-
-    // Gather off process coefficients
-    const Array<uint> wrapped_off_process_dofs(_off_process_dofs.size(),
-                                               &_off_process_dofs[0]);
-    _vector->gather(*_off_process_vector, wrapped_off_process_dofs);
-  }
-}
-//-----------------------------------------------------------------------------
-void Function::compute_off_process_dofs() const
-{
-  // Clear data
-  _off_process_dofs.clear();
-  global_to_local.clear();
-
-  // Get mesh
-  assert(_function_space);
-  const Mesh& mesh = _function_space->mesh();
-
-  // Storage for each cell dofs
-  const GenericDofMap& dofmap = _function_space->dofmap();
-  const uint num_dofs_per_cell = _function_space->element().space_dimension();
-  const uint num_dofs_global = vector().size();
-  boost::scoped_array<uint> dofs(new uint[num_dofs_per_cell]);
-
-  // Iterate over mesh and check which dofs are needed
-  UFCCell ufc_cell(mesh);
-  uint i = 0;
-  for (CellIterator cell(mesh); !cell.end(); ++cell)
-  {
-    // Update to current cell
-    ufc_cell.update(*cell);
-
-    // Tabulate dofs on cell
-    dofmap.tabulate_dofs(dofs.get(), ufc_cell, cell->index());
-
-    for (uint d = 0; d < num_dofs_per_cell; ++d)
-    {
-      const uint dof = dofs[d];
-      const uint index_owner = MPI::index_owner(dof, num_dofs_global);
-      if (index_owner != MPI::process_number())
-      {
-        if (std::find(_off_process_dofs.begin(), _off_process_dofs.end(), dof) == _off_process_dofs.end())
-        {
-          _off_process_dofs.push_back(dof);
-          global_to_local[dof] = i++;
-        }
-      }
-    }
-  }
+    _vector->update_ghost_values();
 }
 //-----------------------------------------------------------------------------
 void Function::init_vector()
 {
-  // Get size
+  // Check that function space is not a subspace (view)
+  if (function_space().dofmap().is_view())
+    error("Cannot create a Function from a subspace. The subspace needs to be collapsed.");
+
+  // Get global size
   const uint N = _function_space->dofmap().global_dimension();
 
+  // Get local range
+  const std::pair<uint, uint> range = _function_space->dofmap().ownership_range();
+  const uint local_size = range.second - range.first;
+
+  // Determine ghost vertices if dof map is distributed
+  std::vector<uint> ghost_indices;
+  if (N  > local_size)
+    compute_ghost_indices(range, ghost_indices);
+
   // Create vector of dofs
   if (!_vector)
   {
     DefaultFactory factory;
     _vector.reset(factory.create_vector());
   }
+  assert(_vector);
 
   // Initialize vector of dofs
-  assert(_vector);
-  _vector->resize(N);
+  _vector->resize(range, ghost_indices);
   _vector->zero();
 }
 //-----------------------------------------------------------------------------
-void Function::get(double* block, uint m, const uint* rows) const
+void Function::compute_ghost_indices(std::pair<uint, uint> range,
+                                     std::vector<uint>& ghost_indices) const
 {
-  // Get local ownership range
-  const std::pair<uint, uint> range = _vector->local_range();
+  // Clear data
+  ghost_indices.clear();
 
-  // Get local values when running in serial or collect values in parallel
-  if (range.first == 0 && range.second == _vector->size())
-    _vector->get(block, m, rows);
-  else
+  // Get mesh
+  assert(_function_space);
+  const Mesh& mesh = _function_space->mesh();
+
+  // Get dof map
+  const GenericDofMap& dofmap = _function_space->dofmap();
+
+  // Dofs per cell and total dofs
+  const uint num_dofs_per_cell = _function_space->element().space_dimension();
+
+  // Get local range
+  const uint n0 = range.first;
+  const uint n1 = range.second;
+
+  // Iterate over local mesh and check which dofs are needed
+  for (CellIterator cell(mesh); !cell.end(); ++cell)
   {
-    if (!_off_process_vector.get())
-      error("Function has not been prepared with off-process data. Did you forget to call Function::gather()?");
+    // Get dofs on cell
+    const std::vector<uint>& dofs = dofmap.cell_dofs(cell->index());
 
-    // FIXME: Perform some more sanity checks
-
-    // Build lists of local and nonlocal coefficients
-    uint n_local = 0;
-    uint n_nonlocal = 0;
-    for (uint i = 0; i < m; ++i)
+    for (uint d = 0; d < num_dofs_per_cell; ++d)
     {
-      if (rows[i] >= range.first && rows[i] < range.second)
+      const uint dof = dofs[d];
+      if (dof < n0 || dof >= n1)
       {
-        gather_scratch.local_index[n_local]  = i;
-        gather_scratch.local_rows[n_local++] = rows[i];
-     }
-      else
-      {
-        gather_scratch.nonlocal_index[n_nonlocal]  = i;
-        gather_scratch.nonlocal_rows[n_nonlocal++] = global_to_local[rows[i]];
+        // FIXME: Could we use dolfin::Set here? Or unordered_set?
+        if (std::find(ghost_indices.begin(), ghost_indices.end(), dof) == ghost_indices.end())
+          ghost_indices.push_back(dof);
       }
     }
-
-    // Get local coefficients
-    _vector->get_local(gather_scratch.local_block, n_local, gather_scratch.local_rows);
-
-    // Get off process coefficients
-    _off_process_vector->get_local(gather_scratch.nonlocal_block, n_nonlocal, gather_scratch.nonlocal_rows);
-
-    // Copy result into block
-    for (uint i = 0; i < n_local; ++i)
-      block[gather_scratch.local_index[i]] = gather_scratch.local_block[i];
-    for (uint i = 0; i < n_nonlocal; ++i)
-      block[gather_scratch.nonlocal_index[i]] = gather_scratch.nonlocal_block[i];
   }
 }
 //-----------------------------------------------------------------------------
-//-----------------------------------------------------------------------------
-Function::LocalScratch::LocalScratch(const FiniteElement& element)
-  : size(0), dofs(0), coefficients(0), values(0)
-{
-  init(element);
-}
-//-----------------------------------------------------------------------------
-Function::LocalScratch::LocalScratch()
-  : size(0), dofs(0), coefficients(0), values(0)
-{
-  // Do nothing
-}
-//-----------------------------------------------------------------------------
-Function::LocalScratch::~LocalScratch()
-{
-  delete [] dofs;
-  delete [] coefficients;
-  delete [] values;
-}
-//-----------------------------------------------------------------------------
-void Function::LocalScratch::init(const FiniteElement& element)
-{
-  // Compute size of value (number of entries in tensor value)
-  size = 1;
-  for (uint i = 0; i < element.value_rank(); i++)
-    size *= element.value_dimension(i);
-
-  // Initialize local array for mapping of dofs
-  delete [] dofs;
-  dofs = new uint[element.space_dimension()];
-  for (uint i = 0; i < element.space_dimension(); i++)
-    dofs[i] = 0;
-
-  // Initialize local array for expansion coefficients
-  delete [] coefficients;
-  coefficients = new double[element.space_dimension()];
-  for (uint i = 0; i < element.space_dimension(); i++)
-    coefficients[i] = 0.0;
-
-  // Initialize local array for values
-  delete [] values;
-  values = new double[size];
-  for (uint i = 0; i < size; i++)
-    values[i] = 0.0;
-}
-//-----------------------------------------------------------------------------
