@@ -1,22 +1,24 @@
 // Copyright (C) 2003-2009 Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
-// Modified by Garth N. Wells, 2005-2009.
+// Modified by Garth N. Wells, 2005-2010.
 // Modified by Kristian B. Oelgaard, 2007.
 // Modified by Martin Sandve Alnes, 2008.
 // Modified by Andre Massing, 2009.
 //
 // First added:  2003-11-28
-// Last changed: 2010-04-29
+// Last changed: 2011-01-31
 
 #ifndef __FUNCTION_H
 #define __FUNCTION_H
 
-#include <map>
+#include <utility>
 #include <vector>
 #include <boost/ptr_container/ptr_map.hpp>
 #include <boost/scoped_ptr.hpp>
 #include <boost/shared_ptr.hpp>
+
+#include <dolfin/common/Hierarchical.h>
 #include "GenericFunction.h"
 
 namespace ufc
@@ -29,7 +31,6 @@ namespace dolfin
 {
 
   // Forward declarations
-  class Data;
   class DirichletBC;
   class Expression;
   class FunctionSpace;
@@ -37,7 +38,6 @@ namespace dolfin
   class SubDomain;
   template<class T> class Array;
 
-
   /// This class represents a function u_h in a finite element
   /// function space V_h, given by
   ///
@@ -46,7 +46,7 @@ namespace dolfin
   /// where {phi_i}_i is a basis for V_h, and U is a vector of
   /// expansion coefficients for u_h.
 
-  class Function : public GenericFunction
+  class Function : public GenericFunction, public Hierarchical<Function>
 
   {
   public:
@@ -140,14 +140,13 @@ namespace dolfin
     virtual uint value_dimension(uint i) const;
 
     /// Evaluate function for given data
-    virtual void eval(Array<double>& values, const Data& data) const;
+    virtual void eval(Array<double>& values, const Array<double>& x, const ufc::cell& cell) const;
 
     /// Restrict function to local cell (compute expansion coefficients w)
     virtual void restrict(double* w,
                           const FiniteElement& element,
                           const Cell& dolfin_cell,
-                          const ufc::cell& ufc_cell,
-                          int local_facet) const;
+                          const ufc::cell& ufc_cell) const;
 
     /// Compute values at all mesh vertices
     virtual void compute_vertex_values(Array<double>& vertex_values,
@@ -171,7 +170,8 @@ namespace dolfin
     void init_vector();
 
     // Get coefficients from the vector(s)
-    void get(double* block, uint m, const uint* rows) const;
+    void compute_ghost_indices(std::pair<uint, uint> range,
+                               std::vector<uint>& ghost_indices) const;
 
     // The function space
     boost::shared_ptr<const FunctionSpace> _function_space;
@@ -179,93 +179,9 @@ namespace dolfin
     // The vector of expansion coefficients (local)
     boost::shared_ptr<GenericVector> _vector;
 
-    // The vector of expansion coefficients (off-process)
-    mutable boost::shared_ptr<GenericVector> _off_process_vector;
-    mutable std::map<uint, uint> global_to_local;
-    mutable std::vector<uint> _off_process_dofs;
-
     // True if extrapolation should be allowed
     bool allow_extrapolation;
 
-    // Scratch space, used for storing temporary local data
-    class LocalScratch
-    {
-    public:
-
-      // Constructor
-      LocalScratch(const FiniteElement& element);
-
-      // Constructor
-      LocalScratch();
-
-      // Destructor
-      ~LocalScratch();
-
-      // Initialize scratch space
-      void init(const FiniteElement& element);
-
-      // Value size (number of entries in tensor value)
-      uint size;
-
-      // Local array for mapping of dofs
-      uint* dofs;
-
-      // Local array for expansion coefficients
-      double* coefficients;
-
-      // Local array for values
-      double* values;
-
-    };
-
-    mutable LocalScratch local_scratch;
-
-    // Scratch data used in extracting coefficients from parallel vectors
-    class GatherScratch
-    {
-    public:
-
-      GatherScratch() : local_rows(0), nonlocal_rows(0), local_block(0),
-                        nonlocal_block(0), local_index(0), nonlocal_index(0)
-      {}
-
-      ~GatherScratch()
-      {
-        clear();
-      }
-
-      void init(uint dim)
-      {
-        clear();
-        local_rows     = new uint[dim];
-        nonlocal_rows  = new uint[dim];
-        local_block    = new double[dim];
-        nonlocal_block = new double[dim];
-        local_index    = new uint[dim];
-        nonlocal_index = new uint[dim];
-      }
-
-      void clear()
-      {
-        delete [] local_rows;
-        delete [] nonlocal_rows;
-        delete [] local_block;
-        delete [] nonlocal_block;
-        delete [] local_index;
-        delete [] nonlocal_index;
-      }
-
-      uint* local_rows;
-      uint* nonlocal_rows;
-      double* local_block;
-      double* nonlocal_block;
-      uint* local_index;
-      uint* nonlocal_index;
-
-    };
-
-    mutable GatherScratch gather_scratch;
-
   };
 
 }
