@@ -8,16 +8,14 @@
 // Modified by Ola Skavhaug, 2009.
 //
 // First added:  2008-09-11
-// Last changed: 2010-05-20
+// Last changed: 2011-02-03
 
 #include <iostream>
-#include <boost/scoped_array.hpp>
 
 #include <dolfin/common/utils.h>
-#include <dolfin/common/NoDeleter.h>
-#include <dolfin/main/MPI.h>
+#include <dolfin/common/MPI.h>
 #include <dolfin/fem/GenericDofMap.h>
-#include <dolfin/fem/UFC.h>
+#include <dolfin/fem/UFCCell.h>
 #include <dolfin/log/log.h>
 #include <dolfin/mesh/Vertex.h>
 #include <dolfin/mesh/Mesh.h>
@@ -36,7 +34,8 @@ using namespace dolfin;
 FunctionSpace::FunctionSpace(boost::shared_ptr<const Mesh> mesh,
                              boost::shared_ptr<const FiniteElement> element,
                              boost::shared_ptr<const GenericDofMap> dofmap)
-  : _mesh(mesh), _element(element), _dofmap(dofmap)
+  : Hierarchical<FunctionSpace>(*this),
+    _mesh(mesh), _element(element), _dofmap(dofmap)
 {
   // Do nothing
 }
@@ -44,22 +43,25 @@ FunctionSpace::FunctionSpace(boost::shared_ptr<const Mesh> mesh,
 FunctionSpace::FunctionSpace(boost::shared_ptr<Mesh> mesh,
                              boost::shared_ptr<const FiniteElement> element,
                              boost::shared_ptr<const GenericDofMap> dofmap)
-  : _mesh(mesh), _element(element), _dofmap(dofmap)
+  : Hierarchical<FunctionSpace>(*this),
+    _mesh(mesh), _element(element), _dofmap(dofmap)
 {
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-FunctionSpace::FunctionSpace(boost::shared_ptr<const Mesh> mesh) : _mesh(mesh)
+FunctionSpace::FunctionSpace(boost::shared_ptr<const Mesh> mesh)
+  : Hierarchical<FunctionSpace>(*this), _mesh(mesh)
 {
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-FunctionSpace::FunctionSpace(boost::shared_ptr<Mesh> mesh) : _mesh(mesh)
+FunctionSpace::FunctionSpace(boost::shared_ptr<Mesh> mesh)
+  : Hierarchical<FunctionSpace>(*this), _mesh(mesh)
 {
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-FunctionSpace::FunctionSpace(const FunctionSpace& V)
+FunctionSpace::FunctionSpace(const FunctionSpace& V) : Hierarchical<FunctionSpace>(*this)
 {
   // Assign data (will be shared)
   *this = V;
@@ -85,6 +87,9 @@ const FunctionSpace& FunctionSpace::operator=(const FunctionSpace& V)
   _dofmap    = V._dofmap;
   _component = V._component;
 
+  // Call assignment operator for base class
+  Hierarchical<FunctionSpace>::operator=(V);
+
   return *this;
 }
 //-----------------------------------------------------------------------------
@@ -123,9 +128,7 @@ void FunctionSpace::interpolate(GenericVector& expansion_coefficients,
   expansion_coefficients.zero();
 
   // Initialize local arrays
-  const uint max_local_dimension = _dofmap->max_local_dimension();
-  boost::scoped_array<double> cell_coefficients(new double[max_local_dimension]);
-  boost::scoped_array<uint> cell_dofs(new uint[max_local_dimension]);
+  std::vector<double> cell_coefficients(_dofmap->max_cell_dimension());
 
   // Iterate over mesh and interpolate on each cell
   UFCCell ufc_cell(*_mesh);
@@ -135,15 +138,15 @@ void FunctionSpace::interpolate(GenericVector& expansion_coefficients,
     ufc_cell.update(*cell);
 
     // Restrict function to cell
-    v.restrict(cell_coefficients.get(), this->element(), *cell, ufc_cell);
+    v.restrict(&cell_coefficients[0], this->element(), *cell, ufc_cell);
 
     // Tabulate dofs
-    _dofmap->tabulate_dofs(cell_dofs.get(), ufc_cell, cell->index());
+    const std::vector<uint>& cell_dofs =  _dofmap->cell_dofs(cell->index());
 
     // Copy dofs to vector
-    expansion_coefficients.set(cell_coefficients.get(),
-                               _dofmap->local_dimension(ufc_cell),
-                               cell_dofs.get());
+    expansion_coefficients.set(&cell_coefficients[0],
+                               _dofmap->cell_dimension(cell->index()),
+                               &cell_dofs[0]);
   }
 
   // Finalise changes
@@ -227,13 +230,11 @@ std::string FunctionSpace::str(bool verbose) const
 //-----------------------------------------------------------------------------
 void FunctionSpace::print_dofmap() const
 {
-  UFCCell ufc_cell(*_mesh);
   for (CellIterator cell(*_mesh); !cell.end(); ++cell)
   {
-    ufc_cell.update(*cell);
-    const uint n = _dofmap->local_dimension(ufc_cell);
-    boost::scoped_array<uint> dofs(new uint[n]);
-    _dofmap->tabulate_dofs(dofs.get(), ufc_cell, cell->index());
+    const uint n = _dofmap->cell_dimension(cell->index());
+    std::vector<uint> dofs(n);
+    _dofmap->tabulate_dofs(&dofs[0], *cell);
 
     cout << cell->index() << ":";
     for (uint i = 0; i < n; i++)
