@@ -4,7 +4,7 @@
 // Modified by Garth N. Wells, 2009.
 //
 // First added:  2009-09-28
-// Last changed: 2009-12-11
+// Last changed: 2011-01-19
 
 #ifndef __GENERIC_FUNCTION_H
 #define __GENERIC_FUNCTION_H
@@ -12,7 +12,6 @@
 #include <ufc.h>
 #include <dolfin/common/Array.h>
 #include <dolfin/common/Variable.h>
-#include "Data.h"
 
 namespace dolfin
 {
@@ -50,15 +49,18 @@ namespace dolfin
     /// Return value dimension for given axis
     virtual uint value_dimension(uint i) const = 0;
 
-    /// Evaluate function for given data
-    virtual void eval(Array<double>& values, const Data& data) const = 0;
+    /// Evaluate at given point in given cell
+    virtual void eval(Array<double>& values, const Array<double>& x,
+                      const ufc::cell& cell) const;
+
+    /// Evaluate at given point
+    virtual void eval(Array<double>& values, const Array<double>& x) const;
 
     /// Restrict function to local cell (compute expansion coefficients w)
     virtual void restrict(double* w,
                           const FiniteElement& element,
                           const Cell& dolfin_cell,
-                          const ufc::cell& ufc_cell,
-                          int local_facet) const = 0;
+                          const ufc::cell& ufc_cell) const = 0;
 
     /// Compute values at all mesh vertices
     virtual void compute_vertex_values(Array<double>& vertex_values,
@@ -71,16 +73,35 @@ namespace dolfin
 
     //--- Convenience functions ---
 
+    /// Evaluation at given point (scalar function)
+    double operator() (double x);
+
+    /// Evaluation at given point (scalar function)
+    double operator() (double x, double y);
+
+    /// Evaluation at given point (scalar function)
+    double operator() (double x, double y, double z);
+
+    /// Evaluation at given point (scalar function)
+    double operator() (const Point& p);
+
+    /// Evaluation at given point (vector-valued function)
+    void operator() (Array<double>& values, double x);
+
+    /// Evaluation at given point (vector-valued function)
+    void operator() (Array<double>& values, double x, double y);
+
+    /// Evaluation at given point (vector-valued function)
+    void operator() (Array<double>& values, double x, double y, double z);
+
+    /// Evaluation at given point (vector-valued function)
+    void operator() (Array<double>& values, const Point& p);
+
+    /// Evaluation at given point
+
     /// Return value size (product of value dimensions)
     uint value_size() const;
 
-    /// Convenience function for restriction when facet is unknown
-    void restrict(double* w,
-                  const FiniteElement& element,
-                  const Cell& dolfin_cell,
-                  const ufc::cell& ufc_cell) const
-    { restrict(w, element, dolfin_cell, ufc_cell, -1); }
-
     //--- Implementation of ufc::function interface ---
 
     /// Evaluate function at given point in cell
@@ -94,13 +115,7 @@ namespace dolfin
     void restrict_as_ufc_function(double* w,
                                   const FiniteElement& element,
                                   const Cell& dolfin_cell,
-                                  const ufc::cell& ufc_cell,
-                                  int local_facet) const;
-
-  private:
-
-    // Function call data
-    mutable Data data;
+                                  const ufc::cell& ufc_cell) const;
 
   };
 
