@@ -28,7 +28,7 @@ namespace dolfin
     MeshCoordinates(const Mesh& mesh);
 
     /// Evaluate function
-    void eval(Array<double>& values, const Data& data) const;
+    void eval(Array<double>& values, const Array<double>& x, const ufc::cell& cell) const;
 
   private:
 
@@ -37,6 +37,7 @@ namespace dolfin
 
   };
 
+  /*
   /// This Function represents the local cell size on a given mesh.
   class CellSize : public Expression
   {
@@ -54,6 +55,8 @@ namespace dolfin
     const Mesh& mesh;
 
   };
+*/
+
 
   /// This function represents the area/length of a cell facet on a given mesh.
   class FacetArea : public Expression
@@ -64,7 +67,8 @@ namespace dolfin
     FacetArea(const Mesh& mesh);
 
     /// Evaluate function
-    void eval(Array<double>& values, const Data& data) const;
+    void eval(Array<double>& values, const Array<double>& x,
+              const ufc::cell& cell) const;
 
   private:
 
