@@ -8,27 +8,141 @@
 #include <numeric>
 #include <set>
 #include <vector>
-#include <tr1/unordered_set>
-#include <tr1/unordered_map>
+#include <boost/unordered_set.hpp>
+#include <boost/unordered_map.hpp>
 #include <dolfin/log/log.h>
 #include <dolfin/common/types.h>
-#include <dolfin/main/MPI.h>
+#include <dolfin/common/MPI.h>
 #include <dolfin/mesh/LocalMeshData.h>
 #include <dolfin/mesh/Vertex.h>
 #include <dolfin/mesh/Cell.h>
 #include <dolfin/common/Timer.h>
 #include "GraphBuilder.h"
 
-
-#ifdef HAS_SCOTCH
-extern "C"
-{
-#include <ptscotch.h>
-}
-#endif
-
 using namespace dolfin;
 
+//-----------------------------------------------------------------------------
+Graph GraphBuilder::local_graph(const Mesh& mesh,
+                                const std::vector<uint>& coloring_type)
+{
+  // Check coloring type
+  assert(coloring_type.size() >= 2);
+  assert(coloring_type.front() == coloring_type.back());
+
+  // Create graph
+  const uint num_verticies = mesh.num_entities(coloring_type[0]);
+  Graph graph(num_verticies);
+
+  // Build graph
+  for (MeshEntityIterator vertex_entity(mesh, coloring_type[0]); !vertex_entity.end(); ++vertex_entity)
+  {
+    boost::unordered_set<uint> entity_list0;
+    boost::unordered_set<uint> entity_list1;
+    entity_list0.insert(vertex_entity->index());
+
+    // Build list of entities, moving between levels
+    for (uint level = 1; level < coloring_type.size(); ++level)
+    {
+      for (boost::unordered_set<uint>::const_iterator entity_index = entity_list0.begin(); entity_index != entity_list0.end(); ++entity_index)
+      {
+        const MeshEntity entity(mesh, coloring_type[level -1], *entity_index);
+        for (MeshEntityIterator neighbor(entity, coloring_type[level]); !neighbor.end(); ++neighbor)
+          entity_list1.insert(neighbor->index());
+      }
+      entity_list0 = entity_list1;
+      entity_list1.clear();
+    }
+
+    // Add edges to graph
+    const uint vertex_entity_index = vertex_entity->index();
+    //graph[vertex_entity_index] = entity_list0;
+    //for (boost::unordered_set<uint>::const_iterator entity_index = entity_list0.begin(); entity_index != entity_list0.end(); ++entity_index)
+    graph[vertex_entity_index].insert(entity_list0.begin(), entity_list0.end());
+  }
+
+  return graph;
+}
+//-----------------------------------------------------------------------------
+Graph GraphBuilder::local_graph(const Mesh& mesh, uint dim0, uint dim1)
+{
+  // Create graph
+  const uint num_verticies = mesh.num_entities(dim0);
+  Graph graph(num_verticies);
+
+  // Build graph
+  for (MeshEntityIterator colored_entity(mesh, dim0); !colored_entity.end(); ++colored_entity)
+  {
+    const uint colored_entity_index = colored_entity->index();
+    for (MeshEntityIterator entity(*colored_entity, dim1); !entity.end(); ++entity)
+    {
+      for (MeshEntityIterator neighbor(*entity, dim0); !neighbor.end(); ++neighbor)
+        graph[colored_entity_index].insert(neighbor->index());
+    }
+  }
+
+  return graph;
+}
+//-----------------------------------------------------------------------------
+BoostBidirectionalGraph GraphBuilder::local_boost_graph(const Mesh& mesh,
+                                        const std::vector<uint>& coloring_type)
+{
+  // Check coloring type
+  assert(coloring_type.size() >= 2);
+  assert(coloring_type.front() == coloring_type.back());
+
+  // Create graph
+  const uint num_verticies = mesh.num_entities(coloring_type[0]);
+  BoostBidirectionalGraph graph(num_verticies);
+
+  // Build graph
+  for (MeshEntityIterator vertex_entity(mesh, coloring_type[0]); !vertex_entity.end(); ++vertex_entity)
+  {
+    boost::unordered_set<uint> entity_list0;
+    boost::unordered_set<uint> entity_list1;
+    entity_list0.insert(vertex_entity->index());
+
+    // Build list of entities, moving between levels
+    for (uint level = 1; level < coloring_type.size(); ++level)
+    {
+      for (boost::unordered_set<uint>::const_iterator entity_index = entity_list0.begin(); entity_index != entity_list0.end(); ++entity_index)
+      {
+        const MeshEntity entity(mesh, coloring_type[level -1], *entity_index);
+        for (MeshEntityIterator neighbor(entity, coloring_type[level]); !neighbor.end(); ++neighbor)
+          entity_list1.insert(neighbor->index());
+      }
+      entity_list0 = entity_list1;
+      entity_list1.clear();
+    }
+
+    // Add edges to graph
+    const uint vertex_entity_index = vertex_entity->index();
+    for (boost::unordered_set<uint>::const_iterator entity_index = entity_list0.begin(); entity_index != entity_list0.end(); ++entity_index)
+      boost::add_edge(vertex_entity_index, *entity_index, graph);
+  }
+
+  return graph;
+}
+//-----------------------------------------------------------------------------
+BoostBidirectionalGraph GraphBuilder::local_boost_graph(const Mesh& mesh,
+                                                        uint dim0, uint dim1)
+{
+  // Create graph
+  const uint num_verticies = mesh.num_entities(dim0);
+  BoostBidirectionalGraph graph(num_verticies);
+
+  // Build graph
+  for (MeshEntityIterator colored_entity(mesh, dim0); !colored_entity.end(); ++colored_entity)
+  {
+    const uint colored_entity_index = colored_entity->index();
+    for (MeshEntityIterator entity(*colored_entity, dim1); !entity.end(); ++entity)
+    {
+      for (MeshEntityIterator neighbor(*entity, dim0); !neighbor.end(); ++neighbor)
+        boost::add_edge(colored_entity_index, neighbor->index(), graph);
+    }
+  }
+
+  return graph;
+}
 //-----------------------------------------------------------------------------
 void GraphBuilder::compute_dual_graph(const LocalMeshData& mesh_data,
                                 std::vector<std::set<uint> >& local_graph,
@@ -61,9 +175,9 @@ void GraphBuilder::compute_dual_graph(const LocalMeshData& mesh_data,
                        local_graph);
   cout << "Finished computing local cell-cell connections" << endl;
 
-  ///-----------------------------------------------
-  /// The rest only applies when running in parallel
-  ///-----------------------------------------------
+  //-----------------------------------------------
+  // The rest only applies when running in parallel
+  //-----------------------------------------------
 
   // Determine candidate ghost cells (graph ghost vertices)
   info("Preparing data to to send off-process.");
@@ -108,6 +222,10 @@ void GraphBuilder::compute_dual_graph(const LocalMeshData& mesh_data,
   // Set number of candidate ghost cells on this process to zero (not communicated to self)
   boundary_cells_per_process[MPI::process_number()] = 0;
 
+  // FIXME: Make the communication cleverer and more scalable. Send to
+  // one process at a time, and remove cells when it is know that all
+  // neighbors have been found.
+
   // Distribute data to all processes
   cout << "Send off-process data" << endl;
   MPI::distribute(transmit_data, partition);
@@ -119,16 +237,16 @@ void GraphBuilder::compute_dual_graph(const LocalMeshData& mesh_data,
 
   // Unpack data
   uint _offset = 0;
-  for (uint i = 0; i < MPI::num_processes()-1; ++i)
+  for (uint i = 0; i < MPI::num_processes() - 1; ++i)
   {
     const uint p = partition[_offset];
-    const uint data_length = (num_cell_vertices+1)*boundary_cells_per_process[p];
+    const uint data_length = (num_cell_vertices + 1)*boundary_cells_per_process[p];
 
     std::vector<uint>& _global_cell_indices         = candidate_ghost_cell_global_indices[p];
     std::vector<std::vector<uint> >& _cell_vertices = candidate_ghost_cell_vertices[p];
 
     // Loop over data for each cell
-    for (uint j = _offset; j < _offset + data_length; j += num_cell_vertices+1)
+    for (uint j = _offset; j < _offset + data_length; j += num_cell_vertices + 1)
     {
       assert(partition[j] == p);
 
@@ -138,7 +256,7 @@ void GraphBuilder::compute_dual_graph(const LocalMeshData& mesh_data,
       // Get cell vertices
       std::vector<uint> vertices;
       for (uint k = 0; k < num_cell_vertices; ++k)
-        vertices.push_back(transmit_data[(j+1)+k]);
+        vertices.push_back(transmit_data[(j + 1) +k]);
       _cell_vertices.push_back(vertices);
     }
 
@@ -172,8 +290,8 @@ void GraphBuilder::compute_connectivity(const std::vector<std::vector<uint> >& c
   std::vector<uint>::const_iterator c_vertex;
   std::vector<uint>::const_iterator connected_cell;
 
-  std::tr1::unordered_map<uint, std::vector<uint> > vertex_connectivity;
-  std::pair<std::tr1::unordered_map<uint, std::vector<uint> >::iterator, bool> ret;
+  boost::unordered_map<uint, std::vector<uint> > vertex_connectivity;
+  std::pair<boost::unordered_map<uint, std::vector<uint> >::iterator, bool> ret;
 
   // Build (global vertex)-(local cell) connectivity
   double tt = time();
@@ -233,7 +351,7 @@ void GraphBuilder::compute_connectivity(const std::vector<std::vector<uint> >& c
 
   tt = time();
   // Iterate over all vertices
-  std::tr1::unordered_map<uint, std::vector<uint> >::const_iterator _vertex;
+  boost::unordered_map<uint, std::vector<uint> >::const_iterator _vertex;
   for (_vertex = vertex_connectivity.begin(); _vertex != vertex_connectivity.end(); ++_vertex)
   {
     const std::vector<uint>& cell_list = _vertex->second;
@@ -282,8 +400,8 @@ dolfin::uint GraphBuilder::compute_ghost_connectivity(const std::vector<std::vec
   std::vector<uint>::const_iterator c_vertex;
   std::vector<uint>::const_iterator connected_cell;
 
-  std::tr1::unordered_map<uint, std::pair<std::vector<uint>, std::vector<uint> > > vertex_connectivity;
-  std::pair<std::tr1::unordered_map<uint, std::pair<std::vector<uint>, std::vector<uint> > >::iterator, bool> ret;
+  boost::unordered_map<uint, std::pair<std::vector<uint>, std::vector<uint> > > vertex_connectivity;
+  std::pair<boost::unordered_map<uint, std::pair<std::vector<uint>, std::vector<uint> > >::iterator, bool> ret;
 
   // Build boundary (global vertex)-(local cell) connectivity
   double tt = time();
