@@ -2,17 +2,16 @@
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2010-02-10
-// Last changed:
+// Last changed: 2010-12-31
 
 #include <algorithm>
 #include <map>
 #include <numeric>
 #include <set>
-#include <tr1/unordered_map>
-#include <tr1/unordered_set>
+
 #include <dolfin/common/Set.h>
 #include <dolfin/common/Timer.h>
-#include <dolfin/main/MPI.h>
+#include <dolfin/common/MPI.h>
 #include <dolfin/mesh/LocalMeshData.h>
 #include "GraphBuilder.h"
 #include "SCOTCH.h"
@@ -54,52 +53,50 @@ void SCOTCH::compute_partition(std::vector<uint>& cell_partition,
 void SCOTCH::partition(const std::vector<std::set<uint> >& local_graph,
                const std::set<uint>& ghost_vertices,
                const std::vector<uint>& global_cell_indices,
-               uint num_global_vertices,
+               const uint num_global_vertices,
                std::vector<uint>& cell_partition)
 {
   // C-style array indexing
-  const SCOTCH_Num baseval = 0;
+  const int baseval = 0;
 
   // Local data ---------------------------------
 
   // Number of local graph vertices (cells)
-  const SCOTCH_Num vertlocnbr = local_graph.size();
+  const int vertlocnbr = local_graph.size();
 
-  // Number of local + ghost graph vertices (cells)
-  // Number of local edges + edges connecting to ghost vertices
-  SCOTCH_Num edgelocnbr = 0;
+  // Data structures for graph input to SCOTCH
+  std::vector<int> vertloctab;
+  vertloctab.reserve(local_graph.size() + 1);
+  std::vector<int> edgeloctab;
+
+  // Build local graph input for SCOTCH
+  // (number of local + ghost graph vertices (cells),
+  //  number of local edges + edges connecting to ghost vertices)
+  int edgelocnbr = 0;
+  vertloctab.push_back(0);
   std::vector<std::set<uint> >::const_iterator vertex;
   for(vertex = local_graph.begin(); vertex != local_graph.end(); ++vertex)
-    edgelocnbr += vertex->size();
-
-  // Local graph layout
-  std::vector<SCOTCH_Num> vertloctab;
-  std::vector<SCOTCH_Num> edgeloctab;
-  vertloctab.push_back(0);
-  for (uint i = 0; i < local_graph.size(); ++i)
   {
-    const std::set<uint>& vertices = local_graph[i];
-    vertloctab.push_back(vertloctab[i] + vertices.size());
-    edgeloctab.insert( edgeloctab.end(), vertices.begin(), vertices.end() );
+    edgelocnbr += vertex->size();
+    vertloctab.push_back(vertloctab.back() + vertex->size());
+    edgeloctab.insert(edgeloctab.end(), vertex->begin(), vertex->end());
   }
 
   // Global data ---------------------------------
 
   // Number of local vertices (cells) on each process
-  std::vector<uint> tmp = MPI::gather(local_graph.size());
-  std::vector<SCOTCH_Num> proccnttab(MPI::num_processes());
-  for (uint i = 0; i < MPI::num_processes(); ++i)
-    proccnttab[i] = tmp[i];
+  std::vector<uint> proccnttab = MPI::gather(local_graph.size());
 
- // Array containing . . . .
-  std::vector<SCOTCH_Num> procvrttab(MPI::num_processes() + 1);
+  // FIXME: explain this test
+  // Array containing . . . . (some sanity checks)
+  std::vector<uint> procvrttab(MPI::num_processes() + 1);
   for (uint i = 0; i < MPI::num_processes(); ++i)
     procvrttab[i] = std::accumulate(proccnttab.begin(), proccnttab.begin() + i, 0);
-  procvrttab[MPI::num_processes()] = procvrttab[MPI::num_processes()-1] + proccnttab[MPI::num_processes()-1];
+  procvrttab[MPI::num_processes()] = procvrttab[MPI::num_processes() - 1] + proccnttab[MPI::num_processes() - 1];
 
   // Sanity check
   for (uint i = 1; i <= MPI::process_number(); ++i)
-    assert( procvrttab[i] >= (procvrttab[i-1] + proccnttab[i-1]) );
+    assert(procvrttab[i] >= (procvrttab[i - 1] + proccnttab[i - 1]));
 
   /*
   // Print data ---------------
@@ -115,7 +112,6 @@ void SCOTCH::partition(const std::vector<std::set<uint> >& local_graph,
   // Number of processes
   const SCOTCH_Num procglbnbr = MPI::num_processes();
 
-  //------ Print data
   cout << "Num vertices      : " << vertglbnbr << endl;
   cout << "Num edges         : " << edgeglbnbr << endl;
   cout << "Num of processes  : " << procglbnbr << endl;
@@ -133,6 +129,7 @@ void SCOTCH::partition(const std::vector<std::set<uint> >& local_graph,
   cout << "(*) Num vert (inc ghost): " << vertgstnbr << endl;
   cout << "(*) Num edges           : " << edgelocnbr << endl;
   cout << "(*) Vertloctab          : " << endl;
+
   for (uint i = 0; i < vertloctab.size(); ++i)
     cout << vertloctab[i] << " " ;
   cout << endl;
@@ -140,7 +137,7 @@ void SCOTCH::partition(const std::vector<std::set<uint> >& local_graph,
   for (uint i = 0; i < edgeloctab.size(); ++i)
     cout << edgeloctab[i] << " ";
   cout << endl;
-  // ---------------------------
+  // -----
   */
 
   // Construct communicator (copy of MPI_COMM_WORLD)
@@ -151,7 +148,8 @@ void SCOTCH::partition(const std::vector<std::set<uint> >& local_graph,
   if (SCOTCH_dgraphInit(&dgrafdat, *comm) != 0)
     error("Error initialising SCOTCH graph.");
 
-  // Build SCOTCH Dgraph
+  // Build SCOTCH distributed graph
+  info("Start SCOTCH graph building.");
   if (SCOTCH_dgraphBuild(&dgrafdat, baseval, vertlocnbr, vertlocnbr,
                               &vertloctab[0], NULL, NULL, NULL,
                               edgelocnbr, edgelocnbr,
@@ -159,44 +157,42 @@ void SCOTCH::partition(const std::vector<std::set<uint> >& local_graph,
   {
     error("Error buidling SCOTCH graph.");
   }
+  info("End SCOTCH graph building.");
 
   // Check graph
   if (SCOTCH_dgraphCheck(&dgrafdat))
     error("Consistency error in SCOTCH graph.");
 
+  // Deal with ghost vertices (write a better description)
   SCOTCH_dgraphGhst(&dgrafdat);
 
   // Number of partitions (set equal to number of processes)
-  SCOTCH_Num npart = MPI::num_processes();
+  const int npart = MPI::num_processes();
 
   // Partitioning strategy
   SCOTCH_Strat strat;
   SCOTCH_stratInit(&strat);
 
-  // Set strategy (SCOTCH uses very crytic strings for this)
-  std::string strategy = "b{sep=m{asc=b{bnd=q{strat=f},org=q{strat=f}},low=q{strat=m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}|m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}},seq=q{strat=m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}|m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}}},seq=b{job=t,map=t,poli=S,sep=m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}|m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}}}";
-  SCOTCH_stratDgraphMap (&strat, strategy.c_str());
+  // Set strategy (SCOTCH uses very crytic strings for this, and they can change between versions)
+  //std::string strategy = "b{sep=m{asc=b{bnd=q{strat=f},org=q{strat=f}},low=q{strat=m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}|m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}},seq=q{strat=m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}|m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}}},seq=b{job=t,map=t,poli=S,sep=m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}|m{type=h,vert=80,low=h{pass=10}f{bal=0.0005,move=80},asc=b{bnd=d{dif=1,rem=1,pass=40}f{bal=0.005,move=80},org=f{bal=0.005,move=80}}}}}";
+  //SCOTCH_stratDgraphMap (&strat, strategy.c_str());
 
-  // Hold partition data
-  std::vector<SCOTCH_Num> partloctab(vertlocnbr);
+  // Resize vector to hold cell partition indices
+  cell_partition.resize(vertlocnbr);
+  int* _cell_partition = reinterpret_cast<int*>(&cell_partition[0]);
 
+  // Reset SCOTCH random number generator to produce determinstic partitions
+  SCOTCH_randomReset();
+
+  // Partition graph
   info("Start SCOTCH partitioning.");
-  if (SCOTCH_dgraphPart(&dgrafdat, npart, &strat, &partloctab[0]))
+  if (SCOTCH_dgraphPart(&dgrafdat, npart, &strat, _cell_partition))
     error("Error during partitioning.");
   info("End SCOTCH partitioning.");
 
   // Clean up SCOTCH objects
   SCOTCH_dgraphExit(&dgrafdat);
   SCOTCH_stratExit(&strat);
-
-  // Copy partiton datap
-  cell_partition.resize(vertlocnbr);
-  for (uint i = 0; i < cell_partition.size(); ++i)
-  {
-    if ( cell_partition[i] < 0 || cell_partition[i] >= (uint) npart )
-      error("Problem with SCOTCH partition.");
-    cell_partition[i] = partloctab[i];
-  }
 }
 //-----------------------------------------------------------------------------
 #else
@@ -217,4 +213,3 @@ void SCOTCH::partition(const std::vector<std::set<uint> >& local_graph,
 }
 //-----------------------------------------------------------------------------
 #endif
-
