@@ -15,6 +15,7 @@
 #include <ostream>
 #include <string>
 #include <utility>
+#include <boost/scoped_ptr.hpp>
 #include "GenericFile.h"
 
 namespace dolfin
@@ -32,7 +33,7 @@ namespace dolfin
   public:
 
     /// File formats
-    enum Type {xml, matlab, octave, vtk, python, raw, xyz, binary};
+    enum Type {xml, vtk, python, raw, xyz, binary};
 
     /// Create a file with given name
     File(const std::string filename, std::string encoding = "ascii");
@@ -72,7 +73,7 @@ namespace dolfin
   private:
 
     // Pointer to implementation (envelop-letter design)
-    GenericFile* file;
+    boost::scoped_ptr<GenericFile> file;
 
   };
 
