@@ -42,21 +42,20 @@ namespace dolfin
 
     void finalize(std::string vtu_filename, double time);
 
-    void mesh_write(const Mesh& mesh, std::string file) const;
-
     void results_write(const Function& u, std::string file) const;
 
     void write_point_data(const GenericFunction& u, const Mesh& mesh,
                           std::string file) const;
 
-    void write_cell_data(const Function& u, std::string file) const;
-
     void pvd_file_write(uint step, double time, std::string file);
 
     void pvtu_mesh_write(std::string pvtu_filename, std::string vtu_filename) const;
 
     void pvtu_results_write(const Function& u, std::string pvtu_filename) const;
 
+    void pvtu_results_write(uint dim, uint rank, std::string data_type,
+                            std::string name, std::string pvtu_filename) const;
+
     void vtk_header_open(uint num_vertices, uint num_cells, std::string file) const;
     void vtk_header_close(std::string file) const;
 
@@ -93,8 +92,11 @@ namespace dolfin
 
     // File encoding
     const std::string encoding;
-
     std::string encode_string;
+
+    bool binary;
+    bool compress;
+
   };
 
 }
