@@ -7,6 +7,7 @@
 #ifndef __XMLARRAY_H
 #define __XMLARRAY_H
 
+#include <utility>
 #include <vector>
 #include "XMLHandler.h"
 
@@ -19,14 +20,14 @@ namespace dolfin
   {
   public:
 
-    XMLArray(std::vector<int>& ix, XMLFile& parser);
-    XMLArray(std::vector<int>& ix, XMLFile& parser, uint size);
+    XMLArray(std::vector<int>& ix, XMLFile& parser, bool distributed = false);
+    XMLArray(std::vector<int>& ix, XMLFile& parser, uint size, bool distributed = false);
 
-    XMLArray(std::vector<uint>& ux, XMLFile& parser);
-    XMLArray(std::vector<uint>& ux, XMLFile& parser, uint size);
+    XMLArray(std::vector<uint>& ux, XMLFile& parser, bool distributed = false);
+    XMLArray(std::vector<uint>& ux, XMLFile& parser, uint size, bool distributed = false);
 
-    XMLArray(std::vector<double>& dx, XMLFile& parser);
-    XMLArray(std::vector<double>& dx, XMLFile& parser, uint size);
+    XMLArray(std::vector<double>& dx, XMLFile& parser, bool distributed = false);
+    XMLArray(std::vector<double>& dx, XMLFile& parser, uint size, bool distributed = false);
 
     void start_element (const xmlChar *name, const xmlChar **attrs);
     void end_element   (const xmlChar *name);
@@ -46,19 +47,35 @@ namespace dolfin
 
   private:
 
+    friend class XMLVector;
+
     enum parser_state { OUTSIDE_ARRAY, INSIDE_ARRAY, ARRAY_DONE };
     enum array_type { INT, UINT, DOUBLE, UNSET };
 
-    void read_entry  (const xmlChar *name, const xmlChar **attrs);
+    void read_entry(const xmlChar *name, const xmlChar **attrs);
 
-    std::vector<int>*  ix;
+    // Element indices
+    std::vector<uint> element_index;
+
+    // Integer data
+    std::vector<int>* ix;
+
+    // Unsigned integer data
     std::vector<uint>* ux;
+
+    // Double data
     std::vector<double>* dx;
+
     parser_state state;
     array_type atype;
 
+    // Range for indices
+    std::pair<uint, uint> range;
+
     uint size;
 
+    bool distributed;
+
   };
 
 }
