@@ -10,10 +10,12 @@
 
 #include <dolfin/common/types.h>
 #include <dolfin/common/constants.h>
-#include <dolfin/log/log.h>
-#include <dolfin/mesh/Mesh.h>
-#include <dolfin/mesh/LocalMeshData.h>
 #include <dolfin/la/GenericMatrix.h>
+#include <dolfin/log/log.h>
+#include <dolfin/mesh/LocalMeshData.h>
+#include <dolfin/mesh/Mesh.h>
+#include <dolfin/mesh/MeshEntity.h>
+#include <dolfin/mesh/MeshFunction.h>
 #include <dolfin/plot/FunctionPlotData.h>
 #include "XMLArray.h"
 #include "XMLMap.h"
@@ -27,7 +29,7 @@ using namespace dolfin;
 
 //-----------------------------------------------------------------------------
 XMLFile::XMLFile(const std::string filename, bool gzip)
-  : GenericFile(filename), sax(0), outstream(0), gzip(gzip)
+               : GenericFile(filename), sax(0), outstream(0), gzip(gzip)
 {
   // Set up the output stream (to file)
   outstream = new std::ofstream();
@@ -45,8 +47,7 @@ XMLFile::XMLFile(const std::string filename, bool gzip)
   sax->fatalError    = sax_fatal_error;
 }
 //-----------------------------------------------------------------------------
-XMLFile::XMLFile(std::ostream& s)
-  : GenericFile(""), sax(0), outstream(&s)
+XMLFile::XMLFile(std::ostream& s) : GenericFile(""), sax(0), outstream(&s)
 {
   // Set up the sax handler.
   sax = new xmlSAXHandler();
@@ -65,9 +66,9 @@ XMLFile::~XMLFile()
 {
   delete sax;
 
-  // Only delete outstream if it is a ofstream
+  // Only delete outstream if it is an 'ofstream'
   std::ofstream* outfile = dynamic_cast<std::ofstream*>(outstream);
-  if ( outfile != 0)
+  if (outfile)
   {
     outfile = 0;
     delete outstream;
@@ -82,7 +83,7 @@ void XMLFile::validate(const std::string filename)
   xmlDocPtr document;
   document = xmlParseFile(filename.c_str());
   int ret = 1;
-  parser = xmlRelaxNGNewParserCtxt("http://fenics.org/pub/misc/dolfin.rng");
+  parser = xmlRelaxNGNewParserCtxt("http://fenicsproject.org/pub/misc/dolfin.rng");
   xmlRelaxNGSetParserStructuredErrors(parser,
                                       (xmlStructuredErrorFunc)rng_parser_error,
                                       stderr);
@@ -143,10 +144,9 @@ void XMLFile::end_element(const xmlChar *name)
 //-----------------------------------------------------------------------------
 void XMLFile::open_file()
 {
-
   // Convert to ofstream
   std::ofstream* outfile = dynamic_cast<std::ofstream*>(outstream);
-  if ( outfile != 0 )
+  if (outfile)
   {
     // Open file
     outfile->open(filename.c_str());
@@ -160,9 +160,10 @@ void XMLFile::open_file()
 void XMLFile::close_file()
 {
   XMLDolfin::write_end(*outstream);
+
   // Convert to ofstream
   std::ofstream* outfile = dynamic_cast<std::ofstream*>(outstream);
-  if ( outfile != 0 )
+  if (outfile)
     outfile->close();
 }
 //-----------------------------------------------------------------------------
@@ -178,9 +179,8 @@ void dolfin::sax_end_document(void *ctx)
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-void dolfin::sax_start_element(void *ctx,
-                                   const xmlChar *name,
-                                   const xmlChar **attrs)
+void dolfin::sax_start_element(void *ctx, const xmlChar *name,
+                               const xmlChar **attrs)
 {
   ( (XMLFile*) ctx )->start_element(name, attrs);
 }
@@ -232,8 +232,10 @@ void dolfin::rng_parser_error(void *user_data, xmlErrorPtr error)
   int length = buffer.length();
   buffer.erase(length-1);
   if (node != NULL)
+  {
     warning("%s:%d: element %s: Relax-NG parser error: %s",
             file, line, node->name, buffer.c_str());
+  }
 }
 //-----------------------------------------------------------------------------
 void dolfin::rng_valid_error(void *user_data, xmlErrorPtr error)
