@@ -4,6 +4,7 @@
 // First added: 2009-03-09
 // Last changed: 2009-03-17
 
+#include "dolfin/mesh/MeshData.h"
 #include "XMLMeshFunction.h"
 #include "XMLIndent.h"
 #include "XMLArray.h"
@@ -96,7 +97,8 @@ void XMLMeshData::end_element (const xmlChar* name)
   }
 }
 //-----------------------------------------------------------------------------
-void XMLMeshData::write(const MeshData& data, std::ostream& outfile, uint indentation_level)
+void XMLMeshData::write(const MeshData& data, std::ostream& outfile,
+                        uint indentation_level)
 {
   if ( data.mesh_functions.size() > 0 || data.arrays.size() > 0)
   {
