@@ -5,19 +5,19 @@
 //
 // First added:  2008-07-02
 
-#include <sstream>
 #include <fstream>
+#include <sstream>
 #include <boost/scoped_array.hpp>
 
 #include <dolfin/common/Array.h>
 #include <dolfin/fem/FiniteElement.h>
+#include <dolfin/function/Function.h>
+#include <dolfin/function/FunctionSpace.h>
+#include <dolfin/la/Vector.h>
 #include <dolfin/mesh/Mesh.h>
 #include <dolfin/mesh/MeshFunction.h>
 #include <dolfin/mesh/Vertex.h>
 #include <dolfin/mesh/Cell.h>
-#include <dolfin/function/Function.h>
-#include <dolfin/function/FunctionSpace.h>
-#include <dolfin/la/Vector.h>
 #include "XYZFile.h"
 
 using namespace dolfin;
@@ -36,10 +36,10 @@ XYZFile::~XYZFile()
 void XYZFile::operator<<(const Function& u)
 {
   // Update xyz file name and clear file
-  xyzNameUpdate(counter);
+  xyz_name_update(counter);
 
   // Write results
-  ResultsWrite(u);
+  results_write(u);
 
   // Increase the number of times we have saved the function
   counter++;
@@ -48,7 +48,7 @@ void XYZFile::operator<<(const Function& u)
        << ") to file " << filename << " in xd3d xyz format." << endl;
 }
 //----------------------------------------------------------------------------
-void XYZFile::ResultsWrite(const Function& u) const
+void XYZFile::results_write(const Function& u) const
 {
   // Open file
   std::ofstream fp(xyz_filename.c_str(), std::ios_base::app);
@@ -87,7 +87,7 @@ void XYZFile::ResultsWrite(const Function& u) const
   }
 }
 //----------------------------------------------------------------------------
-void XYZFile::xyzNameUpdate(const int counter)
+void XYZFile::xyz_name_update(int counter)
 {
   std::string filestart, extension;
   std::ostringstream fileid, newfilename;
@@ -104,17 +104,16 @@ void XYZFile::xyzNameUpdate(const int counter)
   xyz_filename = newfilename.str();
 
   // Make sure file is empty
-  FILE* fp = fopen(xyz_filename.c_str(), "w");
+  std::ofstream fp(xyz_filename.c_str(), std::ios_base::trunc);
   if (!fp)
     error("Unable to open file %s", filename.c_str());
-  fclose(fp);
 }
 //----------------------------------------------------------------------------
 template<class T>
-void XYZFile::MeshFunctionWrite(T& meshfunction)
+void XYZFile::mesh_function_write(T& meshfunction)
 {
   // Update xyz file name and clear file
-  xyzNameUpdate(counter);
+  xyz_name_update(counter);
 
   Mesh& mesh = meshfunction.mesh();
 
@@ -123,19 +122,17 @@ void XYZFile::MeshFunctionWrite(T& meshfunction)
 
   // Open file
   std::ofstream fp(xyz_filename.c_str(), std::ios_base::app);
+  if (!fp)
+    error("Unable to open file %s", filename.c_str());
 
-  fp<<mesh.num_cells( ) <<std::endl;
+  fp << mesh.num_cells() <<std::endl;
   for (CellIterator cell(mesh); !cell.end(); ++cell)
     fp << meshfunction.get( cell->index() )  << std::endl;
 
-  // Close file
-  fp.close();
-
   // Increase the number of times we have saved the mesh function
   counter++;
 
   cout << "saved mesh function " << counter << " times." << endl;
-
   cout << "Saved mesh function " << mesh.name() << " (" << mesh.label()
        << ") to file " << filename << " in XYZ format." << endl;
 }
