@@ -1,45 +1,42 @@
 // Copyright (C) 2008 Kent-Andre Mardal.
 // Licensed under the GNU LGPL Version 2.1.
 //
-// First added:  2008-08-25
-// Last changed: 2009-09-08
-//
 // Modified by Anders Logg, 2008.
+// Modified by Garth N. Wells, 2011.
+//
+// First added:  2008-08-25
+// Last changed: 2011-01-22
 
 #ifndef __BLOCKMATRIX_H
 #define __BLOCKMATRIX_H
 
-#include <map>
-#include "Matrix.h"
+#include <boost/multi_array.hpp>
+#include <boost/shared_ptr.hpp>
 
 namespace dolfin
 {
-  // Forward declaration
-  class SubMatrix;
+
+  /// Forward declarations
+  class GenericMatrix;
 
   class BlockMatrix
   {
   public:
 
-    // FIXME: Change order of m and n
-
     // Constructor
-    BlockMatrix(uint n=0, uint m=0, bool owner=false);
+    BlockMatrix(uint m = 0, uint n = 0);
 
     // Destructor
     ~BlockMatrix();
 
-    /// Return SubMatrix reference number (i,j)
-    SubMatrix operator() (uint i, uint j);
-
     /// Set block
-    void set(uint i, uint j, Matrix& m);
+    void set_block(uint i, uint j, boost::shared_ptr<GenericMatrix> m);
 
     /// Get block (const version)
-    const Matrix& get(uint i, uint j) const;
+    const boost::shared_ptr<GenericMatrix> get_block(uint i, uint j) const;
 
     /// Get block
-    Matrix& get(uint i, uint j);
+    boost::shared_ptr<GenericMatrix> get_block(uint i, uint j);
 
     /// Return size of given dimension
     uint size(uint dim) const;
@@ -56,34 +53,13 @@ namespace dolfin
     /// Matrix-vector product, y = Ax
     void mult(const BlockVector& x, BlockVector& y, bool transposed=false) const;
 
-  private:
-
-    bool owner;
-    uint n, m;
-    //    std::map<std::pair<int,int>, Matrix*> matrices;
-    Matrix** matrices;
-
-  };
-
-  // SubMatrix
-  // Rip off of the design in Table and TableEntry for giving nice operators
-  // A(0,0) = A00 also in the case with external storage.
-
-  class SubMatrix
-  {
-  public:
-
-    SubMatrix(uint row, uint col, BlockMatrix& bm);
-    ~SubMatrix();
-
-    /// Assign Matrix to SubMatrix
-    const SubMatrix& operator= (Matrix& m);
-    //          Matrix& operator() const;
+    /// Create a crude explicit Schur approximation of S = D - C A^-1 B of (A B; C D)
+    /// If symmetry != 0, then the caller promises that B = symmetry * transpose(C).
+    boost::shared_ptr<GenericMatrix> schur_approximation(double symmetry=1) const;
 
   private:
 
-    uint row, col;
-    BlockMatrix& bm;
+    boost::multi_array<boost::shared_ptr<GenericMatrix>, 2> matrices;
 
   };
 
