@@ -45,6 +45,10 @@ namespace dolfin
     /// Destructor
     ~CholmodCholeskySolver();
 
+    /// Solve the operator (matrix)
+    void set_operator(const GenericMatrix& A)
+    { error("set_operator(A) is not implemented."); }
+
     /// Solve linear system Ax = b for a sparse matrix using CHOLMOD
     virtual uint solve(const GenericMatrix& A, GenericVector& x, const GenericVector& b);
 
@@ -93,7 +97,7 @@ namespace dolfin
 
       /// Compute residual norm
       double residual_norm(cholmod_dense* r, cholmod_dense* x,
-			   cholmod_dense* b);
+			                     cholmod_dense* b);
 
       /// Perform one refinement
       void refine_once(cholmod_dense* x, cholmod_dense* r);
