@@ -30,11 +30,22 @@ GenericVector* DefaultFactory::create_local_vector() const
   return factory().create_local_vector();
 }
 //-----------------------------------------------------------------------------
-GenericSparsityPattern * DefaultFactory::create_pattern() const
+GenericSparsityPattern* DefaultFactory::create_pattern() const
 {
   return factory().create_pattern();
 }
 //-----------------------------------------------------------------------------
+GenericLinearSolver* DefaultFactory::create_lu_solver() const
+{
+  return factory().create_lu_solver();
+}
+//-----------------------------------------------------------------------------
+GenericLinearSolver* DefaultFactory::create_krylov_solver(std::string method,
+                                                          std::string pc) const
+{
+  return factory().create_krylov_solver(method, pc);
+}
+//-----------------------------------------------------------------------------
 LinearAlgebraFactory& DefaultFactory::factory() const
 {
   // Fallback
@@ -42,7 +53,7 @@ LinearAlgebraFactory& DefaultFactory::factory() const
   typedef uBLASFactory<> DefaultFactory;
 
   // Get backend from parameter system
-  const std::string backend = parameters["linear_algebra_backend"];
+  const std::string backend = dolfin::parameters["linear_algebra_backend"];
 
   // Choose backend
   if (backend == "uBLAS")
