@@ -7,6 +7,7 @@
 #ifndef __DEFAULT_FACTORY_H
 #define __DEFAULT_FACTORY_H
 
+#include <string>
 #include "LinearAlgebraFactory.h"
 
 namespace dolfin
@@ -14,6 +15,9 @@ namespace dolfin
 
   class DefaultFactory : public LinearAlgebraFactory
   {
+
+    class GenericLinearSolver;
+
     public:
 
     /// Constructor
@@ -32,7 +36,14 @@ namespace dolfin
     virtual dolfin::GenericVector* create_local_vector() const;
 
     /// Create empty sparsity pattern
-    virtual dolfin::GenericSparsityPattern * create_pattern() const;
+    virtual dolfin::GenericSparsityPattern* create_pattern() const;
+
+    /// Create LU solver
+    virtual dolfin::GenericLinearSolver* create_lu_solver() const;
+
+    /// Create Krylov solver
+    virtual dolfin::GenericLinearSolver*
+        create_krylov_solver(std::string method, std::string pc) const;
 
   private:
 
