@@ -9,13 +9,12 @@
 #include <Epetra_MpiComm.h>
 #include <Epetra_SerialComm.h>
 
-#include "dolfin/main/MPI.h"
+#include "dolfin/common/MPI.h"
 #include "EpetraSparsityPattern.h"
 #include "SparsityPattern.h"
+#include "EpetraLUSolver.h"
 #include "EpetraMatrix.h"
 #include "EpetraVector.h"
-#include "EpetraVector.h"
-
 #include "EpetraFactory.h"
 
 using namespace dolfin;
@@ -55,7 +54,18 @@ EpetraVector* EpetraFactory::create_local_vector() const
 //-----------------------------------------------------------------------------
 SparsityPattern* EpetraFactory::create_pattern() const
 {
-  return new SparsityPattern(SparsityPattern::sorted);
+  return new SparsityPattern;
+}
+//-----------------------------------------------------------------------------
+EpetraLUSolver* EpetraFactory::create_lu_solver() const
+{
+  return new EpetraLUSolver();
+}
+//-----------------------------------------------------------------------------
+EpetraKrylovSolver* EpetraFactory::create_krylov_solver(std::string method,
+                                                          std::string pc) const
+{
+  return new EpetraKrylovSolver(method, pc);
 }
 //-----------------------------------------------------------------------------
 Epetra_SerialComm& EpetraFactory::get_serial_comm() const
@@ -68,5 +78,4 @@ Epetra_MpiComm& EpetraFactory::get_mpi_comm() const
   return *mpi_comm;
 }
 //-----------------------------------------------------------------------------
-
 #endif
