@@ -9,11 +9,13 @@
 #ifndef __EPETRA_FACTORY_H
 #define __EPETRA_FACTORY_H
 
+#include <string>
+#include "EpetraKrylovSolver.h"
+#include "EpetraLUSolver.h"
 #include "EpetraMatrix.h"
 #include "EpetraVector.h"
-#include "EpetraSparsityPattern.h"
-#include "LinearAlgebraFactory.h"
 #include "SparsityPattern.h"
+#include "LinearAlgebraFactory.h"
 
 // Forwad declarations
 class Epetra_MpiComm;
@@ -22,6 +24,8 @@ class Epetra_SerialComm;
 namespace dolfin
 {
 
+  class GenericLinearSolver;
+
   class EpetraFactory : public LinearAlgebraFactory
   {
   public:
@@ -29,6 +33,9 @@ namespace dolfin
     /// Destructor
     virtual ~EpetraFactory();
 
+
+    /// --- LinearAlgebraFactory interface
+
     /// Create empty matrix
     EpetraMatrix* create_matrix() const;
 
@@ -39,9 +46,17 @@ namespace dolfin
     EpetraVector* create_local_vector() const;
 
     /// Create empty sparsity pattern
-    //EpetraSparsityPattern* create_pattern() const;
     SparsityPattern* create_pattern() const;
 
+    /// Create LU solver
+    EpetraLUSolver* create_lu_solver() const;
+
+    /// Create Krylov solver
+    EpetraKrylovSolver* create_krylov_solver(std::string method,
+                                             std::string pc) const;
+
+    /// --- EpetraFactory interface
+
     // Return Epetra Communicator
     Epetra_SerialComm& get_serial_comm() const;
 
