@@ -7,7 +7,9 @@
 
 #ifdef HAS_TRILINOS
 
-#include <dolfin/main/MPI.h>
+#include <boost/assign/list_of.hpp>
+
+#include <dolfin/common/MPI.h>
 
 #include <Epetra_ConfigDefs.h>
 #include <Epetra_CrsMatrix.h>
@@ -20,7 +22,6 @@
 #include <AztecOO.h>
 #include <Epetra_LinearProblem.h>
 
-#include <boost/assign/list_of.hpp>
 #include <dolfin/log/dolfin_log.h>
 #include "GenericMatrix.h"
 #include "GenericVector.h"
@@ -37,6 +38,7 @@ const std::map<std::string, int> EpetraKrylovSolver::methods
   = boost::assign::map_list_of("default",  AZ_gmres)
                               ("cg",       AZ_cg)
                               ("gmres",    AZ_gmres)
+                              ("tfqmr",    AZ_tfqmr)
                               ("bicgstab", AZ_bicgstab);
 //-----------------------------------------------------------------------------
 Parameters EpetraKrylovSolver::default_parameters()
@@ -54,12 +56,12 @@ EpetraKrylovSolver::EpetraKrylovSolver(std::string method, std::string pc_type)
   parameters = default_parameters();
 
   // Check that requsted solver is supported
-  if (methods.count(method) == 0 )
+  if (methods.count(method) == 0)
     error("Requested EpetraKrylovSolver method '%s' in unknown", method.c_str());
 
   // Set solver type
   solver->SetAztecOption(AZ_solver, methods.find(method)->second);
-  solver->SetAztecOption(AZ_kspace, parameters["gmres_restart"]);
+  solver->SetAztecOption(AZ_kspace, parameters("gmres")["restart"]);
 }
 //-----------------------------------------------------------------------------
 EpetraKrylovSolver::EpetraKrylovSolver(std::string method,
@@ -72,12 +74,12 @@ EpetraKrylovSolver::EpetraKrylovSolver(std::string method,
   parameters = default_parameters();
 
   // Check that requsted solver is supported
-  if (methods.count(method) == 0 )
+  if (methods.count(method) == 0)
     error("Requested EpetraKrylovSolver method '%s' in unknown", method.c_str());
 
   // Set solver type
   solver->SetAztecOption(AZ_solver, methods.find(method)->second);
-  solver->SetAztecOption(AZ_kspace, parameters["gmres_restart"]);
+  solver->SetAztecOption(AZ_kspace, parameters("gmres")["restart"]);
 }
 //-----------------------------------------------------------------------------
 EpetraKrylovSolver::~EpetraKrylovSolver()
@@ -85,21 +87,35 @@ EpetraKrylovSolver::~EpetraKrylovSolver()
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-dolfin::uint EpetraKrylovSolver::solve(const GenericMatrix& A, GenericVector& x,
-                                       const GenericVector& b)
+void EpetraKrylovSolver::set_operator(const GenericMatrix& A)
 {
-  return solve(A.down_cast<EpetraMatrix>(), x.down_cast<EpetraVector>(),
-               b.down_cast<EpetraVector>());
+  set_operators(A, A);
 }
 //-----------------------------------------------------------------------------
-dolfin::uint EpetraKrylovSolver::solve(const EpetraMatrix& A, EpetraVector& x,
-                                       const EpetraVector& b)
+void EpetraKrylovSolver::set_operators(const GenericMatrix& A,
+                                       const GenericMatrix& P)
+{
+  this->A = reference_to_no_delete_pointer(A.down_cast<EpetraMatrix>());
+  this->P = reference_to_no_delete_pointer(P.down_cast<EpetraMatrix>());
+  assert(this->A);
+  assert(this->P);
+}
+//-----------------------------------------------------------------------------
+dolfin::uint EpetraKrylovSolver::solve(GenericVector& x,
+                                       const GenericVector& b)
+{
+  return solve(x.down_cast<EpetraVector>(), b.down_cast<EpetraVector>());
+}
+//-----------------------------------------------------------------------------
+dolfin::uint EpetraKrylovSolver::solve(EpetraVector& x, const EpetraVector& b)
 {
   assert(solver);
+  assert(A);
+  assert(P);
 
   // Check dimensions
-  const uint M = A.size(0);
-  const uint N = A.size(1);
+  const uint M = A->size(0);
+  const uint N = A->size(1);
   if (N != b.size())
     error("Non-matching dimensions for linear system.");
 
@@ -117,7 +133,7 @@ dolfin::uint EpetraKrylovSolver::solve(const EpetraMatrix& A, EpetraVector& x,
   // FIXME: permit initial guess
 
   // Create linear problem
-  Epetra_LinearProblem linear_problem(A.mat().get(), x.vec().get(),
+  Epetra_LinearProblem linear_problem(A->mat().get(), x.vec().get(),
                                       b.vec().get());
   // Set-up linear solver
   solver->SetProblem(linear_problem);
@@ -129,24 +145,31 @@ dolfin::uint EpetraKrylovSolver::solve(const EpetraMatrix& A, EpetraVector& x,
     solver->SetAztecOption(AZ_output, AZ_none);
 
   // Configure preconditioner
-  if (preconditioner && !preconditioner_set)
-  {
-    preconditioner->set(*this);
-    preconditioner_set = true;
-  }
+  //if (preconditioner && !preconditioner_set)
+  //{
+    assert(P);
+    preconditioner->set(*this, *P);
+    //preconditioner_set = true;
+  //}
 
   // Start solve
   solver->Iterate(parameters["maximum_iterations"], parameters["relative_tolerance"]);
+
+  // Check solve status
   const double* status = solver->GetAztecStatus();
   if ((int) status[AZ_why] != AZ_normal)
   {
-    bool error_on_nonconvergence = parameters["error_on_nonconvergence"];
+    const bool error_on_nonconvergence = parameters["error_on_nonconvergence"];
     if (error_on_nonconvergence)
+    {
       error("Epetra (Aztec00) Krylov solver failed to converge (error code %i).",
             status[AZ_why]);
+    }
     else
+    {
       warning("Epetra (Aztec00) Krylov solver failed to converge (error code %i).",
               status[AZ_why]);
+    }
   }
   else
   {
@@ -157,6 +180,20 @@ dolfin::uint EpetraKrylovSolver::solve(const EpetraMatrix& A, EpetraVector& x,
   return solver->NumIters();
 }
 //-----------------------------------------------------------------------------
+dolfin::uint EpetraKrylovSolver::solve(const GenericMatrix& A, GenericVector& x,
+                                       const GenericVector& b)
+{
+  return solve(A.down_cast<EpetraMatrix>(), x.down_cast<EpetraVector>(),
+               b.down_cast<EpetraVector>());
+}
+//-----------------------------------------------------------------------------
+dolfin::uint EpetraKrylovSolver::solve(const EpetraMatrix& A, EpetraVector& x,
+                                       const EpetraVector& b)
+{
+  set_operator(A);
+  return solve(x, b);
+}
+//-----------------------------------------------------------------------------
 std::string EpetraKrylovSolver::str(bool verbose) const
 {
   dolfin_not_implemented();
