@@ -51,6 +51,18 @@ namespace dolfin
     /// Destructor
     ~EpetraKrylovSolver();
 
+    /// Solve the operator (matrix)
+    void set_operator(const GenericMatrix& A);
+
+    /// Solve the operator (matrix)
+    void set_operators(const GenericMatrix& A, const GenericMatrix& P);
+
+    /// Solve linear system Ax = b and return number of iterations
+    uint solve(GenericVector& x, const GenericVector& b);
+
+    /// Solve linear system Ax = b and return number of iterations
+    uint solve(EpetraVector& x, const EpetraVector& b);
+
     /// Solve linear system Ax = b and return number of iterations
     uint solve(const GenericMatrix& A, GenericVector& x, const GenericVector& b);
 
@@ -81,6 +93,12 @@ namespace dolfin
     // Preconditioner
     boost::shared_ptr<TrilinosPreconditioner> preconditioner;
 
+    // Operator (the matrix)
+    boost::shared_ptr<const EpetraMatrix> A;
+
+    // Matrix used to construct the preconditoner
+    boost::shared_ptr<const EpetraMatrix> P;
+
     bool preconditioner_set;
 
   };
