@@ -1,13 +1,19 @@
-// Copyright (C) 2008 Kent-Andre Mardal.
+// Copyright (C) 2008-2010 Kent-Andre Mardal and Garth N. Wells
 // Licensed under the GNU LGPL Version 2.1.
 //
-// Last changed: 2009-09-08
+// Last changed: 2010-07-16
 
 #ifdef HAS_TRILINOS
+
 #ifndef __EPETRA_LU_SOLVER_H
 #define __EPETRA_LU_SOLVER_H
 
-#include "GenericLinearSolver.h"
+#include <boost/scoped_ptr.hpp>
+#include "GenericLUSolver.h"
+
+/// Forward declaration
+class Amesos_BaseSolver;
+class Epetra_LinearProblem;
 
 namespace dolfin
 {
@@ -21,16 +27,25 @@ namespace dolfin
   /// linear systems of the form Ax = b. It is a wrapper for the LU
   /// solver of Epetra.
 
-  class EpetraLUSolver : public GenericLinearSolver
+  class EpetraLUSolver : public GenericLUSolver
   {
   public:
 
     /// Constructor
     EpetraLUSolver();
 
+    /// Constructor
+    EpetraLUSolver(const GenericMatrix& A);
+
     /// Destructor
     ~EpetraLUSolver();
 
+    /// Set operator (matrix)
+    void set_operator(const GenericMatrix& A);
+
+    /// Solve linear system Ax = b
+    uint solve(GenericVector& x, const GenericVector& b);
+
     /// Solve linear system Ax = b
     uint solve(const GenericMatrix& A, GenericVector& x, const GenericVector& b);
 
@@ -43,6 +58,18 @@ namespace dolfin
     /// Return informal string representation (pretty-print)
     std::string str(bool verbose) const;
 
+  private:
+
+    bool symbolic_factorized, numeric_factorized;
+
+    // Epetra linear problem
+    boost::scoped_ptr<Epetra_LinearProblem> linear_problem;
+
+    // Linear solver
+    boost::scoped_ptr<Amesos_BaseSolver> solver;
+
+    std::string solver_type;
+
   };
 
 }
