@@ -2,19 +2,23 @@
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Anders Logg, 2008.
-// Modified by Garth N. Wells, 2008, 2009.
+// Modified by Garth N. Wells, 2008-2010.
 //
 // First added:  2008-04-21
-// Last changed: 2009-09-08
+// Last changed: 2010-11-26
 
 #ifdef HAS_TRILINOS
 
 #include <cstring>
 #include <iostream>
 #include <iomanip>
+#include <set>
 #include <sstream>
 #include <utility>
 
+#include <boost/unordered_set.hpp>
+
+#include <Epetra_BlockMap.h>
 #include <Epetra_CrsGraph.h>
 #include <Epetra_CrsMatrix.h>
 #include <Epetra_FECrsGraph.h>
@@ -24,9 +28,7 @@
 #include <Epetra_SerialComm.h>
 #include <EpetraExt_MatrixMatrix.h>
 
-#include <ml_epetra_utils.h>
-
-#include <dolfin/main/MPI.h>
+#include <dolfin/common/MPI.h>
 #include <dolfin/log/dolfin_log.h>
 #include "EpetraVector.h"
 #include "GenericSparsityPattern.h"
@@ -43,11 +45,6 @@ EpetraMatrix::EpetraMatrix()
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-EpetraMatrix::EpetraMatrix(uint M, uint N)
-{
-  resize(M, N);
-}
-//-----------------------------------------------------------------------------
 EpetraMatrix::EpetraMatrix(const EpetraMatrix& A)
 {
   if (A.mat())
@@ -70,11 +67,6 @@ EpetraMatrix::~EpetraMatrix()
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-void EpetraMatrix::resize(uint M, uint N)
-{
-  error("EpetraMatrix::resize(uint, unit) not yet implemented.");
-}
-//-----------------------------------------------------------------------------
 void EpetraMatrix::init(const GenericSparsityPattern& sparsity_pattern)
 {
   if (A && !A.unique())
@@ -86,42 +78,67 @@ void EpetraMatrix::init(const GenericSparsityPattern& sparsity_pattern)
   const uint n0 = range.first;
 
   const SparsityPattern& _pattern = dynamic_cast<const SparsityPattern&>(sparsity_pattern);
-  const std::vector<dolfin::Set<dolfin::uint> >& d_pattern = _pattern.diagonal_pattern();
-  const std::vector<dolfin::Set<dolfin::uint> >& o_pattern = _pattern.off_diagonal_pattern();
+  const std::vector<std::vector<dolfin::uint> >& d_pattern = _pattern.diagonal_pattern(SparsityPattern::sorted);
+  const std::vector<std::vector<dolfin::uint> >& o_pattern = _pattern.off_diagonal_pattern(SparsityPattern::sorted);
 
   // Get number of non-zeroes per row (on and off diagonal)
   std::vector<uint> dnum_nonzeros(num_local_rows);
   std::vector<uint> onum_nonzeros(num_local_rows);
-  sparsity_pattern.num_nonzeros_diagonal(&dnum_nonzeros[0]);
-  sparsity_pattern.num_nonzeros_off_diagonal(&onum_nonzeros[0]);
+  sparsity_pattern.num_nonzeros_diagonal(dnum_nonzeros);
+  sparsity_pattern.num_nonzeros_off_diagonal(onum_nonzeros);
 
   // Create row map
   EpetraFactory& f = EpetraFactory::instance();
   Epetra_MpiComm comm = f.get_mpi_comm();
   Epetra_Map row_map(sparsity_pattern.size(0), num_local_rows, 0, comm);
 
+  // For rectangular matrices with more columns than rows, the columns which are
+  // larger than those in row_map are marked as nonlocal (and assembly fails).
+  // The domain_map fixes that problem, at least in the serial case.
+  // FIXME: Needs attention in the parallel case. Maybe range_map is also req'd.
+  const std::pair<uint,uint> colrange = sparsity_pattern.local_range(1);
+  const int num_local_cols = colrange.second - colrange.first;
+  Epetra_Map domain_map(sparsity_pattern.size(1), num_local_cols, 0, comm);
+
   // Create Epetra_FECrsGraph
   Epetra_FECrsGraph matrix_map(Copy, row_map, reinterpret_cast<int*>(&dnum_nonzeros[0]));
 
   // Add diagonal block indices
-  std::vector<dolfin::Set<dolfin::uint> >::const_iterator row_set;
+  std::vector<std::vector<dolfin::uint> >::const_iterator row_set;
   for (row_set = d_pattern.begin(); row_set != d_pattern.end(); ++row_set)
   {
     const uint global_row = row_set - d_pattern.begin() + n0;
-    const std::vector<dolfin::uint>& nz_entries = row_set->set();
+    const std::vector<dolfin::uint>& nz_entries = *row_set;
     std::vector<dolfin::uint>& _nz_entries = const_cast<std::vector<dolfin::uint>& >(nz_entries);
     matrix_map.InsertGlobalIndices(global_row, row_set->size(), reinterpret_cast<int*>(&_nz_entries[0]));
   }
 
-  // Add off-diagonal block indices
-  for (row_set = o_pattern.begin(); row_set != o_pattern.end(); ++row_set)
+  for (uint local_row = 0; local_row < d_pattern.size(); local_row++)
   {
-    const uint global_row = row_set - o_pattern.begin() + n0;
-    const std::vector<dolfin::uint>& nz_entries = row_set->set();
-    std::vector<dolfin::uint>& _nz_entries = const_cast<std::vector<dolfin::uint>& >(nz_entries);
-    matrix_map.InsertGlobalIndices(global_row, row_set->size(), reinterpret_cast<int*>(&_nz_entries[0]));
+    const uint global_row = local_row + n0;
+    std::vector<uint> &entries = const_cast<std::vector<uint>&>(d_pattern[local_row]);
+    matrix_map.InsertGlobalIndices(global_row, entries.size(), reinterpret_cast<int*>(&entries[0]));
+  }
+
+  // Add off-diagonal block indices (parallel only)
+  for (uint local_row = 0; local_row < o_pattern.size(); local_row++)
+  {
+    const uint global_row = local_row + n0;
+    std::vector<uint> &entries = const_cast<std::vector<uint>&>(o_pattern[local_row]);
+    matrix_map.InsertGlobalIndices(global_row, entries.size(), reinterpret_cast<int*>(&entries[0]));
+  }
+
+  try
+  {
+    // Finalise map. Here, row_map is standing in for RangeMap, which is
+    // probably ok but should be double-checked.
+    matrix_map.GlobalAssemble(domain_map, row_map);
+    matrix_map.OptimizeStorage();
+  }
+  catch (int err)
+  {
+    error("Epetra threw error %d in assemble", err);
   }
-  matrix_map.GlobalAssemble();
 
   // Create matrix
   A.reset( new Epetra_FECrsMatrix(Copy, matrix_map) );
@@ -135,16 +152,44 @@ EpetraMatrix* EpetraMatrix::copy() const
 dolfin::uint EpetraMatrix::size(uint dim) const
 {
   assert(A);
-  int M = A->NumGlobalRows();
-  int N = A->NumGlobalCols();
+  const int M = A->NumGlobalRows();
+  const int N = A->NumGlobalCols();
   return (dim == 0 ? M : N);
 }
 //-----------------------------------------------------------------------------
+std::pair<dolfin::uint, dolfin::uint> EpetraMatrix::local_range(uint dim) const
+{
+  assert(dim < 2);
+  if (dim == 1)
+    error("Cannot compute columns range for Epetra matrices.");
+
+  const Epetra_BlockMap& row_map = A->RowMap();
+  assert(row_map.LinearMap());
+
+  return std::make_pair(row_map.MinMyGID(), row_map.MaxMyGID() + 1);
+}
+//-----------------------------------------------------------------------------
+void EpetraMatrix::resize(GenericVector& y, uint dim) const
+{
+  assert(A);
+
+  // Get map appropriate map
+  const Epetra_Map* map = 0;
+  if (dim == 0)
+    map = &(A->RangeMap());
+  else if (dim == 1)
+    map = &(A->DomainMap());
+  else
+    error("dim must be <= 1 to resize vector.");
+
+  // Reset vector with new map
+  EpetraVector& _y = y.down_cast<EpetraVector>();
+  _y.reset(*map);
+}
+//-----------------------------------------------------------------------------
 void EpetraMatrix::get(double* block, uint m, const uint* rows,
 		                   uint n, const uint* cols) const
 {
-  not_working_in_parallel("EpetraMatrix::get");
-
   assert(A);
 
   int num_entities = 0;
@@ -157,14 +202,13 @@ void EpetraMatrix::get(double* block, uint m, const uint* rows,
     // Extract the values and indices from row: rows[i]
     if (A->IndicesAreLocal())
     {
-      int err = A->ExtractMyRowView(rows[i], num_entities, values, indices);
-      if (err != 0) {
+      const int err = A->ExtractMyRowView(rows[i], num_entities, values, indices);
+      if (err != 0)
         error("EpetraMatrix::get: Did not manage to perform Epetra_CrsMatrix::ExtractMyRowView.");
-      }
     }
     else
     {
-      int err = A->ExtractGlobalRowView(rows[i], num_entities, values, indices);
+      const int err = A->ExtractGlobalRowView(rows[i], num_entities, values, indices);
       if (err != 0)
         error("EpetraMatrix::get: Did not manage to perform Epetra_CRSMatrix::ExtractGlobalRowView.");
     }
@@ -193,7 +237,7 @@ void EpetraMatrix::set(const double* block,
                        uint n, const uint* cols)
 {
   assert(A);
-  int err = A->ReplaceGlobalValues(m, reinterpret_cast<const int*>(rows),
+  const int err = A->ReplaceGlobalValues(m, reinterpret_cast<const int*>(rows),
                                    n, reinterpret_cast<const int*>(cols), block);
   if (err != 0)
     error("EpetraMatrix::set: Did not manage to perform Epetra_CrsMatrix::ReplaceGlobalValues.");
@@ -204,7 +248,7 @@ void EpetraMatrix::add(const double* block,
                        uint n, const uint* cols)
 {
   assert(A);
-  int err = A->SumIntoGlobalValues(m, reinterpret_cast<const int*>(rows),
+  const int err = A->SumIntoGlobalValues(m, reinterpret_cast<const int*>(rows),
                                    n, reinterpret_cast<const int*>(cols), block,
                                    Epetra_FECrsMatrix::ROW_MAJOR);
 
@@ -215,11 +259,10 @@ void EpetraMatrix::add(const double* block,
 void EpetraMatrix::axpy(double a, const GenericMatrix& A, bool same_nonzero_pattern)
 {
   const EpetraMatrix* AA = &A.down_cast<EpetraMatrix>();
-
   if (!AA->mat()->Filled())
     error("Epetramatrix is not in the correct state for addition.");
 
-  int err = EpetraExt::MatrixMatrix::Add(*(AA->mat()), false, a, *(this->A), 1.0);
+  const int err = EpetraExt::MatrixMatrix::Add(*(AA->mat()), false, a, *(this->A), 1.0);
   if (err != 0)
     error("EpetraMatrDid::axpy: Did not manage to perform EpetraExt::MatrixMatrix::Add. If the matrix has been assembled, the nonzero patterns must match.");
 }
@@ -242,7 +285,7 @@ double EpetraMatrix::norm(std::string norm_type) const
 void EpetraMatrix::zero()
 {
   assert(A);
-  int err = A->PutScalar(0.0);
+  const int err = A->PutScalar(0.0);
   if (err != 0)
     error("EpetraMatrix::zero: Did not manage to perform Epetra_CrsMatrix::PutScalar.");
 }
@@ -280,31 +323,83 @@ std::string EpetraMatrix::str(bool verbose) const
 void EpetraMatrix::ident(uint m, const uint* rows)
 {
   assert(A);
+  assert(A->Filled() == true);
 
-  // FIXME: Is this the best way to do this?
+  // FIXME: This is a major hack and will not scale for large numbers of
+  // processes. The problem is that a dof is not guaranteed to reside on
+  // the same process as one of cells to which it belongs (which is bad,
+  // but is due to the sparsity pattern computation). This function only
+  // work for locally owned rows (the PETSc version works for any row).
 
-  // FIXME: This can be made more efficient by eliminating creation of some
-  //        obejcts inside the loop
+  typedef boost::unordered_set<uint> MySet;
 
-  const Epetra_CrsGraph& graph = A->Graph();
+  // Build lists of local and nonlocal rows
+  MySet local_rows;
+  std::vector<uint> non_local_rows;
   for (uint i = 0; i < m; ++i)
   {
-    int row = rows[i];
-    int num_nz = graph.NumGlobalIndices(row);
-    std::vector<int> indices(num_nz);
+    if (A->MyGlobalRow(rows[i]))
+      local_rows.insert(rows[i]);
+    else
+      non_local_rows.push_back(rows[i]);
+  }
 
-    int out_num = 0;
-    graph.ExtractGlobalRowCopy(row, num_nz, out_num, &indices[0]);
+  // If parallel, send non_local rows to all processes
+  if (MPI::num_processes() > 1)
+  {
+    // Send list of nonlocal rows to all processes
+    std::vector<uint> partition;
+    std::vector<uint> transmit_data;
+    for (uint i = 0; i < MPI::num_processes(); ++i)
+    {
+      if (i != MPI::process_number())
+      {
+        transmit_data.insert(transmit_data.end(), non_local_rows.begin(),
+                             non_local_rows.end());
+        partition.insert(partition.end(), non_local_rows.size(), i);
+      }
+    }
+    MPI::distribute(transmit_data, partition);
 
-    // Zero row
-    std::vector<double> block(num_nz);
-    int err = A->ReplaceGlobalValues(row, num_nz, &block[0], &indices[0]);
-    if (err != 0)
-      error("EpetraMatrix::ident: Did not manage to perform Epetra_CrsMatrix::ReplaceGlobalValues.");
+    // Unpack data
+    assert(transmit_data.size() == partition.size());
+    for (uint i = 0; i < transmit_data.size(); ++i)
+    {
+      // Insert row into set if it's local
+      const uint new_index = transmit_data[i];
+      if (A->MyGlobalRow(new_index))
+        local_rows.insert(new_index);
+    }
+  }
+  //-------------------------
 
-    // Place one on the diagonal
-    double one = 1.0;
-    A->ReplaceGlobalValues(row, 1, &one, &row);
+  const Epetra_CrsGraph& graph = A->Graph();
+  MySet::const_iterator global_row;
+  for (global_row = local_rows.begin(); global_row != local_rows.end(); ++global_row)
+  {
+    // Get local row index
+    const int local_row = A->LRID(*global_row);
+
+    // If this process owns row, then zero row
+    if (local_row >= 0)
+    {
+      // Get row map
+      int num_nz = 0;
+      int* column_indices;
+      int err = graph.ExtractMyRowView(local_row, num_nz, column_indices);
+      if (err != 0)
+        error("EpetraMatrix::ident: Did not manage to extract row map.");
+
+      // Zero row
+      std::vector<double> block(num_nz, 0.0);
+      err = A->ReplaceMyValues(local_row, num_nz, &block[0], column_indices);
+      if (err != 0)
+        error("EpetraMatrix::ident: Did not manage to perform Epetra_CrsMatrix::ReplaceGlobalValues.");
+
+      // Place one on the diagonal
+      const double one = 1.0;
+      A->ReplaceMyValues(local_row, 1, &one, &local_row);
+    }
   }
 }
 //-----------------------------------------------------------------------------
@@ -317,15 +412,15 @@ void EpetraMatrix::zero(uint m, const uint* rows)
   const Epetra_CrsGraph& graph = A->Graph();
   for (uint i = 0; i < m; ++i)
   {
-    int row = rows[i];
-    int num_nz = graph.NumGlobalIndices(row);
+    const int row = rows[i];
+    const int num_nz = graph.NumGlobalIndices(row);
     std::vector<int> indices(num_nz);
 
     int out_num = 0;
     graph.ExtractGlobalRowCopy(row, num_nz, out_num, &indices[0]);
 
     std::vector<double> block(num_nz);
-    int err = A->ReplaceGlobalValues(row, num_nz, &block[0], &indices[0]);
+    const int err = A->ReplaceGlobalValues(row, num_nz, &block[0], &indices[0]);
     if (err != 0)
       error("EpetraMatrix::zero: Did not manage to perform Epetra_CrsMatrix::ReplaceGlobalValues.");
   }
@@ -334,20 +429,18 @@ void EpetraMatrix::zero(uint m, const uint* rows)
 void EpetraMatrix::mult(const GenericVector& x_, GenericVector& Ax_) const
 {
   assert(A);
-  const EpetraVector* x = dynamic_cast<const EpetraVector*>(x_.instance());
-  if (!x)
-    error("EpetraMatrix::mult: The vector x should be of type EpetraVector.");
+  const EpetraVector& x = x_.down_cast<EpetraVector>();
+  EpetraVector& Ax = Ax_.down_cast<EpetraVector>();
 
-  EpetraVector* Ax = dynamic_cast<EpetraVector*>(Ax_.instance());
-  if (!Ax)
-    error("EpetraMatrix::mult: The vector Ax should be of type EpetraVector.");
-
-  if (size(1) != x->size())
+  if (x.size() != size(1))
     error("EpetraMatrix::mult: Matrix and vector dimensions don't match for matrix-vector product.");
 
-  Ax->resize(size(0));
+  // Resize RHS
+  this->resize(Ax, 0);
 
-  int err = A->Multiply(false, *(x->vec()), *(Ax->vec()));
+  assert(x.vec());
+  assert(Ax.vec());
+  const int err = A->Multiply(false, *(x.vec()), *(Ax.vec()));
   if (err != 0)
     error("EpetraMatrix::mult: Did not manage to perform Epetra_CRSMatrix::Multiply.");
 }
@@ -355,20 +448,16 @@ void EpetraMatrix::mult(const GenericVector& x_, GenericVector& Ax_) const
 void EpetraMatrix::transpmult(const GenericVector& x_, GenericVector& Ax_) const
 {
   assert(A);
-  const EpetraVector* x = dynamic_cast<const EpetraVector*>(x_.instance());
-  if (!x)
-    error("EpetraMatrix::transpmult: The vector x should be of type EpetraVector.");
+  const EpetraVector& x = x_.down_cast<EpetraVector>();
+  EpetraVector& Ax = Ax_.down_cast<EpetraVector>();
 
-  EpetraVector* Ax = dynamic_cast<EpetraVector*>(Ax_.instance());
-  if (!Ax)
-    error("EpetraMatrix::transpmult: The vector Ax should be of type EpetraVector.");
-
-  if (size(0) != x->size())
+  if (x.size() != size(0))
     error("EpetraMatrix::transpmult: Matrix and vector dimensions don't match for (transposed) matrix-vector product.");
 
-  Ax->resize(size(1));
+  // Resize RHS
+  this->resize(Ax, 1);
 
-  int err = A->Multiply(true, *(x->vec()), *(Ax->vec()));
+  const int err = A->Multiply(true, *(x.vec()), *(Ax.vec()));
   if (err != 0)
     error("EpetraMatrix::transpmult: Did not manage to perform Epetra_CRSMatrix::Multiply.");
 }
@@ -376,34 +465,52 @@ void EpetraMatrix::transpmult(const GenericVector& x_, GenericVector& Ax_) const
 void EpetraMatrix::getrow(uint row, std::vector<uint>& columns,
                           std::vector<double>& values) const
 {
-  not_working_in_parallel("EpetraMatrix::getrow");
-
   assert(A);
 
-  // Temporary variables
-  int* indices;
-  double* vals;
-  int num_entries;
+  // Get local row index
+  const int local_row_index = A->LRID(row);
 
-  // Extract data from Epetra matrix
-  int err = A->ExtractMyRowView(row, num_entries, vals, indices);
-  if (err != 0)
-    error("EpetraMatrix::getrow: Did not manage to perform Epetra_CrsMatrix::ExtractMyRowView.");
-
-  // Put data in columns and values
-  columns.resize(num_entries);
-  values.resize(num_entries);
-  for (int i = 0; i < num_entries; i++)
+  // If this process has part of the row, get values
+  if (local_row_index >= 0)
   {
-    columns[i] = indices[i];
-    values[i]  = vals[i];
+    // Temporary variables
+    int* indices;
+    double* vals;
+    int num_entries;
+
+    // Extract data from Epetra matrix
+    const int err = A->ExtractMyRowView(local_row_index, num_entries, vals, indices);
+    if (err != 0)
+      error("EpetraMatrix::getrow: Did not manage to perform Epetra_CrsMatrix::ExtractMyRowView.");
+
+    // Put data in columns and values
+    columns.resize(num_entries);
+    values.resize(num_entries);
+    for (int i = 0; i < num_entries; i++)
+    {
+      columns[i] = indices[i];
+      values[i]  = vals[i];
+    }
+  }
+  else
+  {
+    columns.resize(0);
+    values.resize(0);
   }
 }
 //-----------------------------------------------------------------------------
 void EpetraMatrix::setrow(uint row, const std::vector<uint>& columns,
                           const std::vector<double>& values)
 {
-  dolfin_not_implemented();
+  static bool print_msg_once=true;
+  if (print_msg_once)
+  {
+    info("EpetraMatrix::setrow is implemented inefficiently");
+    print_msg_once = false;
+  }
+
+  for (uint i=0; i<columns.size(); i++)
+    set(&values[i], 1, &row, 1, &columns[i]);
 }
 //-----------------------------------------------------------------------------
 LinearAlgebraFactory& EpetraMatrix::factory() const
@@ -426,7 +533,7 @@ boost::shared_ptr<Epetra_FECrsMatrix> EpetraMatrix::mat() const
 const EpetraMatrix& EpetraMatrix::operator*= (double a)
 {
   assert(A);
-  int err = A->Scale(a);
+  const int err = A->Scale(a);
   if (err !=0)
     error("EpetraMatrix::operator*=: Did not manage to perform Epetra_CrsMatrix::Scale.");
   return *this;
