@@ -13,8 +13,6 @@
 #ifdef HAS_TRILINOS
 
 #include <boost/shared_ptr.hpp>
-
-#include <dolfin/common/Variable.h>
 #include "GenericMatrix.h"
 
 class Epetra_FECrsMatrix;
@@ -23,6 +21,7 @@ class Epetra_CrsGraph;
 namespace dolfin
 {
 
+  /// Forward declarations
   class EpetraSparsityPattern;
   class GenericSparsityPattern;
 
@@ -34,16 +33,13 @@ namespace dolfin
   /// access the Epetra_FECrsMatrix object using the function mat() and
   /// use the standard Epetra interface.
 
-  class EpetraMatrix: public GenericMatrix
+  class EpetraMatrix : public GenericMatrix
   {
   public:
 
     /// Create empty matrix
     EpetraMatrix();
 
-    /// Create M x N matrix
-    EpetraMatrix(uint M, uint N);
-
     /// Copy constuctor
     EpetraMatrix(const EpetraMatrix& A);
 
@@ -67,6 +63,9 @@ namespace dolfin
     /// Return size of given dimension
     virtual uint size(uint dim) const;
 
+    /// Return local ownership range
+    virtual std::pair<uint, uint> local_range(uint dim) const;
+
     /// Set all entries to zero and keep any sparse structure
     virtual void zero();
 
@@ -78,8 +77,10 @@ namespace dolfin
 
     //--- Implementation of the GenericMatrix interface ---
 
-    /// Resize matrix to M x N
-    virtual void resize(uint M, uint N);
+    /// Resize vector y such that is it compatible with matrix for
+    /// multuplication Ax = b (dim = 0 -> b, dim = 1 -> x) In parallel
+    /// case, size and layout are important.
+    virtual void resize(GenericVector& y, uint dim) const;
 
     /// Get block of values
     virtual void get(double* block, uint m, const uint* rows, uint n, const uint* cols) const;
@@ -148,5 +149,5 @@ namespace dolfin
 
 }
 
-#endif //HAS_TRILINOS
-#endif //__EPETRA_MATRIX_H
+#endif // HAS_TRILINOS
+#endif // __EPETRA_MATRIX_H
