@@ -4,13 +4,15 @@
 // Modified by Garth N. Wells, 2008-2010.
 //
 // First added:  2008-04-21
-// Last changed: 2009-08-22
+// Last changed: 2011-01-14
 
 #ifdef HAS_TRILINOS
 
 #include <cmath>
 #include <cstring>
 #include <numeric>
+#include <utility>
+#include <boost/scoped_ptr.hpp>
 
 #include <Epetra_FEVector.h>
 #include <Epetra_Export.h>
@@ -20,10 +22,11 @@
 #include <Epetra_MpiComm.h>
 #include <Epetra_SerialComm.h>
 #include <Epetra_Vector.h>
+#include <Epetra_DataAccess.h>
 
 #include <dolfin/common/Array.h>
 #include <dolfin/common/Set.h>
-#include <dolfin/main/MPI.h>
+#include <dolfin/common/MPI.h>
 #include <dolfin/math/dolfin_math.h>
 #include <dolfin/log/dolfin_log.h>
 #include "uBLASVector.h"
@@ -31,9 +34,6 @@
 #include "EpetraFactory.h"
 #include "EpetraVector.h"
 
-// FIXME: A review is needed with respect to correct use of parallel vectors.
-//        It would be useful to store the Epetra map.
-
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
@@ -58,7 +58,7 @@ EpetraVector::EpetraVector(const Epetra_Map& map)
   x.reset(new Epetra_FEVector(map));
 }
 //-----------------------------------------------------------------------------
-EpetraVector::EpetraVector(const EpetraVector& v)
+EpetraVector::EpetraVector(const EpetraVector& v) : type(v.type)
 {
   *this = v;
 }
@@ -73,27 +73,78 @@ void EpetraVector::resize(uint N)
   if (x && this->size() == N)
     return;
 
-  if (x && !x.unique())
-      error("Cannot resize EpetraVector. More than one object points to the underlying Epetra object.");
+  // Create empty ghost vertices vector
+  std::vector<uint> ghost_indices;
 
-  // Get local range
-  const std::pair<uint, uint> range = MPI::local_range(N);
-  const uint n = range.second - range.first;
-
-  if (N == n || type == "local")
+  if (type == "global")
   {
-    EpetraFactory& f = EpetraFactory::instance();
-    Epetra_SerialComm serial_comm = f.get_serial_comm();
-    Epetra_Map map(N, N, 0, serial_comm);
-    x.reset(new Epetra_FEVector(map));
+    const std::pair<uint, uint> range = MPI::local_range(N);
+    resize(range, ghost_indices);
+  }
+  else if (type == "local")
+  {
+    const std::pair<uint, uint> range(0, N);
+    resize(range, ghost_indices);
+  }
+  else
+    error("Epetra vector type unkown.");
+}
+//-----------------------------------------------------------------------------
+void EpetraVector::resize(std::pair<uint, uint> range)
+{
+  std::vector<uint> ghost_indices;
+  resize(range, ghost_indices);
+}
+//-----------------------------------------------------------------------------
+void EpetraVector::resize(std::pair<uint, uint> range,
+                          const std::vector<uint>& ghost_indices)
+{
+  if (x && !x.unique())
+    error("Cannot resize EpetraVector. More than one object points to the underlying Epetra object.");
+
+  // Create ghost data structures
+  ghost_global_to_local.clear();
+
+  // Pointer to Epetra map
+  boost::scoped_ptr<Epetra_Map> epetra_map;
+
+  // Epetra factory and serial communicator
+  const EpetraFactory& f = EpetraFactory::instance();
+  Epetra_SerialComm serial_comm = f.get_serial_comm();
+
+  // Compute local size
+  const uint local_size = range.second - range.first;
+  assert(range.second - range.first >= 0);
+
+  // Create vector
+  if (type == "local")
+  {
+    if (ghost_indices.size() != 0)
+      error("Serial EpetraVectors do not suppprt ghost points.");
+
+    // Create map
+    epetra_map.reset(new Epetra_Map(-1, local_size, 0, serial_comm));
   }
   else
   {
-    EpetraFactory& f = EpetraFactory::instance();
+    // Create map
     Epetra_MpiComm mpi_comm = f.get_mpi_comm();
-    Epetra_Map map(N, n, 0, mpi_comm);
-    x.reset(new Epetra_FEVector(map));
+    epetra_map.reset(new Epetra_Map(-1, local_size, 0, mpi_comm));
+
+    // Build global-to-local map for ghost indices
+    for (uint i = 0; i < ghost_indices.size(); ++i)
+      ghost_global_to_local.insert(std::pair<uint, uint>(ghost_indices[i], i));
   }
+
+  // Create vector
+  x.reset(new Epetra_FEVector(*epetra_map));
+
+  // Create local ghost vector
+  const int num_ghost_entries = ghost_indices.size();
+  const int* ghost_entries = reinterpret_cast<const int*>(&ghost_indices[0]);
+  Epetra_Map ghost_map(num_ghost_entries, num_ghost_entries,
+                       ghost_entries, 0, serial_comm);
+  x_ghost.reset(new Epetra_Vector(ghost_map));
 }
 //-----------------------------------------------------------------------------
 EpetraVector* EpetraVector::copy() const
@@ -107,24 +158,29 @@ dolfin::uint EpetraVector::size() const
   return x ? x->GlobalLength(): 0;
 }
 //-----------------------------------------------------------------------------
+dolfin::uint EpetraVector::local_size() const
+{
+  return x ? x->MyLength(): 0;
+}
+//-----------------------------------------------------------------------------
 std::pair<dolfin::uint, dolfin::uint> EpetraVector::local_range() const
 {
   assert(x);
-
-  if ( x->Comm().NumProc() == 1 )
-    return std::make_pair<uint, uint>(0, size());
-  else
-  {
-    assert(x->Map().LinearMap());
-    const Epetra_BlockMap& map = x->Map();
-    return std::make_pair<uint, uint>(map.MinMyGID(), map.MaxMyGID() + 1);
-  }
+  assert(x->Map().LinearMap());
+  const Epetra_BlockMap& map = x->Map();
+  return std::make_pair<uint, uint>(map.MinMyGID(), map.MaxMyGID() + 1);
+}
+//-----------------------------------------------------------------------------
+bool EpetraVector::owns_index(uint i) const
+{
+  return x->Map().MyGID(i);
 }
 //-----------------------------------------------------------------------------
 void EpetraVector::zero()
 {
   assert(x);
-  int err = x->PutScalar(0.0);
+  const int err = x->PutScalar(0.0);
+  //apply("add");
   if (err != 0)
     error("EpetraVector::zero: Did not manage to perform Epetra_Vector::PutScalar.");
 }
@@ -132,16 +188,57 @@ void EpetraVector::zero()
 void EpetraVector::apply(std::string mode)
 {
   assert(x);
-  int err = 0;
-  if (mode == "add")
-    err = x->GlobalAssemble(Add);
-  else if (mode == "insert")
-    err = x->GlobalAssemble(Insert);
-  else
-    error("Unknown apply mode in EpetraVector::apply");
 
-  if (err != 0)
-    error("EpetraVector::apply: Did not manage to perform Epetra_Vector::GlobalAssemble.");
+  // Special treatement required for values applied using 'set'
+  // This would be simpler if we required that only local values (on this process) can be set
+  if (MPI::sum(static_cast<uint>(off_process_set_values.size())) > 0)
+  {
+    // Create communicator
+    const EpetraFactory& f = EpetraFactory::instance();
+    Epetra_MpiComm mpi_comm = f.get_mpi_comm();
+    Epetra_SerialComm serial_comm = f.get_serial_comm();
+
+    std::vector<int> non_local_indices; non_local_indices.reserve(off_process_set_values.size());
+    std::vector<double> non_local_values; non_local_values.reserve(off_process_set_values.size());
+    boost::unordered_map<uint, double>::const_iterator entry;
+    for (entry = off_process_set_values.begin(); entry != off_process_set_values.end(); ++entry)
+    {
+      non_local_indices.push_back(entry->first);
+      non_local_values.push_back(entry->second);
+    }
+
+    // Create map for y
+    const int* _non_local_indices = reinterpret_cast<const int*>(&non_local_indices[0]);
+    Epetra_Map target_map(-1, non_local_indices.size(), _non_local_indices, 0, mpi_comm);
+
+    // Create vector y (view of non_local_values)
+    Epetra_Vector y(View, target_map, &non_local_values[0]);
+
+    // Create importer
+    Epetra_Import importer(x->Map(), target_map);
+
+    // Import off-process 'set' data
+    if (mode == "add")
+      x->Import(y, importer, Add);
+    else if (mode == "insert")
+      x->Import(y, importer, InsertAdd);
+
+    // Clear map of off-process set values
+    off_process_set_values.clear();
+  }
+  else
+  {
+    int err = 0;
+    if (mode == "add")
+      err = x->GlobalAssemble(Add);
+    else if (mode == "insert")
+      err = x->GlobalAssemble(Insert);
+    else
+      error("Unknown apply mode in EpetraVector::apply");
+
+    if (err != 0)
+      error("EpetraVector::apply: Did not manage to perform Epetra_Vector::GlobalAssemble.");
+  }
 }
 //-----------------------------------------------------------------------------
 std::string EpetraVector::str(bool verbose) const
@@ -163,8 +260,11 @@ std::string EpetraVector::str(bool verbose) const
 void EpetraVector::get_local(Array<double>& values) const
 {
   assert(x);
-  assert( (int)values.size() >= x->MyLength() );
-  int err = x->ExtractCopy(values.data().get(), 0);
+  const uint local_size = x->MyLength();
+  if (values.size() != local_size)
+    error("EpetraVector::get_local: length of values array is not equal to local vector size.");
+
+  const int err = x->ExtractCopy(values.data().get(), 0);
   if (err!= 0)
     error("EpetraVector::get: Did not manage to perform Epetra_Vector::ExtractCopy.");
 }
@@ -173,7 +273,9 @@ void EpetraVector::set_local(const Array<double>& values)
 {
   assert(x);
   const uint local_size = x->MyLength();
-  assert(values.size() >= local_size);
+
+ if (values.size() != local_size)
+    error("EpetraVector::set_local: length of values array is not equal to local vector size.");
 
   for (uint i = 0; i < local_size; ++i)
     (*x)[0][i] = values[i];
@@ -183,53 +285,47 @@ void EpetraVector::add_local(const Array<double>& values)
 {
   assert(x);
   const uint local_size = x->MyLength();
-  assert(values.size() >= local_size);
+  if (values.size() != local_size)
+    error("EpetraVector::add_local: length of values array is not equal to local vector size.");
 
   for (uint i = 0; i < local_size; ++i)
     (*x)[0][i] += values[i];
 }
 //-----------------------------------------------------------------------------
-void EpetraVector::get(double* block, uint m, const uint* rows) const
-{
-  // If vector is local this function will call get_local. For distributed
-  // vectors, perform first a gather into a local vector
-
-  if (local_range().first == 0 && local_range().second == size())
-    get_local(block, m, rows);
-  else
-  {
-    EpetraVector y("local");
-    std::vector<uint> indices;
-    std::vector<uint> local_indices;
-    indices.reserve(m);
-    local_indices.reserve(m);
-    for (uint i = 0; i < m; ++i)
-    {
-      indices.push_back(rows[i]);
-      local_indices.push_back(i);
-    }
-
-    // Gather values into y
-    const Array<uint> _indices(indices.size(), &indices[0]);
-    gather(y, _indices);
-
-    // Get entries of y
-    y.get_local(block, m, &local_indices[0]);
-  }
-}
-//-----------------------------------------------------------------------------
 void EpetraVector::set(const double* block, uint m, const uint* rows)
 {
   assert(x);
-  int err = x->ReplaceGlobalValues(m, reinterpret_cast<const int*>(rows), block, 0);
+  const int err = x->ReplaceGlobalValues(m, reinterpret_cast<const int*>(rows),
+                                         block, 0);
+
   if (err != 0)
     error("EpetraVector::set: Did not manage to perform Epetra_Vector::ReplaceGlobalValues.");
+
+  assert(x);
+  const Epetra_BlockMap& map = x->Map();
+  assert(x->Map().LinearMap());
+  const uint n0 = map.MinMyGID();
+  const uint n1 = map.MaxMyGID();
+
+  // Set local values, or add to off-process cache
+  for (uint i = 0; i < m; ++i)
+  {
+    if (rows[i] >= n0 && rows[i] <= n1)
+      (*x)[0][rows[i] - n0] = block[i];
+    else
+      off_process_set_values[rows[i]] = block[i];
+  }
 }
 //-----------------------------------------------------------------------------
 void EpetraVector::add(const double* block, uint m, const uint* rows)
 {
+  if (off_process_set_values.size() > 0)
+    error("EpetraVector:: must be called between calling EpetraVector::set and EpetraVector::add.");
+
   assert(x);
-  int err = x->SumIntoGlobalValues(m, reinterpret_cast<const int*>(rows), block, 0);
+  int err = x->SumIntoGlobalValues(m, reinterpret_cast<const int*>(rows),
+                                   block, 0);
+
   if (err != 0)
     error("EpetraVector::add: Did not manage to perform Epetra_Vector::SumIntoGlobalValues.");
 }
@@ -237,42 +333,95 @@ void EpetraVector::add(const double* block, uint m, const uint* rows)
 void EpetraVector::get_local(double* block, uint m, const uint* rows) const
 {
   assert(x);
-  const uint n0 = local_range().first;
-  for (uint i = 0; i < m; ++i)
-    block[i] = (*x)[0][rows[i] - n0];
+  const Epetra_BlockMap& map = x->Map();
+  assert(x->Map().LinearMap());
+  const uint n0 = map.MinMyGID();
+
+  // Get values
+  if (ghost_global_to_local.size() == 0)
+  {
+    for (uint i = 0; i < m; ++i)
+      block[i] = (*x)[0][rows[i] - n0];
+  }
+  else
+  {
+    assert(x_ghost);
+    const uint n1 = map.MaxMyGID();
+    const Epetra_BlockMap& ghost_map = x_ghost->Map();
+    for (uint i = 0; i < m; ++i)
+    {
+      if (rows[i] >= n0 && rows[i] <= n1)
+        block[i] = (*x)[0][rows[i] - n0];
+      else
+      {
+        // FIXME: Check if look-up in std::map is faster than Epetra_BlockMap::LID
+        // Get local index
+        const int local_index = ghost_map.LID(rows[i]);
+        assert(local_index != -1);
+
+        //boost::unordered_map<uint, uint>::const_iterator _local_index = ghost_global_to_local.find(rows[i]);
+        //assert(_local_index != ghost_global_to_local.end());
+        //const int local_index = _local_index->second;
+
+        // Get value
+        block[i] = (*x_ghost)[local_index];
+      }
+    }
+  }
 }
 //-----------------------------------------------------------------------------
 void EpetraVector::gather(GenericVector& y,
                           const Array<dolfin::uint>& indices) const
 {
-  // FIXME: This can be done better. Problem is that the GenericVector interface
-  //        is PETSc-centric for the parallel case. It should be improved.
-
   assert(x);
 
-  EpetraFactory& f = EpetraFactory::instance();
-  Epetra_SerialComm serial_comm = f.get_serial_comm();
-
-  // Down cast to a EpetraVector and resize
+  // Down cast to an EpetraVector
   EpetraVector& _y = y.down_cast<EpetraVector>();
 
-  // Check that y is a local vector (check communicator)
+  // Create serial communicator
+  const EpetraFactory& f = EpetraFactory::instance();
+  Epetra_SerialComm serial_comm = f.get_serial_comm();
 
-  // Create map
-  std::vector<int> _indices(indices.size());
-  for (uint i = 0; i < indices.size(); ++i)
-    _indices[i] = indices[i];
+  // Create map for y
+  const int* _indices = reinterpret_cast<const int*>(indices.data().get());
+  Epetra_Map target_map(indices.size(), indices.size(), _indices, 0, serial_comm);
 
-  Epetra_Map target_map(indices.size(), indices.size(), &_indices[0], 0, serial_comm);
-
-  // FIXME: Check that the data belonging to y is not shared
+  // Reset vector y
   _y.reset(target_map);
-  Epetra_Import importer(_y.vec()->Map(), x->Map());
+  assert(_y.vec());
+
+  // Create importer
+  Epetra_Import importer(target_map, x->Map());
+
+  // Import values into y
   _y.vec()->Import(*x, importer, Insert);
 }
 //-----------------------------------------------------------------------------
+void EpetraVector::gather(Array<double>& x, const Array<uint>& indices) const
+{
+  const uint _size = indices.size();
+  x.resize(_size);
+  assert(x.size() == _size);
+
+  // Gather values into a vector
+  EpetraVector y;
+  gather(y, indices);
+
+  assert(y.size() == _size);
+  const Epetra_FEVector& _y = *(y.vec());
+
+  // Copy values into x
+  for (uint i = 0; i < _size; ++i)
+    x[i] = (_y)[0][i];
+}
+//-----------------------------------------------------------------------------
 void EpetraVector::reset(const Epetra_Map& map)
 {
+  // Clear ghost data
+  x_ghost.reset();
+  ghost_global_to_local.clear();
+  off_process_set_values.clear();
+
   x.reset(new Epetra_FEVector(map));
 }
 //-----------------------------------------------------------------------------
@@ -290,9 +439,10 @@ double EpetraVector::inner(const GenericVector& y) const
     error("Given vector is not initialized.");
 
   double a;
-  int err = x->Dot(*(v.x), &a);
+  const int err = x->Dot(*(v.x), &a);
   if (err!= 0)
     error("EpetraVector::inner: Did not manage to perform Epetra_Vector::Dot.");
+
   return a;
 }
 //-----------------------------------------------------------------------------
@@ -307,12 +457,17 @@ void EpetraVector::axpy(double a, const GenericVector& y)
   if (size() != _y.size())
     error("The vectors must be of the same size.");
 
-  int err = x->Update(a, *(_y.vec()), 1.0);
-
+  const int err = x->Update(a, *(_y.vec()), 1.0);
   if (err != 0)
     error("EpetraVector::axpy: Did not manage to perform Epetra_Vector::Update.");
 }
 //-----------------------------------------------------------------------------
+void EpetraVector::abs()
+{
+  assert(x);
+  x->Abs(*x);
+}
+//-----------------------------------------------------------------------------
 LinearAlgebraFactory& EpetraVector::factory() const
 {
   return EpetraFactory::instance();
@@ -331,6 +486,19 @@ const EpetraVector& EpetraVector::operator= (double a)
   return *this;
 }
 //-----------------------------------------------------------------------------
+void EpetraVector::update_ghost_values()
+{
+  assert(x);
+  assert(x_ghost);
+  assert(x_ghost->MyLength() == (int) ghost_global_to_local.size());
+
+  // Create importer
+  Epetra_Import importer(x_ghost->Map(), x->Map());
+
+  // Import into ghost vector
+  x_ghost->Import(*x, importer, Insert);
+}
+//-----------------------------------------------------------------------------
 const EpetraVector& EpetraVector::operator= (const EpetraVector& v)
 {
   // FIXME: Epetra assignment operator leads to an errror. Must vectors have
@@ -338,7 +506,15 @@ const EpetraVector& EpetraVector::operator= (const EpetraVector& v)
 
   assert(v.x);
   if (this != &v)
-    x.reset(new Epetra_FEVector(*(v.vec())));
+  {
+    // Copy vector
+    x.reset(new Epetra_FEVector(*(v.x)));
+
+    // Copy ghost data
+    if (v.x_ghost)
+      x_ghost.reset(new Epetra_Vector(*(v.x_ghost)));
+    ghost_global_to_local = v.ghost_global_to_local;
+  }
   return *this;
 }
 //-----------------------------------------------------------------------------
@@ -359,7 +535,7 @@ const EpetraVector& EpetraVector::operator-= (const GenericVector& y)
 const EpetraVector& EpetraVector::operator*= (double a)
 {
   assert(x);
-  int err = x->Scale(a);
+  const int err = x->Scale(a);
   if (err!= 0)
     error("EpetraVector::operator*=: Did not manage to perform Epetra_Vector::Scale.");
   return *this;
@@ -376,7 +552,7 @@ const EpetraVector& EpetraVector::operator*= (const GenericVector& y)
   if (size() != v.size())
     error("The vectors must be of the same size.");
 
-  int err = x->Multiply(1.0, *x, *v.x, 0.0);
+  const int err = x->Multiply(1.0, *x, *v.x, 0.0);
   if (err!= 0)
     error("EpetraVector::operator*=: Did not manage to perform Epetra_Vector::Multiply.");
   return *this;
@@ -410,7 +586,7 @@ double EpetraVector::min() const
 {
   assert(x);
   double value = 0.0;
-  int err = x->MinValue(&value);
+  const int err = x->MinValue(&value);
   if (err!= 0)
     error("EpetraVector::min: Did not manage to perform Epetra_Vector::MinValue.");
   return value;
@@ -420,7 +596,7 @@ double EpetraVector::max() const
 {
   assert(x);
   double value = 0.0;
-  int err = x->MaxValue(&value);
+  const int err = x->MaxValue(&value);
   if (err != 0)
     error("EpetraVector::min: Did not manage to perform Epetra_Vector::MinValue.");
   return value;
@@ -436,7 +612,6 @@ double EpetraVector::sum() const
   get_local(x_local);
 
   // Compute local sum
-  //double local_sum = std::accumulate(&x_local[0], &x_local[local_size], 0.0);
   double local_sum = 0.0;
   for (uint i = 0; i < local_size; ++i)
     local_sum += x_local[i];
@@ -465,7 +640,7 @@ double EpetraVector::sum(const Array<uint>& rows) const
       nonlocal_rows.insert(rows[i]);
   }
 
-  // Send nonlocal rows indices to other processes
+  // Send nonlocal row indices to other processes
   const uint num_processes  = MPI::num_processes();
   const uint process_number = MPI::process_number();
   for (uint i = 1; i < num_processes; ++i)
@@ -505,5 +680,4 @@ double EpetraVector::sum(const Array<uint>& rows) const
   return global_sum;
 }
 //-----------------------------------------------------------------------------
-
 #endif
