@@ -1,24 +1,29 @@
 // Copyright (C) 2008 Martin Sandve Alnes, Kent-Andre Mardal and Johannes Ring.
 // Licensed under the GNU LGPL Version 2.1.
 //
-// Modified by Anders Logg, 2008.
+// Modified by Anders Logg, 2008-2010.
 // Modified by Garth N. Wells, 2008-2009.
 //
 // First added:  2008-04-21
-// Last changed: 2009-09-08
+// Last changed: 2011-01-14
 
 #ifndef __EPETRA_VECTOR_H
 #define __EPETRA_VECTOR_H
 
 #ifdef HAS_TRILINOS
 
+#include <map>
 #include <string>
 #include <vector>
 #include <boost/shared_ptr.hpp>
-#include <dolfin/common/Variable.h>
+#include <boost/unordered_map.hpp>
+#include <boost/unordered_set.hpp>
+
 #include "GenericVector.h"
 
 class Epetra_FEVector;
+class Epetra_MultiVector;
+class Epetra_Vector;
 class Epetra_Map;
 
 namespace dolfin
@@ -76,14 +81,24 @@ namespace dolfin
     /// Resize vector to size N
     virtual void resize(uint N);
 
+    /// Resize vector with given ownership range
+    virtual void resize(std::pair<uint, uint> range);
+
+    /// Resize vector with given ownership range and with ghost values
+    virtual void resize(std::pair<uint, uint> range,
+                        const std::vector<uint>& ghost_indices);
+
     /// Return size of vector
     virtual uint size() const;
 
+    /// Return size of local vector
+    virtual uint local_size() const;
+
     /// Return local ownership range of a vector
     virtual std::pair<uint, uint> local_range() const;
 
-    /// Get block of values
-    virtual void get(double* block, uint m, const uint* rows) const;
+    /// Determine whether global vector index is owned by this process
+    virtual bool owns_index(uint i) const;
 
     /// Set block of values
     virtual void set(const double* block, uint m, const uint* rows);
@@ -105,9 +120,15 @@ namespace dolfin
     /// Gather entries into local vector x
     virtual void gather(GenericVector& x, const Array<uint>& indices) const;
 
+    /// Gather entries into Array x
+    virtual void gather(Array<double>& x, const Array<uint>& indices) const;
+
     /// Add multiple of given vector (AXPY operation)
     virtual void axpy(double a, const GenericVector& x);
 
+    /// Replace all entries in the vector by their absolute values
+    virtual void abs();
+
     /// Return inner product with given vector
     virtual double inner(const GenericVector& vector) const;
 
@@ -147,6 +168,8 @@ namespace dolfin
     /// Assignment operator
     virtual const EpetraVector& operator= (double a);
 
+    virtual void update_ghost_values();
+
     //--- Special functions ---
 
     /// Return linear algebra backend factory
@@ -168,6 +191,15 @@ namespace dolfin
     // Epetra_FEVector pointer
     boost::shared_ptr<Epetra_FEVector> x;
 
+    // Epetra_FEVector pointer
+    boost::shared_ptr<Epetra_Vector> x_ghost;
+
+    // Global-to-local map for ghost values
+    boost::unordered_map<uint, uint> ghost_global_to_local;
+
+    // Cache of off-process 'set' values (versus 'add') to be communicated
+    boost::unordered_map<uint, double> off_process_set_values;
+
     // Local/global vector
     const std::string type;
 
