@@ -5,7 +5,7 @@
 //
 // First added:  2008-08-26
 // Last changed: 2009-06-29
-// Last changed: 2010-06-04
+// Last changed: 2010-07-16
 
 #ifndef __GENERIC_LINEAR_SOLVER_H
 #define __GENERIC_LINEAR_SOLVER_H
@@ -26,6 +26,13 @@ namespace dolfin
   {
   public:
 
+    /// Solve the operator (matrix)
+    virtual void set_operator(const GenericMatrix& A) = 0;
+
+    /// Solve the operator (matrix) and preconditioner matrix
+    virtual void set_operators(const GenericMatrix& A, const GenericMatrix& P)
+    { error("Linear algebra backend solver does not support 'set_operators'."); }
+
     /// Solve linear system Ax = b
     virtual uint solve(const GenericMatrix& A, GenericVector& x, const GenericVector& b)
     { error("solve(A, x, b) is not implemented. Consider trying solve(x, b)."); return 0; }
