@@ -1,4 +1,4 @@
-// Copyright (C) 2006-2009 Garth N. Wells.
+// Copyright (C) 2006-2011 Garth N. Wells.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Johan Jansson, 2006.
@@ -8,7 +8,7 @@
 // Modified by Martin Alnæs, 2008.
 //
 // First added:  2006-04-24
-// Last changed: 2010-02-24
+// Last changed: 2011-02-05
 
 #ifndef __GENERIC_MATRIX_H
 #define __GENERIC_MATRIX_H
@@ -36,7 +36,7 @@ namespace dolfin
 
     /// Resize tensor with given dimensions
     virtual void resize(uint rank, const uint* dims)
-    { assert(rank == 2); resize(dims[0], dims[1]); }
+    { error("Generic matrices must be re-sized using a GenericSparsityPattern"); }
 
     /// Initialize zero tensor using sparsity pattern
     virtual void init(const GenericSparsityPattern& sparsity_pattern) = 0;
@@ -51,6 +51,9 @@ namespace dolfin
     /// Return size of given dimension
     virtual uint size(uint dim) const = 0;
 
+    /// Return local ownership range
+    virtual std::pair<uint, uint> local_range(uint dim) const = 0;
+
     /// Get block of values
     virtual void get(double* block, const uint* num_rows,
                      const uint * const * rows) const
@@ -66,6 +69,14 @@ namespace dolfin
                      const uint * const * rows)
     { add(block, num_rows[0], rows[0], num_rows[1], rows[1]); }
 
+    /// Add block of values
+    virtual void add(const double* block, const std::vector<const std::vector<uint>* >& rows)
+    { add(block, (rows[0])->size(), &(*rows[0])[0], (rows[1])->size(), &(*rows[1])[0]); }
+
+    /// Add block of values
+    virtual void add(const double* block, const std::vector<std::vector<uint> >& rows)
+    { add(block, rows[0].size(), &(rows[0])[0], rows[1].size(), &(rows[1])[0]); }
+
     /// Set all entries to zero and keep any sparse structure
     virtual void zero() = 0;
 
@@ -77,8 +88,10 @@ namespace dolfin
 
     //--- Matrix interface ---
 
-    /// Resize matrix to  M x N
-    virtual void resize(uint M, uint N) = 0;
+    /// Resize vector y such that is it compatible with matrix for
+    /// multuplication Ax = b (dim = 0 -> b, dim = 1 -> x) In parallel
+    /// case, size and layout are important.
+    virtual void resize(GenericVector& y, uint dim) const = 0;
 
     /// Get block of values
     virtual void get(double* block, uint m, const uint* rows, uint n,
@@ -99,11 +112,11 @@ namespace dolfin
     /// Return norm of matrix
     virtual double norm(std::string norm_type) const = 0;
 
-    /// Get non-zero values of given row
+    /// Get non-zero values of given row on local process
     virtual void getrow(uint row, std::vector<uint>& columns,
                         std::vector<double>& values) const = 0;
 
-    /// Set values for given row
+    /// Set values for given row on local process
     virtual void setrow(uint row, const std::vector<uint>& columns,
                         const std::vector<double>& values) = 0;
 
