@@ -3,13 +3,18 @@
 //
 // Modified by Magnus Vikstrom, 2008.
 // Modified by Anders Logg, 2009.
+// Modified by Garth N. Wells, 2010.
 //
 // First added:  2007-11-30
-// Last changed: 2009-08-06
+// Last changed: 2010-12-30
 
 #ifndef __GENERIC_SPARSITY_PATTERN_H
 #define __GENERIC_SPARSITY_PATTERN_H
 
+#include <utility>
+#include <vector>
+#include <boost/unordered_map.hpp>
+
 #include <dolfin/common/types.h>
 #include <dolfin/common/Variable.h>
 
@@ -23,6 +28,8 @@ namespace dolfin
   {
   public:
 
+    enum Type {sorted, unsorted};
+
     /// Create empty sparsity pattern
     GenericSparsityPattern() {}
 
@@ -30,10 +37,13 @@ namespace dolfin
     virtual ~GenericSparsityPattern() {};
 
     /// Initialize sparsity pattern for a generic tensor
-    virtual void init(uint rank, const uint* dims) = 0;
+    virtual void init(const std::vector<uint>& dims,
+                      const std::vector<std::pair<uint, uint> >& ownership_range,
+                      const std::vector<const boost::unordered_map<uint, uint>* > off_process_owner) = 0;
+
 
     /// Insert non-zero entries
-    virtual void insert(const uint* num_rows, const uint * const * rows) = 0;
+    virtual void insert(const std::vector<const std::vector<uint>* >& entries) = 0;
 
     /// Return rank
     virtual uint rank() const = 0;
@@ -47,11 +57,19 @@ namespace dolfin
     /// Return total number of nonzeros in local_range for dimension 0
     virtual uint num_nonzeros() const = 0;
 
-    /// Fill array with number of nonzeros for diagonal block in local_range for dimension 0 
-    virtual void num_nonzeros_diagonal(uint* num_nonzeros) const = 0;
+    /// Fill vector with number of nonzeros for diagonal block in local_range for dimension 0
+    virtual void num_nonzeros_diagonal(std::vector<uint>& num_nonzeros) const = 0;
 
-    /// Fill array with number of nonzeros for off-diagonal block in local_range for dimension 0 
-    virtual void num_nonzeros_off_diagonal(uint* num_nonzeros) const = 0;
+    /// Fill vector with number of nonzeros for off-diagonal block in local_range for dimension 0
+    virtual void num_nonzeros_off_diagonal(std::vector<uint>& num_nonzeros) const = 0;
+
+    /// Return underlying sparsity pattern (diagonal). Options are
+    /// 'sorted' and 'unsorted'.
+    virtual std::vector<std::vector<uint> > diagonal_pattern(Type type) const = 0;
+
+    /// Return underlying sparsity pattern (off-diagional). Options are
+    /// 'sorted' and 'unsorted'.
+    virtual std::vector<std::vector<uint> > off_diagonal_pattern(Type type) const = 0;
 
     /// Finalize sparsity pattern
     virtual void apply() = 0;
