@@ -6,11 +6,13 @@
 // Modified by Martin Alnæs, 2008.
 //
 // First added:  2007-01-17
-// Last changed: 2009-09-08
+// Last changed: 2010-11-29
 
 #ifndef __GENERIC_TENSOR_H
 #define __GENERIC_TENSOR_H
 
+#include <exception>
+#include <typeinfo>
 #include <dolfin/log/log.h>
 #include <dolfin/common/types.h>
 #include <dolfin/common/Variable.h>
@@ -23,7 +25,7 @@ namespace dolfin
 
   /// This class defines a common interface for arbitrary rank tensors.
 
-  class GenericTensor : public Variable
+  class GenericTensor : public virtual Variable
   {
   public:
 
@@ -32,9 +34,6 @@ namespace dolfin
 
     //--- Basic GenericTensor interface ---
 
-    /// Resize tensor with given dimensions
-    virtual void resize(uint rank, const uint* dims) = 0;
-
     /// Initialize zero tensor using sparsity pattern
     virtual void init(const GenericSparsityPattern& sparsity_pattern) = 0;
 
@@ -47,6 +46,9 @@ namespace dolfin
     /// Return size of given dimension
     virtual uint size(uint dim) const = 0;
 
+    /// Return local ownership range
+    virtual std::pair<uint, uint> local_range(uint dim) const = 0;
+
     /// Get block of values
     virtual void get(double* block, const uint* num_rows,
                      const uint * const * rows) const = 0;
@@ -55,6 +57,14 @@ namespace dolfin
     virtual void set(const double* block, const uint* num_rows,
                      const uint * const * rows) = 0;
 
+    /// Add block of values
+    virtual void add(const double* block,
+                     const std::vector<const std::vector<uint>* >& rows) = 0;
+
+    /// Add block of values
+    virtual void add(const double* block,
+                     const std::vector<std::vector<uint> >& rows) = 0;
+
     /// Add block of values
     virtual void add(const double* block, const uint* num_rows,
                      const uint * const * rows) = 0;
@@ -76,19 +86,33 @@ namespace dolfin
     /// Cast a GenericTensor to its derived class (const version)
     template<class T> const T& down_cast() const
     {
-      const T* t = dynamic_cast<const T*>(instance());
-      if (!t)
-        error("GenericTensor cannot be cast to the requested type.");
-      return *t;
+      try
+      {
+        return dynamic_cast<const T&>(*instance());
+      }
+      catch (std::exception& e)
+      {
+        error("GenericTensor cannot be cast to the requested type: %s", e.what());
+      }
+
+      // Return something to keep the compiler happy. Code will never be reached.
+      return dynamic_cast<const T&>(*instance());
     }
 
     /// Cast a GenericTensor to its derived class (non-const version)
     template<class T> T& down_cast()
     {
-      T* t = dynamic_cast<T*>(instance());
-      if (!t)
-        error("GenericTensor cannot be cast to the requested type.");
-      return *t;
+      try
+      {
+        return dynamic_cast<T&>(*instance());
+      }
+      catch (std::exception& e)
+      {
+        error("GenericTensor cannot be cast to the requested type: %s", e.what());
+      }
+
+      // Return something to keep the compiler happy. Code will never be reached.
+      return dynamic_cast<T&>(*instance());
     }
 
     /// Check whether the GenericTensor instance matches a specific type
