@@ -1,14 +1,14 @@
-// Copyright (C) 2006-2009 Garth N. Wells.
+// Copyright (C) 2006-2010 Garth N. Wells.
 // Licensed under the GNU LGPL Version 2.1.
 //
-// Modified by Anders Logg, 2006-2008.
+// Modified by Anders Logg, 2006-2010.
 // Modified by Kent-Andre Mardal, 2008.
 // Modified by Ola Skavhaug, 2008.
 // Modified by Martin Sandve Alnes, 2009.
 // Modified by Johan Hake, 2009-2010.
 //
 // First added:  2006-04-25
-// Last changed: 2010-03-07
+// Last changed: 2011-01-14
 
 #ifndef __GENERIC_VECTOR_H
 #define __GENERIC_VECTOR_H
@@ -16,7 +16,7 @@
 #include <algorithm>
 #include <utility>
 #include <vector>
-#include <boost/lambda/lambda.hpp>
+#include <dolfin/log/log.h>
 #include "GenericSparsityPattern.h"
 #include "GenericTensor.h"
 
@@ -41,8 +41,8 @@ namespace dolfin
     { assert(rank == 1); resize(dims[0]); }
 
     /// Initialize zero tensor using sparsity pattern
-    inline void init(const GenericSparsityPattern& sparsity_pattern)
-    { resize(sparsity_pattern.size(0)); zero(); }
+    virtual void init(const GenericSparsityPattern& sparsity_pattern)
+    { resize(sparsity_pattern.local_range(0)); zero(); }
 
     /// Return copy of tensor
     virtual GenericVector* copy() const = 0;
@@ -55,10 +55,14 @@ namespace dolfin
     virtual uint size(uint dim) const
     { assert(dim == 0); return size(); }
 
+    /// Return local ownership range
+    virtual std::pair<uint, uint> local_range(uint dim) const
+    { assert(dim == 0); return local_range(); }
+
     /// Get block of values
     virtual void get(double* block, const uint* num_rows,
                      const uint * const * rows) const
-    { get(block, num_rows[0], rows[0]); }
+    { get_local(block, num_rows[0], rows[0]); }
 
     /// Set block of values
     virtual void set(const double* block, const uint* num_rows,
@@ -70,6 +74,14 @@ namespace dolfin
                      const uint * const * rows)
     { add(block, num_rows[0], rows[0]); }
 
+    /// Add block of values
+    virtual void add(const double* block, const std::vector<const std::vector<uint>* >& rows)
+    { add(block, rows[0]->size(), &(*rows[0])[0]); }
+
+    /// Add block of values
+    virtual void add(const double* block, const std::vector<std::vector<uint> >& rows)
+    { add(block, rows[0].size(), &(rows[0])[0]); }
+
     /// Set all entries to zero and keep any sparse structure
     virtual void zero() = 0;
 
@@ -81,21 +93,37 @@ namespace dolfin
 
     //--- Vector interface ---
 
-    /// Resize vector to size N
+    /// Resize vector to global size N
     virtual void resize(uint N) = 0;
 
-    /// Return size of vector
+    /// Resize vector with given ownership range
+    virtual void resize(std::pair<uint, uint> range) = 0;
+
+    /// Resize vector with given ownership range and with ghost values
+    virtual void resize(std::pair<uint, uint> range,
+                        const std::vector<uint>& ghost_indices) = 0;
+
+    /// Return global size of vector
     virtual uint size() const = 0;
 
+    /// Return local size of vector
+    virtual uint local_size() const = 0;
+
     /// Return local ownership range of a vector
     virtual std::pair<uint, uint> local_range() const = 0;
 
+    /// Determine whether global vector index is owned by this process
+    virtual bool owns_index(uint i) const = 0;
+
     /// Get block of values (values may live on any process)
-    virtual void get(double* block, uint m, const uint* rows) const = 0;
+    virtual void get(double* block, uint m, const uint* rows) const
+    {
+      warning("GenericVector::get is redirected to GenericVector::get_local. Use GenericVector::gather for get off-process entries. GenericVector::get will be removed");
+      get_local(block, m, rows);
+    }
 
     /// Get block of values (values must all live on the local process)
-    virtual void get_local(double* block, uint m, const uint* rows) const
-    { error("GenericVector::get_local not yet implemented for this backend."); }
+    virtual void get_local(double* block, uint m, const uint* rows) const = 0;
 
     /// Set block of values
     virtual void set(const double* block, uint m, const uint* rows) = 0;
@@ -115,9 +143,15 @@ namespace dolfin
     /// Gather entries into local vector x
     virtual void gather(GenericVector& x, const Array<uint>& indices) const = 0;
 
+    /// Gather entries into Array x
+    virtual void gather(Array<double>& x, const Array<uint>& indices) const = 0;
+
     /// Add multiple of given vector (AXPY operation)
     virtual void axpy(double a, const GenericVector& x) = 0;
 
+    /// Replace all entries in the vector by their absolute values
+    virtual void abs() = 0;
+
     /// Return inner product with given vector
     virtual double inner(const GenericVector& x) const = 0;
 
@@ -158,34 +192,6 @@ namespace dolfin
     /// Assignment operator
     virtual const GenericVector& operator= (double a) = 0;
 
-    /// Apply lambda function
-    template<typename T> void lambda(const T function)
-    {
-      // FIXME: This could be more efficient by acting on the underling vector
-      //        data
-      std::vector<double> values(size());
-      Array<double> _values(size(), &values[0]);
-      get_local(_values);
-      std::for_each(values.begin(), values.end(), function);
-      set_local(_values);
-    }
-
-    /// Apply lambda function
-    template<typename T> void lambda(const GenericVector& x, const T function)
-    {
-      // FIXME: This could be more efficient by acting on the underling vector
-      //        data
-      assert(x.size() == this->size());
-      std::vector<double> values(size());
-      Array<double> _values(size(), values);
-      std::vector<double> x_values(size());
-      Array<double> _x_values(size(), x_values);
-      this->get_local(_values);
-      x.get_local(_x_values);
-      std::transform(x_values.begin(), x_values.end(), values.begin(), function);
-      this->set_local(_values);
-    }
-
     /// Return pointer to underlying data (const version)
     virtual const double* data() const
     {
@@ -200,15 +206,19 @@ namespace dolfin
       return 0;
     }
 
+    /// Update ghost values
+    virtual void update_ghost_values()
+    { error("GenericVector::update_ghost_values not implemented for this backend."); }
+
     //--- Convenience functions ---
 
     /// Get value of given entry
     virtual double operator[] (uint i) const
-    { double value(0); get(&value, 1, &i); return value; }
+    { double value(0); get_local(&value, 1, &i); return value; }
 
     /// Get value of given entry
     virtual double getitem(uint i) const
-    { double value(0); get(&value, 1, &i); return value; }
+    { double value(0); get_local(&value, 1, &i); return value; }
 
     /// Set given entry to value. apply("insert") should be called before using
     /// using the object.
