@@ -11,20 +11,19 @@
 #ifndef __MTL4_MATRIX_H
 #define __MTL4_MATRIX_H
 
+#include <utility>
 #include <tr1/tuple>
 #include "GenericMatrix.h"
 #include "mtl4.h"
 
-/*
-  Developers note:
-
-  This class implements a minimal backend for MTL4.
-
-  There are certain inline decisions that have been deferred.
-  Due to the extensive calling of this backend through the generic LA
-  interface, it is not clear where inlining will be possible and
-  improve performance.
-*/
+//  Developers note:
+//
+//  This class implements a minimal backend for MTL4.
+//
+//  There are certain inline decisions that have been deferred.
+//  Due to the extensive calling of this backend through the generic LA
+//  interface, it is not clear where inlining will be possible and
+//  improve performance.
 
 namespace dolfin
 {
@@ -36,9 +35,6 @@ namespace dolfin
     /// Create empty matrix
     MTL4Matrix();
 
-    /// Create M x N matrix
-    MTL4Matrix(uint M, uint N);
-
     /// Copy constuctor
     MTL4Matrix(const MTL4Matrix& A);
 
@@ -56,6 +52,10 @@ namespace dolfin
     /// Return size of given dimension
     virtual uint size(uint dim) const;
 
+    /// Return local ownership range
+    virtual std::pair<uint, uint> local_range(uint dim) const
+    { return std::make_pair(0, size(dim)); }
+
     /// Set all entries to zero and keep any sparse structure
     virtual void zero();
 
@@ -70,6 +70,11 @@ namespace dolfin
     /// Resize matrix to M x N
     virtual void resize(uint M, uint N);
 
+    /// Resize vector y such that is it compatible with matrix for
+    /// multuplication Ax = b (dim = 0 -> b, dim = 1 -> x) In parallel
+    /// case, size and layout are important.
+    virtual void resize(GenericVector& y, uint dim) const;
+
     /// Get block of values
     virtual void get(double* block, uint m, const uint* rows, uint n, const uint* cols) const;
 
