@@ -2,9 +2,10 @@
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Garth N. Wells, 2008-2009.
+// Modified by Anders Logg, 2010.
 //
 // First added:  2008-07-06
-// Last changed: 2009-08-22
+// Last changed: 2011-01-14
 
 #ifdef HAS_MTL4
 
@@ -44,6 +45,23 @@ void MTL4Vector::resize(uint N)
     x.change_dim(N);
 }
 //-----------------------------------------------------------------------------
+void MTL4Vector::resize(std::pair<uint, uint> range)
+{
+  if (range.first != 0)
+    error("MTL4Vector does not support distributed vectors.");
+
+  resize(range.second - range.first);
+}
+//-----------------------------------------------------------------------------
+void MTL4Vector::resize(std::pair<uint, uint> range,
+                    const std::vector<uint>& ghost_indices)
+{
+  if (ghost_indices.size() != 0)
+    error("MTL4SVector does not support ghost entries.");
+
+  resize(range);
+}
+//-----------------------------------------------------------------------------
 MTL4Vector* MTL4Vector::copy() const
 {
   return new MTL4Vector(*this);
@@ -59,6 +77,14 @@ std::pair<dolfin::uint, dolfin::uint> MTL4Vector::local_range() const
   return std::make_pair<uint, uint>(0, size());
 }
 //-----------------------------------------------------------------------------
+bool MTL4Vector::owns_index(uint i) const
+{
+  if (i < size())
+    return true;
+  else
+    return false;
+}
+//-----------------------------------------------------------------------------
 void MTL4Vector::zero()
 {
   x = 0.0;
@@ -94,6 +120,12 @@ std::string MTL4Vector::str(bool verbose) const
   return s.str();
 }
 //-----------------------------------------------------------------------------
+void MTL4Vector::get_local(double* block, uint m, const uint* rows) const
+{
+  for (uint i = 0; i < m; ++i)
+    block[i] = x[ rows[i] ];
+}
+//-----------------------------------------------------------------------------
 void MTL4Vector::get_local(Array<double>& values) const
 {
   values.resize(size());
@@ -115,12 +147,6 @@ void MTL4Vector::add_local(const Array<double>& values)
     x(i) += values[i];
 }
 //-----------------------------------------------------------------------------
-void MTL4Vector::get(double* block, uint m, const uint* rows) const
-{
-  for (uint i = 0; i < m; i++)
-    block[i] = x[ rows[i] ];
-}
-//-----------------------------------------------------------------------------
 void MTL4Vector::set(const double* block, uint m, const uint* rows)
 {
   for (uint i = 0; i < m; i++)
@@ -133,6 +159,30 @@ void MTL4Vector::add(const double* block, uint m, const uint* rows)
     x[ rows[i] ] += block[i];
 }
 //-----------------------------------------------------------------------------
+void MTL4Vector::gather(GenericVector& x, const Array<uint>& indices) const
+{
+  not_working_in_parallel("MTL4Vector::gather)");
+
+  const uint _size = indices.size();
+  assert(this->size() >= _size);
+
+  x.resize(_size);
+  mtl4_vector& _x = x.down_cast<MTL4Vector>().vec();
+  for (uint i = 0; i < _size; i++)
+    _x[i] = this->x[ indices[i] ];
+}
+//-----------------------------------------------------------------------------
+void MTL4Vector::gather(Array<double>& x, const Array<uint>& indices) const
+{
+  not_working_in_parallel("MTL4Vector::gather)");
+
+  const uint _size = indices.size();
+  x.resize(_size);
+  assert(x.size() == _size);
+  for (uint i = 0; i < _size; i++)
+    x[i] = this->x[ indices[i] ];
+}
+//-----------------------------------------------------------------------------
 const mtl4_vector& MTL4Vector::vec() const
 {
   return x;
@@ -152,11 +202,13 @@ double MTL4Vector::inner(const GenericVector& v) const
 //-----------------------------------------------------------------------------
 void MTL4Vector::axpy(double a, const GenericVector& v)
 {
-  // Developers note: This is a hack. One would like:
-  // x += a*v.down_cast<MTL4Vector>().vec();
-  mtl4_vector vv =  v.down_cast<MTL4Vector>().vec();
-  vv *= a;
-  x  += vv;
+  x += a*v.down_cast<MTL4Vector>().vec();
+}
+//-----------------------------------------------------------------------------
+void MTL4Vector::abs()
+{
+  for (uint i = 0; i < size(); ++i)
+    x[i] = std::abs(x[i]);
 }
 //-----------------------------------------------------------------------------
 LinearAlgebraFactory& MTL4Vector::factory() const
@@ -190,12 +242,12 @@ const MTL4Vector& MTL4Vector::operator*= (double a)
 //-----------------------------------------------------------------------------
 const MTL4Vector& MTL4Vector::operator*= (const GenericVector& y)
 {
-  if ( size() != y.size() )
+  if (size() != y.size())
     error("Vectors must be of same size.");
 
-  const MTL4Vector& _y = y.down_cast<MTL4Vector>();
+  const mtl4_vector& _y = y.down_cast<MTL4Vector>().vec();
   for (uint i = 0; i < size(); ++i)
-    x[i] *= _y.vec()[i];
+    x[i] *= _y[i];
   return *this;
 }
 //-----------------------------------------------------------------------------
