@@ -1,30 +1,32 @@
 // Copyright (C) 2008 Dag Lindbo
 // Licensed under the GNU LGPL Version 2.1.
 //
-// Modified by Anders Logg, 2008.
+// Modified by Anders Logg, 2008-2010.
 // Modified by Garth N. Wells, 2009.
 //
 // First added:  2008-07-06
-// Last changed: 2009-09-08
+// Last changed: 2011-01-14
 
 #ifdef HAS_MTL4
 
 #ifndef __MTL4_VECTOR_H
 #define __MTL4_VECTOR_H
 
+#include <vector>
+#include <utility>
+
+#include <dolfin/common/types.h>
 #include "mtl4.h"
 #include "GenericVector.h"
 
-/*
-  Developers note:
-
-  This class implements a minimal backend for MTL4.
-
-  There are certain inline decisions that have been deferred.
-  Due to the extensive calling of this backend through the generic LA
-  interface, it is not clear where inlining will be possible and
-  improve performance.
-*/
+//  Developers note:
+//
+//  This class implements a minimal backend for MTL4.
+//
+//  There are certain inline decisions that have been deferred.
+//  Due to the extensive calling of this backend through the generic LA
+//  interface, it is not clear where inlining will be possible and
+//  improve performance.
 
 namespace dolfin
 {
@@ -66,14 +68,28 @@ namespace dolfin
     /// Resize vector to size N
     virtual void resize(uint N);
 
+    /// Resize vector with given ownership range
+    virtual void resize(std::pair<uint, uint> range);
+
+    /// Resize vector with given ownership range and with ghost values
+    virtual void resize(std::pair<uint, uint> range,
+                        const std::vector<uint>& ghost_indices);
+
     /// Return size of vector
     virtual uint size() const;
 
+    /// Return local size of vector
+    virtual uint local_size() const
+    { return size(); }
+
     /// Return local ownership range of a vector
     virtual std::pair<uint, uint> local_range() const;
 
+    /// Determine whether global vector index is owned by this process
+    virtual bool owns_index(uint i) const;
+
     /// Get block of values
-    virtual void get(double* block, uint m, const uint* rows) const;
+    virtual void get_local(double* block, uint m, const uint* rows) const;
 
     /// Set block of values
     virtual void set(const double* block, uint m, const uint* rows);
@@ -91,12 +107,16 @@ namespace dolfin
     virtual void add_local(const Array<double>& values);
 
     /// Gather entries into local vector x
-    virtual void gather(GenericVector& x, const Array<uint>& indices) const
-    { not_working_in_parallel("MTL4::gather()"); }
+    virtual void gather(GenericVector& x, const Array<uint>& indices) const;
+
+    virtual void gather(Array<double>& x, const Array<uint>& indices) const;
 
     /// Add multiple of given vector (AXPY operation)
     virtual void axpy(double a, const GenericVector& x);
 
+    /// Replace all entries in the vector by their absolute values
+    virtual void abs();
+
     /// Return inner product with given vector
     virtual double inner(const GenericVector& vector) const;
 
