@@ -29,10 +29,6 @@ namespace dolfin
     Matrix() : matrix(0)
     { DefaultFactory factory; matrix = factory.create_matrix(); }
 
-    /// Create M x N matrix
-    Matrix(uint M, uint N) : matrix(0)
-    { DefaultFactory factory; matrix = factory.create_matrix(); matrix->resize(M, N); }
-
     /// Copy constructor
     Matrix(const Matrix& A) : matrix(A.matrix->copy()) {}
 
@@ -54,6 +50,10 @@ namespace dolfin
     virtual uint size(uint dim) const
     { return matrix->size(dim); }
 
+    /// Return local ownership range
+    virtual std::pair<uint, uint> local_range(uint dim) const
+    { return matrix->local_range(dim); }
+
     /// Set all entries to zero and keep any sparse structure
     virtual void zero()
     { matrix->zero(); }
@@ -64,13 +64,15 @@ namespace dolfin
 
     /// Return informal string representation (pretty-print)
     virtual std::string str(bool verbose) const
-    { return matrix->str(verbose); }
+    { return "<Matrix wrapper of " + matrix->str(verbose) + ">"; }
 
     //--- Implementation of the GenericMatrix interface ---
 
-    /// Resize matrix to M x N
-    virtual void resize(uint M, uint N)
-    { matrix->resize(M, N); }
+    /// Resize vector y such that is it compatible with matrix for
+    /// multuplication Ax = b (dim = 0 -> b, dim = 1 -> x) In parallel
+    /// case, size and layout are important.
+    virtual void resize(GenericVector& y, uint dim) const
+    { matrix->resize(y, dim); }
 
     /// Get block of values
     virtual void get(double* block, uint m, const uint* rows, uint n, const uint* cols) const
