@@ -9,6 +9,7 @@
 #ifdef HAS_PETSC
 
 #include "SparsityPattern.h"
+#include "PETScLUSolver.h"
 #include "PETScMatrix.h"
 #include "PETScVector.h"
 #include "PETScFactory.h"
@@ -36,7 +37,18 @@ PETScVector* PETScFactory:: create_local_vector() const
 //-----------------------------------------------------------------------------
 SparsityPattern* PETScFactory::create_pattern() const
 {
-  return new SparsityPattern(SparsityPattern::unsorted);
+  return new SparsityPattern;
+}
+//-----------------------------------------------------------------------------
+PETScLUSolver* PETScFactory::create_lu_solver() const
+{
+  return new PETScLUSolver();
+}
+//-----------------------------------------------------------------------------
+PETScKrylovSolver* PETScFactory::create_krylov_solver(std::string method,
+                                                      std::string pc) const
+{
+  return new PETScKrylovSolver(method, pc);
 }
 //-----------------------------------------------------------------------------
 
