@@ -9,6 +9,8 @@
 #ifndef __PETSC_FACTORY_H
 #define __PETSC_FACTORY_H
 
+#include "PETScKrylovSolver.h"
+#include "PETScLUSolver.h"
 #include "PETScMatrix.h"
 #include "PETScVector.h"
 #include "SparsityPattern.h"
@@ -36,6 +38,13 @@ namespace dolfin
     /// Create empty sparsity pattern
     SparsityPattern* create_pattern() const;
 
+    /// Create LU solver
+    PETScLUSolver* create_lu_solver() const;
+
+    /// Create Krylov solver
+    PETScKrylovSolver* create_krylov_solver(std::string method,
+                                            std::string pc) const;
+
     /// Return singleton instance
     static PETScFactory& instance()
     { return factory; }
