@@ -12,9 +12,11 @@
 #include <boost/assign/list_of.hpp>
 #include <dolfin/common/NoDeleter.h>
 #include <dolfin/log/dolfin_log.h>
-#include <dolfin/main/MPI.h>
+#include <dolfin/common/MPI.h>
+#include "GenericMatrix.h"
+#include "GenericVector.h"
 #include "KrylovSolver.h"
-#include "PETScKrylovMatrix.h"
+#include "PETScBaseMatrix.h"
 #include "PETScMatrix.h"
 #include "PETScPreconditioner.h"
 #include "PETScUserPreconditioner.h"
@@ -43,6 +45,8 @@ const std::map<std::string, const KSPType> PETScKrylovSolver::methods
   = boost::assign::map_list_of("default",  "")
                               ("cg",         KSPCG)
                               ("gmres",      KSPGMRES)
+                              ("minres",     KSPMINRES)
+                              ("tfqmr",      KSPTFQMR)
                               ("richardson", KSPRICHARDSON)
                               ("bicgstab",   KSPBCGS);
 //-----------------------------------------------------------------------------
@@ -57,8 +61,7 @@ Parameters PETScKrylovSolver::default_parameters()
 }
 //-----------------------------------------------------------------------------
 PETScKrylovSolver::PETScKrylovSolver(std::string method, std::string pc_type)
-  : method(method), pc_dolfin(0),
-    preconditioner(new PETScPreconditioner(pc_type)),
+  : pc_dolfin(0), preconditioner(new PETScPreconditioner(pc_type)),
     preconditioner_set(false)
 {
   // Check that the requested method is known
@@ -67,30 +70,34 @@ PETScKrylovSolver::PETScKrylovSolver(std::string method, std::string pc_type)
 
   // Set parameter values
   parameters = default_parameters();
+
+  init(method);
 }
 //-----------------------------------------------------------------------------
 PETScKrylovSolver::PETScKrylovSolver(std::string method,
 				                             PETScPreconditioner& preconditioner)
-  : method(method),
-    preconditioner(reference_to_no_delete_pointer(preconditioner)),
+  : preconditioner(reference_to_no_delete_pointer(preconditioner)),
     preconditioner_set(false)
 
 {
   // Set parameter values
   parameters = default_parameters();
+
+  init(method);
 }
 //-----------------------------------------------------------------------------
 PETScKrylovSolver::PETScKrylovSolver(std::string method,
 				                             PETScUserPreconditioner& preconditioner)
-  : method(method), pc_dolfin(&preconditioner), preconditioner_set(false)
+  : pc_dolfin(&preconditioner), preconditioner_set(false)
 {
   // Set parameter values
   parameters = default_parameters();
+
+  init(method);
 }
 //-----------------------------------------------------------------------------
 PETScKrylovSolver::PETScKrylovSolver(boost::shared_ptr<KSP> _ksp)
-  : method("default"), pc_dolfin(0), _ksp(_ksp),
-    preconditioner_set(true)
+  : pc_dolfin(0), _ksp(_ksp), preconditioner_set(true)
 {
   // Set parameter values
   parameters = default_parameters();
@@ -101,29 +108,58 @@ PETScKrylovSolver::~PETScKrylovSolver()
   // Do nothing
 }
 //-----------------------------------------------------------------------------
+void PETScKrylovSolver::set_operator(const GenericMatrix& A)
+{
+  set_operators(A, A);
+}
+//-----------------------------------------------------------------------------
+void PETScKrylovSolver::set_operator(const PETScBaseMatrix& A)
+{
+  set_operators(A, A);
+}
+//-----------------------------------------------------------------------------
+void PETScKrylovSolver::set_operators(const GenericMatrix& A,
+                                      const GenericMatrix& P)
+{
+  set_operators(A.down_cast<PETScBaseMatrix>(), P.down_cast<PETScBaseMatrix>());
+}
+//-----------------------------------------------------------------------------
+void PETScKrylovSolver::set_operators(const PETScBaseMatrix& A,
+                                      const PETScBaseMatrix& P)
+{
+  this->A = reference_to_no_delete_pointer(A);
+  this->P = reference_to_no_delete_pointer(P);
+  assert(this->A);
+  assert(this->P);
+}
+//-----------------------------------------------------------------------------
+dolfin::uint PETScKrylovSolver::solve(GenericVector& x, const GenericVector& b)
+{
+  return solve(x.down_cast<PETScVector>(), b.down_cast<PETScVector>());
+}
+//-----------------------------------------------------------------------------
 dolfin::uint PETScKrylovSolver::solve(const GenericMatrix& A, GenericVector& x,
                                        const GenericVector& b)
 {
-  return solve(A.down_cast<PETScMatrix>(), x.down_cast<PETScVector>(),
+  return solve(A.down_cast<PETScBaseMatrix>(), x.down_cast<PETScVector>(),
                b.down_cast<PETScVector>());
 }
 //-----------------------------------------------------------------------------
-dolfin::uint PETScKrylovSolver::solve(const PETScMatrix& A, PETScVector& x,
-                                      const PETScVector& b)
+dolfin::uint PETScKrylovSolver::solve(PETScVector& x, const PETScVector& b)
 {
+  assert(A);
+  assert(_ksp);
+
   // Check dimensions
-  const uint M = A.size(0);
-  const uint N = A.size(1);
+  const uint N = A->size(1);
+  const uint M = A->size(0);
   if (N != b.size())
     error("Non-matching dimensions for linear system.");
 
   // Write a message
-  if (parameters["report"])
+  if (parameters["report"] && dolfin::MPI::process_number() == 0)
     info(PROGRESS, "Solving linear system of size %d x %d (PETSc Krylov solver).", M, N);
 
-  // Reinitialize KSP solver if necessary
-  init(M, N);
-
   // Reinitialize solution vector if necessary
   if (x.size() != M)
   {
@@ -131,30 +167,14 @@ dolfin::uint PETScKrylovSolver::solve(const PETScMatrix& A, PETScVector& x,
     x.zero();
   }
 
-  if (!_ksp)
-    _ksp.reset(new KSP, PETScKSPDeleter());
+  // Set some PETSc-specific options
+  set_petsc_options();
 
-  if (parameters["monitor_convergence"])
-    KSPMonitorSet(*_ksp, KSPMonitorTrueResidualNorm, 0, 0);
-
-  // Set tolerances
-  KSPSetTolerances(*_ksp,
-                   parameters["relative_tolerance"],
-                   parameters["absolute_tolerance"],
-                   parameters["divergence_limit"],
-                   parameters["maximum_iterations"]);
-
-  const std::string pc_side = parameters["preconditioner_side"];
-  if (pc_side == "left")
-    KSPSetPreconditionerSide(*_ksp, PC_LEFT);
-  else if (pc_side == "right")
-    KSPSetPreconditionerSide(*_ksp, PC_RIGHT);
-  else
-    error("Unknown preconditioning side for PETSc Krylov solver. Options are \"left\" or \"right\".");
-
-  KSPGMRESSetRestart(*_ksp, parameters["gmres_restart"]);
-  KSPSetOperators(*_ksp, *A.mat(), *A.mat(), SAME_NONZERO_PATTERN);
+  // Set operators
+  set_petsc_operators();
 
+  // FIXME: Improve check for re-setting preconditoner, e.g. if parameters change
+  // Set preconditioner if necessary
   if (preconditioner && !preconditioner_set)
   {
     preconditioner->set(*this);
@@ -162,6 +182,11 @@ dolfin::uint PETScKrylovSolver::solve(const PETScMatrix& A, PETScVector& x,
   }
 
   // Solve linear system
+  if (MPI::process_number() == 0)
+  {
+    info(PROGRESS, "PETSc Krylov solver starting to solve %i x %i system.",
+                    A->size(0), A->size(1));
+  }
   KSPSolve(*_ksp, *b.vec(), *x.vec());
 
   // Get the number of iterations
@@ -176,11 +201,12 @@ dolfin::uint PETScKrylovSolver::solve(const PETScMatrix& A, PETScVector& x,
     // Get solver residual norm
     double rnorm = 0.0;
     KSPGetResidualNorm(*_ksp, &rnorm);
+    const char *reason_str = KSPConvergedReasons[reason];
     bool error_on_nonconvergence = parameters["error_on_nonconvergence"];
     if (error_on_nonconvergence)
-      error("PETSc Krylov solver did not converge in %i iterations (PETSc reason %i, norm %e).", num_iterations, reason, rnorm);
+      error("PETSc Krylov solver did not converge in %i iterations (PETSc reason %s, norm %e).", num_iterations, reason_str, rnorm);
     else
-      warning("Krylov solver did not converge  in %i iterations (PETSc reason %i, norm %e).", num_iterations, reason, rnorm);
+      warning("Krylov solver did not converge in %i iterations (PETSc reason %s, norm %e).", num_iterations, reason_str, rnorm);
   }
 
   // Report results
@@ -189,6 +215,20 @@ dolfin::uint PETScKrylovSolver::solve(const PETScMatrix& A, PETScVector& x,
   return num_iterations;
 }
 //-----------------------------------------------------------------------------
+dolfin::uint PETScKrylovSolver::solve(const PETScBaseMatrix& A, PETScVector& x,
+                                      const PETScVector& b)
+{
+  // Check dimensions
+  const uint N = A.size(1);
+  if (N != b.size())
+    error("Non-matching dimensions for linear system.");
+
+  // Set operator
+  set_operator(A);
+
+  return solve(x, b);
+}
+//-----------------------------------------------------------------------------
 boost::shared_ptr<KSP> PETScKrylovSolver::ksp() const
 {
   return _ksp;
@@ -208,19 +248,8 @@ std::string PETScKrylovSolver::str(bool verbose) const
   return s.str();
 }
 //-----------------------------------------------------------------------------
-void PETScKrylovSolver::init(uint M, uint N)
+void PETScKrylovSolver::init(const std::string& method)
 {
-  // Check if we need to reinitialize
-  if (_ksp)
-  {
-    Mat Amat;
-    KSPGetOperators(*_ksp, &Amat, PETSC_NULL, PETSC_NULL);
-    uint _M(0), _N(0);
-    MatGetSize(Amat, (int*)&_M, (int*)&_N);
-    if (M == _M && N == _N)
-      return;
-  }
-
   // Check that nobody else shares this solver
   if (_ksp && !_ksp.unique())
     error("Cannot create new KSP Krylov solver. More than one object points to the underlying PETSc object.");
@@ -236,13 +265,53 @@ void PETScKrylovSolver::init(uint M, uint N)
 
   // Set some options
   KSPSetFromOptions(*_ksp);
-  KSPSetInitialGuessNonzero(*_ksp, PETSC_TRUE);
 
   // Set solver type
   if (method != "default")
     KSPSetType(*_ksp, methods.find(method)->second);
 }
 //-----------------------------------------------------------------------------
+void PETScKrylovSolver::set_petsc_operators()
+{
+  assert(A);
+  assert(P);
+
+  // Get some parameters
+  const bool reuse_precon = parameters("preconditioner")["reuse"];
+  const bool same_pattern = parameters("preconditioner")["same_nonzero_pattern"];
+
+  // Set operators with appropriate option
+  if (reuse_precon)
+    KSPSetOperators(*_ksp, *A->mat(), *P->mat(), SAME_PRECONDITIONER);
+  else if (same_pattern)
+    KSPSetOperators(*_ksp, *A->mat(), *P->mat(), SAME_NONZERO_PATTERN);
+  else
+    KSPSetOperators(*_ksp, *A->mat(), *P->mat(), DIFFERENT_NONZERO_PATTERN);
+}
+//-----------------------------------------------------------------------------
+void PETScKrylovSolver::set_petsc_options()
+{
+  // GMRES restart parameter
+  KSPGMRESSetRestart(*_ksp, parameters("gmres")["restart"]);
+
+  // Non-zero initial guess
+  const bool nonzero_guess = parameters["nonzero_initial_guess"];
+  if (nonzero_guess)
+    KSPSetInitialGuessNonzero(*_ksp, PETSC_TRUE);
+  else
+    KSPSetInitialGuessNonzero(*_ksp, PETSC_FALSE);
+
+  if (parameters["monitor_convergence"])
+    KSPMonitorSet(*_ksp, KSPMonitorTrueResidualNorm, 0, 0);
+
+  // Set tolerances
+  KSPSetTolerances(*_ksp,
+                   parameters["relative_tolerance"],
+                   parameters["absolute_tolerance"],
+                   parameters["divergence_limit"],
+                   parameters["maximum_iterations"]);
+}
+//-----------------------------------------------------------------------------
 void PETScKrylovSolver::write_report(int num_iterations,
                                      KSPConvergedReason reason)
 {
@@ -272,22 +341,37 @@ void PETScKrylovSolver::write_report(int num_iterations,
     PCGetType(sub_pc, &sub_pc_type);
   }
 
-  // Report number of iterations and solver type
-  if (reason >= 0)
-  {
-    info(PROGRESS, "PETSc Krylov solver (%s, %s) converged in %d iterations.",
-            ksp_type, pc_type, num_iterations);
-  }
-  else
-  {
-    info(PROGRESS, "PETSc Krylov solver (%s, %s) failed to converge in %d iterations.",
-            ksp_type, pc_type, num_iterations);
-  }
+  // FIXME: Get preconditioner description from PETScPreconditioner
 
-  if (pc_type_str == PCASM || pc_type_str == PCBJACOBI)
+  // Report number of iterations and solver type
+  if (MPI::process_number() == 0)
   {
-    info(PROGRESS, "PETSc Krylov solver preconditioner (%s) sub-methods: (%s, %s)",
-            pc_type, sub_ksp_type, sub_pc_type);
+    if (reason >= 0)
+    {
+      info(PROGRESS, "PETSc Krylov solver (%s, %s) converged in %d iterations.",
+              ksp_type, pc_type, num_iterations);
+    }
+    else
+    {
+      info(PROGRESS, "PETSc Krylov solver (%s, %s) failed to converge in %d iterations.",
+              ksp_type, pc_type, num_iterations);
+    }
+
+    if (pc_type_str == PCASM || pc_type_str == PCBJACOBI)
+    {
+      info(PROGRESS, "PETSc Krylov solver preconditioner (%s) sub-methods: (%s, %s)",
+              pc_type, sub_ksp_type, sub_pc_type);
+    }
+
+    #if PETSC_HAVE_HYPRE
+    if (pc_type_str == PCHYPRE)
+    {
+      const char* hypre_sub_type;
+      PCHYPREGetType(pc, &hypre_sub_type);
+
+      info(PROGRESS, "  Hypre preconditioner method: %s", hypre_sub_type);
+    }
+    #endif
   }
 }
 //-----------------------------------------------------------------------------
