@@ -28,9 +28,9 @@ namespace dolfin
   /// Forward declarations
   class GenericMatrix;
   class GenericVector;
+  class PETScBaseMatrix;
   class PETScMatrix;
   class PETScVector;
-  class PETScKrylovMatrix;
   class PETScPreconditioner;
   class PETScUserPreconditioner;
 
@@ -57,32 +57,53 @@ namespace dolfin
     /// Destructor
     ~PETScKrylovSolver();
 
+    /// Set operator (matrix)
+    void set_operator(const GenericMatrix& A);
+
+    /// Set operator (matrix)
+    void set_operator(const PETScBaseMatrix& A);
+
+    /// Set operator (matrix) and preconditioner matrix
+    void set_operators(const GenericMatrix& A, const GenericMatrix& P);
+
+    /// Set operator (matrix) and preconditioner matrix
+    void set_operators(const PETScBaseMatrix& A, const PETScBaseMatrix& P);
+
+    /// Solve linear system Ax = b and return number of iterations
+    uint solve(GenericVector& x, const GenericVector& b);
+
+    /// Solve linear system Ax = b and return number of iterations
+    uint solve(PETScVector& x, const PETScVector& b);
+
     /// Solve linear system Ax = b and return number of iterations
     uint solve(const GenericMatrix& A, GenericVector& x, const GenericVector& b);
 
     /// Solve linear system Ax = b and return number of iterations
-    uint solve(const PETScMatrix& A, PETScVector& x, const PETScVector& b);
+    uint solve(const PETScBaseMatrix& A, PETScVector& x, const PETScVector& b);
 
     /// Return informal string representation (pretty-print)
     std::string str(bool verbose) const;
 
-    /// Default parameter values
-    static Parameters default_parameters();
-
     /// Return PETSc KSP pointer
     boost::shared_ptr<KSP> ksp() const;
 
+    /// Default parameter values
+    static Parameters default_parameters();
+
   private:
 
     /// Initialize KSP solver
-    void init(uint M, uint N);
+    void init(const std::string& method);
+
+    // Set PETSc operators
+    void set_petsc_operators();
+
+    // Set options
+    void set_petsc_options();
 
     /// Report the number of iterations
     void write_report(int num_iterations, KSPConvergedReason reason);
 
-    /// Krylov method
-    std::string method;
-
     // Available solvers and preconditioners
     static const std::map<std::string, const KSPType> methods;
     static const std::map<std::string, const PCType> pc_methods;
@@ -96,6 +117,12 @@ namespace dolfin
     /// Preconditioner
     boost::shared_ptr<PETScPreconditioner> preconditioner;
 
+    /// Operator (the matrix)
+    boost::shared_ptr<const PETScBaseMatrix> A;
+
+    /// Matrix used to construct the preconditoner
+    boost::shared_ptr<const PETScBaseMatrix> P;
+
     bool preconditioner_set;
 
   };
