@@ -14,7 +14,8 @@
 #include <map>
 #include <boost/shared_ptr.hpp>
 #include <petscksp.h>
-#include "GenericLinearSolver.h"
+#include <petscpc.h>
+#include "GenericLUSolver.h"
 #include "PETScObject.h"
 
 namespace dolfin
@@ -30,16 +31,32 @@ namespace dolfin
   /// linear systems of the form Ax = b. It is a wrapper for the LU
   /// solver of PETSc.
 
-  class PETScLUSolver : public GenericLinearSolver, public PETScObject
+  class PETScLUSolver : public GenericLUSolver, public PETScObject
   {
   public:
 
     /// Constructor
     PETScLUSolver(std::string lu_package="default");
 
+    /// Constructor
+    PETScLUSolver(const GenericMatrix& A, std::string lu_package="default");
+
+    /// Constructor
+    PETScLUSolver(boost::shared_ptr<const PETScMatrix> A,
+                  std::string lu_package="default");
+
     /// Destructor
     ~PETScLUSolver();
 
+    /// Set operator (matrix)
+    void set_operator(const GenericMatrix& A);
+
+    /// Set operator (matrix)
+    void set_operator(const PETScMatrix& A);
+
+    /// Solve linear system Ax = b
+    uint solve(GenericVector& x, const GenericVector& b);
+
     /// Solve linear system Ax = b
     uint solve(const GenericMatrix& A, GenericVector& x, const GenericVector& b);
 
@@ -49,22 +66,35 @@ namespace dolfin
     /// Return informal string representation (pretty-print)
     std::string str(bool verbose) const;
 
+    /// Return PETSc KSP pointer
+    boost::shared_ptr<KSP> ksp() const;
+
     /// Default parameter values
     static Parameters default_parameters();
 
   private:
 
-    /// LU method
-    std::string lu_package;
-
     // Available LU solvers
     static const std::map<std::string, const MatSolverPackage> lu_packages;
 
+    // Select LU solver type
+    const MatSolverPackage select_solver(std::string& lu_package) const;
+
     // Initialise solver
-    void init();
+    void init_solver(std::string& lu_package);
+
+    // Set PETSc operators
+    void set_petsc_operators();
+
+    // Print pre-solve report
+    void pre_report(const PETScMatrix& A) const;
 
     /// PETSc solver pointer
-    boost::shared_ptr<KSP> ksp;
+    boost::shared_ptr<KSP> _ksp;
+
+    // Operator (the matrix)
+    boost::shared_ptr<const PETScMatrix> A;
+
   };
 
 }
