@@ -18,9 +18,8 @@
 #include <string>
 #include <boost/shared_ptr.hpp>
 #include <petscmat.h>
-
-#include "PETScObject.h"
 #include "GenericMatrix.h"
+#include "PETScBaseMatrix.h"
 
 namespace dolfin
 {
@@ -34,21 +33,14 @@ namespace dolfin
   /// The interface is intentionally simple. For advanced usage,
   /// access the PETSc Mat pointer using the function mat() and
   /// use the standard PETSc interface.
-  ///
-  /// std::string type is the type of PETSc matrix. Options are "default",
-  /// "spooles", "superlu" and "umfpack". Note: Setting the matrix type may
-  /// not be necessary in the future with PETSc version 3.
 
-  class PETScMatrix : public GenericMatrix, public PETScObject
+  class PETScMatrix : public GenericMatrix, public PETScBaseMatrix
   {
   public:
 
     /// Create empty matrix
     PETScMatrix();
 
-    /// Create M x N matrix
-    PETScMatrix(uint M, uint N);
-
     /// Copy constructor
     PETScMatrix(const PETScMatrix& A);
 
@@ -67,7 +59,11 @@ namespace dolfin
     virtual PETScMatrix* copy() const;
 
     /// Return size of given dimension
-    virtual uint size(uint dim) const;
+    uint size(uint dim) const { return PETScBaseMatrix::size(dim); }
+
+    /// Return local ownership range
+    std::pair<uint, uint> local_range(uint dim) const
+    { return PETScBaseMatrix::local_range(dim); };
 
     /// Set all entries to zero and keep any sparse structure
     virtual void zero();
@@ -83,6 +79,12 @@ namespace dolfin
     /// Resize matrix to M x N
     virtual void resize(uint M, uint N);
 
+    /// Resize vector y such that is it compatible with matrix for
+    /// multuplication Ax = b (dim = 0 -> b, dim = 1 -> x) In parallel
+    /// case, size and layout are important.
+    void resize(GenericVector& y, uint dim) const
+    { PETScBaseMatrix::resize(y, dim); }
+
     /// Get block of values
     virtual void get(double* block, uint m, const uint* rows, uint n, const uint* cols) const;
 
@@ -129,23 +131,20 @@ namespace dolfin
 
     //--- Special PETScFunctions ---
 
-    /// Return PETSc Mat pointer
-    boost::shared_ptr<Mat> mat() const;
-
     /// Return norm of matrix
     double norm(std::string norm_type) const;
 
     /// Assignment operator
     const PETScMatrix& operator= (const PETScMatrix& A);
 
+    /// Dump matrix to PETSc binary format
+    void binary_dump(std::string file_name) const;
+
   private:
 
     // PETSc norm types
     static const std::map<std::string, NormType> norm_types;
 
-    // PETSc Mat pointer
-    boost::shared_ptr<Mat> A;
-
   };
 
 }
