@@ -2,16 +2,16 @@
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2008-01-07
-// Last changed:
+// Last changed: 2011-01-24
 
 #ifndef __PETSC_OBJECT_H
 #define __PETSC_OBJECT_H
 
-#include <dolfin/main/SubSystemsManager.h>
+#include <dolfin/common/SubSystemsManager.h>
 
 namespace dolfin
 {
-  /// This class calls SubSystemsManger to initialise PETSc.
+  /// This class calls SubSystemsManager to initialise PETSc.
   ///
   /// All PETSc objects must be derived from this class.
 
@@ -26,4 +26,3 @@ namespace dolfin
 }
 
 #endif
-
