@@ -4,11 +4,12 @@
 // Modified by Anders Logg, 2010.
 //
 // First added:  2010-02-25
-// Last changed: 2010-03-02
+// Last changed: 2010-08-25
 
 #ifdef HAS_PETSC
 
 #include <boost/assign/list_of.hpp>
+#include <boost/lexical_cast.hpp>
 #include <petscksp.h>
 #include <petscmat.h>
 #include <dolfin/la/KrylovSolver.h>
@@ -29,15 +30,28 @@ const std::map<std::string, const PCType> PETScPreconditioner::methods
                               ("jacobi",           PCJACOBI)
                               ("sor",              PCSOR)
                               ("icc",              PCICC)
+                              ("hypre_amg",        PCHYPRE)
+                              ("hypre_euclid",     PCHYPRE)
+                              ("hypre_parasails",  PCHYPRE)
                               ("amg_hypre",        PCHYPRE)
                               ("amg_ml",           PCML);
 //-----------------------------------------------------------------------------
 Parameters PETScPreconditioner::default_parameters()
 {
-  Parameters p(KrylovSolver::default_parameters());
+  Parameters p(KrylovSolver::default_parameters()("preconditioner"));
   p.rename("petsc_preconditioner");
-  p.add("ilu_fill_level", 0);
-  p.add("schwarz_overlap", 1);
+
+  // Hypre/parasails parameters
+  Parameters p_parasails("parasails");
+  p_parasails.add("threshold", 0.1);
+  p_parasails.add("levels", 1);
+
+  // Hypre package parameters
+  Parameters p_hypre("hypre");
+  p_hypre.add(p_parasails);
+
+  p.add(p_hypre);
+
   return p;
 }
 //-----------------------------------------------------------------------------
@@ -65,18 +79,41 @@ void PETScPreconditioner::set(PETScKrylovSolver& solver) const
   KSPGetPC(*(solver.ksp()), &pc);
 
   // Treat special cases  first
-  if (type == "amg_hypre")
+  if (type.find("hypre") != std::string::npos)
   {
     #if PETSC_HAVE_HYPRE
     PCSetType(pc, PCHYPRE);
-    PCHYPRESetType(pc, "boomeramg");
-    PCSetFromOptions(pc);
+
+    #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR == 1
+    PCFactorSetShiftType(pc, MAT_SHIFT_NONZERO);
+    PCFactorSetShiftAmount(pc, PETSC_DECIDE);
+    #else
+    PCFactorSetShiftNonzero(pc, PETSC_DECIDE);
+    #endif
+
+    if (type == "amg_hypre" || type == "hypre_amg")
+      PCHYPRESetType(pc, "boomeramg");
+    else if (type == "hypre_parasails")
+    {
+      PCHYPRESetType(pc, "parasails");
+      const double thresh = parameters("hypre")("parasails")["threshold"];
+      const int levels = parameters("hypre")("parasails")["levels"];
+
+      PetscOptionsSetValue("-pc_hypre_parasails_thresh", boost::lexical_cast<std::string>(thresh).c_str());
+      PetscOptionsSetValue("-pc_hypre_parasails_nlevels", boost::lexical_cast<std::string>(levels).c_str());
+    }
+    else if (type == "hypre_euclid")
+    {
+      PCHYPRESetType(pc, "euclid");
+    }
+    else
+      error("Requested Hypre preconditioner unknown (Note: pilut is not supported).");
+
     #else
     warning("PETSc has not been compiled with the HYPRE library for "
             "algebraic multigrid. Default PETSc solver will be used. "
             "For performance, installation of HYPRE is recommended.");
     #endif
-    return;
   }
   else if (type == "amg_ml")
   {
@@ -93,13 +130,12 @@ void PETScPreconditioner::set(PETScKrylovSolver& solver) const
             "algerbraic multigrid. Default PETSc solver will be used. "
             "For performance, installation of ML is recommended.");
     #endif
-    return;
   }
   else if (type == "additive_schwarz")
   {
     // Select method and and overlap
     PCSetType(pc, methods.find("additive_schwarz")->second);
-    PCASMSetOverlap(pc, parameters["schwarz_overlap"]);
+    PCASMSetOverlap(pc, parameters("schwarz")["overlap"]);
 
     // Make sure the data structures have been constructed
     KSPSetUp(*solver.ksp());
@@ -116,7 +152,7 @@ void PETScPreconditioner::set(PETScKrylovSolver& solver) const
       //PCFactorSetMatSolverPackage(sub_pc, MAT_SOLVER_UMFPACK);
       //PCSetType(sub_pc, PCILU);
       //KSPSetType(sub_ksps[i], KSPGMRES);
-      PCFactorSetLevels(sub_pc, parameters["ilu_fill_level"]);
+      PCFactorSetLevels(sub_pc, parameters("ilu")["fill_level"]);
       //PCFactorSetLevels(sub_pc, 4);
       //PCView(sub_pc, PETSC_VIEWER_STDOUT_WORLD);
     }
@@ -135,12 +171,15 @@ void PETScPreconditioner::set(PETScKrylovSolver& solver) const
     #else
     PCFactorSetShiftNonzero(pc, parameters["shift_nonzero"]);
     #endif
-    PCFactorSetLevels(pc, parameters["ilu_fill_level"]);
   }
 
+  PCFactorSetLevels(pc, parameters("ilu")["fill_level"]);
+
   // Make sure options are set
   PCSetFromOptions(pc);
-  //PCView(pc, PETSC_VIEWER_STDOUT_WORLD);
+
+  if (parameters["report"])
+    PCView(pc, PETSC_VIEWER_STDOUT_WORLD);
 }
 //-----------------------------------------------------------------------------
 std::string PETScPreconditioner::str(bool verbose) const
