@@ -1,11 +1,11 @@
 // Copyright (C) 2004-2007 Johan Hoffman, Johan Jansson and Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
-// Modified by Garth N. Wells 2005-2009.
+// Modified by Garth N. Wells 2005-2010.
 // Modified by Martin Sandve Alnes 2008
 //
 // First added:  2004
-// Last changed: 2009-09-07
+// Last changed: 2011-01-14
 
 #ifdef HAS_PETSC
 
@@ -19,8 +19,9 @@
 #include "PETScVector.h"
 #include "uBLASVector.h"
 #include "PETScFactory.h"
-#include <dolfin/main/MPI.h>
+#include <dolfin/common/MPI.h>
 
+/*
 namespace dolfin
 {
   class PETScVectorDeleter
@@ -28,13 +29,13 @@ namespace dolfin
   public:
     void operator() (Vec* x)
     {
-      if (x)
+      if (*x)
         VecDestroy(*x);
       delete x;
     }
   };
 }
-
+*/
 using namespace dolfin;
 
 const std::map<std::string, NormType> PETScVector::norm_types
@@ -45,32 +46,51 @@ const std::map<std::string, NormType> PETScVector::norm_types
 //-----------------------------------------------------------------------------
 PETScVector::PETScVector(std::string type)
 {
+  if (type != "global" && type != "local")
+    error("PETSc vector type unkown.");
+
+  // Empty ghost indices vector
+  const std::vector<uint> ghost_indices;
+
+  // Trivial range
+  const std::pair<uint, uint> range(0, 0);
+
   if (type == "global" && dolfin::MPI::num_processes() > 1)
-    init(0, 0, "mpi");
+    init(range, ghost_indices, true);
   else
-    init(0, 0, "sequential");
+    init(range, ghost_indices, false);
 }
 //-----------------------------------------------------------------------------
 PETScVector::PETScVector(uint N, std::string type)
 {
+  // Empty ghost indices vector
+  const std::vector<uint> ghost_indices;
+
   if (type == "global")
   {
-    // Get local range
+    // Compute a local range
     const std::pair<uint, uint> range = MPI::local_range(N);
+
     if (range.first == 0 && range.second == N)
-      init(N, 0, "sequential");
+      init(range, ghost_indices, false);
     else
-    {
-      const uint n = range.second - range.first;
-      init(N, n, "mpi");
-    }
+      init(range, ghost_indices, true);
   }
   else if (type == "local")
-    init(0, 0, "sequential");
+  {
+    const std::pair<uint, uint> range(0, N);
+    init(range, ghost_indices, false);
+  }
   else
     error("PETScVector type not known.");
 }
 //-----------------------------------------------------------------------------
+PETScVector::PETScVector(const GenericSparsityPattern& sparsity_pattern)
+{
+  std::vector<uint> ghost_indices;
+  resize(sparsity_pattern.local_range(0), ghost_indices);
+}
+//-----------------------------------------------------------------------------
 PETScVector::PETScVector(boost::shared_ptr<Vec> x): x(x)
 {
   // Do nothing
@@ -94,24 +114,47 @@ void PETScVector::resize(uint N)
   if (!x)
     error("PETSc vector has not been initialised. Cannot call PETScVector::resize.");
 
-  // Figure out vector type
-  std::string type;
-  uint n = 0;
-  const VecType petsc_type;
-  VecGetType(*x, &petsc_type);
-  if (strcmp(petsc_type, VECSEQ) == 0)
-    type = "sequential";
-  else if (strcmp(petsc_type, VECMPI) == 0)
+  // Get vector type
+  bool _distributed = distributed();
+
+  // Create vector
+  if (_distributed)
   {
     const std::pair<uint, uint> range = MPI::local_range(N);
-    n = range.second - range.first;
-    type = "mpi";
+    resize(range);
   }
   else
-    error("Unknown PETSc vector type.");
+  {
+    const std::pair<uint, uint> range(0, N);
+    resize(range);
+  }
+}
+//-----------------------------------------------------------------------------
+void PETScVector::resize(std::pair<uint, uint> range)
+{
+  // Empty ghost indices vector
+  std::vector<uint> ghost_indices;
 
-  // Initialise vector
-  init(N, n, type);
+  resize(range, ghost_indices);
+}
+//-----------------------------------------------------------------------------
+void PETScVector::resize(std::pair<uint, uint> range,
+                         const std::vector<uint>& ghost_indices)
+{
+  // Get local size
+  assert(range.second - range.first >= 0);
+
+  // FIXME: Can this check be made robust? Need to avoid parallel lock-up.
+  //        Cannot just check size because range may change.
+  // Check if resizing is required
+  //if (x && (this->local_range().first == range.first && this->local_range().second == range.second))
+  //  return;
+
+  // Save type
+  bool _distributed = distributed();
+
+  // Re-initialise vector
+  init(range, ghost_indices, _distributed);
 }
 //-----------------------------------------------------------------------------
 PETScVector* PETScVector::copy() const
@@ -123,12 +166,12 @@ PETScVector* PETScVector::copy() const
 void PETScVector::get_local(Array<double>& values) const
 {
   assert(x);
-  if (size() == 0)
-    return;
-
   const uint n0 = local_range().first;
   const uint local_size = local_range().second - local_range().first;
-  assert(values.size() >= local_size);
+  assert(values.size() == local_size);
+
+  if (local_size == 0)
+    return;
 
   std::vector<int> rows(local_size);
   for (uint i = 0; i < local_size; ++i)
@@ -140,12 +183,15 @@ void PETScVector::get_local(Array<double>& values) const
 void PETScVector::set_local(const Array<double>& values)
 {
   assert(x);
-  assert(values.size() == size());
-  if (size() == 0)
-    return;
-
   const uint n0 = local_range().first;
   const uint local_size = local_range().second - local_range().first;
+  if (values.size() != local_size)
+    error("PETScVector::set_local: length of values array is not equal to local vector size.");
+
+  if (local_size == 0)
+    return;
+
+  // Build array of global indices
   std::vector<int> rows(local_size);
   for (uint i = 0; i < local_size; ++i)
     rows[i] = i + n0;
@@ -156,12 +202,15 @@ void PETScVector::set_local(const Array<double>& values)
 void PETScVector::add_local(const Array<double>& values)
 {
   assert(x);
-  assert(values.size() == size());
-  if (size() == 0)
-    return;
-
   const uint n0 = local_range().first;
   const uint local_size = local_range().second - local_range().first;
+  if (values.size() != local_size)
+    error("PETScVector::add_local: length of values array is not equal to local vector size.");
+
+  if (local_size == 0)
+    return;
+
+  // Build array of global indices
   std::vector<int> rows(local_size);
   for (uint i = 0; i < local_size; ++i)
     rows[i] = i + n0;
@@ -169,70 +218,72 @@ void PETScVector::add_local(const Array<double>& values)
   VecSetValues(*x, local_size, &rows[0], values.data().get(), ADD_VALUES);
 }
 //-----------------------------------------------------------------------------
-void PETScVector::get(double* block, uint m, const uint* rows) const
-{
-  assert(x);
-
-  // If vector is local, just get the values. For distributed vectors, perform
-  // first a gather into a local vector
-  if (local_range().first == 0 && local_range().second == size())
-    get_local(block, m, rows);
-  else
-  {
-    PETScVector y("local");
-    std::vector<uint> local_indices(m);
-    for (uint i = 0; i < m; ++i)
-      local_indices[i] = i;
-
-    // Gather values into y
-    const Array<uint> _rows(m, const_cast<uint*>(rows));
-    gather(y, _rows);
-
-    // Get entries of y
-    y.get_local(block, m, &local_indices[0]);
-  }
-}
-//-----------------------------------------------------------------------------
 void PETScVector::get_local(double* block, uint m, const uint* rows) const
 {
   assert(x);
   int _m = static_cast<int>(m);
-  const int* _rows = reinterpret_cast<int*>(const_cast<uint*>(rows));
+  const int* _rows = reinterpret_cast<const int*>(rows);
+
+  // Handle case that m = 0 (VecGetValues is collective -> must be called be
+  //                         all processes)
   if (m == 0)
   {
     _rows = &_m;
     double tmp = 0.0;
     block = &tmp;
   }
-  VecGetValues(*x, _m, _rows, block);
+
+  // Use VecGetValues if no ghost points, otherwise check for ghost values
+  if (ghost_global_to_local.size() == 0 || m == 0)
+  {
+    VecGetValues(*x, _m, _rows, block);
+  }
+  else
+  {
+    assert(x_ghosted);
+
+    // Get local range
+    const uint n0 = local_range().first;
+    const uint n1 = local_range().second;
+    const uint local_size = n1 - n0;
+
+    // Build list of rows, and get from ghosted vector
+    std::vector<int> local_rows(m);
+    for (uint i = 0; i < m; ++i)
+    {
+      if (rows[i] >= n0 && rows[i] < n1)
+        local_rows[i] = rows[i] - n0;
+      else
+      {
+        boost::unordered_map<uint, uint>::const_iterator local_index = ghost_global_to_local.find(rows[i]);
+        assert(local_index != ghost_global_to_local.end());
+        local_rows[i] = local_index->second + local_size;
+      }
+    }
+
+    // Pick values from ghosted vector
+    VecGetValues(*x_ghosted, _m, &local_rows[0], block);
+  }
 }
 //-----------------------------------------------------------------------------
 void PETScVector::set(const double* block, uint m, const uint* rows)
 {
   assert(x);
-  int _m =  static_cast<int>(m);
-  const int* _rows = reinterpret_cast<int*>(const_cast<uint*>(rows));
+
   if (m == 0)
-  {
-    _rows = &_m;
-    double tmp = 0.0;
-    block = &tmp;
-  }
-  VecSetValues(*x, _m, _rows, block, INSERT_VALUES);
+    return;
+
+  VecSetValues(*x, m, reinterpret_cast<const int*>(rows), block, INSERT_VALUES);
 }
 //-----------------------------------------------------------------------------
 void PETScVector::add(const double* block, uint m, const uint* rows)
 {
   assert(x);
-  int _m =  static_cast<int>(m);
-  const int* _rows = reinterpret_cast<int*>(const_cast<uint*>(rows));
+
   if (m == 0)
-  {
-    _rows = &_m;
-    double tmp = 0;
-    block = &tmp;
-  }
-  VecSetValues(*x, _m, _rows, block, ADD_VALUES);
+    return;
+
+  VecSetValues(*x, m, reinterpret_cast<const int*>(rows), block, ADD_VALUES);
 }
 //-----------------------------------------------------------------------------
 void PETScVector::apply(std::string mode)
@@ -257,6 +308,14 @@ dolfin::uint PETScVector::size() const
   return static_cast<uint>(n);
 }
 //-----------------------------------------------------------------------------
+dolfin::uint PETScVector::local_size() const
+{
+  int n = 0;
+  if (x)
+    VecGetLocalSize(*x, &n);
+  return static_cast<uint>(n);
+}
+//-----------------------------------------------------------------------------
 std::pair<dolfin::uint, dolfin::uint> PETScVector::local_range() const
 {
   std::pair<uint, uint> range;
@@ -265,6 +324,14 @@ std::pair<dolfin::uint, dolfin::uint> PETScVector::local_range() const
   return range;
 }
 //-----------------------------------------------------------------------------
+bool PETScVector::owns_index(uint i) const
+{
+  if (i >= local_range().first && i < local_range().second)
+    return true;
+  else
+    return false;
+}
+//-----------------------------------------------------------------------------
 const GenericVector& PETScVector::operator= (const GenericVector& v)
 {
   *this = v.down_cast<PETScVector>();
@@ -278,13 +345,21 @@ const PETScVector& PETScVector::operator= (const PETScVector& v)
   // Check for self-assignment
   if (this != &v)
   {
-    x.reset(new Vec, PETScVectorDeleter());
+    x.reset(new Vec(0), PETScVectorDeleter());
 
     // Create new vector
     VecDuplicate(*(v.x), x.get());
 
     // Copy data
     VecCopy(*(v.x), *x);
+
+    // Copy ghost data
+    this->ghost_global_to_local = v.ghost_global_to_local;
+
+    // Create ghost view
+    this->x_ghosted.reset(new Vec(0), PETScVectorDeleter());
+    if (ghost_global_to_local.size() > 0)
+      VecGhostGetLocalForm(*x, x_ghosted.get());
   }
   return *this;
 }
@@ -296,6 +371,35 @@ const PETScVector& PETScVector::operator= (double a)
   return *this;
 }
 //-----------------------------------------------------------------------------
+void PETScVector::update_ghost_values()
+{
+  if (ghost_global_to_local.size() > 0)
+  {
+    VecGhostUpdateBegin(*x, INSERT_VALUES, SCATTER_FORWARD);
+    VecGhostUpdateEnd(*x, INSERT_VALUES, SCATTER_FORWARD);
+  }
+
+  /*
+  std::vector<uint> gdof;
+  std::vector<uint> ldof;
+  for (std::map<uint, uint>::const_iterator ghost = ghost_global_to_local.begin();
+           ghost != ghost_global_to_local.end(); ++ghost)
+  {
+    gdof.push_back(ghost->first);
+    ldof.push_back(ghost->second);
+  }
+
+  std::vector<double> gvals(gdof.size());
+  get_local(&gvals[0], gdof.size(), &gdof[0]);
+  if (MPI::process_number() == 0)
+  {
+    cout << "Local   " << "Global    " << "value" << endl;
+    for (uint i = 0; i < gdof.size(); ++i)
+      cout << ldof[i] << "   " << gdof[i] << "   " << gvals[i] << endl;
+  }
+  */
+}
+//-----------------------------------------------------------------------------
 const PETScVector& PETScVector::operator+= (const GenericVector& x)
 {
   this->axpy(1.0, x);
@@ -343,11 +447,11 @@ double PETScVector::inner(const GenericVector& y) const
 {
   assert(x);
 
-  const PETScVector& v = y.down_cast<PETScVector>();
-  assert(v.x);
+  const PETScVector& _y = y.down_cast<PETScVector>();
+  assert(_y.x);
 
   double a;
-  VecDot(*(v.x), *x, &a);
+  VecDot(*(_y.x), *x, &a);
   return a;
 }
 //-----------------------------------------------------------------------------
@@ -355,13 +459,19 @@ void PETScVector::axpy(double a, const GenericVector& y)
 {
   assert(x);
 
-  const PETScVector& v = y.down_cast<PETScVector>();
-  assert(v.x);
+  const PETScVector& _y = y.down_cast<PETScVector>();
+  assert(_y.x);
 
-  if (size() != v.size())
+  if (size() != _y.size())
     error("The vectors must be of the same size.");
 
-  VecAXPY(*x, a, *(v.x));
+  VecAXPY(*x, a, *(_y.x));
+}
+//-----------------------------------------------------------------------------
+void PETScVector::abs()
+{
+  assert(x);
+  VecAbs(*x);
 }
 //-----------------------------------------------------------------------------
 double PETScVector::norm(std::string norm_type) const
@@ -454,7 +564,7 @@ double PETScVector::sum(const Array<uint>& rows) const
   get_local(&local_values[0], local_rows.size(), &local_rows.set()[0]);
 
   // Compute local sum
-  double local_sum = std::accumulate(local_values.begin(), local_values.end(), 0.0);
+  const double local_sum = std::accumulate(local_values.begin(), local_values.end(), 0.0);
 
   return MPI::sum(local_sum);
 }
@@ -465,7 +575,7 @@ std::string PETScVector::str(bool verbose) const
 
   if (verbose)
   {
-    warning("Verbose output for PETScVector not implemented, calling PETSc VecView directly.");
+    //warning("Verbose output for PETScVector not implemented, calling PETSc VecView directly.");
 
     // Get vector type
     const VecType petsc_type;
@@ -495,22 +605,25 @@ void PETScVector::gather(GenericVector& y, const Array<uint>& indices) const
   if (strcmp(petsc_type, VECSEQ) != 0)
     error("PETScVector::gather can only gather into local vectors");
 
-  // Prepare data for index sets
-  const int* global_indices = reinterpret_cast<int*>(const_cast<uint*>(indices.data().get()));
+  // Prepare data for index sets (global indices)
+  const int* global_indices = reinterpret_cast<const int*>(indices.data().get());
+
+  // Prepare data for index sets (local indices)
   const int n = indices.size();
-  Array<int> local_indices(n);
-  for (int i = 0; i < n; ++i)
-    local_indices[i] = i;
 
   // PETSc will bail out if it receives a NULL pointer even though m == 0.
-  // Can't return from function as this will cause a lock up in parallel
+  // Can't return from function since function calls are collective.
   if (n == 0)
     global_indices = &n;
 
-  // Create index sets
+  // Create local index sets
   IS from, to;
+  #if (PETSC_VERSION_RELEASE == 0)
+  ISCreateGeneral(PETSC_COMM_SELF, n, global_indices, PETSC_COPY_VALUES, &from);
+  #else
   ISCreateGeneral(PETSC_COMM_SELF, n, global_indices,    &from);
-  ISCreateGeneral(PETSC_COMM_SELF, n, local_indices.data().get(), &to);
+  #endif
+  ISCreateStride(PETSC_COMM_SELF, n, 0 , 1, &to);
 
   // Resize vector if required
   y.resize(n);
@@ -527,23 +640,78 @@ void PETScVector::gather(GenericVector& y, const Array<uint>& indices) const
   VecScatterDestroy(scatter);
 }
 //-----------------------------------------------------------------------------
-void PETScVector::init(uint N, uint n, std::string type)
+void PETScVector::gather(Array<double>& x, const Array<uint>& indices) const
+{
+  x.resize(indices.size());
+  PETScVector y("local");
+  gather(y, indices);
+  assert(y.local_size() == x.size());
+
+  y.get_local(x);
+
+  double sum = 0.0;
+  for (uint i = 0; i < x.size(); ++i)
+    sum += x[i]*x[i];
+}
+//-----------------------------------------------------------------------------
+bool PETScVector::distributed() const
+{
+  assert(x);
+
+  // Get type
+  const VecType petsc_type;
+  VecGetType(*x, &petsc_type);
+
+  // Return type string
+  if (strcmp(petsc_type, VECMPI) == 0)
+    return true;
+  else if (strcmp(petsc_type, VECSEQ) == 0)
+    return false;
+  else
+  {
+    error("Unknown PETSc vector type.");
+    return false;
+  }
+}
+//-----------------------------------------------------------------------------
+void PETScVector::init(std::pair<uint, uint> range,
+                       const std::vector<uint>& ghost_indices, bool distributed)
 {
   // Create vector
   if (x && !x.unique())
     error("Cannot init/resize PETScVector. More than one object points to the underlying PETSc object.");
-  x.reset(new Vec, PETScVectorDeleter());
+  x.reset(new Vec(0), PETScVectorDeleter());
+
+  const uint local_size = range.second - range.first;
+  assert(range.second - range.first >= 0);
 
   // Initialize vector, either default or MPI vector
-  if (type == "sequential")
+  if (!distributed)
   {
-    VecCreateSeq(PETSC_COMM_SELF, N, x.get());
+    VecCreateSeq(PETSC_COMM_SELF, local_size, x.get());
     VecSetFromOptions(*x);
   }
-  else if (type == "mpi")
-    VecCreateMPI(PETSC_COMM_WORLD, n, N, x.get());
   else
-    error("Unknown vector type in PETScVector::init.");
+  {
+    // Clear ghost indices map
+    ghost_global_to_local.clear();
+
+    if (ghost_indices.size() > 0)
+    {
+      VecCreateGhost(PETSC_COMM_WORLD, local_size, PETSC_DECIDE, ghost_indices.size(),
+                     reinterpret_cast<const int*>(&ghost_indices[0]), x.get());
+
+      // Build global-to-local map for ghost indices
+      for (uint i = 0; i < ghost_indices.size(); ++i)
+        ghost_global_to_local.insert(std::pair<uint, uint>(ghost_indices[i], i));
+
+      // Create ghost view
+      x_ghosted.reset(new Vec(0), PETScVectorDeleter());
+      VecGhostGetLocalForm(*x, x_ghosted.get());
+    }
+    else
+      VecCreateMPI(PETSC_COMM_WORLD, local_size, PETSC_DECIDE, x.get());
+  }
 }
 //-----------------------------------------------------------------------------
 boost::shared_ptr<Vec> PETScVector::vec() const
@@ -551,10 +719,16 @@ boost::shared_ptr<Vec> PETScVector::vec() const
   return x;
 }
 //-----------------------------------------------------------------------------
+void PETScVector::reset()
+{
+  x.reset();
+  x_ghosted.reset();
+  ghost_global_to_local.clear();
+}
+//-----------------------------------------------------------------------------
 LinearAlgebraFactory& PETScVector::factory() const
 {
   return PETScFactory::instance();
 }
 //-----------------------------------------------------------------------------
-
 #endif
