@@ -1,13 +1,13 @@
-// Copyright (C) 2004-2008 Johan Hoffman, Johan Jansson and Anders Logg.
+// Copyright (C) 2004-2010 Johan Hoffman, Johan Jansson and Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
-// Modified by Garth N. Wells, 2005-2009.
+// Modified by Garth N. Wells, 2005-2010.
 // Modified by Kent-Andre Mardal, 2008.
 // Modified by Ola Skavhaug, 2008.
 // Modified by Martin Alnæs, 2008.
 //
 // First added:  2004-01-01
-// Last changed: 2009-09-08
+// Last changed: 2011-01-14
 
 #ifndef __PETSC_VECTOR_H
 #define __PETSC_VECTOR_H
@@ -17,14 +17,30 @@
 #include <map>
 #include <string>
 #include <boost/shared_ptr.hpp>
+#include <boost/unordered_map.hpp>
+
 #include <petscvec.h>
 
 #include "PETScObject.h"
 #include "GenericVector.h"
 
+#include <dolfin/log/dolfin_log.h>
+
 namespace dolfin
 {
 
+  class PETScVectorDeleter
+  {
+  public:
+    void operator() (Vec* x)
+    {
+      if (*x)
+        VecDestroy(*x);
+      delete x;
+    }
+  };
+
+  class GenericSparsityPattern;
   template<class T> class Array;
 
   /// This class provides a simple vector class based on PETSc.
@@ -45,6 +61,9 @@ namespace dolfin
     /// Create vector of size N
     PETScVector(uint N, std::string type="global");
 
+    /// Create vector
+    PETScVector(const GenericSparsityPattern& sparsity_pattern);
+
     /// Copy constructor
     PETScVector(const PETScVector& x);
 
@@ -70,17 +89,27 @@ namespace dolfin
 
     //--- Implementation of the GenericVector interface ---
 
-    /// Resize vector ro size N
+    /// Resize vector to global size N
     virtual void resize(uint N);
 
+    /// Resize vector with given ownership range
+    virtual void resize(std::pair<uint, uint> range);
+
+    /// Resize vector with given ownership range and with ghost values
+    virtual void resize(std::pair<uint, uint> range,
+                        const std::vector<uint>& ghost_indices);
+
     /// Return size of vector
     virtual uint size() const;
 
+    /// Return local size of vector
+    virtual uint local_size() const;
+
     /// Return ownership range of a vector
     virtual std::pair<uint, uint> local_range() const;
 
-    /// Get block of values (values may live on any process)
-    virtual void get(double* block, uint m, const uint* rows) const;
+    /// Determine whether global vector index is owned by this process
+    virtual bool owns_index(uint i) const;
 
     /// Get block of values (values must all live on the local process)
     virtual void get_local(double* block, uint m, const uint* rows) const;
@@ -103,6 +132,9 @@ namespace dolfin
     /// Add multiple of given vector (AXPY operation)
     virtual void axpy(double a, const GenericVector& x);
 
+    /// Replace all entries in the vector by their absolute values
+    virtual void abs();
+
     /// Return inner product with given vector
     virtual double inner(const GenericVector& v) const;
 
@@ -142,8 +174,13 @@ namespace dolfin
     /// Assignment operator
     virtual const PETScVector& operator= (double a);
 
+    virtual void update_ghost_values();
+
     //--- Special functions ---
 
+    /// Reset data and PETSc vector object
+    void reset();
+
     /// Return linear algebra backend factory
     virtual LinearAlgebraFactory& factory() const;
 
@@ -155,22 +192,33 @@ namespace dolfin
     /// Assignment operator
     const PETScVector& operator= (const PETScVector& x);
 
-    /// Gather vector entries into a local vector. If local_indices is
-    /// 0, then a local index array is created such that the order of
-    /// the values in the return array is the same as the order in
-    /// global_indices.
+    /// Gather vector entries into a local vector
     virtual void gather(GenericVector& y, const Array<uint>& indices) const;
 
+    /// Gather entries into Array x
+    virtual void gather(Array<double>& x, const Array<uint>& indices) const;
+
+    // Test vector type (distributed/local)
+    bool distributed() const;
+
+    friend class PETScBaseMatrix;
     friend class PETScMatrix;
 
   private:
 
     // Initialise PETSc vector
-    void init(uint N, uint n, std::string type);
+    void init(std::pair<uint, uint> range, const std::vector<uint>& ghost_indices,
+              bool distributed);
 
     // PETSc Vec pointer
     boost::shared_ptr<Vec> x;
 
+    // PETSc Vec pointer (local ghosted)
+    mutable boost::shared_ptr<Vec> x_ghosted;
+
+    // Global-to-local map for ghost values
+    boost::unordered_map<uint, uint> ghost_global_to_local;
+
     // PETSc norm types
     static const std::map<std::string, NormType> norm_types;
 
