@@ -1,4 +1,4 @@
-// Copyright (C) 2005-2010 Garth N. Wells.
+// Copyright (C) 2005-2011 Garth N. Wells.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Ola Skavhaug, 2008.
@@ -6,13 +6,14 @@
 // Modified by Marie Rognes, 2009.
 //
 // First added:  2005-08-31
-// Last changed: 2010-06-01
+// Last changed: 2011-02-02
 
 #ifdef HAS_SLEPC
 
 #include <slepcversion.h>
 #include <dolfin/log/dolfin_log.h>
-#include <dolfin/main/MPI.h>
+#include <dolfin/common/MPI.h>
+#include <dolfin/common/NoDeleter.h>
 #include "PETScMatrix.h"
 #include "PETScVector.h"
 #include "SLEPcEigenSolver.h"
@@ -20,8 +21,62 @@
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
-SLEPcEigenSolver::SLEPcEigenSolver() : system_size(0)
+SLEPcEigenSolver::SLEPcEigenSolver(const PETScMatrix& A)
+      : A(reference_to_no_delete_pointer(const_cast<PETScMatrix&>(A)))
 {
+  assert(A.size(0) == A.size(1));
+
+  // Set default parameter values
+  parameters = default_parameters();
+
+  // Set up solver environment
+  if (dolfin::MPI::num_processes() > 1)
+    EPSCreate(PETSC_COMM_WORLD, &eps);
+  else
+    EPSCreate(PETSC_COMM_SELF, &eps);
+}
+//-----------------------------------------------------------------------------
+SLEPcEigenSolver::SLEPcEigenSolver(const PETScMatrix& A, const PETScMatrix& B)
+   : A(reference_to_no_delete_pointer(A)),
+     B(reference_to_no_delete_pointer(B))
+
+{
+  assert(A.size(0) == A.size(1));
+  assert(B.size(0) == A.size(0));
+  assert(B.size(1) == A.size(1));
+
+  // Set default parameter values
+  parameters = default_parameters();
+
+  // Set up solver environment
+  if (dolfin::MPI::num_processes() > 1)
+    EPSCreate(PETSC_COMM_WORLD, &eps);
+  else
+    EPSCreate(PETSC_COMM_SELF, &eps);
+}
+//-----------------------------------------------------------------------------
+SLEPcEigenSolver::SLEPcEigenSolver(boost::shared_ptr<const PETScMatrix> A) : A(A)
+{
+  assert(A->size(0) == A->size(1));
+
+  // Set default parameter values
+  parameters = default_parameters();
+
+  // Set up solver environment
+  if (dolfin::MPI::num_processes() > 1)
+    EPSCreate(PETSC_COMM_WORLD, &eps);
+  else
+    EPSCreate(PETSC_COMM_SELF, &eps);
+}
+//-----------------------------------------------------------------------------
+SLEPcEigenSolver::SLEPcEigenSolver(boost::shared_ptr<const PETScMatrix> A,
+                           boost::shared_ptr<const PETScMatrix> B) : A(A), B(B)
+
+{
+  assert(A->size(0) == A->size(1));
+  assert(B->size(0) == A->size(0));
+  assert(B->size(1) == A->size(1));
+
   // Set default parameter values
   parameters = default_parameters();
 
@@ -39,38 +94,67 @@ SLEPcEigenSolver::~SLEPcEigenSolver()
     EPSDestroy(eps);
 }
 //-----------------------------------------------------------------------------
-void SLEPcEigenSolver::solve(const PETScMatrix& A)
+void SLEPcEigenSolver::solve()
 {
-  solve(&A, 0, A.size(0));
+  solve(A->size(0));
 }
 //-----------------------------------------------------------------------------
-void SLEPcEigenSolver::solve(const PETScMatrix& A, uint n)
+void SLEPcEigenSolver::solve(uint n)
 {
-  solve(&A, 0, n);
+  assert(A);
+
+  // Associate matrix (matrices) with eigenvalue solver
+  assert(A->size(0) == A->size(1));
+  if (B)
+  {
+    assert(B->size(0) == B->size(1) && B->size(0) == A->size(0));
+    EPSSetOperators(eps, *A->mat(), *B->mat());
+  }
+  else
+    EPSSetOperators(eps, *A->mat(), PETSC_NULL);
+
+  // Set number of eigenpairs to compute
+  assert(n <= A->size(0));
+  const uint nn = static_cast<int>(n);
+  EPSSetDimensions(eps, nn, PETSC_DECIDE, PETSC_DECIDE);
+
+  // Set parameters from local parameters
+  read_parameters();
+
+  // Set parameters from PETSc parameter database
+  EPSSetFromOptions(eps);
+
+  // Solve
+  EPSSolve(eps);
+
+  // Check for convergence
+  EPSConvergedReason reason;
+  EPSGetConvergedReason(eps, &reason);
+  if (reason < 0)
+    warning("Eigenvalue solver did not converge");
+
+  // Report solver status
+  int num_iterations = 0;
+  EPSGetIterationNumber(eps, &num_iterations);
+
+  const EPSType eps_type = 0;
+  EPSGetType(eps, &eps_type);
+  info(PROGRESS, "Eigenvalue solver (%s) converged in %d iterations.",
+       eps_type, num_iterations);
 }
 //-----------------------------------------------------------------------------
-void SLEPcEigenSolver::solve(const PETScMatrix& A, const PETScMatrix& B)
-{
-  solve(&A, &B, A.size(0));
-}
-//-----------------------------------------------------------------------------
-void SLEPcEigenSolver::solve(const PETScMatrix& A, const PETScMatrix& B, uint n)
-{
-  solve(&A, &B, n);
-}
-//-----------------------------------------------------------------------------
-void SLEPcEigenSolver::get_eigenvalue(double& lr, double& lc)
+void SLEPcEigenSolver::get_eigenvalue(double& lr, double& lc) const
 {
   get_eigenvalue(lr, lc, 0);
 }
 //-----------------------------------------------------------------------------
 void SLEPcEigenSolver::get_eigenpair(double& lr, double& lc,
-                                     PETScVector& r, PETScVector& c)
+                                     PETScVector& r, PETScVector& c) const
 {
   get_eigenpair(lr, lc, r, c, 0);
 }
 //-----------------------------------------------------------------------------
-void SLEPcEigenSolver::get_eigenvalue(double& lr, double& lc, uint i)
+void SLEPcEigenSolver::get_eigenvalue(double& lr, double& lc, uint i) const
 {
   const int ii = static_cast<int>(i);
 
@@ -80,7 +164,7 @@ void SLEPcEigenSolver::get_eigenvalue(double& lr, double& lc, uint i)
 
   if (ii < num_computed_eigenvalues)
   {
-    #if SLEPC_VERSION_RELEASE == 0
+    #if SLEPC_VERSION_MAJOR == 3 && SLEPC_VERSION_MINOR == 1
     EPSGetEigenvalue(eps, ii, &lr, &lc);
     #else
     EPSGetValue(eps, ii, &lr, &lc);
@@ -92,7 +176,7 @@ void SLEPcEigenSolver::get_eigenvalue(double& lr, double& lc, uint i)
 //-----------------------------------------------------------------------------
 void SLEPcEigenSolver::get_eigenpair(double& lr, double& lc,
                                     PETScVector& r, PETScVector& c,
-                                    uint i)
+                                    uint i) const
 {
   const int ii = static_cast<int>(i);
 
@@ -102,79 +186,32 @@ void SLEPcEigenSolver::get_eigenpair(double& lr, double& lc,
 
   if (ii < num_computed_eigenvalues)
   {
-    // Check size of passed vectors
-    if (system_size != r.size())
-      r.resize(system_size);
-
-    if (system_size != c.size())
-      c.resize(system_size);
+    assert(A);
+    A->resize(r, 0);
+    A->resize(c, 0);
 
+    assert(r.vec());
+    assert(c.vec());
     EPSGetEigenpair(eps, ii, &lr, &lc, *r.vec(), *c.vec());
   }
   else
     error("Requested eigenvalue/vector has not been computed");
 }
-
-
-int SLEPcEigenSolver::get_number_converged()
+//-----------------------------------------------------------------------------
+int SLEPcEigenSolver::get_number_converged() const
 {
   int num_conv;
   EPSGetConverged(eps, &num_conv);
   return num_conv;
 }
-
 //-----------------------------------------------------------------------------
-void SLEPcEigenSolver::solve(const PETScMatrix* A,
-                             const PETScMatrix* B,
-                             uint n)
+void SLEPcEigenSolver::set_deflation_space(const PETScVector& deflation_space)
 {
-  // Associate matrix (matrices) with eigenvalue solver
-  assert(A);
-  assert(A->size(0) == A->size(1));
-  if (B)
-  {
-    assert(B->size(0) == B->size(1) && B->size(0) == A->size(0));
-    EPSSetOperators(eps, *A->mat(), *B->mat());
-  }
-  else
-  {
-    EPSSetOperators(eps, *A->mat(), PETSC_NULL);
-  }
-
-  // Store the size of the eigenvalue system
-  system_size = A->size(0);
-
-  // Set number of eigenpairs to compute
-  assert(n <= system_size);
-  const uint nn = static_cast<int>(n);
-  EPSSetDimensions(eps, nn, PETSC_DECIDE, PETSC_DECIDE);
-
-  // Set parameters from PETSc parameter database
-  EPSSetFromOptions(eps);
-
-  // Set parameters from local parameters
-  read_parameters();
-
-  // Solve
-  EPSSolve(eps);
-
-  // Check for convergence
-  EPSConvergedReason reason;
-  EPSGetConvergedReason(eps, &reason);
-  if (reason < 0)
-  {
-    warning("Eigenvalue solver did not converge");
-    return;
-  }
-
-  // Report solver status
-  int num_iterations = 0;
-  EPSGetIterationNumber(eps, &num_iterations);
-
-  const EPSType eps_type = 0;
-  EPSGetType(eps, &eps_type);
-  info(PROGRESS, "Eigenvalue solver (%s) converged in %d iterations.",
-       eps_type, num_iterations);
+  #if SLEPC_VERSION_MAJOR == 3 && SLEPC_VERSION_MINOR == 1
+  EPSSetDeflationSpace(eps, 1, deflation_space.vec().get());
+  #else
+  error("Setting a deflation space requires SLEPc 3.1 or newer version");
+  #endif
 }
 //-----------------------------------------------------------------------------
 void SLEPcEigenSolver::read_parameters()
@@ -188,7 +225,6 @@ void SLEPcEigenSolver::read_parameters()
 
 }
 //-----------------------------------------------------------------------------
-
 void SLEPcEigenSolver::set_problem_type(std::string type)
 {
   // Do nothing if default type is specified
@@ -206,7 +242,7 @@ void SLEPcEigenSolver::set_problem_type(std::string type)
   else
     error("Unknown problem type: \"%s\".", type.c_str());
 }
-
+//-----------------------------------------------------------------------------
 void SLEPcEigenSolver::set_spectral_transform(std::string transform,
                                               double shift)
 {
@@ -216,14 +252,18 @@ void SLEPcEigenSolver::set_spectral_transform(std::string transform,
   ST st;
   EPSGetST(eps, &st);
   if (transform == "shift-and-invert")
-    {
-      STSetType(st, STSINV);
-      STSetShift(st, shift);
-    }
+  {
+    #if SLEPC_VERSION_MAJOR == 3 && SLEPC_VERSION_MINOR == 1
+    STSetType(st, STSINVERT);
+    #else
+    STSetType(st, STSINV);
+    #endif
+    STSetShift(st, shift);
+  }
   else
     error("Unknown transform: \"%s\".", transform.c_str());
 }
-
+//-----------------------------------------------------------------------------
 void SLEPcEigenSolver::set_spectrum(std::string spectrum)
 {
   // Do nothing if default type is specified
@@ -243,6 +283,25 @@ void SLEPcEigenSolver::set_spectrum(std::string spectrum)
     EPSSetWhichEigenpairs(eps, EPS_LARGEST_IMAGINARY);
   else if (spectrum == "smallest imaginary")
     EPSSetWhichEigenpairs(eps, EPS_SMALLEST_IMAGINARY);
+
+  #if SLEPC_VERSION_MAJOR == 3 && SLEPC_VERSION_MINOR == 1
+  else if (spectrum == "target magnitude")
+  {
+    EPSSetWhichEigenpairs(eps, EPS_TARGET_MAGNITUDE);
+    EPSSetTarget(eps, parameters["spectral_shift"]);
+  }
+  else if (spectrum == "target real")
+  {
+    EPSSetWhichEigenpairs(eps, EPS_TARGET_REAL);
+    EPSSetTarget(eps, parameters["spectral_shift"]);
+  }
+  else if (spectrum == "target imaginary")
+  {
+    EPSSetWhichEigenpairs(eps, EPS_TARGET_IMAGINARY);
+    EPSSetTarget(eps, parameters["spectral_shift"]);
+  }
+  #endif
+
   else
     error("Unknown spectrum: \"%s\".", spectrum.c_str());
 
@@ -256,7 +315,7 @@ void SLEPcEigenSolver::set_solver(std::string solver)
   if (solver == "default")
     return;
 
-  // Choose solver.
+  // Choose solver
 
   // (Note that lanczos will give PETSc error unless problem_type is
   // set to 'hermitian' or 'gen_hermitian')
@@ -282,12 +341,11 @@ void SLEPcEigenSolver::set_tolerance(double tolerance, uint maxiter)
   EPSSetTolerances(eps, tolerance, static_cast<int>(maxiter));
 }
 //-----------------------------------------------------------------------------
-int SLEPcEigenSolver::get_iteration_number()
+int SLEPcEigenSolver::get_iteration_number() const
 {
   int num_iter;
   EPSGetIterationNumber(eps, &num_iter);
   return num_iter;
 }
 //-----------------------------------------------------------------------------
-
 #endif
