@@ -1,4 +1,4 @@
-// Copyright (C) 2005-2010 Garth N. Wells.
+// Copyright (C) 2005-2011 Garth N. Wells.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Ola Skavhaug, 2008.
@@ -6,14 +6,17 @@
 // Modified by Marie Rognes, 2009.
 //
 // First added:  2005-08-31
-// Last changed: 2010-06-01
+// Last changed: 2011-02-02
 
 #ifndef __SLEPC_EIGEN_SOLVER_H
 #define __SLEPC_EIGEN_SOLVER_H
 
 #ifdef HAS_SLEPC
 
+#include <string>
+#include <boost/shared_ptr.hpp>
 #include <slepceps.h>
+#include "dolfin/common/types.h"
 #include "PETScObject.h"
 
 namespace dolfin
@@ -28,7 +31,7 @@ namespace dolfin
   ///
   /// The following parameters may be specified to control the solver.
   ///
-  /// 1. "eigenvalue spectrum"
+  /// 1. "spectrum"
   ///
   /// This parameter controls which part of the spectrum to compute.
   /// Possible values are
@@ -39,9 +42,16 @@ namespace dolfin
   ///   "smallest real"       (eigenvalues with smallest double part)
   ///   "largest imaginary"   (eigenvalues with largest imaginary part)
   ///   "smallest imaginary"  (eigenvalues with smallest imaginary part)
-  ///   "default spectrum"    (default spectrum)
   ///
-  /// 2. "eigenvalue solver"
+  /// For SLEPc versions >= 3.1 , the following values are also possible
+  ///
+  ///   "target magnitude"    (eigenvalues closest to target in magnitude)
+  ///   "target real"         (eigenvalues closest to target in real part)
+  ///   "target imaginary"    (eigenvalues closest to target in imaginary part)
+  ///
+  /// The default is "largest magnitude"
+  ///
+  /// 2. "solver"
   ///
   /// This parameter controls which algorithm is used by SLEPc.
   /// Possible values are
@@ -51,61 +61,104 @@ namespace dolfin
   ///   "arnoldi"             (Arnoldi)
   ///   "lanczos"             (Lanczos)
   ///   "krylov-schur"        (Krylov-Schur)
-  ///   "lapack"              (LAPACK, all values, direct, only for small systems)
-  ///   "default"             (default algorithm)
+  ///   "lapack"              (LAPACK, all values, direct, small systems only)
   ///
-  /// 3. "eigenvalue tolerance"
+  /// The default is "krylov-schur"
+  ///
+  /// 3. "tolerance"
   ///
   /// This parameter controls the tolerance used by SLEPc.
   /// Possible values are positive double numbers.
   ///
-  /// 4. "eigenvalue iterations"
+  /// The default is 1e-15;
+  ///
+  /// 4. "maximum_iterations"
   ///
   /// This parameter controls the maximum number of iterations used by SLEPc.
   /// Possible values are positive integers.
   ///
   /// Note that both the tolerance and the number of iterations must be
   /// specified if either one is specified.
+  ///
+  /// 5. "problem_type"
+  ///
+  /// This parameter can be used to give extra information about the
+  /// type of the eigenvalue problem. Some solver types require this
+  /// extra piece of information. Possible values are:
+  ///
+  ///   "hermitian"               (Hermitian)
+  ///   "non_hermitian"           (Non-Hermitian)
+  ///   "gen_hermitian"           (Generalized Hermitian)
+  ///   "gen_non_hermitian"       (Generalized Non-Hermitian)
+  ///
+  /// 6. "spectral_transform"
+  ///
+  /// This parameter controls the application of a spectral transform. A
+  /// spectral transform can be used to enhance the convergence of the
+  /// eigensolver and in particular to only compute eigenvalues in the
+  /// interior of the spectrum. Possible values are:
+  ///
+  ///   "shift-and-invert"      (A shift-and-invert transform)
+  ///
+  /// Note that if a spectral transform is given, then also a non-zero
+  /// spectral shift parameter has to be provided.
+  ///
+  /// The default is no spectral transform.
+  ///
+  /// 7. "spectral_shift"
+  ///
+  /// This parameter controls the spectral shift used by the spectral
+  /// transform and must be provided if a spectral transform is given. The
+  /// possible values are real numbers.
+  ///
 
   class SLEPcEigenSolver : public Variable, public PETScObject
   {
   public:
 
-    /// Create eigenvalue solver
-    SLEPcEigenSolver();
+    /// Create eigenvalue solver for Ax = \lambda x
+    SLEPcEigenSolver(const PETScMatrix& A);
+
+    /// Create eigenvalue solver Ax = \lambda Bx
+    SLEPcEigenSolver(const PETScMatrix& A, const PETScMatrix& B);
+
+    /// Create eigenvalue solver for Ax = \lambda x
+    SLEPcEigenSolver(boost::shared_ptr<const PETScMatrix> A);
+
+    /// Create eigenvalue solver for Ax = \lambda x
+    SLEPcEigenSolver(boost::shared_ptr<const PETScMatrix> A,
+                     boost::shared_ptr<const PETScMatrix> B);
 
     /// Destructor
     ~SLEPcEigenSolver();
 
     /// Compute all eigenpairs of the matrix A (solve Ax = \lambda x)
-    void solve(const PETScMatrix& A);
+    void solve();
 
     /// Compute the n first eigenpairs of the matrix A (solve Ax = \lambda x)
-    void solve(const PETScMatrix& A, uint n);
-
-    /// Compute all eigenpairs of the generalised problem Ax = \lambda Bx
-    void solve(const PETScMatrix& A, const PETScMatrix& B);
-
-    /// Compute the n first eigenpairs of the generalised problem Ax = \lambda Bx
-    void solve(const PETScMatrix& A, const PETScMatrix& B, uint n);
+    void solve(uint n);
 
     /// Get the first eigenvalue
-    void get_eigenvalue(double& lr, double& lc);
+    void get_eigenvalue(double& lr, double& lc) const;
 
     /// Get the first eigenpair
-    void get_eigenpair(double& lr, double& lc, PETScVector& r, PETScVector& c);
+    void get_eigenpair(double& lr, double& lc, PETScVector& r, PETScVector& c) const;
 
     /// Get eigenvalue i
-    void get_eigenvalue(double& lr, double& lc, uint i);
+    void get_eigenvalue(double& lr, double& lc, uint i) const;
 
     /// Get eigenpair i
-    void get_eigenpair(double& lr, double& lc, PETScVector& r, PETScVector& c, uint i);
+    void get_eigenpair(double& lr, double& lc,
+                       PETScVector& r, PETScVector& c, uint i) const;
 
     // Get the number of iterations used by the solver
-    int get_iteration_number();
+    int get_iteration_number() const;
 
     // Get the number of converged eigenvalues
-    int get_number_converged();
+    int get_number_converged() const;
+
+    // Set deflation space
+    void set_deflation_space(const PETScVector& deflation_space);
 
     /// Default parameter values
     static Parameters default_parameters()
@@ -125,9 +178,6 @@ namespace dolfin
 
   private:
 
-    /// Compute eigenpairs
-    void solve(const PETScMatrix* A, const PETScMatrix* B, uint n);
-
     /// Callback for changes in parameter values
     void read_parameters();
 
@@ -146,12 +196,13 @@ namespace dolfin
     // Set tolerance
     void set_tolerance(double tolerance, uint maxiter);
 
+    // Operators (A x = \lambda x or Ax = \lambda B x)
+    boost::shared_ptr<const PETScMatrix> A;
+    boost::shared_ptr<const PETScMatrix> B;
+
     // SLEPc solver pointer
     EPS eps;
 
-    // System size
-    unsigned int system_size;
-
   };
 
 }
