@@ -1,4 +1,4 @@
-// Copyright (C) 2007-2008 Anders Logg.
+// Copyright (C) 2007-2010 Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Ola Skavhaug, 2007.
@@ -6,24 +6,33 @@
 // Modified by Ilmar Wilbers, 2008.
 //
 // First added:  2007-01-17
-// Last changed: 2010-06-07
+// Last changed: 2010-11-08
 
 #ifndef __STL_MATRIX_H
 #define __STL_MATRIX_H
 
 #include <map>
 #include <string>
+#include <utility>
 #include <vector>
+#include <boost/unordered_map.hpp>
+
 #include <dolfin/log/log.h>
 #include "GenericSparsityPattern.h"
 #include "GenericMatrix.h"
+#include "GenericVector.h"
 
 namespace dolfin
 {
 
-  /// Simple implementation of a GenericMatrix for experimenting
-  /// with new assembly. Not sure this will be used later but it
-  /// might be useful.
+  /// Simple STL-based implementation of the GenericMatrix interface.
+  /// The sparse matrix is stored as a pair of std::vector of
+  /// std::vector, one for the columns and one for the values.
+  ///
+  /// Historically, this class has undergone a number of different
+  /// incarnations, based on various combinations of std::vector,
+  /// std::set and std::map. The current implementation has proven to
+  /// be the fastest.
 
   class STLMatrix : public GenericMatrix
   {
@@ -56,18 +65,21 @@ namespace dolfin
 
     /// Return size of given dimension
     virtual uint size(uint dim) const
-    { return dims[dim]; }
+    { assert(dim < 2); return dims[dim]; }
+
+    /// Return local ownership range
+    virtual std::pair<uint, uint> local_range(uint dim) const
+    { return std::make_pair(0, size(dim)); }
 
     /// Set all entries to zero and keep any sparse structure
     virtual void zero()
     {
-      for (uint i = 0; i < A.size(); i++)
-        for (std::map<uint, double>::iterator it = A[i].begin(); it != A[i].end(); it++)
-          it->second = 0.0;
+      for (std::vector<std::vector<double> >::iterator row = vals.begin(); row != vals.end(); ++row)
+        std::fill(row->begin(), row->end(), 0);
     }
 
     /// Finalize assembly of tensor
-    virtual void apply(std::string mode) {}
+    virtual void apply(std::string mode);
 
     /// Return informal string representation (pretty-print)
     virtual std::string str(bool verbose) const;
@@ -76,7 +88,20 @@ namespace dolfin
 
     /// Initialize M x N matrix
     virtual void resize(uint M, uint N)
-    { dims[0] = M; dims[1] = N; A.resize(M); }
+    {
+      cols.clear();
+      vals.clear();
+      cols.resize(M);
+      vals.resize(M);
+      dims[0] = M;
+      dims[1] = N;
+    }
+
+    /// Resize vector y such that is it compatible with matrix for
+    /// multuplication Ax = b (dim = 0 -> b, dim = 1 -> x) In parallel
+    /// case, size and layout are important.
+    virtual void resize(GenericVector& y, uint dim) const
+    { y.resize(dim); }
 
     /// Get block of values
     virtual void get(double* block, uint m, const uint* rows, uint n, const uint* cols) const
@@ -87,23 +112,7 @@ namespace dolfin
     { dolfin_not_implemented(); }
 
     /// Add block of values
-    virtual void add(const double* block, uint m, const uint* rows, uint n, const uint* cols)
-    {
-      uint pos = 0;
-      for (uint i = 0; i < m; i++)
-      {
-        std::map<uint, double>& row = A[rows[i]];
-        for (uint j = 0; j < n; j++)
-        {
-          const uint col = cols[j];
-          const std::map<uint, double>::iterator it = row.find(col);
-          if ( it == row.end() )
-            row.insert(it, std::map<uint, double>::value_type(col, block[pos++]));
-          else
-            it->second += block[pos++];
-        }
-      }
-    }
+    virtual void add(const double* block, uint m, const uint* rows, uint n, const uint* cols);
 
     /// Add multiple of given matrix (AXPY operation)
     virtual void axpy(double a, const GenericMatrix& A, bool same_nonzero_pattern)
@@ -116,14 +125,9 @@ namespace dolfin
     /// Get non-zero values of given row
     virtual void getrow(uint row, std::vector<uint>& columns, std::vector<double>& values) const
     {
-      columns.clear();
-      values.clear();
-      const std::map<uint, double>& rowid = A[row];
-      for (std::map<uint, double>::const_iterator it = rowid.begin(); it != rowid.end(); it++)
-      {
-        columns.push_back(it->first);
-        values.push_back(it->second);
-      }
+      assert(row < dims[0]);
+      columns = this->cols[row];
+      values = this->vals[row];
     }
 
     /// Set values for given row
@@ -180,8 +184,14 @@ namespace dolfin
 
   private:
 
-    // The matrix representation
-    std::vector<std::map<uint, double> > A;
+    // Storages of columns
+    std::vector<std::vector<uint> > cols;
+
+    // Storage of values
+    std::vector<std::vector<double> > vals;
+
+    //std::vector<std::map<uint, double> > matrix;
+    //std::vector<boost::unordered_map<uint, double> > matrix;
 
     // The size of the matrix
     uint dims[2];
