@@ -1,18 +1,18 @@
-// Copyright (C) 2007-2008 Anders Logg.
+// Copyright (C) 2007-2011 Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Garth N. Wells, 2007-2009.
 // Modified by Ola Skavhaug, 2007.
 //
 // First added:  2007-03-15
-// Last changed: 2009-08-18
+// Last changed: 2011-01-24
 
 #ifndef __SCALAR_H
 #define __SCALAR_H
 
 #include <numeric>
 #include <vector>
-#include <dolfin/main/MPI.h>
+#include <dolfin/common/MPI.h>
 #include "uBLASFactory.h"
 #include "GenericTensor.h"
 
@@ -58,6 +58,14 @@ namespace dolfin
     uint size(uint dim) const
     { error("The size() function is not available for scalars."); return 0; }
 
+    /// Return local ownership range
+    virtual std::pair<uint, uint> local_range(uint dim) const
+    {
+      error("The size() function is not available for scalars.");
+      return std::make_pair(0, 0);
+    }
+
+
     /// Get block of values
     void get(double* block, const uint* num_rows, const uint * const * rows) const
     { block[0] = value; }
@@ -68,7 +76,24 @@ namespace dolfin
 
     /// Add block of values
     void add(const double* block, const uint* num_rows, const uint * const * rows)
-    { value += block[0]; }
+    {
+      assert(block);
+      value += block[0];
+    }
+
+    /// Add block of values
+    void add(const double* block, const std::vector<const std::vector<uint>* >& rows)
+    {
+      assert(block);
+      value += block[0];
+    }
+
+    /// Add block of values
+    void add(const double* block, const std::vector<std::vector<uint> >& rows)
+    {
+      assert(block);
+      value += block[0];
+    }
 
     /// Set all entries to zero and keep any sparse structure
     void zero()
