@@ -5,7 +5,7 @@
 // Modified by Dag Lindbo 2008.
 //
 // First added:  2006-06-01
-// Last changed: 2008-07-08
+// Last changed: 2010-11-29
 
 #include <dolfin/common/NoDeleter.h>
 #include <dolfin/log/dolfin_log.h>
@@ -22,6 +22,63 @@ extern "C"
 #endif
 }
 
+#ifdef HAS_UMFPACK
+namespace dolfin
+{
+  class UmfpackIntSymbolicDeleter
+  {
+  public:
+    void operator() (void* symbolic)
+    {
+      if (symbolic)
+      {
+        umfpack_di_free_symbolic(&symbolic);
+        symbolic = 0;
+      }
+    }
+  };
+
+  class UmfpackLongIntSymbolicDeleter
+  {
+  public:
+    void operator() (void* symbolic)
+    {
+      if (symbolic)
+      {
+        umfpack_dl_free_symbolic(&symbolic);
+        symbolic = 0;
+      }
+    }
+  };
+
+  class UmfpackIntNumericDeleter
+  {
+  public:
+    void operator() (void* numeric)
+    {
+      if (numeric)
+      {
+        umfpack_di_free_numeric(&numeric);
+        numeric = 0;
+      }
+    }
+  };
+
+  class UmfpackLongIntNumericDeleter
+  {
+  public:
+    void operator() (void* numeric)
+    {
+      if (numeric)
+      {
+        umfpack_dl_free_numeric(&numeric);
+        numeric = 0;
+      }
+    }
+  };
+}
+#endif
+
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
@@ -32,22 +89,21 @@ Parameters UmfpackLUSolver::default_parameters()
   return p;
 }
 //-----------------------------------------------------------------------------
-UmfpackLUSolver::UmfpackLUSolver() : symbolic(0), numeric(0)
+UmfpackLUSolver::UmfpackLUSolver()
 {
   // Set parameter values
   parameters = default_parameters();
 }
 //-----------------------------------------------------------------------------
 UmfpackLUSolver::UmfpackLUSolver(const GenericMatrix& A)
-                               : symbolic(0), numeric(0),
-                                 A(reference_to_no_delete_pointer(A))
+                               : A(reference_to_no_delete_pointer(A))
 {
   // Set parameter values
   parameters = default_parameters();
 }
 //-----------------------------------------------------------------------------
 UmfpackLUSolver::UmfpackLUSolver(boost::shared_ptr<const GenericMatrix> A)
-                               : symbolic(0), numeric(0), A(A)
+                               : A(A)
 {
   // Set parameter values
   parameters = default_parameters();
@@ -55,29 +111,56 @@ UmfpackLUSolver::UmfpackLUSolver(boost::shared_ptr<const GenericMatrix> A)
 //-----------------------------------------------------------------------------
 UmfpackLUSolver::~UmfpackLUSolver()
 {
-  clear();
+  // Do nothing
 }
 //-----------------------------------------------------------------------------
 void UmfpackLUSolver::set_operator(const GenericMatrix& A)
 {
-  clear();
+  symbolic.reset();
+  numeric.reset();
   this->A = reference_to_no_delete_pointer(A);
 }
 //-----------------------------------------------------------------------------
 dolfin::uint UmfpackLUSolver::solve(GenericVector& x, const GenericVector& b)
 {
-  // Factorize matrix
-  factorize();
+  assert(A);
+
+  // Get some parameters
+  const bool reuse_fact   = parameters["reuse_factorization"];
+  const bool same_pattern = parameters["same_nonzero_pattern"];
+
+  // Perform symbolic factorization if required
+  if (!symbolic)
+    symbolic_factorize();
+  else if (!reuse_fact && !same_pattern)
+    symbolic_factorize();
+
+  // Perform numerical factorization if required
+  if (!numeric)
+    numeric_factorize();
+  else if (!reuse_fact)
+    numeric_factorize();
 
   // Solve
   return solve_factorized(x, b);
 }
 //-----------------------------------------------------------------------------
+dolfin::uint UmfpackLUSolver::solve(const GenericMatrix& A, GenericVector& x,
+                                    const GenericVector& b)
+{
+  set_operator(A);
+  return solve(x, b);
+}
+//-----------------------------------------------------------------------------
 void UmfpackLUSolver::symbolic_factorize()
 {
   if (!A)
     error("A matrix must be assocoated with UmfpackLUSolver to peform a symbolic factorisation.");
 
+  // Clear any old factorizations
+  symbolic.reset();
+  numeric.reset();
+
   // Get matrix data
   std::tr1::tuple<const std::size_t*, const std::size_t*, const double*, int> data = A->data();
   const std::size_t* Ap  = std::tr1::get<0>(data);
@@ -90,17 +173,14 @@ void UmfpackLUSolver::symbolic_factorize()
   const uint N   = A->size(1);
   assert(nnz >= M);
 
-  // Clear any old factorizations
-  clear();
-
   // Factorize and solve
   info(PROGRESS, "Symbolic factorization of a matrix of size %d x %d (UMFPACK).", M, N);
 
   // Perform symbolic factorisation
-  symbolic = umfpack_factorize_symbolic(M, N, (const long int*)Ap, (const long int*)Ai, Ax);
+  symbolic = umfpack_factorize_symbolic(M, N, Ap, Ai, Ax);
 }
 //-----------------------------------------------------------------------------
-void UmfpackLUSolver::factorize()
+void UmfpackLUSolver::numeric_factorize()
 {
   if (!A)
     error("A matrix must be assocoated with UmfpackLUSolver to peform a factorisation.");
@@ -120,12 +200,11 @@ void UmfpackLUSolver::factorize()
   // Factorize and solve
   info(PROGRESS, "LU factorization of a matrix of size %d x %d (UMFPACK).", M, N);
 
-  // Create symbolic factoriation if required
-  if (!symbolic || !parameters["same_nonzero_pattern"])
-    symbolic_factorize();
+  assert(symbolic);
+  numeric.reset();
 
   // Perform LU factorisation
-  numeric = umfpack_factorize_numeric((const long int*)Ap, (const long int*)Ai, Ax, symbolic);
+  numeric = umfpack_factorize_numeric(Ap, Ai, Ax, symbolic.get());
 }
 //-----------------------------------------------------------------------------
 dolfin::uint UmfpackLUSolver::solve_factorized(GenericVector& x,
@@ -153,54 +232,56 @@ dolfin::uint UmfpackLUSolver::solve_factorized(GenericVector& x,
   const std::size_t* Ai  = std::tr1::get<1>(data);
   const double*      Ax  = std::tr1::get<2>(data);
 
+  info(PROGRESS, "Solving linear system of size %d x %d (UMFPACK LU solver).",
+       A->size(0), A->size(1));
+
   // Solve for tranpose since we use compressed rows and UMFPACK expected compressed columns
-  umfpack_solve((const long int*)Ap, (const long int*)Ai, Ax, x.data(), b.data(), numeric);
+  umfpack_solve(Ap, Ai, Ax, x.data(), b.data(), numeric.get());
 
   return 1;
 }
 //-----------------------------------------------------------------------------
 #ifdef HAS_UMFPACK
 //-----------------------------------------------------------------------------
-void UmfpackLUSolver::clear()
-{
-  if (numeric)
-  {
-    umfpack_dl_free_numeric(&numeric);
-    numeric = 0;
-  }
-
-  if(symbolic)
-  {
-    umfpack_dl_free_symbolic(&symbolic);
-    symbolic = 0;
-  }
-}
-//-----------------------------------------------------------------------------
-void* UmfpackLUSolver::umfpack_factorize_symbolic(uint M, uint N,
-                                                   const long int* Ap,
-                                                   const long int* Ai,
-                                                   const double* Ax)
+boost::shared_ptr<void> UmfpackLUSolver::umfpack_factorize_symbolic(uint M, uint N,
+                                                                    const std::size_t* Ap,
+                                                                    const std::size_t* Ai,
+                                                                    const double* Ax)
 {
   assert(Ap);
   assert(Ai);
   assert(Ax);
 
   void* symbolic = 0;
-  double* dnull = 0;
+  boost::scoped_ptr<double> dnull;
 
   // Symbolic factorisation step (reordering, etc)
-  long int status = umfpack_dl_symbolic(M, N, Ap, Ai, Ax, &symbolic, dnull, dnull);
-  delete dnull;
+  if (sizeof(std::size_t) == sizeof(int))
+  {
+    const int* _Ap = reinterpret_cast<const int*>(Ap);
+    const int* _Ai = reinterpret_cast<const int*>(Ai);
+    long int status = umfpack_di_symbolic(M, N, _Ap, _Ai, Ax, &symbolic, dnull.get(), dnull.get());
+    umfpack_check_status(status, "symbolic");
+    return boost::shared_ptr<void>(symbolic, UmfpackIntSymbolicDeleter());
+  }
+  else if (sizeof(std::size_t) == sizeof(UF_long))
+  {
+    const UF_long* _Ap = reinterpret_cast<const UF_long*>(Ap);
+    const UF_long* _Ai = reinterpret_cast<const UF_long*>(Ai);
+    long int status = umfpack_dl_symbolic(M, N, _Ap, _Ai, Ax, &symbolic, dnull.get(), dnull.get());
+    umfpack_check_status(status, "symbolic");
+    return boost::shared_ptr<void>(symbolic, UmfpackLongIntSymbolicDeleter());
+  }
+  else
+    error("Could not determine correct types for casting integers to pass to UMFPACK.");
 
-  umfpack_check_status(status, "symbolic");
-
-  return symbolic;
+  return boost::shared_ptr<void>();
 }
 //-----------------------------------------------------------------------------
-void* UmfpackLUSolver::umfpack_factorize_numeric(const long int* Ap,
-                                                  const long int* Ai,
-                                                  const double* Ax,
-                                                  void* symbolic)
+boost::shared_ptr<void> UmfpackLUSolver::umfpack_factorize_numeric(const std::size_t* Ap,
+                                                 const std::size_t* Ai,
+                                                 const double* Ax,
+                                                 void* symbolic)
 {
   assert(Ap);
   assert(Ai);
@@ -208,18 +289,34 @@ void* UmfpackLUSolver::umfpack_factorize_numeric(const long int* Ap,
   assert(symbolic);
 
   void* numeric = 0;
-  double* dnull = 0;
+  boost::scoped_ptr<double> dnull;
 
   // Factorization step
-  long int status = umfpack_dl_numeric(Ap, Ai, Ax, symbolic, &numeric, dnull, dnull);
-  delete dnull;
+  long int status = 0;
+  if (sizeof(std::size_t) == sizeof(int))
+  {
+    const int* _Ap = reinterpret_cast<const int*>(Ap);
+    const int* _Ai = reinterpret_cast<const int*>(Ai);
+    status = umfpack_di_numeric(_Ap, _Ai, Ax, symbolic, &numeric, dnull.get(), dnull.get());
+    umfpack_check_status(status, "numeric");
+    return boost::shared_ptr<void>(numeric, UmfpackIntNumericDeleter());
+  }
+  else if (sizeof(std::size_t) == sizeof(UF_long))
+  {
+    const UF_long* _Ap = reinterpret_cast<const UF_long*>(Ap);
+    const UF_long* _Ai = reinterpret_cast<const UF_long*>(Ai);
+    status = umfpack_dl_numeric(_Ap, _Ai, Ax, symbolic, &numeric, dnull.get(), dnull.get());
+    umfpack_check_status(status, "numeric");
+    return boost::shared_ptr<void>(numeric, UmfpackLongIntNumericDeleter());
+  }
+  else
+    error("Could not determine correct types for casting integers to pass to UMFPACK.");
 
-  umfpack_check_status(status, "numeric");
-
-  return numeric;
+  return boost::shared_ptr<void>();
 }
 //-----------------------------------------------------------------------------
-void UmfpackLUSolver::umfpack_solve(const long int* Ap, const long int* Ai,
+void UmfpackLUSolver::umfpack_solve(const std::size_t* Ap,
+                                    const std::size_t* Ai,
                                     const double* Ax, double* x,
                                     const double* b, void* numeric)
 {
@@ -229,12 +326,26 @@ void UmfpackLUSolver::umfpack_solve(const long int* Ap, const long int* Ai,
   assert(x);
   assert(b);
   assert(numeric);
-  double* dnull = 0;
+
+  boost::scoped_ptr<double> dnull;
 
   // Solve system. We assume CSR storage, but UMFPACK expects CSC, so solve
   // for the transpose
-  long int status = umfpack_dl_solve(UMFPACK_At, Ap, Ai, Ax, x, b, numeric, dnull, dnull);
-  delete dnull;
+  long int status = 0;
+  if (sizeof(std::size_t) == sizeof(int))
+  {
+    const int* _Ap = reinterpret_cast<const int*>(Ap);
+    const int* _Ai = reinterpret_cast<const int*>(Ai);
+    status = umfpack_di_solve(UMFPACK_At, _Ap, _Ai, Ax, x, b, numeric, dnull.get(), dnull.get());
+  }
+  else if (sizeof(std::size_t) == sizeof(UF_long))
+  {
+    const UF_long* _Ap = reinterpret_cast<const UF_long*>(Ap);
+    const UF_long* _Ai = reinterpret_cast<const UF_long*>(Ai);
+    status = umfpack_dl_solve(UMFPACK_At, _Ap, _Ai, Ax, x, b, numeric, dnull.get(), dnull.get());
+  }
+  else
+    error("Could not determine correct types for casting integers to pass to UMFPACK.");
 
   umfpack_check_status(status, "solve");
 }
@@ -264,30 +375,26 @@ void UmfpackLUSolver::umfpack_check_status(long int status,
 //-----------------------------------------------------------------------------
 #else
 //-----------------------------------------------------------------------------
-void UmfpackLUSolver::clear()
+boost::shared_ptr<void> UmfpackLUSolver::umfpack_factorize_symbolic(uint M, uint N,
+                                                  const std::size_t* Ap,
+                                                  const std::size_t* Ai,
+                                                  const double* Ax)
 {
   error("Umfpack not installed. Cannot perform LU solver using Umfpack.");
+  return boost::shared_ptr<void>();
 }
 //-----------------------------------------------------------------------------
-void* UmfpackLUSolver::umfpack_factorize_symbolic(uint M, uint N,
-                                                   const long int* Ap,
-                                                   const long int* Ai,
-                                                   const double* Ax)
+boost::shared_ptr<void> UmfpackLUSolver::umfpack_factorize_numeric(const std::size_t* Ap,
+                                                 const std::size_t* Ai,
+                                                 const double* Ax,
+                                                 void* symbolic)
 {
   error("Umfpack not installed. Cannot perform LU solver using Umfpack.");
-  return 0;
+  return boost::shared_ptr<void>();
 }
 //-----------------------------------------------------------------------------
-void* UmfpackLUSolver::umfpack_factorize_numeric(const long int* Ap,
-                                                  const long int* Ai,
-                                                  const double* Ax,
-                                                  void* symbolic)
-{
-  error("Umfpack not installed. Cannot perform LU solver using Umfpack.");
-  return 0;
-}
-//-----------------------------------------------------------------------------
-void UmfpackLUSolver::umfpack_solve(const long int* Ap, const long int* Ai,
+void UmfpackLUSolver::umfpack_solve(const std::size_t* Ap,
+                                    const std::size_t* Ai,
                                     const double* Ax, double* x,
                                     const double* b, void* numeric)
 {
