@@ -10,9 +10,10 @@
 #ifndef __UMFPACK_LU_SOLVER_H
 #define __UMFPACK_LU_SOLVER_H
 
+#include <boost/scoped_ptr.hpp>
 #include <boost/shared_ptr.hpp>
 #include "ublas.h"
-#include "GenericLinearSolver.h"
+#include "GenericLUSolver.h"
 
 namespace dolfin
 {
@@ -28,7 +29,7 @@ namespace dolfin
   /// linear systems of the form Ax = b using UMFPACK
   /// (http://www.cise.ufl.edu/research/sparse/umfpack/) if installed.
 
-  class UmfpackLUSolver : public GenericLinearSolver
+  class UmfpackLUSolver : public GenericLUSolver
   {
 
   public:
@@ -45,47 +46,50 @@ namespace dolfin
     /// Destructor
     ~UmfpackLUSolver();
 
-    /// Ste operator (matrix)
+    /// Set operator (matrix)
     void set_operator(const GenericMatrix& A);
 
     /// Solve linear system Ax = b for a sparse matrix using UMFPACK if installed
     uint solve(GenericVector& x, const GenericVector& b);
 
-    // Perform symbolic factorisation if UMFPACK is installed
-    void symbolic_factorize();
-
-    /// LU-factor the sparse matrix A if UMFPACK is installed
-    void factorize();
-
-    /// Solve factorized system (UMFPACK).
-    uint solve_factorized(GenericVector& x, const GenericVector& b) const;
+    /// Solve linear system
+    uint solve(const GenericMatrix& A, GenericVector& x, const GenericVector& b);
 
     /// Default parameter values
     static Parameters default_parameters();
 
   private:
 
-    // Clear data
-    void clear();
+    // Perform symbolic factorisation
+    void symbolic_factorize();
+
+    /// LU factorisation
+    void numeric_factorize();
+
+    /// Solve factorized system (UMFPACK).
+    uint solve_factorized(GenericVector& x, const GenericVector& b) const;
 
     // Return pointer to symbolic factorisation
-    static void* umfpack_factorize_symbolic(uint M, uint N, const long int* Ap,
-                                            const long int* Ai, const double* Ax);
+    static boost::shared_ptr<void> umfpack_factorize_symbolic(uint M, uint N,
+                                                         const std::size_t* Ap,
+                                                         const std::size_t* Ai,
+                                                         const double* Ax);
 
     // Return pointer to the numerical factorisation
-    static void* umfpack_factorize_numeric(const long int* Ap, const long int* Ai,
+    static boost::shared_ptr<void> umfpack_factorize_numeric(const std::size_t* Ap,
+                                           const std::size_t* Ai,
                                            const double* Ax, void* symbolic);
 
-    static void umfpack_solve(const long int* Ap, const long int* Ai,
+    static void umfpack_solve(const std::size_t* Ap, const std::size_t* Ai,
                               const double* Ax, double* x, const double* b,
                               void* numeric);
 
     /// Check status flag returned by an UMFPACK function
     static void umfpack_check_status(long int status, std::string function);
 
-    // UMFPACK data
-    void* symbolic;
-    void* numeric;
+    // UMFPACK data (note that boost::scoped_ptr cannot hold a void pointer)
+    boost::shared_ptr<void> symbolic;
+    boost::shared_ptr<void> numeric;
 
     // Operator (the matrix)
     boost::shared_ptr<const GenericMatrix> A;
