@@ -1,13 +1,13 @@
 // Copyright (C) 2007 Garth N. Wells.
 // Licensed under the GNU LGPL Version 2.1.
 //
-// Modified by Anders Logg, 2007-2008.
+// Modified by Anders Logg, 2007-2010.
 // Modified by Kent-Andre Mardal, 2008.
 // Modified by Ola Skavhaug, 2008.
 // Modified by Martin Sandve Alnes, 2008.
 //
 // First added:  2007-07-03
-// Last changed: 2009-09-08
+// Last changed: 2011-01-14
 
 #ifndef __VECTOR_H
 #define __VECTOR_H
@@ -62,7 +62,7 @@ namespace dolfin
 
     /// Return informal string representation (pretty-print)
     virtual std::string str(bool verbose) const
-    { return vector->str(verbose); }
+    { return "<Vector wrapper of " + vector->str(verbose) + ">"; }
 
     //--- Implementation of the GenericVector interface ---
 
@@ -70,17 +70,29 @@ namespace dolfin
     virtual void resize(uint N)
     { vector->resize(N); }
 
+    /// Resize vector with given ownership range
+    virtual void resize(std::pair<uint, uint> range)
+    { vector->resize(range); }
+
+    /// Resize vector with given ownership range and with ghost values
+    virtual void resize(std::pair<uint, uint> range, const std::vector<uint>& ghost_indices)
+    { vector->resize(range, ghost_indices); }
+
     /// Return size of vector
     virtual uint size() const
     { return vector->size(); }
 
+    /// Return local size of vector
+    virtual uint local_size() const
+    { return vector->local_size(); }
+
     /// Return local ownership range of a vector
     virtual std::pair<uint, uint> local_range() const
     { return vector->local_range(); }
 
-    /// Get block of values
-    virtual void get(double* block, uint m, const uint* rows) const
-    { vector->get(block, m, rows); }
+    /// Determine whether global vector index is owned by this process
+    virtual bool owns_index(uint i) const
+    { return vector->owns_index(i); }
 
     /// Get block of values (values must all live on the local process)
     virtual void get_local(double* block, uint m, const uint* rows) const
@@ -110,10 +122,18 @@ namespace dolfin
     virtual void gather(GenericVector& x, const Array<uint>& indices) const
     { vector->gather(x, indices); }
 
+    /// Gather entries into Array x
+    virtual void gather(Array<double>& x, const Array<uint>& indices) const
+    { vector->gather(x, indices); }
+
     /// Add multiple of given vector (AXPY operation)
     virtual void axpy(double a, const GenericVector& x)
     { vector->axpy(a, x); }
 
+    /// Replace all entries in the vector by their absolute values
+    virtual void abs()
+    { vector->abs(); }
+
     /// Return inner product with given vector
     virtual double inner(const GenericVector& x) const
     { return vector->inner(x); }
