@@ -9,18 +9,21 @@
 #ifndef __UBLAS_FACTORY_H
 #define __UBLAS_FACTORY_H
 
+#include <string>
+#include "uBLASKrylovSolver.h"
 #include "uBLASMatrix.h"
 #include "uBLASVector.h"
 #include "SparsityPattern.h"
+#include "UmfpackLUSolver.h"
 #include "LinearAlgebraFactory.h"
 
 namespace dolfin
 {
   // Forward declaration
-  template<class T> class uBLASMatrix;
+  class GenericLinearSolver;
 
   template<class Mat = ublas_sparse_matrix>
-  class uBLASFactory: public LinearAlgebraFactory
+  class uBLASFactory : public LinearAlgebraFactory
   {
   public:
 
@@ -31,10 +34,6 @@ namespace dolfin
     uBLASMatrix<Mat>* create_matrix() const
     { return new uBLASMatrix<Mat>(); }
 
-    /// Create empty sparsity pattern
-    SparsityPattern* create_pattern() const
-    { return new SparsityPattern(SparsityPattern::sorted); }
-
     /// Create empty vector
     uBLASVector* create_vector() const
     { return new uBLASVector(); }
@@ -43,6 +42,19 @@ namespace dolfin
     uBLASVector* create_local_vector() const
     { return new uBLASVector(); }
 
+    /// Create empty sparsity pattern
+    SparsityPattern* create_pattern() const
+    { return new SparsityPattern(); }
+
+    /// Create LU solver
+    UmfpackLUSolver* create_lu_solver() const
+    { return new UmfpackLUSolver(); }
+
+    /// Create Krylov solver
+    GenericLinearSolver* create_krylov_solver(std::string method, std::string pc) const
+    //{ return 0; }
+    { return new uBLASKrylovSolver(method, pc); }
+
     static uBLASFactory<Mat>& instance()
     { return factory; }
 
