@@ -9,8 +9,9 @@
 #ifndef __UBLAS_KRYLOV_SOLVER_H
 #define __UBLAS_KRYLOV_SOLVER_H
 
-#include <string>
 #include <set>
+#include <string>
+#include <boost/shared_ptr.hpp>
 #include <dolfin/common/types.h>
 #include "ublas.h"
 #include "GenericLinearSolver.h"
@@ -22,6 +23,9 @@
 namespace dolfin
 {
 
+  class GenericMatrix;
+  class GenericVector;
+
   /// This class implements Krylov methods for linear systems
   /// of the form Ax = b using uBLAS data types.
 
@@ -37,11 +41,16 @@ namespace dolfin
     uBLASKrylovSolver(uBLASPreconditioner& pc);
 
     /// Create Krylov solver for a particular method and uBLASPreconditioner
-    uBLASKrylovSolver(std::string solver_type, uBLASPreconditioner& preconditioner);
+    uBLASKrylovSolver(std::string solver_type,
+                      uBLASPreconditioner& preconditioner);
 
     /// Destructor
     ~uBLASKrylovSolver();
 
+    /// Solve the operator (matrix)
+    void set_operator(const GenericMatrix& A)
+    { error("set_operator(A) is not implemented."); }
+
     /// Solve linear system Ax = b and return number of iterations
     uint solve(const GenericMatrix& A, GenericVector& x, const GenericVector& b);
 
@@ -90,14 +99,11 @@ namespace dolfin
     std::string solver_type;
 
     /// Preconditioner
-    uBLASPreconditioner* pc;
+    boost::shared_ptr<uBLASPreconditioner> pc;
 
     // Available solver types
     static const std::set<std::string> solver_types;
 
-    /// True if a user has provided a preconditioner
-    bool pc_user;
-
     /// Solver parameters
     double rtol, atol, div_tol;
     uint max_it, restart;
