@@ -17,12 +17,11 @@
 #include <iomanip>
 #include <tr1/tuple>
 
-#include "LinearAlgebraFactory.h"
+#include "GenericMatrix.h"
 #include "SparsityPattern.h"
 #include "ublas.h"
-#include "uBLASFactory.h"
 #include "uBLASVector.h"
-#include "GenericMatrix.h"
+#include "uBLASFactory.h"
 
 namespace dolfin
 {
@@ -77,6 +76,10 @@ namespace dolfin
     /// Return size of given dimension
     virtual uint size(uint dim) const;
 
+    /// Return local ownership range
+    virtual std::pair<uint, uint> local_range(uint dim) const
+    { return std::make_pair(0, size(dim)); }
+
     /// Set all entries to zero and keep any sparse structure
     virtual void zero();
 
@@ -91,6 +94,11 @@ namespace dolfin
     /// Resize matrix to M x N
     virtual void resize(uint M, uint N);
 
+    /// Resize vector y such that is it compatible with matrix for
+    /// multuplication Ax = b (dim = 0 -> b, dim = 1 -> x) In parallel
+    /// case, size and layout are important.
+    virtual void resize(GenericVector& y, uint dim) const;
+
     /// Get block of values
     virtual void get(double* block, uint m, const uint* rows, uint n, const uint* cols) const;
 
@@ -264,7 +272,7 @@ namespace dolfin
     columns.clear();
     values.clear();
     typename ublas::matrix_row<const Mat>::const_iterator component;
-    for (component=row.begin(); component != row.end(); ++component)
+    for (component = row.begin(); component != row.end(); ++component)
     {
       columns.push_back(component.index());
       values.push_back(*component );
@@ -287,8 +295,14 @@ namespace dolfin
   }
   //-----------------------------------------------------------------------------
   template <class Mat>
+  void uBLASMatrix<Mat>::resize(GenericVector& y, uint dim) const
+  {
+    y.resize(size(dim));
+  }
+  //-----------------------------------------------------------------------------
+  template <class Mat>
   void uBLASMatrix<Mat>::set(const double* block, uint m, const uint* rows,
-                                                uint n, const uint* cols)
+                             uint n, const uint* cols)
   {
     for (uint i = 0; i < m; i++)
       for (uint j = 0; j < n; j++)
@@ -297,7 +311,7 @@ namespace dolfin
   //---------------------------------------------------------------------------
   template <class Mat>
   void uBLASMatrix<Mat>::add(const double* block, uint m, const uint* rows,
-                                                uint n, const uint* cols)
+                             uint n, const uint* cols)
   {
     for (uint i = 0; i < m; i++)
       for (uint j = 0; j < n; j++)
@@ -306,7 +320,7 @@ namespace dolfin
   //---------------------------------------------------------------------------
   template <class Mat>
   void uBLASMatrix<Mat>::get(double* block, uint m, const uint* rows,
-                                          uint n, const uint* cols) const
+                             uint n, const uint* cols) const
   {
     for(uint i = 0; i < m; ++i)
       for(uint j = 0; j < n; ++j)
@@ -516,10 +530,10 @@ namespace dolfin
     const SparsityPattern* pattern_pointer = dynamic_cast<const SparsityPattern*>(&sparsity_pattern);
     if (!pattern_pointer)
       error("Cannot convert GenericSparsityPattern to concrete SparsityPattern type. Aborting.");
-    const std::vector<Set<uint> >& pattern = pattern_pointer->diagonal_pattern();
+    const std::vector<std::vector<uint> > pattern = pattern_pointer->diagonal_pattern(SparsityPattern::sorted);
 
     // Add entries
-    std::vector<Set<uint> >::const_iterator row;
+    std::vector<std::vector<uint> >::const_iterator row;
     Set<uint>::const_iterator element;
     for(row = pattern.begin(); row != pattern.end(); ++row)
       for(element = row->begin(); element != row->end(); ++element)
