@@ -1,13 +1,13 @@
-// Copyright (C) 2006-2008 Garth N. Wells.
+// Copyright (C) 2006-2010 Garth N. Wells.
 // Licensed under the GNU LGPL Version 2.1.
 //
-// Modified by Anders Logg, 2006-2008.
+// Modified by Anders Logg, 2006-2010.
 // Modified by Kent-Andre Mardal, 2008.
 // Modified by Ola Skavhaug, 2008.
 // Modified by Martin Alnæs, 2008.
 //
 // First added:  2006-03-04
-// Last changed: 2009-09-08
+// Last changed: 2011-01-14
 
 #ifndef __UBLAS_VECTOR_H
 #define __UBLAS_VECTOR_H
@@ -74,14 +74,28 @@ namespace dolfin
     /// Resize vector to size N
     virtual void resize(uint N);
 
+    /// Resize vector with given ownership range
+    virtual void resize(std::pair<uint, uint> range);
+
+    /// Resize vector with given ownership range and with ghost values
+    virtual void resize(std::pair<uint, uint> range,
+                        const std::vector<uint>& ghost_indices);
+
     /// Return size of vector
     virtual uint size() const;
 
+    /// Return local size of vector
+    virtual uint local_size() const
+    { return size(); }
+
     /// Return local ownership range of a vector
     virtual std::pair<uint, uint> local_range() const;
 
+    /// Determine whether global vector index is owned by this process
+    virtual bool owns_index(uint i) const;
+
     /// Get block of values
-    virtual void get(double* block, uint m, const uint* rows) const;
+    virtual void get_local(double* block, uint m, const uint* rows) const;
 
     /// Set block of values
     virtual void set(const double* block, uint m, const uint* rows);
@@ -99,12 +113,17 @@ namespace dolfin
     virtual void add_local(const Array<double>& values);
 
     /// Gather entries into local vector x
-    virtual void gather(GenericVector& x, const Array<uint>& indices) const
-    { not_working_in_parallel("uBLASVector::gather)"); }
+    virtual void gather(GenericVector& x, const Array<uint>& indices) const;
+
+    /// Gather entries into Array x
+    virtual void gather(Array<double>& x, const Array<uint>& indices) const;
 
     /// Add multiple of given vector (AXPY operation)
     virtual void axpy(double a, const GenericVector& x);
 
+    /// Replace all entries in the vector by their absolute values
+    virtual void abs();
+
     /// Return inner product with given vector
     virtual double inner(const GenericVector& x) const;
 
