@@ -4,17 +4,17 @@
 // Modified by Ola Skavhaug, 2007, 2009.
 //
 // First added:  2003-03-13
-// Last changed: 2010-04-06
+// Last changed: 2010-09-01
 
-#include <string>
-#include <iostream>
-#include <sstream>
 #include <iomanip>
+#include <iostream>
+#include <string>
+#include <sstream>
 #include <stdexcept>
 
-#include <dolfin/main/MPI.h>
 #include <dolfin/common/constants.h>
 #include <dolfin/common/types.h>
+#include <dolfin/common/MPI.h>
 #include "Table.h"
 #include "Logger.h"
 
