@@ -1,4 +1,4 @@
-// Copyright (C) 2003-2008 Anders Logg.
+// Copyright (C) 2003-2010 Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Thanks to Jim Tilander for many helpful hints.
@@ -7,7 +7,7 @@
 // Modified by Ola Skavhaug, 2009.
 //
 // First added:  2003-03-14
-// Last changed: 2009-04-30
+// Last changed: 2010-06-28
 
 #include <dolfin/common/constants.h>
 #include <dolfin/common/timing.h>
@@ -19,8 +19,9 @@ using namespace dolfin;
 
 //-----------------------------------------------------------------------------
 Progress::Progress(std::string title, unsigned int n)
-  : title(title), n(n), i(0), p_step(0.1), t_step(1.0), p(0), t(0),
-    always(false), finished(false), displayed(false)
+  : title(title), n(n), i(0), p_step(0.1), t_step(0.5), c_step(1),
+    p(0), t(0), tc(0),
+    always(false), finished(false), displayed(false), counter(0)
 {
   if (n <= 0)
     error("Number of steps for progress session must be positive.");
@@ -28,26 +29,28 @@ Progress::Progress(std::string title, unsigned int n)
   // LogManager::logger.progress(title, 0.0);
   t = time();
 
-  // When log level is more than 0, progress is always visible
-  if (LogManager::logger.get_log_level() > 0 )
+  // When log level is TRACE or lower, always display at least the 100% message
+  if (LogManager::logger.get_log_level() <= TRACE )
     always = true;
 }
 //-----------------------------------------------------------------------------
 Progress::Progress(std::string title)
-  : title(title), n(0), i(0), p_step(0.1), t_step(1.0), p(0), t(0),
-    always(false), finished(false), displayed(false)
+  : title(title), n(0), i(0), p_step(0.1), t_step(0.5), p(0), t(0),
+    always(false), finished(false), displayed(false), counter(0)
 {
   // LogManager::logger.progress(title, 0.0);
   t = time();
 
-  // When log level is more than 0, progress is always visible
-  if (LogManager::logger.get_log_level() > 0 )
+  // When log level is TRACE or lower, always display at least the 100% message
+  if (LogManager::logger.get_log_level() <= TRACE )
     always = true;
 }
 //-----------------------------------------------------------------------------
 Progress::~Progress()
 {
-  // Do nothing
+  // Display last progress bar if not displayed
+  if (displayed && !finished)
+    LogManager::logger.progress(title, 1.0);
 }
 //-----------------------------------------------------------------------------
 void Progress::operator=(double p)
@@ -71,25 +74,67 @@ void Progress::operator++(int)
 //-----------------------------------------------------------------------------
 void Progress::update(double p)
 {
-  //p = std::max(std::min(p, 1.0), 0.0);
-  //const bool p_check = p - this->p >= p_step - DOLFIN_EPS;
+  // FIXME: We should be able to simplify this...
 
+  // Check that enough number of updates have passed so we don't call
+  // time() to often which is costly
+  if (counter++ < c_step)
+    return;
+  counter = 0;
+
+  // Check if we have already finished
+  if (finished)
+    return;
+
+  // Check that enough time has passed since last update
   const double t = time();
-  const bool t_check = t - this->t >= t_step - DOLFIN_EPS;
+  const bool t_check = (t - this->t >= t_step - DOLFIN_EPS);
 
-  // Only update when the increase is significant
-  if (t_check || always || (p >= 1.0 && displayed && !finished))
+  // Adjust counter step
+  const bool fraction = 0.1;
+  if (t - this->tc < fraction * t_step)
+    c_step *= 2;
+  else if (t - this->tc > t_step && c_step >= 2)
+    c_step /= 2;
+  this->tc = t;
+
+  // Reset time for next update
+  if (t_check)
   {
-    LogManager::logger.progress(title, p);
     this->p = p;
     this->t = t;
-    always = false;
+  }
+
+  // Assume we want to update the progress
+  bool do_log_update = t_check;
+
+  // Nontrivial check for first update
+  if (t_check && !always && !displayed && p >= 0.7)
+  {
+    // skip the first update, since it will probably reach 100%
+    // before the next time t_check is true
+    do_log_update = false;
+
+    // ...but don't skip the next (pretend we displayed this one)
     displayed = true;
   }
 
-  // Update finished flag
+  // Nontrivial check for last update
   if (p >= 1.0)
-    finished = true;
+  {
+    // always display 100% message if a message has already been shown
+    if (displayed || always)
+      do_log_update = true;
 
+    // ...but don't show more than one
+    finished = true;
+  }
+
+  // Only update when the increase is significant
+  if (do_log_update)
+  {
+    LogManager::logger.progress(title, p);
+    displayed = true;
+  }
 }
 //-----------------------------------------------------------------------------
