@@ -2,38 +2,46 @@
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2003-03-14
-// Last changed: 2010-06-28
+// Last changed: 2010-11-10
 
 #ifndef __PROGRESS_H
 #define __PROGRESS_H
 
 #include <string>
-#include <stdarg.h>
-
 #include <dolfin/common/types.h>
 
 namespace dolfin
 {
 
   /// This class provides a simple way to create and update progress
-  /// bars during a computation. A progress bar may be used either
-  /// in an iteration with a known number of steps:
+  /// bars during a computation.
   ///
-  ///     Progress p("Iterating...", n);
-  ///     for (int i = 0; i < n; i++)
-  ///     {
-  ///       ...
-  ///       p++;
-  ///     }
+  /// *Example*
+  ///     A progress bar may be used either in an iteration with a known number
+  ///     of steps:
   ///
-  /// or in an iteration with an unknown number of steps:
+  ///     .. code-block:: c++
   ///
-  ///     Progress p("Iterating...");
-  ///     while (t < T)
-  ///     {
-  ///       ...
-  ///       p = t / T;
-  ///     }
+  ///         Progress p("Iterating...", n);
+  ///         for (int i = 0; i < n; i++)
+  ///         {
+  ///           ...
+  ///           p++;
+  ///         }
+  ///
+  ///     or in an iteration with an unknown number of steps:
+  ///
+  ///     .. code-block:: c++
+  ///
+  ///         Progress p("Iterating...");
+  ///         while (t < T)
+  ///         {
+  ///           ...
+  ///           p = t / T;
+  ///         }
+
+  // FIXME: Replace implementation with wrapper for boost::progress_display
+  // FIXME: See http://www.boost.org/doc/libs/1_42_0/libs/timer/timer.htm
 
   class Progress
   {
