@@ -1,4 +1,4 @@
-// Copyright (C) 2003-2009 Anders Logg.
+// Copyright (C) 2003-2011 Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Thanks to Jim Tilander for many helpful hints.
@@ -7,17 +7,19 @@
 // Modified by Garth N. Wells, 2009.
 //
 // First added:  2003-03-13
-// Last changed: 2009-12-03
+// Last changed: 2011-01-21
 
-#include <boost/scoped_array.hpp>
 #include <cstdarg>
 #include <cstdlib>
-#include <stdio.h>
 #include <sstream>
+#include <stdarg.h>
+#include <stdio.h>
+
+#include <boost/scoped_array.hpp>
 #include <dolfin/common/types.h>
 #include <dolfin/common/constants.h>
 #include <dolfin/common/Variable.h>
-#include <dolfin/main/MPI.h>
+#include <dolfin/common/MPI.h>
 #include <dolfin/parameter/Parameters.h>
 #include "LogManager.h"
 #include "log.h"
@@ -55,21 +57,24 @@ void allocate_buffer(std::string msg)
 //-----------------------------------------------------------------------------
 void dolfin::info(std::string msg, ...)
 {
-  if (!LogManager::logger.is_active()) return; // optimization
+  if (!LogManager::logger.is_active())
+    return; // optimization
   read(buffer.get(), msg);
   LogManager::logger.info(buffer.get());
 }
 //-----------------------------------------------------------------------------
 void dolfin::info(int log_level, std::string msg, ...)
 {
-  if (!LogManager::logger.is_active()) return; // optimization
+  if (!LogManager::logger.is_active())
+    return; // optimization
   read(buffer.get(), msg);
   LogManager::logger.info(buffer.get(), log_level);
 }
 //-----------------------------------------------------------------------------
 void dolfin::info(const Variable& variable, bool verbose)
 {
-  if (!LogManager::logger.is_active()) return; // optimization
+  if (!LogManager::logger.is_active())
+    return; // optimization
   info(variable.str(verbose));
 }
 //-----------------------------------------------------------------------------
@@ -78,13 +83,15 @@ void dolfin::info(const Parameters& parameters, bool verbose)
   // Need separate function for Parameters since we can't make Parameters
   // a subclass of Variable (gives cyclic dependencies)
 
-  if (!LogManager::logger.is_active()) return; // optimization
+  if (!LogManager::logger.is_active())
+    return; // optimization
   info(parameters.str(verbose));
 }
 //-----------------------------------------------------------------------------
 void dolfin::info_stream(std::ostream& out, std::string msg)
 {
-  if (!LogManager::logger.is_active()) return; // optimization
+  if (!LogManager::logger.is_active())
+    return; // optimization
   std::ostream& old_out = LogManager::logger.get_output_stream();
   LogManager::logger.set_output_stream(out);
   LogManager::logger.info(msg);
@@ -93,14 +100,16 @@ void dolfin::info_stream(std::ostream& out, std::string msg)
 //-----------------------------------------------------------------------------
 void dolfin::info_underline(std:: string msg, ...)
 {
-  if (!LogManager::logger.is_active()) return; // optimization
+  if (!LogManager::logger.is_active())
+    return; // optimization
   read(buffer.get(), msg);
   LogManager::logger.info_underline(buffer.get());
 }
 //-----------------------------------------------------------------------------
 void dolfin::warning(std::string msg, ...)
 {
-  if (!LogManager::logger.is_active()) return; // optimization
+  if (!LogManager::logger.is_active())
+    return; // optimization
   read(buffer.get(), msg);
   LogManager::logger.warning(buffer.get());
 }
@@ -113,7 +122,8 @@ void dolfin::error(std::string msg, ...)
 //-----------------------------------------------------------------------------
 void dolfin::begin(std::string msg, ...)
 {
-  if (!LogManager::logger.is_active()) return; // optimization
+  if (!LogManager::logger.is_active())
+    return; // optimization
   read(buffer.get(), msg);
   LogManager::logger.begin(buffer.get());
 }
@@ -127,7 +137,8 @@ void dolfin::begin(int log_level, std::string msg, ...)
 //-----------------------------------------------------------------------------
 void dolfin::end()
 {
-  if (!LogManager::logger.is_active()) return; // optimization
+  if (!LogManager::logger.is_active())
+    return; // optimization
   LogManager::logger.end();
 }
 //-----------------------------------------------------------------------------
@@ -141,6 +152,11 @@ void dolfin::set_log_level(int level)
   LogManager::logger.set_log_level(level);
 }
 //-----------------------------------------------------------------------------
+void dolfin::set_output_stream(std::ostream& out)
+{
+  LogManager::logger.set_output_stream(out);
+}
+//-----------------------------------------------------------------------------
 int dolfin::get_log_level()
 {
   return LogManager::logger.get_log_level();
