@@ -4,7 +4,7 @@
 // Modified by Ola Skavhaug, 2007, 2009.
 //
 // First added:  2003-03-13
-// Last changed: 2010-06-07
+// Last changed: 2011-01-24
 
 #ifndef __LOG_H
 #define __LOG_H
@@ -67,6 +67,9 @@ namespace dolfin
   /// Set log level
   void set_log_level(int level);
 
+  /// Set output stream
+  void set_output_stream(std::ostream& out);
+
   /// Get log level
   int get_log_level();
 
@@ -87,6 +90,10 @@ namespace dolfin
 
 }
 
+// The following two macros are the only "functions" in DOLFIN
+// named dolfin_foo. Other functions can be placed inside the
+// DOLFIN namespace and therefore don't require a prefix.
+
 // Debug macros (with varying number of arguments)
 #define dolfin_debug(msg)              do { dolfin::__debug(__FILE__, __LINE__, __FUNCTION__, msg); } while (false)
 #define dolfin_debug1(msg, a0)         do { dolfin::__debug(__FILE__, __LINE__, __FUNCTION__, msg, a0); } while (false)
