@@ -1,19 +1,22 @@
-// Copyright (C) 2006-2009 Anders Logg.
+// Copyright (C) 2006-2010 Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Johan Hoffman 2006.
 // Modified by Andre Massing 2009.
+// Modified by Garth N. Wells 2010.
 //
 // First added:  2006-06-01
-// Last changed: 2010-02-11
+// Last changed: 2010-12-06
 
 #ifndef __CELL_H
 #define __CELL_H
 
-#include "Point.h"
 #include "CellType.h"
+#include "Mesh.h"
 #include "MeshEntity.h"
 #include "MeshEntityIterator.h"
+#include "MeshFunction.h"
+#include "Point.h"
 
 namespace dolfin
 {
@@ -28,44 +31,46 @@ namespace dolfin
     Cell() : MeshEntity() {}
 
     /// Create cell on given mesh with given index
-    Cell(const Mesh& mesh, uint index) : MeshEntity(mesh, mesh.topology().dim(), index) {}
+    Cell(const Mesh& mesh, uint index)
+      : MeshEntity(mesh, mesh.topology().dim(), index) {}
 
     /// Destructor
     ~Cell() {}
 
     /// Return type of cell
-    inline CellType::Type type() const { return _mesh->type().cell_type(); }
+    CellType::Type type() const
+    { return _mesh->type().cell_type(); }
 
     /// Compute orientation of cell (0 is right, 1 is left)
-    inline double orientation() const
+    double orientation() const
     { return _mesh->type().orientation(*this); }
 
     /// Compute (generalized) volume of cell
-    inline double volume() const
+    double volume() const
     { return _mesh->type().volume(*this); }
 
     /// Compute diameter of cell
-    inline double diameter() const
+    double diameter() const
     { return _mesh->type().diameter(*this); }
 
     /// Compute component i of normal of given facet with respect to the cell
-    inline double normal(uint facet, uint i) const
+    double normal(uint facet, uint i) const
     { return _mesh->type().normal(*this, facet, i); }
 
     /// Compute normal of given facet with respect to the cell
-    inline Point normal(uint facet) const
+    Point normal(uint facet) const
     { return _mesh->type().normal(*this, facet); }
 
     /// Compute the area/length of given facet with respect to the cell
-    inline double facet_area(uint facet) const
+    double facet_area(uint facet) const
     { return _mesh->type().facet_area(*this, facet); }
 
     /// Order entities locally
-    inline void order(MeshFunction<uint>* global_vertex_indices)
+    void order(MeshFunction<uint>* global_vertex_indices)
     { _mesh->type().order(*this, global_vertex_indices); }
 
     /// Check if entities are ordered
-    inline bool ordered(MeshFunction<uint>* global_vertex_indices)
+    bool ordered(MeshFunction<uint>* global_vertex_indices)
     { return _mesh->type().ordered(*this, global_vertex_indices); }
 
   };
@@ -84,6 +89,19 @@ namespace dolfin
 
   };
 
+  /// A CellFunction is a MeshFunction of topological codimension 0.
+  template <class T> class CellFunction : public MeshFunction<T>
+  {
+  public:
+
+    CellFunction(const Mesh& mesh)
+      : MeshFunction<T>(mesh, mesh.topology().dim()) {}
+
+    CellFunction(const Mesh& mesh, const T& value)
+      : MeshFunction<T>(mesh, mesh.topology().dim(), value) {}
+
+  };
+
 }
 
 #endif
