@@ -4,8 +4,10 @@
 // First added:  2010-02-08
 // Last changed: 2010-02-08
 
-#include "MeshFunction.h"
 #include "Facet.h"
+#include "Mesh.h"
+#include "MeshEntity.h"
+#include "MeshFunction.h"
 #include "FacetCell.h"
 
 using namespace dolfin;
