@@ -1,56 +1,69 @@
-// Copyright (C) 2006-2009 Anders Logg.
+// Copyright (C) 2006-2011 Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Johan Hoffman, 2007.
-// Modified by Garth N. Wells 2007.
+// Modified by Garth N. Wells 2007-2011.
 // Modified by Niclas Jansson 2008.
 // Modified by Kristoffer Selim 2008.
 // Modified by Andre Massing, 2009-2010.
 //
 // First added:  2006-05-09
-// Last changed: 2010-04-29
+// Last changed: 2011-02-07
 
-#include <sstream>
-
-#include <dolfin/log/log.h>
-#include <dolfin/io/File.h>
-#include <dolfin/main/MPI.h>
 #include <dolfin/ale/ALE.h>
-#include <dolfin/io/File.h>
 #include <dolfin/common/Timer.h>
+#include <dolfin/common/UniqueIdGenerator.h>
 #include <dolfin/common/utils.h>
-#include "IntersectionOperator.h"
-#include "TopologyComputation.h"
-#include "MeshSmoothing.h"
-#include "MeshOrdering.h"
-#include "MeshFunction.h"
-#include "LocalMeshData.h"
-#include "MeshPartitioning.h"
+#include <dolfin/io/File.h>
+#include <dolfin/log/log.h>
+#include <dolfin/common/MPI.h>
 #include "BoundaryMesh.h"
 #include "Cell.h"
+#include "IntersectionOperator.h"
+#include "LocalMeshData.h"
+#include "MeshColoring.h"
+#include "MeshData.h"
+#include "MeshFunction.h"
+#include "MeshOrdering.h"
+#include "MeshPartitioning.h"
+#include "MeshRenumbering.h"
+#include "MeshSmoothing.h"
+#include "ParallelData.h"
+#include "TopologyComputation.h"
 #include "Vertex.h"
 #include "Mesh.h"
 
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
-Mesh::Mesh()
-  : Variable("mesh", "DOLFIN mesh"),
-    _data(*this), _cell_type(0), _intersection_operator(*this), _ordered(false)
+Mesh::Mesh() : Variable("mesh", "DOLFIN mesh"),
+               Hierarchical<Mesh>(*this),
+               _data(*this),
+               _cell_type(0),
+               unique_id(UniqueIdGenerator::id()),
+               _intersection_operator(*this),
+               _ordered(false)
 {
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-Mesh::Mesh(const Mesh& mesh)
-  : Variable("mesh", "DOLFIN mesh"),
-    _data(*this), _cell_type(0), _intersection_operator(*this), _ordered(false)
+Mesh::Mesh(const Mesh& mesh) : Variable("mesh", "DOLFIN mesh"),
+                               Hierarchical<Mesh>(*this),
+                               _data(*this),
+                               _cell_type(0),
+                               unique_id(UniqueIdGenerator::id()),
+                               _intersection_operator(*this),
+                               _ordered(false)
 {
   *this = mesh;
 }
 //-----------------------------------------------------------------------------
-Mesh::Mesh(std::string filename)
-  : Variable("mesh", "DOLFIN mesh"),
-    _data(*this), _cell_type(0), _intersection_operator(*this), _ordered(false)
+Mesh::Mesh(std::string filename) : Variable("mesh", "DOLFIN mesh"),
+                                   Hierarchical<Mesh>(*this),
+                                   _data(*this), _cell_type(0),
+                                   unique_id(UniqueIdGenerator::id()),
+                                   _intersection_operator(*this),
+                                   _ordered(false)
 {
   if (MPI::num_processes() > 1)
   {
@@ -78,20 +91,53 @@ Mesh::~Mesh()
 //-----------------------------------------------------------------------------
 const Mesh& Mesh::operator=(const Mesh& mesh)
 {
+  // Clear all data
   clear();
 
+  // Assign data
   _topology = mesh._topology;
   _geometry = mesh._geometry;
   _data = mesh._data;
-
   if (mesh._cell_type)
     _cell_type = CellType::create(mesh._cell_type->cell_type());
 
+  // Rename
   rename(mesh.name(), mesh.label());
 
-  _ordered = mesh._ordered;
+  // Call assignment operator for base class
+  Hierarchical<Mesh>::operator=(mesh);
 
-   return *this;
+  return *this;
+}
+//-----------------------------------------------------------------------------
+MeshData& Mesh::data()
+{
+  return _data;
+}
+//-----------------------------------------------------------------------------
+const MeshData& Mesh::data() const
+{
+  return _data;
+}
+//-----------------------------------------------------------------------------
+ParallelData& Mesh::parallel_data()
+{
+  // Create if not already created
+  if (!_parallel_data)
+    _parallel_data.reset(new ParallelData(*this));
+
+  // Return data
+  return *_parallel_data;
+}
+//-----------------------------------------------------------------------------
+const ParallelData& Mesh::parallel_data() const
+{
+  // Check that data exists
+  if (!_parallel_data)
+    error("Parallel data not available.");
+
+  // Return data
+  return *_parallel_data;
 }
 //-----------------------------------------------------------------------------
 dolfin::uint Mesh::init(uint dim) const
@@ -171,6 +217,19 @@ void Mesh::clear()
   _ordered = false;
 }
 //-----------------------------------------------------------------------------
+void Mesh::clean()
+{
+  const uint D = topology().dim();
+  for (uint d0 = 0; d0 <= D; d0++)
+  {
+    for (uint d1 = 0; d1 <= D; d1++)
+    {
+      if (!(d0 == D && d1 == 0))
+        _topology.clear(d0, d1);
+    }
+  }
+}
+//-----------------------------------------------------------------------------
 void Mesh::order()
 {
   // Order mesh
@@ -190,34 +249,18 @@ bool Mesh::ordered() const
   return _ordered;
 }
 //-----------------------------------------------------------------------------
-/*
-void Mesh::coarsen()
+void Mesh::renumber_by_color()
 {
-  // FIXME: Move implementation to separate class and just call function here
+  // Define graph type
+  std::vector<uint> coloring_type;
+  coloring_type.push_back(topology().dim());
+  coloring_type.push_back(0);
+  coloring_type.push_back(topology().dim());
 
-  info(DBG, "No cells marked for coarsening, assuming uniform mesh coarsening.");
-  MeshFunction<bool> cell_marker(*this);
-  cell_marker.init(this->topology().dim());
-  for (CellIterator c(*this); !c.end(); ++c)
-    cell_marker[c->index()] = true;
-
-  LocalMeshCoarsening::coarsen_mesh_by_edge_collapse(*this, cell_marker);
-
-  // Mesh may not be ordered
-  _ordered = false;
+  // Renumber by color
+  MeshRenumbering::renumber_by_color(*this, coloring_type);
 }
 //-----------------------------------------------------------------------------
-void Mesh::coarsen(MeshFunction<bool>& cell_markers, bool coarsen_boundary)
-{
-  LocalMeshCoarsening::coarsen_mesh_by_edge_collapse(*this,
-                                                     cell_markers,
-                                                     coarsen_boundary);
-
-  // Mesh may not be ordered
-  _ordered = false;
-}
-*/
-//-----------------------------------------------------------------------------
 void Mesh::move(BoundaryMesh& boundary, ALEType method)
 {
   ALE::move(*this, boundary, method);
@@ -248,40 +291,78 @@ void Mesh::snap_boundary(const SubDomain& sub_domain, bool harmonic_smoothing)
   MeshSmoothing::snap_boundary(*this, sub_domain, harmonic_smoothing);
 }
 //-----------------------------------------------------------------------------
-void Mesh::all_intersected_entities(const Point & point, uint_set & ids_result) const
+const dolfin::MeshFunction<dolfin::uint>& Mesh::color(std::string coloring_type) const
+{
+  // Define graph type
+  const uint dim = MeshColoring::type_to_dim(coloring_type, *this);
+  std::vector<uint> _coloring_type;
+  _coloring_type.push_back(topology().dim());
+  _coloring_type.push_back(dim);
+  _coloring_type.push_back(topology().dim());
+
+  return color(_coloring_type);
+}
+//-----------------------------------------------------------------------------
+const dolfin::MeshFunction<dolfin::uint>& Mesh::color(std::vector<uint> coloring_type) const
+{
+  // Find color data
+  std::map<const std::vector<uint>, std::pair<MeshFunction<uint>,
+           std::vector<std::vector<uint> > > >::const_iterator coloring_data;
+  coloring_data = this->data().coloring.find(coloring_type);
+
+  if (coloring_data != this->data().coloring.end())
+  {
+    dolfin_debug("Mesh has already been colored, not coloring again.");
+    return coloring_data->second.first;
+  }
+
+  // We do the same const-cast trick here as in the init() functions
+  // since we are not really changing the mesh, just attaching some
+  // auxiliary data to it.
+  Mesh* _mesh = const_cast<Mesh*>(this);
+  return MeshColoring::color(*_mesh, coloring_type);
+}
+//-----------------------------------------------------------------------------
+void Mesh::all_intersected_entities(const Point& point,
+                                    uint_set& ids_result) const
 {
   _intersection_operator.all_intersected_entities(point, ids_result);
 }
 //-----------------------------------------------------------------------------
-void Mesh::all_intersected_entities(const std::vector<Point> & points, uint_set & ids_result) const
+void Mesh::all_intersected_entities(const std::vector<Point>& points,
+                                    uint_set& ids_result) const
 {
   _intersection_operator.all_intersected_entities(points, ids_result);
 }
 //-----------------------------------------------------------------------------
-void Mesh::all_intersected_entities(const MeshEntity & entity, std::vector<uint> & ids_result) const
+void Mesh::all_intersected_entities(const MeshEntity & entity,
+                                    std::vector<uint>& ids_result) const
 {
   _intersection_operator.all_intersected_entities(entity, ids_result);
 }
 //-----------------------------------------------------------------------------
-void Mesh::all_intersected_entities(const std::vector<MeshEntity> & entities, uint_set & ids_result) const
+void Mesh::all_intersected_entities(const std::vector<MeshEntity>& entities,
+                                    uint_set& ids_result) const
 {
   _intersection_operator.all_intersected_entities(entities, ids_result);
 }
 //-----------------------------------------------------------------------------
-void Mesh::all_intersected_entities(const Mesh & another_mesh, uint_set & ids_result) const
+void Mesh::all_intersected_entities(const Mesh& another_mesh,
+                                    uint_set& ids_result) const
 {
   _intersection_operator.all_intersected_entities(another_mesh, ids_result);
 }
 //-----------------------------------------------------------------------------
-int Mesh::any_intersected_entity(const Point & point) const
+int Mesh::any_intersected_entity(const Point& point) const
 {
   return _intersection_operator.any_intersected_entity(point);
 }
 //-----------------------------------------------------------------------------
-Point Mesh::closest_point(const Point & point) const
+Point Mesh::closest_point(const Point& point) const
 {
   return _intersection_operator.closest_point(point);
 }
+//-----------------------------------------------------------------------------
 dolfin::uint Mesh::closest_cell(const Point & point) const
 {
   return _intersection_operator.closest_cell(point);
@@ -330,8 +411,8 @@ std::string Mesh::str(bool verbose) const
   {
     s << str(false) << std::endl << std::endl;
 
-    s << indent(_topology.str(true));
     s << indent(_geometry.str(true));
+    s << indent(_topology.str(true));
     s << indent(_data.str(true));
   }
   else
