@@ -2,16 +2,21 @@
 // Licensed under the GNU LGPL Version 2.1.
 //
 // Modified by Niclas Jansson, 2008.
+// Modified by Garth N. Wells, 2011.
 //
 // First added:  2008-05-19
-// Last changed: 2010-04-07
+// Last changed: 2011-01-17
 
 #ifndef __MESH_DATA_H
 #define __MESH_DATA_H
 
 #include <map>
-#include <dolfin/common/Variable.h>
+#include <string>
+#include <utility>
+#include <vector>
+
 #include <dolfin/common/types.h>
+#include <dolfin/common/Variable.h>
 
 namespace dolfin
 {
@@ -56,7 +61,13 @@ namespace dolfin
   ///
   /// Sub meshes
   ///
-  ///    "global vertex indices" - MeshFunction<uint> of dimension 0
+  ///   "global vertex indices" - MeshFunction<uint> of dimension 0
+  ///
+  /// Mesh coloring
+  ///
+  ///   "colors-%D-%d-%1"   - MeshFunction<uint> of dimension D with colors based on connectivity %d
+  ///   "num colored cells" - Array<uint> listing the number of cells of each color
+  ///   "colored cells %d"  - Array<uint> of cell indices with colors 0, 1, 2, ...
 
   class MeshData : public Variable
   {
@@ -82,6 +93,9 @@ namespace dolfin
     /// Create MeshFunction with given name and dimension
     MeshFunction<uint>* create_mesh_function(std::string name, uint dim);
 
+    /// Create empty array (vector) with given name
+    std::vector<uint>* create_array(std::string name);
+
     /// Create array (vector) with given name and size
     std::vector<uint>* create_array(std::string name, uint size);
 
@@ -99,6 +113,9 @@ namespace dolfin
     /// Return array with given name (returning zero if data is not available)
     std::vector<uint>* array(const std::string name) const;
 
+    /// Return array with given name postfixed by " %d" (returning zero if data is not available)
+    std::vector<uint>* array(const std::string name, uint number) const;
+
     /// Return mapping with given name (returning zero if data is not available)
     std::map<uint, uint>* mapping(const std::string name) const;
 
@@ -124,6 +141,16 @@ namespace dolfin
     /// Return informal string representation (pretty-print)
     std::string str(bool verbose) const;
 
+    //--- Coloring data ---
+
+    /// First vector is (colored entity dim - dim0 - .. -  colored entity dim).
+    /// MeshFunction stores mesh entity colors and the vector<vector> a list
+    /// of all mesh entity indices of the same color,
+    /// e.g. vector<vector>[color][i] is the index of the ith entity of
+    /// color 'color'.
+    std::map<const std::vector<uint>,
+             std::pair<MeshFunction<uint>, std::vector<std::vector<uint> > > > coloring;
+
     /// Friends
     friend class XMLFile;
     friend class XMLMeshData;
