@@ -2,11 +2,12 @@
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2006-05-16
-// Last changed: 2009-10-08
+// Last changed: 2011-02-09
 
 #include <dolfin/log/dolfin_log.h>
 #include <dolfin/parameter/dolfin_parameter.h>
 #include "Mesh.h"
+#include "MeshEntity.h"
 #include "MeshFunction.h"
 #include "Point.h"
 #include "MeshEditor.h"
@@ -130,7 +131,7 @@ void MeshEditor::init_cells(uint num_cells)
   mesh->_topology(tdim, 0).init(num_cells, mesh->type().num_vertices(tdim));
 }
 //-----------------------------------------------------------------------------
-void MeshEditor::init_higher_order_cells(uint num_higher_order_cells, 
+void MeshEditor::init_higher_order_cells(uint num_higher_order_cells,
                                          uint num_higher_order_cell_dof)
 {
   // Check if we are currently editing a mesh
@@ -172,7 +173,7 @@ void MeshEditor::add_vertex(uint v, double x)
   add_vertex_common(v, 1);
 
   // Set coordinate, next_vertex doesn't seem to work right
-//  mesh->_geometry.set(next_vertex, 0, x);
+  // mesh->_geometry.set(next_vertex, 0, x);
 
   // Set coordinate
   mesh->_geometry.set(v, 0, x);
@@ -241,6 +242,10 @@ void MeshEditor::add_higher_order_vertex(uint v, double x, double y, double z)
 //-----------------------------------------------------------------------------
 void MeshEditor::add_cell(uint c, const std::vector<uint>& v)
 {
+  // Check vertices
+  for (uint i = 0; i < v.size(); i++)
+    check_vertex(v[i]);
+
   // Add cell
   add_cell_common(c, tdim);
 
@@ -250,6 +255,10 @@ void MeshEditor::add_cell(uint c, const std::vector<uint>& v)
 //-----------------------------------------------------------------------------
 void MeshEditor::add_cell(uint c, uint v0, uint v1)
 {
+  // Check vertices
+  //check_vertex(v0);
+  //check_vertex(v1);
+
   // Add cell
   add_cell_common(c, 1);
 
@@ -261,6 +270,11 @@ void MeshEditor::add_cell(uint c, uint v0, uint v1)
 //-----------------------------------------------------------------------------
 void MeshEditor::add_cell(uint c, uint v0, uint v1, uint v2)
 {
+  // Check vertices
+  check_vertex(v0);
+  check_vertex(v1);
+  check_vertex(v2);
+
   // Add cell
   add_cell_common(c, 2);
 
@@ -273,6 +287,12 @@ void MeshEditor::add_cell(uint c, uint v0, uint v1, uint v2)
 //-----------------------------------------------------------------------------
 void MeshEditor::add_cell(uint c, uint v0, uint v1, uint v2, uint v3)
 {
+  // Check vertices
+  check_vertex(v0);
+  check_vertex(v1);
+  check_vertex(v2);
+  check_vertex(v3);
+
   // Add cell
   add_cell_common(c, 3);
 
@@ -284,9 +304,18 @@ void MeshEditor::add_cell(uint c, uint v0, uint v1, uint v2, uint v3)
   mesh->_topology(tdim, 0).set(c, vertices);
 }
 //-----------------------------------------------------------------------------
-void MeshEditor::add_higher_order_cell_data(uint c, uint v0, uint v1, uint v2,
+void MeshEditor::add_higher_order_cell_data(uint c,
+                                            uint v0, uint v1, uint v2,
                                             uint v3, uint v4, uint v5)
 {
+  // Check vertices
+  check_vertex(v0);
+  check_vertex(v1);
+  check_vertex(v2);
+  check_vertex(v3);
+  check_vertex(v4);
+  check_vertex(v5);
+
   // Add cell
   add_higher_order_cell_common(c, 6);
 
@@ -416,7 +445,7 @@ void MeshEditor::compute_boundary_indicators()
 {
   // Do nothing if mesh function "exterior facet domains" is present
   if (mesh->data().mesh_function("exterior facet domains"))
-    return; 
+    return;
 
   // Extract data for boundary indicators
   std::vector<uint>* facet_cells   = mesh->data().array("boundary facet cells");
@@ -452,10 +481,10 @@ void MeshEditor::compute_boundary_indicators()
     // Get cell index and local facet index
     const uint cell_index = (*facet_cells)[i];
     const uint local_facet = (*facet_numbers)[i];
-    
+
     // Get global facet index
     const uint global_facet = mesh->_topology(D, D - 1)(cell_index)[local_facet];
-        
+
     // Set boundary indicator for facet
     (*exterior_facet_domains)[global_facet] = (*indicators)[i];
   }
@@ -478,3 +507,9 @@ void MeshEditor::clear()
   higher_order_cell_data.clear();
 }
 //-----------------------------------------------------------------------------
+void MeshEditor::check_vertex(uint v)
+{
+  if (num_vertices > 0 && v >= num_vertices)
+    error("Vertex index (%d) out of range [0, %d].", v, num_vertices - 1);
+}
+//-----------------------------------------------------------------------------
