@@ -4,7 +4,7 @@
 // Modified by Andre Massing, 2009.
 //
 // First added:  2006-05-11
-// Last changed: 2010-03-05
+// Last changed: 2010-11-17
 
 #ifndef __MESH_ENTITY_H
 #define __MESH_ENTITY_H
@@ -17,22 +17,25 @@
 
 #include <dolfin/common/types.h>
 #include <dolfin/log/dolfin_log.h>
-#include "Point.h"
 #include "Mesh.h"
+#include "MeshTopology.h"
+#include "Point.h"
 #include "PrimitiveIntersector.h"
 
 namespace dolfin
 {
 
+  //class Mesh;
 
   /// A MeshEntity represents a mesh entity associated with
   /// a specific topological dimension of some mesh.
+
   class MeshEntity
   {
   public:
 
     /// Default Constructor
-    MeshEntity() :_mesh(0), _dim(0), _index(0) {}
+    MeshEntity() : _mesh(0), _dim(0), _index(0) {}
 
     /// Constructor
     MeshEntity(const Mesh& mesh, uint dim, uint index);
@@ -67,6 +70,10 @@ namespace dolfin
     const uint* entities(uint dim) const
     { return _mesh->topology()(_dim, dim)(_index); }
 
+    /// Return unique mesh ID
+    uint mesh_id() const
+    { return _mesh->id(); }
+
     /// Check if given entity is indicent
     bool incident(const MeshEntity& entity) const;
 
@@ -92,10 +99,10 @@ namespace dolfin
     /// Compute midpoint of cell
     Point midpoint() const;
 
-#ifdef HAS_CGAL
+    #ifdef HAS_CGAL
     ///Returns a 3D bounding box of the mesh entity. For lower dimension it may be a degenerated box.
     template <typename K> CGAL::Bbox_3 bbox() const;
-#endif
+    #endif
 
     // Note: Not a subclass of Variable for efficiency!
     /// Return informal string representation (pretty-print)
@@ -105,6 +112,7 @@ namespace dolfin
 
     // Friends
     friend class MeshEntityIterator;
+    friend class SubsetIterator;
 
     // The mesh
     Mesh const * _mesh;
