@@ -1,4 +1,4 @@
-// Copyright (C) 2009 Andre Massing 
+// Copyright (C) 2009 Andre Massing
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2009-09-11
@@ -21,13 +21,14 @@ namespace internal {
 
   template <class K>
   inline
-  bool do_intersect(const typename K::Point_3& pt,
-		    const CGAL::Bbox_3& bbox,
-		    const K &)
+  bool do_intersect(const typename K::Point_3& pt, const CGAL::Bbox_3& bbox,
+                    const K &)
   {
     for(int i = 0; i < 3; ++i)
-      if(bbox.max(i) < pt[i] || bbox.min(i) > pt[i])
-	return false;
+    {
+      if (bbox.max(i) < pt[i] || bbox.min(i) > pt[i])
+        return false;
+    }
     return true;
   }
 
@@ -38,36 +39,40 @@ namespace internal {
 		    const K &)
   {
     for(int i = 0; i < 3; ++i)
+    {
       if(bbox.max(i) < pt[i] || bbox.min(i) > pt[i])
-	return false;
+	      return false;
+    }
     return true;
   }
 
-template <class K>
-inline
-Object
-intersection(const typename K::Point_3 &pt, 
-	     const typename K::Bbox_3 &bbox, 
-	     const K&)
-{
-    if (do_intersect(pt,bbox)) {
-        return make_object(pt);
+  template <class K>
+  inline
+  Object
+  intersection(const typename K::Point_3 &pt,
+	       const typename K::Bbox_3 &bbox,
+	       const K&)
+  {
+    if (do_intersect(pt, bbox))
+    {
+      return make_object(pt);
     }
     return Object();
-}
+  }
 
-template <class K>
-inline
-Object
-intersection( const typename K::Bbox_3 &bbox, 
-	      const typename K::Point_3 &pt, 
-	      const K&)
-{
-    if (do_intersect(pt,bbox)) {
-        return make_object(pt);
+  template <class K>
+  inline
+  Object
+  intersection(const typename K::Bbox_3 &bbox,
+	        const typename K::Point_3 &pt,
+	        const K&)
+  {
+    if (do_intersect(pt, bbox))
+    {
+      return make_object(pt);
     }
     return Object();
-}
+  }
 
 
 } //namespace CGALi
@@ -78,7 +83,6 @@ bool do_intersect(const CGAL::Point_3<K>& point,
 		  const CGAL::Bbox_3& bbox)
 {
   return typename K::Do_intersect_3()(point, bbox);
-//  return CGALi::do_intersect(bbox, point, K());
 }
 
 template <class K>
@@ -87,7 +91,6 @@ bool do_intersect(const CGAL::Bbox_3& bbox,
 		  const CGAL::Point_3<K>& point)
 {
   return typename K::Do_intersect_3()(point, bbox);
-//  return CGALi::do_intersect(bbox, point, K());
 }
 
 template <class K>
