@@ -1,11 +1,11 @@
-// Copyright (C) 2009 Andre Massing 
+// Copyright (C) 2009 Andre Massing
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2009-09-11
 // Last changed: 2010-04-06
 
 #ifndef  CGAL_POINT_3_POINT_3_INTERSECTION_H
-#define	 CGAL_POINT_3_POINT_3_INTERSECTION_H    
+#define	 CGAL_POINT_3_POINT_3_INTERSECTION_H
 
 #include <CGAL/Point_3.h>
 #include <CGAL/Object.h>
@@ -23,7 +23,7 @@ namespace internal {
 #endif
 
   template <class K >
-  inline bool do_intersect(const typename K::Point_3 & pt1, 
+  inline bool do_intersect(const typename K::Point_3 & pt1,
 		    const typename K::Point_3 & pt2,
 		    const K & k)
   {
@@ -32,10 +32,11 @@ namespace internal {
 
   template <class K>
   Object
-  intersection(const typename K::Point_3 &pt1, 
+  intersection(const typename K::Point_3 &pt1,
 	       const typename K::Point_3 &pt2)
   {
-    if (pt1 == pt2) {
+    if (pt1 == pt2)
+    {
       return make_object(pt1);
     }
     return Object();
@@ -45,7 +46,7 @@ namespace internal {
 }// namespace CGALi
 
 template <class K>
-inline 
+inline
 bool
 do_intersect(const Point_3<K> &pt1, const Point_3<K> &pt2)
 {
