@@ -1,4 +1,4 @@
-// Copyright (C) 2009 Andre Massing 
+// Copyright (C) 2009 Andre Massing
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2009-09-11
@@ -19,55 +19,54 @@ namespace CGALi {
 namespace internal {
 #endif
 
-template <class K>
-inline 
-bool
-do_intersect(const typename K::Point_3 &pt, 
-	     const typename K::Ray_3 &ray,
-	     const K&)
-{
-  return ray.has_on(pt);
-}
-
-
-template <class K>
-inline 
-bool
-do_intersect(const typename K::Ray_3 &ray,
-	     const typename K::Point_3 &pt, 
-	     const K&)
-{
-  return ray.has_on(pt);
-}
-
-
-template <class K>
-Object
-intersection(const typename K::Point_3 &pt, 
-	     const typename K::Ray_3 &ray,
-	     const K& k)
-{
-  if (do_intersect(pt,ray, k)) {
-    return make_object(pt);
+  template <class K>
+  inline
+  bool
+  do_intersect(const typename K::Point_3 &pt,
+	       const typename K::Ray_3 &ray,
+	       const K&)
+  {
+    return ray.has_on(pt);
   }
-  return Object();
-}
 
-template <class K>
-Object
-intersection(const typename K::Ray_3 &ray,
-	     const typename K::Point_3 &pt, 
-	     const K& k)
-{
-  if (do_intersect(pt,ray, k)) {
-    return make_object(pt);
+  template <class K>
+  inline
+  bool
+  do_intersect(const typename K::Ray_3 &ray,
+	       const typename K::Point_3 &pt,
+	       const K&)
+  {
+    return ray.has_on(pt);
+  }
+
+  template <class K>
+  Object
+  intersection(const typename K::Point_3 &pt,
+	       const typename K::Ray_3 &ray,
+	       const K& k)
+  {
+    if (do_intersect(pt, ray, k))
+    {
+      return make_object(pt);
+    }
+    return Object();
+  }
+
+  template <class K>
+  Object
+  intersection(const typename K::Ray_3 &ray,
+	       const typename K::Point_3 &pt,
+	       const K& k)
+  {
+    if (do_intersect(pt,ray, k))
+    {
+      return make_object(pt);
+    }
+    return Object();
   }
-  return Object();
-}
 
 } // namespace CGALi
 
-
 template <class K>
 inline
 bool
