@@ -1,4 +1,4 @@
-// Copyright (C) 2009 Andre Massing 
+// Copyright (C) 2009 Andre Massing
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2009-09-11
@@ -19,52 +19,54 @@ namespace CGALi {
 namespace internal {
 #endif
 
-template <class K>
-inline 
-bool
-do_intersect(const typename K::Point_3 &pt, 
-	     const typename K::Segment_3 &seg,
-	     const K&)
-{
+  template <class K>
+  inline
+  bool
+  do_intersect(const typename K::Point_3 &pt,
+	       const typename K::Segment_3 &seg,
+	       const K&)
+  {
     return seg.has_on(pt);
-}
+  }
 
-template <class K>
-inline 
-bool
-do_intersect(const typename K::Segment_3 &seg,
-	     const typename K::Point_3 &pt, 
-	     const K&)
-{
+  template <class K>
+  inline
+  bool
+  do_intersect(const typename K::Segment_3 &seg,
+	       const typename K::Point_3 &pt,
+	       const K&)
+  {
     return seg.has_on(pt);
-}
+  }
 
 
-template <class K>
-inline
-Object
-intersection(const typename K::Point_3 &pt, 
-	     const typename K::Segment_3 &seg, 
-	     const K&)
-{
-    if (do_intersect(pt,seg)) {
-        return make_object(pt);
+  template <class K>
+  inline
+  Object
+  intersection(const typename K::Point_3 &pt,
+	       const typename K::Segment_3 &seg,
+	       const K&)
+  {
+    if (do_intersect(pt,seg))
+    {
+      return make_object(pt);
     }
     return Object();
-}
+  }
 
-template <class K>
-inline
-Object
-intersection( const typename K::Segment_3 &seg, 
-	      const typename K::Point_3 &pt, 
-	      const K&)
-{
-    if (do_intersect(pt,seg)) {
-        return make_object(pt);
+  template <class K>
+  inline
+  Object
+  intersection( const typename K::Segment_3 &seg,
+	        const typename K::Point_3 &pt,
+	        const K&)
+  {
+    if (do_intersect(pt,seg))
+    {
+      return make_object(pt);
     }
     return Object();
-}
+  }
 
 } // namespace CGALi
 
@@ -81,7 +83,7 @@ template <class K>
 inline bool
 do_intersect(const Point_3<K> &pt, const Segment_3<K> &seg)
 {
-  typedef typename K::Do_intersect_3 Do_intersect;
+    typedef typename K::Do_intersect_3 Do_intersect;
     return Do_intersect()(pt, seg);
 }
 
@@ -99,7 +101,7 @@ inline Object
 intersection(const Point_3<K> &pt, const Segment_3<K> &seg)
 {
   typedef typename K::Intersect_3 Intersect;
-    return Intersect()(pt, seg);
+  return Intersect()(pt, seg);
 }
 
 CGAL_END_NAMESPACE
