@@ -4,14 +4,16 @@
 // Modified by Niclas Jansson 2009.
 //
 // First added:  2007-04-24
-// Last changed: 2010-02-11
+// Last changed: 2011-01-25
 
 #include <dolfin/common/Array.h>
 #include <dolfin/log/log.h>
+#include "Mesh.h"
 #include "MeshData.h"
+#include "MeshEntity.h"
 #include "MeshEntityIterator.h"
-#include "Vertex.h"
 #include "SubDomain.h"
+#include "Vertex.h"
 
 using namespace dolfin;
 
@@ -37,7 +39,38 @@ void SubDomain::map(const Array<double>& x, Array<double>&) const
   error("Mapping between subdomains missing for periodic boundary conditions, function map() not implemented by user.");
 }
 //-----------------------------------------------------------------------------
+/// Set sub domain markers (uint) for given subdomain
 void SubDomain::mark(MeshFunction<uint>& sub_domains, uint sub_domain) const
+{ 
+  mark_meshfunction(sub_domains, sub_domain); 
+}
+//-----------------------------------------------------------------------------
+void SubDomain::mark(MeshFunction<int>& sub_domains, int sub_domain) const
+{
+  mark_meshfunction(sub_domains, sub_domain); 
+}
+//-----------------------------------------------------------------------------
+void SubDomain::mark(MeshFunction<double>& sub_domains, double sub_domain) const
+{ 
+  mark_meshfunction(sub_domains, sub_domain); 
+}
+//-----------------------------------------------------------------------------
+void SubDomain::mark(MeshFunction<bool>& sub_domains, bool sub_domain) const
+{ 
+  mark_meshfunction(sub_domains, sub_domain); 
+}
+//-----------------------------------------------------------------------------
+dolfin::uint SubDomain::geometric_dimension() const
+{
+  // Check that dim has been set
+  if (_geometric_dimension == 0)
+    error("Internal error, dimension for subdomain has not been specified.");
+
+  return _geometric_dimension;
+}
+//-----------------------------------------------------------------------------
+template<class T>
+void SubDomain::mark_meshfunction(MeshFunction<T>& sub_domains, T sub_domain) const
 {
   info(TRACE, "Computing sub domain markers for sub domain %d.", sub_domain);
 
@@ -62,7 +95,7 @@ void SubDomain::mark(MeshFunction<uint>& sub_domains, uint sub_domain) const
   // Extract exterior (non shared) facets markers
   MeshFunction<uint>* exterior = mesh.data().mesh_function("exterior facets");
 
-  Array<double> x;  
+  Array<double> x;
 
   // Compute sub domain markers
   Progress p("Computing sub domain markers", mesh.num_entities(dim));
@@ -88,7 +121,7 @@ void SubDomain::mark(MeshFunction<uint>& sub_domains, uint sub_domain) const
         {
           all_points_inside = false;
           break;
-       }
+        }
       }
     }
 
@@ -108,12 +141,3 @@ void SubDomain::mark(MeshFunction<uint>& sub_domains, uint sub_domain) const
   }
 }
 //-----------------------------------------------------------------------------
-dolfin::uint SubDomain::geometric_dimension() const
-{
-  // Check that dim has been set
-  if (_geometric_dimension == 0)
-    error("Internal error, dimension for subdomain has not been specified.");
-
-  return _geometric_dimension;
-}
-//-----------------------------------------------------------------------------
