@@ -2,17 +2,17 @@
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2007-04-10
-// Last changed: 2010-03-02
+// Last changed: 2011-01-25
 
 #ifndef __SUB_DOMAIN_H
 #define __SUB_DOMAIN_H
 
 #include <dolfin/common/types.h>
-#include "MeshFunction.h"
 
 namespace dolfin
 {
 
+  template <class T> class MeshFunction;
   template<class T> class Array;
 
   /// This class defines the interface for definition of sub domains.
@@ -39,14 +39,27 @@ namespace dolfin
     /// Snap coordinate to boundary of sub domain
     virtual void snap(Array<double>& x) const {}
 
-    /// Set sub domain markers for given subdomain
+    /// Set sub domain markers (uint) for given subdomain
     void mark(MeshFunction<uint>& sub_domains, uint sub_domain) const;
 
+    /// Set sub domain markers (int) for given subdomain
+    void mark(MeshFunction<int>& sub_domains, int sub_domain) const;
+
+    /// Set sub domain markers (double) for given subdomain
+    void mark(MeshFunction<double>& sub_domains, double sub_domain) const;
+
+    /// Set sub domain markers (bool) for given subdomain
+    void mark(MeshFunction<bool>& sub_domains, bool sub_domain) const;
+
     /// Return geometric dimension
     uint geometric_dimension() const;
 
   private:
 
+    /// Set sub domain markers for given subdomain
+    template<class T> 
+    void mark_meshfunction(MeshFunction<T>& sub_domains, T sub_domain) const;
+
     // Friends
     friend class DirichletBC;
     friend class PeriodicBC;
