@@ -1,21 +1,33 @@
-// Copyright (C) 2006-2008 Anders Logg.
+// Copyright (C) 2006-2010 Anders Logg.
 // Licensed under the GNU LGPL Version 2.1.
 //
 // First added:  2006-06-02
-// Last changed: 2010-03-02
+// Last changed: 2011-02-09
 
 #include <set>
+#include <vector>
+#include <boost/unordered_set.hpp>
+
+#include <dolfin/common/Timer.h>
+#include <dolfin/common/Timer.h>
+#include <dolfin/common/utils.h>
 #include <dolfin/log/dolfin_log.h>
 #include "CellType.h"
 #include "Mesh.h"
-#include "MeshTopology.h"
 #include "MeshConnectivity.h"
 #include "MeshEntity.h"
 #include "MeshEntityIterator.h"
+#include "MeshTopology.h"
 #include "TopologyComputation.h"
 
 using namespace dolfin;
 
+// Set typedefs
+typedef std::set<dolfin::uint> set;
+typedef std::set<dolfin::uint>::const_iterator set_iterator;
+//typedef boost::unordered_set<dolfin::uint> set;
+//typedef boost::unordered_set<dolfin::uint>::const_iterator set_iterator;
+
 //-----------------------------------------------------------------------------
 dolfin::uint TopologyComputation::compute_entities(Mesh& mesh, uint dim)
 {
@@ -57,6 +69,10 @@ dolfin::uint TopologyComputation::compute_entities(Mesh& mesh, uint dim)
   // Compute connectivity dim - dim if not already computed
   compute_connectivity(mesh, mesh.topology().dim(), mesh.topology().dim());
 
+  // Start timer
+  //info("Creating mesh entities of dimension %d.", dim);
+  Timer timer("compute entities dim = " + to_string(dim));
+
   // Get cell type
   const CellType& cell_type = mesh.type();
 
@@ -130,6 +146,8 @@ void TopologyComputation::compute_connectivity(Mesh& mesh, uint d0, uint d1)
   // Each of these functions assume a set of preconditions that we
   // need to satisfy.
 
+  info(TRACE, "Requesting connectivity %d - %d.", d0, d1);
+
   // Get mesh topology and connectivity
   MeshTopology& topology = mesh.topology();
   MeshConnectivity& connectivity = topology(d0, d1);
@@ -154,12 +172,16 @@ void TopologyComputation::compute_connectivity(Mesh& mesh, uint d0, uint d1)
   if (connectivity.size() > 0)
     return;
 
+  // Start timer
+  //info("Computing mesh connectivity %d - %d.", d0, d1);
+  Timer timer("compute connectivity " + to_string(d0) + " - " + to_string(d1));
+
   // Decide how to compute the connectivity
   if (d0 < d1)
   {
     // Compute connectivity d1 - d0 and take transpose
     compute_connectivity(mesh, d1, d0);
-    computeFromTranspose(mesh, d0, d1);
+    compute_from_transpose(mesh, d0, d1);
   }
   else
   {
@@ -174,11 +196,11 @@ void TopologyComputation::compute_connectivity(Mesh& mesh, uint d0, uint d1)
     // Compute connectivity d0 - d - d1 and take intersection
     compute_connectivity(mesh, d0, d);
     compute_connectivity(mesh, d, d1);
-    computeFromIntersection(mesh, d0, d1, d);
+    compute_from_intersection(mesh, d0, d1, d);
   }
 }
 //----------------------------------------------------------------------------
-void TopologyComputation::computeFromTranspose(Mesh& mesh, uint d0, uint d1)
+void TopologyComputation::compute_from_transpose(Mesh& mesh, uint d0, uint d1)
 {
   // The transpose is computed in three steps:
   //
@@ -190,7 +212,7 @@ void TopologyComputation::computeFromTranspose(Mesh& mesh, uint d0, uint d1)
   //   3. Iterate again over entities of dimension d1 and add connections
   //      for each entity of dimension d0
 
-  //info("Computing mesh connectivity %d - %d from transpose.", d0, d1);
+  info(TRACE, "Computing mesh connectivity %d - %d from transpose.", d0, d1);
 
   // Get mesh topology and connectivity
   MeshTopology& topology = mesh.topology();
@@ -224,48 +246,32 @@ void TopologyComputation::computeFromTranspose(Mesh& mesh, uint d0, uint d1)
       connectivity.set(e0->index(), e1->index(), tmp[e0->index()]++);
 }
 //----------------------------------------------------------------------------
-void TopologyComputation::computeFromIntersection(Mesh& mesh,
-					     uint d0, uint d1, uint d)
+void TopologyComputation::compute_from_intersection(Mesh& mesh,
+                                                    uint d0, uint d1, uint d)
 {
-  // The intersection is computed in three steps:
-  //
-  //   1. Nested iteration over mesh - d0 - d - d1 and count the connections
-  //
-  //   2. Allocate memory / prepare data structures
-  //
-  //   3. Nested iteration over mesh - d0 - d - d1 and add the connections
+  info(TRACE, "Computing mesh connectivity %d - %d from intersection %d - %d - %d.",
+       d0, d1, d0, d, d1);
 
-  //info("Computing mesh connectivity %d - %d from intersection %d - %d - %d.",
-  //            d0, d1, d0, d, d1);
-
-  // Get mesh topology and connectivity
+  // Get mesh topology
   MeshTopology& topology = mesh.topology();
-  MeshConnectivity& connectivity = topology(d0, d1);
 
-  // Need d0 >= d1
+  // Check preconditions
   assert(d0 >= d1);
-
-  // Need connectivity d0 - d and d - d1
   assert(topology(d0, d).size() > 0);
   assert(topology(d, d1).size() > 0);
 
-  // Temporary array
-  std::vector<uint> tmp(topology.size(d0));
-
-  // Reset size for each entity
-  for (uint i = 0; i < tmp.size(); i++)
-    tmp[i] = 0;
-
-  // FIXME: Check efficiency of std::set, compare with std::vector
-
-  // A set with connected entities
-  std::set<uint> entities;
+  // Temporary dynamic storage, later copied into static storage
+  std::vector<std::vector<uint> > connectivity(topology.size(d0));
 
   // Iterate over all entities of dimension d0
+  uint max_size = 1;
   for (MeshEntityIterator e0(mesh, d0); !e0.end(); ++e0)
   {
-    // Clear set of connected entities
-    entities.clear();
+    // Get set of connected entities for current entity
+    std::vector<uint>& entities = connectivity[e0->index()];
+
+    // Reserve space
+    entities.reserve(max_size);
 
     // Iterate over all connected entities of dimension d
     for (MeshEntityIterator e(*e0, d); !e.end(); ++e)
@@ -276,62 +282,30 @@ void TopologyComputation::computeFromIntersection(Mesh& mesh,
         if (d0 == d1)
         {
           // An entity is not a neighbor to itself
-          if (e0->index() != e1->index())
-            entities.insert(e1->index());
+          if (e0->index() != e1->index() && std::find(entities.begin(), entities.end(), e1->index()) == entities.end())
+            entities.push_back(e1->index());
         }
         else
         {
           // Entity e1 must be completely contained in e0
-          if (contains(*e0, *e1))
-            entities.insert(e1->index());
+          if (contains(*e0, *e1) && std::find(entities.begin(), entities.end(), e1->index()) == entities.end())
+            entities.push_back(e1->index());
         }
       }
     }
 
-    // Count the number of connected entities
-    tmp[e0->index()] = entities.size();
+    // Store maximum size
+    if (entities.size() > max_size)
+      max_size = entities.size();
   }
 
-  // Initialize the number of connections
-  connectivity.init(tmp);
-
-  // Iterate over all entities of dimension d
-  for (MeshEntityIterator e0(mesh, d0); !e0.end(); ++e0)
-  {
-    // Clear set of connected entities
-    entities.clear();
-
-    // Iterate over all connected entities of dimension d
-    for (MeshEntityIterator e(*e0, d); !e.end(); ++e)
-    {
-      // Iterate over all connected entities of dimension d1
-      for (MeshEntityIterator e1(*e, d1); !e1.end(); ++e1)
-      {
-        if (d0 == d1)
-        {
-          // An entity is not a neighbor to itself
-          if (e0->index() != e1->index())
-            entities.insert(e1->index());
-        }
-        else
-        {
-          // Entity e1 must be completely contained in e0
-          if (contains(*e0, *e1))
-            entities.insert(e1->index());
-        }
-      }
-    }
-
-    // Add the connected entities
-    uint pos = 0;
-    for (std::set<uint>::iterator it = entities.begin(); it != entities.end(); ++it)
-      connectivity.set(e0->index(), *it, pos++);
-  }
+  // Copy to static storage
+  topology(d0, d1).set(connectivity);
 }
-//----------------------------------------------------------------------------
+//-----------------------------------------------------------------------------
 dolfin::uint TopologyComputation::count_entities(Mesh& mesh, MeshEntity& cell,
-					   uint** entities, uint m, uint n,
-					   uint dim)
+                                                 uint** entities, uint m, uint n,
+                                                 uint dim)
 {
   // For each entity, we iterate over connected and previously visited
   // cells to see if the entity has already been counted.
