@@ -1,13 +1,14 @@
 // Copyright (C) 2010 Garth N. Wells.
 // Licensed under the GNU LGPL Version 2.1.
 //
-// Modified by Anders Logg, 2010.
+// Modified by Anders Logg, 2010-2011.
 //
 // First added:  2010-02-10
-// Last changed: 2010-03-22
+// Last changed: 2011-02-16
 
 #include "LocalMeshRefinement.h"
 #include "Mesh.h"
+#include "MeshEntity.h"
 #include "MeshFunction.h"
 #include "UniformMeshRefinement.h"
 #include "refine.h"
@@ -31,9 +32,7 @@ dolfin::Mesh dolfin::refine(const Mesh& mesh,
                             const MeshFunction<bool>& cell_markers)
 {
   Mesh refined_mesh;
-  LocalMeshRefinement::refineRecursivelyByEdgeBisection(refined_mesh,
-                                                        mesh,
-                                                        cell_markers);
+  refine(refined_mesh, mesh, cell_markers);
   return refined_mesh;
 }
 //-----------------------------------------------------------------------------
@@ -41,8 +40,7 @@ void dolfin::refine(Mesh& refined_mesh,
                     const Mesh& mesh,
                     const MeshFunction<bool>& cell_markers)
 {
-  LocalMeshRefinement::refineRecursivelyByEdgeBisection(refined_mesh,
-                                                        mesh,
-                                                        cell_markers);
+  // Call local mesh refinement algorithm
+  LocalMeshRefinement::refine(refined_mesh, mesh, cell_markers);
 }
 //-----------------------------------------------------------------------------
