@@ -6,7 +6,7 @@
 // Modified by Johan Hake, 2010.
 //
 // First added:  2005-10-23
-// Last changed: 2010-04-22
+// Last changed: 2010-09-20
 
 #include <iostream>
 #include <dolfin/common/constants.h>
@@ -16,6 +16,8 @@
 #include <dolfin/la/Matrix.h>
 #include <dolfin/la/Vector.h>
 #include <dolfin/log/log.h>
+#include <dolfin/log/dolfin_log.h>
+#include <dolfin/common/MPI.h>
 #include "NonlinearProblem.h"
 #include "NewtonSolver.h"
 
@@ -26,7 +28,7 @@ Parameters NewtonSolver::default_parameters()
 {
   Parameters p("newton_solver");
 
-  p.add("maximum_iterations",      50);
+  p.add("maximum_iterations",      10);
   p.add("relative_tolerance",      1e-9);
   p.add("absolute_tolerance",      1e-10);
   p.add("convergence_criterion",   "residual");
@@ -35,21 +37,29 @@ Parameters NewtonSolver::default_parameters()
   p.add("report",                  true);
   p.add("error_on_nonconvergence", true);
 
+  //p.add("reuse_preconditioner", false);
+
   return p;
 }
 //-----------------------------------------------------------------------------
 NewtonSolver::NewtonSolver(std::string solver_type, std::string pc_type)
-             : solver(new LinearSolver(solver_type, pc_type)),
+             : Variable("Newton solver", "unamed"),
+               newton_iteration(0), _residual(0.0), residual0(0.0),
+               solver(new LinearSolver(solver_type, pc_type)),
                A(new Matrix), dx(new Vector), b(new Vector)
 {
   // Set default parameters
   parameters = default_parameters();
-  parameters.add(solver->parameters);
 }
 //-----------------------------------------------------------------------------
-NewtonSolver::NewtonSolver(GenericLinearSolver& solver, LinearAlgebraFactory& factory)
-            : solver(reference_to_no_delete_pointer(solver)), A(factory.create_matrix()),
-              dx(factory.create_vector()), b(factory.create_vector())
+NewtonSolver::NewtonSolver(GenericLinearSolver& solver,
+                           LinearAlgebraFactory& factory)
+                         : Variable("Newton solver", "unamed"),
+                           newton_iteration(0), _residual(0.0), residual0(0.0),
+                           solver(reference_to_no_delete_pointer(solver)),
+                           A(factory.create_matrix()),
+                           dx(factory.create_vector()),
+                           b(factory.create_vector())
 {
   // Set default parameters
   parameters = default_parameters();
@@ -66,6 +76,7 @@ std::pair<dolfin::uint, bool> NewtonSolver::solve(NonlinearProblem& nonlinear_pr
   assert(A);
   assert(b);
   assert(dx);
+  assert(solver);
 
   const uint maxiter = parameters["maximum_iterations"];
 
@@ -79,16 +90,21 @@ std::pair<dolfin::uint, bool> NewtonSolver::solve(NonlinearProblem& nonlinear_pr
   nonlinear_problem.form(*A, *b, x);
   nonlinear_problem.F(*b, x);
 
+  solver->set_operator(*A);
+
   // Start iterations
   while (!newton_converged && newton_iteration < maxiter)
   {
     // Compute Jacobian
     nonlinear_problem.J(*A, x);
 
+    // FIXME: This reset is a hack to handle a deficiency in the Trilinos wrappers
+    solver->set_operator(*A);
+
     // Perform linear solve and update total number of Krylov iterations
     if (dx->size() > 0)
       dx->zero();
-    krylov_iterations += solver->solve(*A, *dx, *b);
+    krylov_iterations += solver->solve(*dx, *b);
 
     // Compute initial residual
     if (newton_iteration == 0)
@@ -114,11 +130,14 @@ std::pair<dolfin::uint, bool> NewtonSolver::solve(NonlinearProblem& nonlinear_pr
   }
 
   if (newton_converged)
-    info(PROGRESS, "Newton solver finished in %d iterations and %d linear solver iterations.",
+  {
+    if (dolfin::MPI::process_number() == 0)
+     info(PROGRESS, "Newton solver finished in %d iterations and %d linear solver iterations.",
             newton_iteration, krylov_iterations);
+  }
   else
   {
-    bool error_on_nonconvergence = parameters["error_on_nonconvergence"];
+    const bool error_on_nonconvergence = parameters["error_on_nonconvergence"];
     if (error_on_nonconvergence)
       error("Newton solver did not converge.");
     else
@@ -135,6 +154,22 @@ dolfin::uint NewtonSolver::iteration() const
   return newton_iteration;
 }
 //-----------------------------------------------------------------------------
+double NewtonSolver::residual() const
+{
+  return _residual;
+}
+//-----------------------------------------------------------------------------
+double NewtonSolver::relative_residual() const
+{
+  return _residual/residual0;
+}
+//-----------------------------------------------------------------------------
+GenericLinearSolver& NewtonSolver::linear_solver() const
+{
+  assert(solver);
+  return *solver;
+}
+//-----------------------------------------------------------------------------
 bool NewtonSolver::converged(const GenericVector& b, const GenericVector& dx,
                              const NonlinearProblem& nonlinear_problem)
 {
@@ -143,42 +178,30 @@ bool NewtonSolver::converged(const GenericVector& b, const GenericVector& dx,
   const double atol = parameters["absolute_tolerance"];
   const bool report = parameters["report"];
 
-  double residual = 1.0;
-
   // Compute resdiual
   if (convergence_criterion == "residual")
-    residual = b.norm("l2");
+    _residual = b.norm("l2");
   else if (convergence_criterion == "incremental")
-    residual = dx.norm("l2");
+    _residual = dx.norm("l2");
   else
     error("Unknown Newton convergence criterion");
 
   // If this is the first iteration step, set initial residual
   if (newton_iteration == 0)
-    residual0 = residual;
+    residual0 = _residual;
 
   // Relative residual
-  double relative_residual = residual / residual0;
+  const double relative_residual = _residual / residual0;
 
   // Output iteration number and residual
-  //FIXME: allow precision to be set for dolfin::cout<<
-  /*
-    std::cout.precision(3);
-    if (report && newton_iteration > 0)
-    std::cout << "  Iteration " << newton_iteration
-              << ":"
-              << " r (abs) = " << std::scientific << residual
-              << " (tol = " << std::scientific << atol << ")"
-              << " r (rel) = " << std::scientific << relative_residual
-              << " (tol = " << std::scientific << rtol << ")"
-              << std::endl;
-  */
-  if (report && newton_iteration > 0)
+  if (report && newton_iteration > 0 && dolfin::MPI::process_number() == 0)
+  {
     info("Newton iteration %d: r (abs) = %.3e (tol = %.3e) r (rel) = %.3e (tol = %.3e)",
-         newton_iteration, residual, atol, relative_residual, rtol);
+         newton_iteration, _residual, atol, relative_residual, rtol);
+  }
 
   // Return true of convergence criterion is met
-  if (relative_residual < rtol || residual < atol)
+  if (relative_residual < rtol || _residual < atol)
     return true;
 
   // Otherwise return false
