@@ -1,10 +1,10 @@
 // Copyright (C) 2005-2008 Garth N. Wells.
 // Licensed under the GNU LGPL Version 2.1.
 //
-// Modified by Anders Logg 2006-2009.
+// Modified by Anders Logg 2006-2011.
 //
 // First added:  2005-10-23
-// Last changed: 2009-06-29
+// Last changed: 2011-01-14
 
 #ifndef __NEWTON_SOLVER_H
 #define __NEWTON_SOLVER_H
@@ -19,10 +19,9 @@ namespace dolfin
 
   // Forward declarations
   class GenericLinearSolver;
-  class GenericLinearAlgebraFactory;
+  class LinearAlgebraFactory;
   class GenericMatrix;
   class GenericVector;
-  class Mesh;
   class NonlinearProblem;
 
   /// This class defines a Newton solver for equations of the form F(u) = 0.
@@ -51,6 +50,15 @@ namespace dolfin
     /// Return Newton iteration number
     uint iteration() const;
 
+    /// Return current residual
+    double residual() const;
+
+    /// Return current relative residual
+    double relative_residual() const;
+
+    /// Return the linear solver
+    GenericLinearSolver& linear_solver() const;
+
     /// Default parameter values
     static Parameters default_parameters();
 
@@ -63,8 +71,8 @@ namespace dolfin
     /// Current number of Newton iterations
     uint newton_iteration;
 
-    /// Residual
-    double residual0;
+    /// Most recent residual and intitial residual
+    double _residual, residual0;
 
     /// Solver
     boost::shared_ptr<GenericLinearSolver> solver;
@@ -82,4 +90,3 @@ namespace dolfin
 }
 
 #endif
-
