@@ -6,63 +6,56 @@ __copyright__ = "Copyright (C) 2008 Joachim B. Haga"
 __license__  = "GNU LGPL Version 2.1"
 
 # Modified by Martin Sandve Alnaes, 2008.
-# Modified by Anders Logg, 2008-2009.
-# Last changed: 2010-06-01
+# Modified by Anders Logg, 2008-2010.
+# Last changed: 2010-12-08
 
 import os
 import dolfin.cpp as cpp
 from donothing import DoNothing
 from dolfin.function.function import Function
+import ufl
 
 __all__ = ['Viper', 'plot', 'update', 'interactive', 'save_plot', 'figure']
 
-# Intelligent plot command that handles projections
-def dolfin_plot(object, *args, **kwargs):
-    "Plot given object (Mesh, MeshFunction or Function)"
+def make_viper_object(object):
+    for plottype,classes in viper_dolfin.plottable:
+        if isinstance(object, classes):
+            return object
 
-    from viper.viper_dolfin import plot as viper_plot
-
-    # Plot mesh
-    if isinstance(object, cpp.Mesh):
-        return viper_plot(object, *args, **kwargs)
-
-    # Plot mesh function
-    if isinstance(object, cpp.MeshFunctionUInt) or \
-       isinstance(object, cpp.MeshFunctionInt)  or \
-       isinstance(object, cpp.MeshFunctionDouble):
-        return viper_plot(object, *args, **kwargs)
-
-    # Plot function
-    if isinstance(object, cpp.Function):
-        return viper_plot(object, *args, **kwargs)
-
-    # Plot expression
-    if isinstance(object, cpp.Expression):
-        if "mesh" not in kwargs:
-            raise TypeError, "expected a mesh when plotting an expression."
-        return viper_plot(object, *args, **kwargs)
-
-    # Plot function plot data
-    if isinstance(object, cpp.FunctionPlotData):
-        return viper_plot(object, *args, **kwargs)
-
-    # Plot boundary condition
     if isinstance(object, cpp.DirichletBC):
         bc = object
         V = bc.function_space()
         v = Function(V)
         bc.apply(v.vector())
-        return viper_plot(v, *args, **kwargs)
+        return v
 
     # Try projecting function
     from dolfin.fem.project import project
     try:
-        print "Object cannot be plotted directly, projecting to piecewise linears."
         u = project(object)
-        return plot(u, *args, **kwargs)
+        cpp.info("Object cannot be plotted directly, projecting to piecewise linears.")
+        return u
     except:
         raise RuntimeError, ("Don't know how to plot given object and projection failed: " + str(object))
 
+
+# Intelligent plot command that handles projections and different objects
+# (aliased as 'plot')
+def dolfin_plot(object, *args, **kwargs):
+    "Plot given object (Mesh, MeshFunction or Function)"
+
+    # Plot element
+    if isinstance(object, ufl.FiniteElementBase):
+        import ffc
+        return ffc.plot(object, *args, **kwargs)
+
+    # Check expression
+    if isinstance(object, cpp.Expression) and "mesh" not in kwargs:
+        raise TypeError, "expected a mesh when plotting an expression."
+
+    return viper_dolfin.plot(make_viper_object(object), *args, **kwargs)
+
+
 # Check DOLFIN_NOPLOT
 do_nothing = False
 if os.environ.has_key('DOLFIN_NOPLOT'):
@@ -72,11 +65,12 @@ else:
 
     # Check for Viper
     try:
+        from viper import viper_dolfin
         for x in __all__:
             exec ('from viper.viper_dolfin import %s' % x)
         plot = dolfin_plot
     except ImportError, details:
-        cpp.info(str(details))
+        cpp.warning(str(details))
         cpp.warning("Unable to import Viper, plotting disabled.")
         do_nothing= True
 
