@@ -1,7 +1,7 @@
 "This module provides functionality for compilation of strings as dolfin Expressions."
 
 __author__ = "Martin Sandve Alnes (martinal@simula.no)"
-__date__ = "2008-06-04 -- 2009-12-16"
+__date__ = "2008-06-04 -- 2011-01-22"
 __copyright__ = "Copyright (C) 2008-2008 Martin Sandve Alnes"
 __license__  = "GNU LGPL Version 2.1"
 
@@ -13,9 +13,9 @@ import hashlib
 import instant
 
 # Import local compile_extension_module
-from dolfin.compilemodules import (compile_extension_module,
-                                   expression_to_code_fragments,
-                                   math_header)
+from dolfin.compilemodules.compilemodules import (compile_extension_module,
+                                                   expression_to_code_fragments,
+                                                   math_header)
 
 __all__ = ["compile_expressions"]
 
