@@ -9,7 +9,7 @@ The C++ PDE classes are reimplemented in Python since the C++ classes
 rely on the dolfin::Form class which is not used on the Python side."""
 
 __author__ = "Anders Logg (logg@simula.no)"
-__date__ = "2007-08-15 -- 2009-12-11"
+__date__ = "2007-08-15 -- 2010-11-04"
 __copyright__ = "Copyright (C) 2007-2008 Anders Logg"
 __license__  = "GNU LGPL Version 2.1"
 
@@ -116,10 +116,10 @@ def assemble(form,
 
     # Extract domains
     cell_domains, exterior_facet_domains, interior_facet_domains = \
-                  _extract_domains(dolfin_form.mesh(),
-                                   cell_domains,
-                                   exterior_facet_domains,
-                                   interior_facet_domains)
+        _extract_domains(dolfin_form.mesh(),
+                         cell_domains,
+                         exterior_facet_domains,
+                         interior_facet_domains)
 
     # Assemble tensor from compiled form
     cpp.assemble(tensor,
@@ -211,6 +211,10 @@ def _create_tensor(form, rank, backend, tensor, reset_sparsity):
     if tensor is not None:
         return (tensor, reset_sparsity)
 
+    # Check that reset_sparsity pattern is not False when not providing tensor
+    if reset_sparsity == False:
+        raise TypeError, "expected a tensor, when 'reset_sparsity' is 'False'"
+
     # Decide if we should reset the tensor
     use_cache = cpp.parameters["optimize_use_tensor_cache"] or \
                 cpp.parameters["optimize"]
@@ -264,15 +268,17 @@ def _extract_domains(mesh,
     # The cell dimension
     cell_dim = mesh.topology().dim()
 
+    # Get cell_domains
     if isinstance(cell_domains, cpp.SubDomain):
         cell_domains = build_mf(cell_domains, cell_dim)
 
-    # If no exterior facet domain is provided check if data is stored in the mesh
+    # Get exterior_facet_domains (may be stored as part of the mesh)
     if exterior_facet_domains is None:
         exterior_facet_domains = mesh.data().mesh_function("exterior facet domains")
     elif isinstance(exterior_facet_domains, cpp.SubDomain):
         exterior_facet_domains = build_mf(exterior_facet_domains, cell_dim-1)
 
+    # Get interior_facet_domains
     if isinstance(interior_facet_domains, cpp.SubDomain):
         interior_facet_domains = build_mf(interior_facet_domains, cell_dim-1)
 
