@@ -32,19 +32,18 @@ class Form(cpp.Form):
         # Compile form if necessary
         if not hasattr(form, "create_cell_integral"):
             (self._compiled_form, module, self.form_data) = jit(form, form_compiler_parameters, common_cell)
-
         else:
             self._compiled_form = form
             self.form_data = None
 
-        # Extract test spaces and coefficients
+        # Extract function spaces
         self.function_spaces = _extract_function_spaces(self.form_data,
                                                         self._compiled_form,
                                                         function_spaces)
 
-        (self.coefficients, self._compiled_coefficients) \
-                             = _extract_coefficients(self.form_data,
-                                                     coefficients)
+        # Extract coefficients
+        (self.coefficients, self._compiled_coefficients) = \
+            _extract_coefficients(self.form_data, coefficients)
 
         # Initialize base class
         cpp.Form.__init__(self, self._compiled_form, self.function_spaces, self.coefficients)
