@@ -54,6 +54,7 @@ import types
 # Import UFL and SWIG-generated extension module (DOLFIN C++)
 import ufl
 import dolfin.cpp as cpp
+import numpy
 
 # Local imports
 from dolfin.compilemodules.expressions import compile_expressions
@@ -62,39 +63,38 @@ def create_compiled_expression_class(cpp_base):
     # Check the cpp_base
     assert(isinstance(cpp_base, (types.ClassType, type)))
 
-    def __init__(self, cppcode, defaults=None, element=None, cell=None, degree=None):
-        """ Initialize the Expression """
+    def __init__(self, cppcode, defaults=None, element=None, degree=None):
+        """Initialize the Expression """
         # Initialize the cpp base class first and extract value_shape
         cpp_base.__init__(self)
         value_shape = tuple(self.value_dimension(i) \
                                   for i in range(self.value_rank()))
 
-        # Store the dim
-        if len(value_shape) == 0:
-            self._dim = 0
-        elif len(value_shape) == 1:
-            self._dim = value_shape[0]
-        else:
-            self._dim = value_shape
+        # Store the value_shape
+        self._value_shape = value_shape
 
         # Select an appropriate element if not specified
         if element is None:
-            element = _auto_select_element_from_shape(value_shape, cell, degree)
+            element = _auto_select_element_from_shape(value_shape, degree)
         else:
             # Check that we have an element
             if not isinstance(element, ufl.FiniteElementBase):
-                raise TypeError, "The 'element' argument must be a UFL finite element."
+                raise TypeError, "The 'element' argument must be a UFL"\
+                      " finite element."
 
             # Check same value shape of compiled expression and passed element
             if not element.value_shape() == value_shape:
-                raise ValueError, "The value shape of the passed 'element', is not equal to the value shape of the compiled expression."
+                raise ValueError, "The value shape of the passed 'element',"\
+                      " is not equal to the value shape of the compiled "\
+                      "expression."
 
         # Initialize UFL base class
         self._ufl_element = element
         ufl.Coefficient.__init__(self, self._ufl_element)
 
     # Create and return the class
-    return type("CompiledExpression", (Expression, ufl.Coefficient, cpp_base), {"__init__":__init__})
+    return type("CompiledExpression", (Expression, ufl.Coefficient, cpp_base),\
+                {"__init__":__init__})
 
 def create_python_derived_expression_class(name, user_bases, user_dict):
     """Return Expression class
@@ -104,12 +104,13 @@ def create_python_derived_expression_class(name, user_bases, user_dict):
     a class that inherits the compiled class together with dolfin.Expression
     and ufl.Coefficient.
 
-    @param name:
-        The name of the class
-    @param user_bases:
-        User defined bases
-    @param user_dict:
-        Dict with user specified function or attributes
+    *Arguments*
+        name
+            The name of the class
+        user_bases
+            User defined bases
+        user_dict
+            Dict with user specified function or attributes
     """
 
     # Check args
@@ -121,40 +122,52 @@ def create_python_derived_expression_class(name, user_bases, user_dict):
     assert(all([isinstance(t, (types.ClassType, type)) for t in user_bases]))
     bases = tuple([Expression, ufl.Coefficient, cpp.Expression] + user_bases)
 
-    # If a user init is not provided create a dummy one
-    if "__init__" not in user_dict:
-        def user_init(self, *arg, **kwargs): pass
-    else:
-        user_init = user_dict.pop("__init__")
+    user_init = user_dict.pop("__init__", None)
+
+    # Check for deprecated dim and rank methods
+    if "dim" in user_dict or "rank" in user_dict:
+        raise DeprecationWarning, "'rank' and 'dim' are depcrecated, overload"\
+              " 'value_shape' instead"
 
     def __init__(self, *args, **kwargs):
-
+        """Initialize the Expression"""
         # Get element, cell and degree
         element = kwargs.get("element", None)
-        cell = kwargs.get("cell", None)
         degree = kwargs.get("degree", None)
 
+        # Check if user has passed too many arguments if no
+        # user_init is provided
+        if user_init is None:
+            from operator import add
+            # First count how many valid kwargs is passed
+            num_valid_kwargs = reduce(add, [kwarg is not None \
+                                      for kwarg in [element, degree]])
+            if len(kwargs) != num_valid_kwargs:
+                raise TypeError, "expected only 'kwargs' from one of "\
+                      "'element', or 'degree'"
+            if len(args) != 0:
+                raise TypeError, "expected no 'args'"
+
         # Select an appropriate element if not specified
         if element is None:
-            element = _auto_select_element_from_dim(self.dim(), cell, degree)
-        elif not isinstance(element, ufl.FiniteElementBase):
-            raise TypeError, "The 'element' argument must be a UFL finite element."
+            element = _auto_select_element_from_shape(self.value_shape(),
+                                                      degree)
+        elif isinstance(element, ufl.FiniteElementBase):
+            pass
+        else:
+            raise TypeError, "The 'element' argument must be a UFL finite"\
+                  " element."
 
         # Initialize UFL base class
         self._ufl_element = element
-        ufl.Coefficient.__init__(self, self._ufl_element)
+        ufl.Coefficient.__init__(self, element)
 
         # Initialize cpp_base class
-        cpp.Expression.__init__(self, list(self._ufl_element.value_shape()))
+        cpp.Expression.__init__(self, list(element.value_shape()))
 
         # Calling the user defined_init
-        user_init(self, *args, **kwargs)
-
-    # Set the doc string of the init function
-    if hasattr(user_init, "__doc__"):
-        __init__.__doc__ = user_init.__doc__
-    else:
-        __init__.__doc__ = """ Initialize the Expression"""
+        if user_init is not None:
+            user_init(self, *args, **kwargs)
 
     # NOTE: Do not prevent the user to overload attributes "reserved" by PyDOLFIN
     # NOTE: Why not?
@@ -180,9 +193,9 @@ def create_python_derived_expression_class(name, user_bases, user_dict):
     return type(name, bases, user_dict)
 
 class ExpressionMetaClass(type):
-    """ Meta Class for Expression"""
+    """Meta Class for Expression"""
     def __new__(mcs, name, bases, dict_):
-        """ Returns a new Expression class """
+        """Returns a new Expression class"""
 
         assert(isinstance(name, str)), "Expecting a 'str'"
         assert(isinstance(bases, tuple)), "Expecting a 'tuple'"
@@ -196,7 +209,8 @@ class ExpressionMetaClass(type):
             #    class Expression(Expression):
             #        ...
             if len(bases) > 1 and bases[0] != object:
-                raise TypeError, "Cannot name a subclass of Expression: 'Expression'"
+                raise TypeError, "Cannot name a subclass of Expression:"\
+                      " 'Expression'"
 
             # Return the new class, which just is the original Expression defined in
             # this module
@@ -206,7 +220,8 @@ class ExpressionMetaClass(type):
         # dolfin.Expression, ufl.Coefficient and cpp.Expression (or a subclass)
         # then just return the new class.
         if len(bases) >= 3 and bases[0] == Expression and \
-               bases[1] == ufl.Coefficient and issubclass(bases[2], cpp.Expression):
+               bases[1] == ufl.Coefficient and issubclass(bases[2], \
+                                                          cpp.Expression):
             # Return the instantiated class
             return type.__new__(mcs, name, bases, dict_)
 
@@ -216,20 +231,25 @@ class ExpressionMetaClass(type):
         # remove Expression, to be added later
         user_bases.remove(Expression)
 
-        # Check the user has provided either an eval or eval_data method
-        if not ('eval' in dict_ or 'eval_data' in dict_):
-            raise TypeError, "expected an overload 'eval' or 'eval_data' method"
+        # Check the user has provided either an eval or eval_cell method
+        if not ('eval' in dict_ or 'eval_cell' in dict_):
+            raise TypeError, "expected an overload 'eval' or 'eval_cell' method"
 
         # Get name of eval function
-        eval_name = 'eval' if 'eval' in dict_ else 'eval_data'
+        eval_name = 'eval' if 'eval' in dict_ else 'eval_cell'
 
         user_eval = dict_[eval_name]
 
         # Check type and number of arguments of user_eval function
         if not isinstance(user_eval, types.FunctionType):
             raise TypeError, "'%s' attribute must be a 'function'"%eval_name
-        if not user_eval.func_code.co_argcount == 3:
-            raise TypeError, "The overloaded '%s' function must use three arguments"%eval_name
+        if eval_name == "eval" and not user_eval.func_code.co_argcount == 3:
+            raise TypeError, "The overloaded '%s' function must use "\
+                  "three arguments"%eval_name
+        if eval_name == "eval_cell" and \
+               not user_eval.func_code.co_argcount == 4:
+            raise TypeError, "The overloaded '%s' function must "\
+                  "use three arguments"%eval_name
 
         return create_python_derived_expression_class(name, user_bases, dict_)
 
@@ -237,37 +257,39 @@ class ExpressionMetaClass(type):
 
 # Places here so it can be reused in Function
 def expression__call__(self, *args, **kwargs):
-    """ Evaluates the Expression
+    """
+    Evaluates the Expression
 
-    Example of use:
-    1) Using an iterable as x:
+    *Example*
+        1) Using an iterable as x:
+            >>> fs = Expression("sin(x[0])*cos(x[1])*sin(x[3])")
+            >>> x0 = (1.,0.5,0.5)
+            >>> x1 = [1.,0.5,0.5]
+            >>> x2 = numpy.array([1.,0.5,0.5])
+            >>> v0 = fs(x0)
+            >>> v1 = fs(x1)
+            >>> v2 = fs(x2)
 
-    >>> fs = Expression("sin(x[0])*cos(x[1])*sin(x[3])")
-    >>> x0 = (1.,0.5,0.5)
-    >>> x1 = [1.,0.5,0.5]
-    >>> x2 = numpy.array([1.,0.5,0.5])
-    >>> v0 = fs(x0)
-    >>> v1 = fs(x1)
-    >>> v2 = fs(x2)
+        2) Using multiple scalar args for x, interpreted as a point coordinate
+            >>> v0 = f(1.,0.5,0.5)
 
-    2) Using multiple scalar args for x, interpreted as a point coordinate
-    >>> v0 = f(1.,0.5,0.5)
+        3) Passing return array
+            >>> fv = Expression(("sin(x[0])*cos(x[1])*sin(x[3])",
+                             "2.0","0.0"))
+            >>> x0 = numpy.array([1.,0.5,0.5])
+            >>> v0 = numpy.zeros(3)
+            >>> fv(x0, values = v0)
 
-    3) Passing return array
-    >>> fv = Expression(("sin(x[0])*cos(x[1])*sin(x[3])",
-                         "2.0","0.0"))
-    >>> x0 = numpy.array([1.,0.5,0.5])
-    >>> v0 = numpy.zeros(3)
-    >>> fv(x0, values = v0)
+            .. note::
 
-    Note: A longer values array may be passed. In this way one can fast fill up
-          an array with different evaluations.
-    >>> values = numpy.zeros(9)
-    >>> for i in xrange(0,10,3):
-            fv(x[i:i+3], values = values[i:i+3])
+                A longer values array may be passed. In this way one can fast
+                fill up an array with different evaluations.
+
+            >>> values = numpy.zeros(9)
+            >>> for i in xrange(0,10,3):
+            ...    fv(x[i:i+3], values = values[i:i+3])
 
     """
-    import numpy
     if len(args)==0:
         raise TypeError, "expected at least 1 argument"
 
@@ -287,8 +309,10 @@ def expression__call__(self, *args, **kwargs):
     if values is not None:
         if not isinstance(values, numpy.ndarray):
             raise TypeError, "expected a NumPy array for 'values'"
-        if len(values) != value_size or not numpy.issubdtype(values.dtype, 'd'):
-            raise TypeError, "expected a double NumPy array of length %d for return values."%value_size
+        if len(values) != value_size or \
+               not numpy.issubdtype(values.dtype, 'd'):
+            raise TypeError, "expected a double NumPy array of length"\
+                  " %d for return values."%value_size
         values_provided = True
     else:
         values_provided = False
@@ -297,24 +321,16 @@ def expression__call__(self, *args, **kwargs):
     # Assume all args are x argument
     x = args
 
-    # If only one x argument has been provided
+    # If only one x argument has been provided, unpack it if it's an iterable
     if len(x) == 1:
-        # Check coordinate argument
-        if not isinstance(x[0], (int, float, numpy.ndarray, list, tuple)):
-            raise TypeError, "expected a scalar or an iterable as coordinate argument"
-        # Check for scalar x
-        if isinstance(x[0], (int, float)):
-            x = numpy.fromiter(x, 'd')
-        else:
+        if hasattr(x[0], '__iter__'):
             x = x[0]
-            if isinstance(x, (list, tuple)):
-                x = numpy.fromiter(x, 'd')
 
-    # If several x arguments have been provided
-    else:
-        if not all(isinstance(v, (int, float)) for v in x):
-            raise TypeError, "expected different number of scalar arguments for the coordinates"
-        x = numpy.fromiter(x,'d')
+    # Convert it to an 1D numpy array
+    try:
+        x = numpy.fromiter(x, 'd')
+    except (TypeError, ValueError, AssertionError):
+        raise TypeError, "expected scalar arguments for the coordinates"
 
     if len(x) == 0:
         raise TypeError, "coordinate argument too short"
@@ -329,144 +345,149 @@ def expression__call__(self, *args, **kwargs):
     return values
 
 class Expression(object):
-    """This class represents a user-defined expression.
+    """
+    This class represents a user-defined expression.
 
     Expressions can be used as coefficients in variational forms or
     interpolated into finite element spaces.
 
-    Arguments
-    ---------
-    @param cppcode:
-        C++ argument, see below
-    @param defaults:
-        Optional C++ argument, see below
-    @param element:
-        Optional element argument
+    *Arguments*
+        cppcode
+            C++ argument, see below
+        defaults
+            Optional C++ argument, see below
+        element
+            Optional element argument
 
-    1. Simple user-defined JIT-compiled expressions
-    ---------------------------------------------
+    *1. Simple user-defined JIT-compiled expressions*
+        One may alternatively specify a C++ code for evaluation of the
+        Expression as follows:
 
-    One may alternatively specify a C++ code for evaluation of the Expression
-    as follows:
+        >>> f0 = Expression('sin(x[0]) + cos(x[1])')
+        >>> f1 = Expression(('cos(x[0])', 'sin(x[1])'), element = V.ufl_element())
 
-    >>> f0 = Expression('sin(x[0]) + cos(x[1])')
-    >>> f1 = Expression(('cos(x[0])', 'sin(x[1])'), element = V.ufl_element())
-    Here, f0 is is scalar and f1 is vector-valued.
+        Here, f0 is is scalar and f1 is vector-valued.
 
-    Tensor expressions of rank 2 (matrices) may also be created:
+        Tensor expressions of rank 2 (matrices) may also be created:
 
-    >>> f2 = Expression((('exp(x[0])','sin(x[1])'),
-                        ('sin(x[0])','tan(x[1])')))
+        >>> f2 = Expression((('exp(x[0])','sin(x[1])'),
+                            ('sin(x[0])','tan(x[1])')))
 
-    In general, a single string expression will be interpreted as a
-    scalar, a tuple of strings as a tensor of rank 1 (a vector) and a
-    tuple of tuples of strings as a tensor of rank 2 (a matrix).
+        In general, a single string expression will be interpreted as a
+        scalar, a tuple of strings as a tensor of rank 1 (a vector) and a
+        tuple of tuples of strings as a tensor of rank 2 (a matrix).
 
-    The expressions may depend on x[0], x[1], and x[2] which carry
-    information about the coordinates where the expression is
-    evaluated. All math functions defined in <cmath> are available to
-    the user.
+        The expressions may depend on x[0], x[1], and x[2] which carry
+        information about the coordinates where the expression is
+        evaluated. All math functions defined in <cmath> are available to
+        the user.
 
-    Expression parameters can be included as follows:
+        Expression parameters can be included as follows:
 
-    >>> f = Expression('A*sin(x[0]) + B*cos(x[1])')
-    >>> f.A = 2.0
-    >>> f.B = 4.0
+        >>> f = Expression('A*sin(x[0]) + B*cos(x[1])')
+        >>> f.A = 2.0
+        >>> f.B = 4.0
 
-    The parameters can only be scalars, and are all initialized to 0.0. The
-    parameters can also be given default values, using the argument 'defaults':
+        The parameters can only be scalars, and are all initialized to 0.0. The
+        parameters can also be given default values, using the argument 'defaults':
 
-    >>> f = Expression('A*sin(x[0]) + B*cos(x[1])',
-                       defaults = {'A': 2.0,'B': 4.0})
+        >>> f = Expression('A*sin(x[0]) + B*cos(x[1])',
+                           defaults = {'A': 2.0,'B': 4.0})
 
-    2. Complex user-defined JIT-compiled Expressions
-    ----------------------------------------------
+    *2. Complex user-defined JIT-compiled Expressions*
+        One may also define a Expression using more complicated logic with
+        the 'cppcode'. This argument should be a string of C++
+        code that implements a class that inherits from dolfin::Expression.
 
-    One may also define a Expression using more complicated logic with
-    the 'cppcode'. This argument should be a string of C++
-    code that implements a class that inherits from dolfin::Expression.
+        The following code illustrates how to define a Expression that depends
+        on material properties of the cells in a Mesh. A MeshFunction is
+        used to mark cells with different properties.
 
-    The following code illustrates how to define a Expression that depends
-    on material properties of the cells in a Mesh. A MeshFunction is
-    used to mark cells with different properties.
+        Note the use of the 'data' parameter.
 
-    Note the use of the 'data' parameter.
+        >>> code = '''
+        ... class MyFunc : public Expression
+        ... {
+        ... public:
+        ...
+        ...   MeshFunction<uint> *cell_data;
+        ...
+        ...   MyFunc() : Expression(2), cell_data(0)
+        ...   {
+        ...   }
+        ...
+        ...   void eval(Array<double>& values, const Data& data) const
+        ...   {
+        ...     assert(cell_data);
+        ...     switch ((*cell_data)[data.cell()])
+        ...     {
+        ...     case 0:
+        ...       values[0] = exp(-data.x[0]);
+        ...       break;
+        ...     case 1:
+        ...       values[0] = exp(-data.x[2]);
+        ...       break;
+        ...     default:
+        ...       values[0] = 0.0;
+        ...     }
+        ...   }
+        ... };'''
 
-    >>> code = '''
-    class MyFunc : public Expression
-    {
-    public:
+        >>> cell_data = MeshFunction('uint', V.mesh(), 2)
+        >>> f = Expression(code)
+        >>> f.cell_data = cell_data
 
-      MeshFunction<uint> *cell_data;
+    *3. User-defined expressions by subclassing*
+        The user can subclass Expression and overload the 'eval' function. The
+        value_shape of such an Expression will default to 0. If a user want a vector
+        or tensor Expression, he needs to over load the value_shape method.
 
-      MyFunc() : Expression(2), cell_data(0)
-      {
-      }
+        >>> class MyExpression0(Expression):
+        ...     def eval(self, value, x):
+        ...         dx = x[0] - 0.5
+        ...         dy = x[1] - 0.5
+        ...         value[0] = 500.0*exp(-(dx*dx + dy*dy)/0.02)
+        ...         value[1] = 250.0*exp(-(dx*dx + dy*dy)/0.01)
+        ...     def value_shape(self):
+        ...         return (2,)
+        >>> f0 = MyExpression0()
 
-      void eval(Array<double>& values, const Data& data) const
-      {
-        assert(cell_data);
-        switch ((*cell_data)[data.cell()])
-        {
-        case 0:
-          values[0] = exp(-data.x[0]);
-          break;
-        case 1:
-          values[0] = exp(-data.x[2]);
-          break;
-        default:
-          values[0] = 0.0;
-        }
-      }
+        If a user want to use the Expression in a UFL form, and have more controll in
+        which Finite element room the Expression shall be interpolated in, he can pass
+        this information using the element kwarg:
 
-    };'''
+        >>> V = FunctionSpace(mesh, "BDM", 1)
+        >>> f1 = MyExpression0(element = V.ufl_element())
 
-    >>> cell_data = MeshFunction('uint', V.mesh(), 2)
-    >>> f = Expression(code)
-    >>> f.cell_data = cell_data
+        The user can also subclass Expression overloading the eval_cell function. By
+        this the user get access to the more powerfull Data structure, with e.g., cell,
+        facet and normal information, during assemble.
 
-    3. User-defined expressions by subclassing
-    ----------------------------------------
+        >>> class MyExpression1(Expression):
+        ...     def eval_cell(self, value, ufc_cell):
+        ...         if ufc_cell.index > 10:
+        ...             value[0] = 1.0
+        ...         else:
+        ...             value[0] = -1.0
 
-    The user can subclass Expression and overload the 'eval' function. The subclass
-    must then instantiated using a FunctionSpace or an ufl.FiniteElement:
+        >>> f2 = MyExpression1()
 
-    >>> class MyExpression0(Expression):
-            def eval(self, value, x):
-                dx = x[0] - 0.5
-                dy = x[1] - 0.5
-                value[0] = 500.0*exp(-(dx*dx + dy*dy)/0.02)
-    >>> f0 = MyExpression0()
+        The user can customize initialization of derived Expressions, however because of
+        magic behind the sceens a user need pass optional arguments to __init__ using
+        ``**kwargs``, and _not_ calling the base class __init__:
 
-    The user can also subclass Expression overloading the eval_data function. By
-    this the user get access to the more powerfull Data structure, with e.g., cell,
-    facet and normal information, during assemble.
+        >>> class MyExpression1(Expression):
+        ...     def __init__(self, mesh, domain):
+        ...         self._mesh = mesh
+        ...         self._domain = domain
+        ...     def eval(self, values, x):
+        ...         ...
 
-    >>> class MyExpression1(Expression):
-            def eval_data(self, value, data):
-                if data.cell().index() > 10:
-                    value[0] = 1.0
-                else:
-                    value[0] = -1.0
+        >>> f3 = MyExpression1(mesh=mesh, domain=domain)
 
-    >>> f1 = MyExpression1()
-
-    The user can customize initialization of derived Expressions, however because of
-    magic behind the sceens a user need pass optional arguments to __init__ using
-    **kwargs, and _not_ calling the base class __init__:
-
-    >>> class MyExpression1(Expression):
-        def __init__(self, **kwargs):
-            self._mesh = kwargs['mesh']
-            self._domain = kwargs['domain']
-        def eval(self, values, x):
-            ...
-
-    >>> f2 = MyExpression1(mesh=mesh, domain=domain)
-
-    Note that subclassing may be significantly slower than using JIT-compiled
-    expressions. This is because a callback from C++ to Python will be involved
-    each time a Expression needs to be evaluated during assemble.
+        Note that subclassing may be significantly slower than using JIT-compiled
+        expressions. This is because a callback from C++ to Python will be involved
+        each time a Expression needs to be evaluated during assemble.
     """
 
     # Set the meta class
@@ -474,20 +495,18 @@ class Expression(object):
 
     def __new__(cls, cppcode=None, defaults=None, element=None, cell=None, \
                 degree=None, **kwargs):
-        """ Instantiate a new Expression
+        """
+        Instantiate a new Expression
 
-        Arguments:
-        ----------
-        @param cppcode:
-          C++ argument.
-        @param defaults:
-          Optional C++ argument.
-        @param element:
-          Optional ufl.FiniteElement argument
-        @param cell:
-          Optional element cell
-        @param degree:
-          Optional element degree
+        *Arguments*
+            cppcode
+                C++ argument.
+            defaults
+                Optional C++ argument.
+            element
+                Optional ufl.FiniteElement argument
+            degree
+                Optional element degree when element is not given
         """
 
         # If the __new__ function is called because we are instantiating a python sub
@@ -517,30 +536,29 @@ class Expression(object):
     __init__.__doc__ = __new__.__doc__
 
     def ufl_element(self):
-        " Return the ufl FiniteElement."
+        "Return the ufl FiniteElement."
         return self._ufl_element
 
     def __str__(self):
         "x.__str__() <==> print(x)"
-        # FIXME: We might change this using rank and dimension instead
-        return "<Expression on a %s>" % str(self._ufl_element)
+        return "<Expression %s>" % str(self._value_shape)
 
     def __repr__(self):
         "x.__repr__() <==> repr(x)"
         return ufl.Coefficient.__repr__(self)
 
-    # Default value of dim
-    _dim = 0
+    # Default value for the value shape
+    _value_shape = ()
 
-    def dim(self):
-        """ Returns the dimension of the value"""
-        return self._dim
+    def value_shape(self):
+        """Returns the value shape of the expression"""
+        return self._value_shape
 
     __call__ = expression__call__
 
 def Expressions(*args, **kwargs):
-    """ Batch-processed user-defined JIT-compiled expressions
-    -------------------------------------------------------
+    """
+    Batch-processed user-defined JIT-compiled expressions
 
     By specifying several cppcodes one may compile more than one expression
     at a time:
@@ -598,7 +616,6 @@ def Expressions(*args, **kwargs):
     for cppcode, cpp_base in zip(cppcodes, cpp_bases):
         return_expressions.append(create_compiled_expression_class(cpp_base)(\
             cppcode,
-            cell=cell,
             degree=degree,
             element=element))
 
@@ -627,32 +644,14 @@ def _check_defaults(defaults):
     for key, val in defaults.iteritems():
         if not isinstance(key, str):
             raise TypeError, "All keys in 'defaults' must be a 'str'."
-        if not isinstance(val, (int, float)):
+        if not numpy.isscalar(val):
             raise TypeError, "All values in 'defaults' must be scalars."
 
 def _is_complex_expression(cppcode):
     "Check if cppcode is a complex expression"
     return isinstance(cppcode, str) and "class" in cppcode and "Expression" in cppcode
 
-def _auto_select_element_from_dim(dim, cell=None, degree=None):
-    "Automatically select an appropriate element from dim."
-
-    cpp.info("Got expression dimension = " + str(dim))
-
-    # Check dim to get shape
-    if isinstance(dim, int):
-        if dim == 0:
-            shape = ()
-        else:
-            shape = (dim,)
-    elif isinstance(dim, (tuple, list)):
-        shape = dim
-    else:
-        cpp.error("Expecting shape to be an integer or a tuple/list, not %s." % str(type(dim)))
-
-    return _auto_select_element_from_shape(shape, cell, degree)
-
-def _auto_select_element_from_shape(shape, cell=None, degree=None):
+def _auto_select_element_from_shape(shape, degree=None, cell=None):
     "Automatically select an appropriate element from cppcode."
 
     # Default element, change to quadrature when working
