@@ -32,8 +32,7 @@ class Function(ufl.Coefficient, cpp.Function):
     where {phi_i}_i is a basis for V_h, and U is a vector of
     expansion coefficients for u_h.
 
-    Initialize Function
-    -------------------
+    *Initialize Function*
 
     Create a Function:
 
@@ -55,7 +54,7 @@ class Function(ufl.Coefficient, cpp.Function):
     __metaclass__ = MetaNoEvalOverloading
 
     def __init__(self, *args):
-        """ Initialize Function"""
+        """Initialize Function"""
         # Check arguments
         if len(args) == 0:
             raise TypeError, "expected 1 or more arguments"
@@ -112,13 +111,14 @@ class Function(ufl.Coefficient, cpp.Function):
             raise TypeError, "too many arguments"
 
     def _sub(self, i, deepcopy = False):
-        """ Return a sub function
+        """Return a sub function
 
         The sub functions are numbered from i = 0..N-1, where N is the total number
         of sub spaces.
 
-        @param i :
-             The number of the sub function
+        *Arguments*
+            i
+                The number of the sub function
         """
         if not isinstance(i, int):
             raise TypeError, "expects an 'int' as first argument"
@@ -135,14 +135,15 @@ class Function(ufl.Coefficient, cpp.Function):
             return Function(self, i)
 
     def split(self, deepcopy = False):
-        """ Extract any sub functions
+        """Extract any sub functions
 
         A sub function can be extracted from a discrete function that is in a
         a MixedFunctionSpace or in a VectorFunctionSpace. The sub function is a
         function that resides in a sub space of the mixed space.
 
-        @param deepcopy :
-             Copy sub function vector instead of sharing
+        *Arguments*
+            deepcopy
+                Copy sub function vector instead of sharing
 
         """
         num_sub_spaces = self.function_space().num_sub_spaces()
@@ -151,7 +152,7 @@ class Function(ufl.Coefficient, cpp.Function):
         return tuple(self._sub(i, deepcopy) for i in xrange(num_sub_spaces))
 
     def ufl_element(self):
-        " Return ufl element"
+        "Return ufl element"
         return self._element
 
     def __str__(self):
@@ -161,37 +162,37 @@ class Function(ufl.Coefficient, cpp.Function):
     def __repr__(self):
         return ufl.Coefficient.__repr__(self)
 
-expression__call__.__doc__ = """ Evaluate the Function by interpolation
+#expression__call__.__doc__ = """ Evaluate the Function by interpolation
 
-    Example of use:
-    1) Using an iterable as x:
+#    Example of use:
+#    1) Using an iterable as x:
 
-    >>> Vs = FunctionSpace(mesh,"CG",1)
-    >>> fs = Function(Vs)
-    >>> x0 = (1.,0.5,0.5)
-    >>> x1 = [1.,0.5,0.5]
-    >>> x2 = numpy.array([1.,0.5,0.5])
-    >>> v0 = fs(x0)
-    >>> v1 = fs(x1)
-    >>> v2 = fs(x2)
+#    >>> Vs = FunctionSpace(mesh,"CG",1)
+#    >>> fs = Function(Vs)
+#    >>> x0 = (1.,0.5,0.5)
+#    >>> x1 = [1.,0.5,0.5]
+#    >>> x2 = numpy.array([1.,0.5,0.5])
+#    >>> v0 = fs(x0)
+#    >>> v1 = fs(x1)
+#    >>> v2 = fs(x2)
 
-    2) Using multiple scalar args for x, interpreted as a point coordinate
-    >>> v0 = f(1.,0.5,0.5)
+#    2) Using multiple scalar args for x, interpreted as a point coordinate
+#    >>> v0 = f(1.,0.5,0.5)
 
-    3) Passing return array
-    >>> Vv = VectorFunctionSpace(mesh,"CG",1)
-    >>> fv = Function(Vv)
-    >>> x0 = numpy.array([1.,0.5,0.5])
-    >>> v0 = numpy.zeros(3)
-    >>> fv(x0, values = v0)
+#    3) Passing return array
+#    >>> Vv = VectorFunctionSpace(mesh,"CG",1)
+#    >>> fv = Function(Vv)
+#    >>> x0 = numpy.array([1.,0.5,0.5])
+#    >>> v0 = numpy.zeros(3)
+#    >>> fv(x0, values = v0)
 
-    Note: A longer values array may be passed. In this way one can fast fill up
-          an array with different evaluations.
-    >>> values = numpy.zeros(9)
-    >>> for i in xrange(0,10,3):
-            fv(x[i:i+3], values = values[i:i+3])
+#    Note: A longer values array may be passed. In this way one can fast fill up
+#          an array with different evaluations.
+#    >>> values = numpy.zeros(9)
+#    >>> for i in xrange(0,10,3):
+#            fv(x[i:i+3], values = values[i:i+3])
 
-    """
+#    """
 
 Function.__call__ = types.MethodType(expression__call__, None, Function)
 
@@ -200,7 +201,6 @@ Function.__call__ = types.MethodType(expression__call__, None, Function)
 class Argument(ufl.Argument):
     """An Argument represents a possibly differentiated component
     of an argument on the reference cell.
-
     """
     def __init__(self, V, index=None):
         if not isinstance(V, FunctionSpaceBase):
@@ -209,7 +209,7 @@ class Argument(ufl.Argument):
         self._V = V
 
     def function_space(self):
-        " Return the FunctionSpace"
+        "Return the FunctionSpace"
         return self._V
 
 def TrialFunction(V):
