@@ -97,11 +97,17 @@ class FunctionSpaceBase(cpp.FunctionSpace):
         if self.num_sub_spaces() == 1:
             raise ValueError, "no SubSpaces to extract"
         if i >= self.num_sub_spaces():
-            raise ValueError, "Can only extract SubSpaces with i = 0..%d"%self.num_sub_spaces()
+            raise ValueError, "Can only extract SubSpaces with i = 0..%d"%(self.num_sub_spaces()-1)
         assert(hasattr(self._ufl_element,"sub_elements"))
         element = self._ufl_element.sub_elements()[i]
         return FunctionSpaceFromCpp(cpp.SubSpace(self,i), element)
 
+    def split(self):
+        S = []
+        for i in range(self.num_sub_spaces()):
+            S.append(self.sub(i))
+        return S
+
 class FunctionSpaceFromCpp(FunctionSpaceBase):
     " FunctionSpace represents a finite element function space."
     def __init__(self, cppV, element=None):
@@ -154,8 +160,8 @@ def create_ufc_function_spaces(mesh, ufc_form, cache=None):
 
         # If the cpp.FunctionSpace does not excists in the cache
         if V is None:
-            # Instantiate a dof_map
-            dm = ufc_form.create_dof_map(i)
+            # Instantiate a dofmap
+            dm = ufc_form.create_dofmap(i)
 
             # Instantiate the UFCFunctionSpace
             V = UFCFunctionSpace(mesh, fe, dm)
@@ -167,19 +173,20 @@ def create_ufc_function_spaces(mesh, ufc_form, cache=None):
 
 class UFCFunctionSpace(cpp.FunctionSpace):
     "FunctionSpace represents a finite element function space."
-    def __init__(self, mesh, ufc_finite_element, ufc_dof_map):
+    def __init__(self, mesh, ufc_finite_element, ufc_dofmap):
         " Initialize a FunctionSpace from ufc data "
         self._mesh = mesh
         self._finite_element     = cpp.FiniteElement(ufc_finite_element)
-        self._dof_map            = cpp.DofMap(ufc_dof_map, mesh)
+        self._dofmap            = cpp.DofMap(ufc_dofmap, mesh)
         self._ufc_finite_element = ufc_finite_element
-        self._ufc_dof_map        = ufc_dof_map
-        cpp.FunctionSpace.__init__(self, self._mesh, self._finite_element, self._dof_map)
+        self._ufc_dofmap        = ufc_dofmap
+        cpp.FunctionSpace.__init__(self, self._mesh, self._finite_element, self._dofmap)
 
 class FunctionSpace(FunctionSpaceBase):
     "FunctionSpace represents a finite element function space."
 
-    def __init__(self, mesh, family, degree, restriction=None):
+    def __init__(self, mesh, family, degree, form_degree=None,
+                 restriction=None):
         """Create finite element function space. The function space
         may be created by
 
@@ -188,6 +195,7 @@ class FunctionSpace(FunctionSpaceBase):
             mesh        : a Mesh
             family      : a string specifying the element family
             degree      : the degree of the element
+            form_degree : used for FEEC notation
             restriction : restriction of the element (e.g. to cell facets)
 
         Which families and degrees that are supported is determined by the
@@ -201,8 +209,6 @@ class FunctionSpace(FunctionSpaceBase):
             Brezzi-Douglas-Marini
             Brezzi-Douglas-Fortin-Marini
             Raviart-Thomas
-            Nedelec 1st kind H(div)
-            Nedelec 2nd kind H(div)
             Nedelec 1st kind H(curl)
             Nedelec 2nd kind H(curl)
             Quadrature
@@ -222,7 +228,8 @@ class FunctionSpace(FunctionSpaceBase):
 
         # Create element
         domain = dim2domain[mesh.topology().dim()]
-        element = ufl.FiniteElement(family, domain, degree)
+        element = ufl.FiniteElement(family, domain, degree,
+                                    form_degree=form_degree)
         if restriction is not None:
             element = element[restriction]
 
@@ -232,9 +239,11 @@ class FunctionSpace(FunctionSpaceBase):
         self.___degree = degree
         self.___family = family
         self.___mesh = mesh
+        self.___form_degree = form_degree
 
     def restriction(self, meshfunction):
-        space = FunctionSpace(self.___mesh, self.___family, self.___degree)
+        space = FunctionSpace(self.___mesh, self.___family, self.___degree,
+                              form_degree=self.___form_degree)
         space.attach(meshfunction)
         return space
 
@@ -308,7 +317,8 @@ class VectorFunctionSpace(MixedFunctionSpace):
 
         # Create subspaces
         dim = dim or mesh.geometry().dim()
-        spaces = dim*[FunctionSpace(mesh, family, degree, restriction)]
+        spaces = dim*[FunctionSpace(mesh, family, degree,
+                                    restriction=restriction)]
 
         # Initialize base class
         MixedFunctionSpace.__init__(self, spaces)
