@@ -4,7 +4,7 @@
 # Modified by Garth N. Wells (gmsh function)
 # Modified by Alexander H. Jarosch (gmsh fix)
 # Modified by Angelo Simone (Gmsh and Medit fix)
-# Modified by Andy R. Terrel (gmsh fix)
+# Modified by Andy R. Terrel (gmsh fix and triangle function)
 # Modified by Magnus Vikstrom (metis and scotch function)
 # Modified by Bartosz Sawicki (diffpack function)
 # Modified by Gideon Simpson (Exodus II function)
@@ -408,6 +408,64 @@ def gmsh2xml(ifilename, ofilename):
     ifile.close()
     ofile.close()
 
+def triangle2xml(ifilename, ofilename):
+    """Convert between triangle format (http://www.cs.cmu.edu/~quake/triangle.html) and .xml.  The
+    given ifilename should be the prefix for the corresponding .node, and .ele files.
+    """
+
+    def get_next_line (fp):
+        """Helper function for skipping comments and blank lines"""
+        line = fp.readline()
+        if line == '':
+            _error("Hit end of file prematurely.")
+        line = line.strip()
+        if not (line.startswith('#') or line == ''):
+            return line
+        return get_next_line(fp)
+
+
+    print "Converting from Triangle format {.node, .ele} to DOLFIN XML format"
+
+    # Open files
+    node_file = open(ifilename+".node", "r")
+    ele_file =  open(ifilename+".ele", "r")
+    ofile = open(ofilename, "w")
+
+    # Read all the nodes
+    nodes = {}
+    num_nodes, dim, attr, bound = map(int, get_next_line(node_file).split())
+    while len(nodes) < num_nodes:
+        node, x, y = get_next_line(node_file).split()[:3]
+        nodes[int(node)] = (float(x), float(y))
+
+    # Read all the triangles
+    tris = {}
+    num_tris, n_per_tri, attrs = map(int, get_next_line(ele_file).split())
+    while len(tris) < num_tris:
+        tri, n1, n2, n3 = map(int, get_next_line(ele_file).split()[:4])
+        tris[tri] = (n1, n2, n3)
+
+    # Write everything out
+    write_header_mesh(ofile, "triangle", 2)
+    write_header_vertices(ofile, num_nodes)
+    node_off = 0 if nodes.has_key(0) else -1
+    for node, node_t in nodes.iteritems():
+        write_vertex(ofile, node+node_off, node_t[0], node_t[1], 0.0)
+    write_footer_vertices(ofile)
+    write_header_cells(ofile, num_tris)
+    tri_off = 0 if tris.has_key(0) else -1
+    for tri, tri_t in tris.iteritems():
+        write_cell_triangle(ofile, tri+tri_off, tri_t[0] + node_off,
+                            tri_t[1] + node_off, tri_t[2] + node_off)
+    write_footer_cells(ofile)
+    write_footer_mesh(ofile)
+
+    # Close files
+    node_file.close()
+    ele_file.close()
+    ofile.close()
+
+
 def xml_old2xml(ifilename, ofilename):
     "Convert from old DOLFIN XML format to new."
 
@@ -448,7 +506,7 @@ def xml_old2xml(ifilename, ofilename):
 
         # Modify line
         if "xmlns" in line:
-            line = "<dolfin xmlns:dolfin=\"http://www.fenics.org/dolfin/\">\n"
+            line = "<dolfin xmlns:dolfin=\"http://www.fenicsproject.org\">\n"
         if "<mesh>" in line:
             line = "  <mesh celltype=\"%s\" dim=\"%d\">\n" % (cell_type, dim)
         if dim == 2 and " z=\"0.0\"" in line:
@@ -1212,6 +1270,9 @@ def convert(ifilename, handler, iformat=None):
     elif iformat == "gmsh":
         # Convert from gmsh to xml format
         gmsh2xml(ifilename, ofilename)
+    elif iformat == "Triangle":
+        # Convert from Triangle to xml format
+        triangle2xml(ifilename, ofilename)
     elif iformat == "xml-old":
         # Convert from old to new xml format
         xml_old2xml(ifilename, ofilename)
