@@ -24,8 +24,8 @@ def refine(mesh, cell_markers=None):
 
     # Call C++ refinement
     if cell_markers is None:
-        cpp._refine(refined_mesh, mesh)
+        cpp.refine(refined_mesh, mesh)
     else:
-        cpp._refine(refined_mesh, mesh, cell_markers)
+        cpp.refine(refined_mesh, mesh, cell_markers)
 
     return refined_mesh
