@@ -23,7 +23,7 @@ cppdemos = []
 pydemos = []
 for dpath, dnames, fnames in os.walk(demodir):
     if os.path.basename(dpath) == 'cpp':
-        if os.path.isfile(os.path.join(dpath, 'SConstruct')):
+        if os.path.isfile(os.path.join(dpath, 'Makefile')):
             cppdemos.append(dpath)
     elif os.path.basename(dpath) == 'python':
         if os.path.isfile(os.path.join(dpath, 'demo.py')):
@@ -40,8 +40,8 @@ print ""
 import pprint
 
 # Remove demos that are known not to work (FIXME's)
-pydemos.remove(os.path.join(demodir, 'ode', 'aliev-panfilov', 'python'))
-pydemos.remove(os.path.join(demodir, 'ode', 'lorenz', 'python'))
+#pydemos.remove(os.path.join(demodir, 'undocumented', 'aliev-panfilov', 'python'))
+#pydemos.remove(os.path.join(demodir, 'undocumented', 'lorenz', 'python'))
 
 # Push slow demos to the end
 pyslow = []
@@ -54,15 +54,13 @@ for s in cppslow:
     cppdemos.append(s)
 
 # Remove overly slow demos
-pydemos.remove(os.path.join(demodir, 'pde', 'cahn-hilliard', 'python'))
-cppdemos.remove(os.path.join(demodir, 'pde', 'cahn-hilliard', 'cpp'))
-cppdemos.remove(os.path.join(demodir, 'pde', 'elastodynamics', 'cpp'))
+#cppdemos.remove(os.path.join(demodir, 'undocumented', 'elastodynamics', 'cpp'))
 
-# Demos that need command-line arguments are treated separately
-pydemos.remove(os.path.join(demodir, 'quadrature', 'python'))
-cppdemos.remove(os.path.join(demodir, 'quadrature', 'cpp'))
-cppdemos.remove(os.path.join(demodir, 'ode', 'method-weights', 'cpp'))
-cppdemos.remove(os.path.join(demodir, 'ode', 'stiff', 'cpp'))
+# Remove demos that need command-line arguments
+pydemos.remove(os.path.join(demodir,  'undocumented', 'quadrature', 'python'))
+cppdemos.remove(os.path.join(demodir, 'undocumented', 'quadrature', 'cpp'))
+cppdemos.remove(os.path.join(demodir, 'undocumented', 'method-weights', 'cpp'))
+cppdemos.remove(os.path.join(demodir, 'undocumented', 'stiff', 'cpp'))
 
 failed = []
 timing = []
@@ -80,10 +78,10 @@ if only_python:
 
 # Build prefix list
 prefixes = [""]
-if has_mpi() and has_parmetis():
-    prefixes.append("mpirun -n 2 ")
+if "RUN_UNIT_TESTS_IN_PARALLEL" in os.environ and has_mpi() and has_parmetis():
+    prefixes.append("mpirun -n 3 ")
 else:
-    print "DOLFIN has not been compiled with mpi and Parmetis. Regression test will not be run in parallel."
+    print "Not running regression tests in parallel."
 
 # Run in serial, then in parallel
 for prefix in prefixes:
@@ -96,10 +94,11 @@ for prefix in prefixes:
         cppdemo_ext = ''
         if platform.system() == 'Windows':
             cppdemo_ext = '.exe'
-        if os.path.isfile(os.path.join(demo, 'demo' + cppdemo_ext)):
+        cppdemo_prefix = demo.split(os.path.sep)[-2]
+        if os.path.isfile(os.path.join(demo, cppdemo_prefix + '-demo' + cppdemo_ext)):
             t1 = time()
-            output = getstatusoutput("cd %s && %s .%sdemo%s" % \
-                                         (demo, prefix, os.path.sep, cppdemo_ext))
+            output = getstatusoutput("cd %s && %s .%s%s-demo%s" % \
+                                         (demo, prefix, os.path.sep, cppdemo_prefix, cppdemo_ext))
             t2 = time()
             timing += [(t2 - t1, demo)]
             if output[0] == 0:
@@ -108,6 +107,7 @@ for prefix in prefixes:
                 print "ok (graceful exit on fail)"
             else:
                 print "*** Failed"
+                print output[1]
                 failed += [(demo, "C++", prefix, output[1])]
         else:
             print "*** Warning: missing demo"
@@ -128,6 +128,7 @@ for prefix in prefixes:
                 print "ok (graceful exit on fail)"
             else:
                 print "*** Failed"
+                print output[1]
                 failed += [(demo, "Python", prefix, output[1])]
         else:
             print "*** Warning: missing demo"
