@@ -13,6 +13,10 @@ class Assembly(unittest.TestCase):
 
     def test_facet_assembly(self):
 
+        if MPI.num_processes() > 1:
+            print "FIXME: This unit test does not work in parallel, skipping"
+            return
+
         mesh = UnitSquare(24, 24)
         V = FunctionSpace(mesh, "DG", 1)
 
@@ -43,6 +47,7 @@ class Assembly(unittest.TestCase):
         b_l2_norm = 1.48087142738768
 
         # Assemble A and b separately
+        parameters["num_threads"] = 0
         self.assertAlmostEqual(assemble(a).norm("frobenius"), A_frobenius_norm, 10)
         self.assertAlmostEqual(assemble(L).norm("l2"), b_l2_norm, 10)
 
@@ -51,6 +56,11 @@ class Assembly(unittest.TestCase):
         self.assertAlmostEqual(A.norm("frobenius"), A_frobenius_norm, 10)
         self.assertAlmostEqual(b.norm("l2"), b_l2_norm, 10)
 
+        # Assemble A and b separately (multi-threaded)
+        parameters["num_threads"] = 4
+        self.assertAlmostEqual(assemble(a).norm("frobenius"), A_frobenius_norm, 10)
+        self.assertAlmostEqual(assemble(L).norm("l2"), b_l2_norm, 10)
+
     def test_cell_assembly(self):
 
         mesh = UnitCube(4, 4, 4)
@@ -66,20 +76,54 @@ class Assembly(unittest.TestCase):
         a = inner(epsilon(v), epsilon(u))*dx
         L = inner(v, f)*dx
 
-        A_frobenius_norm =  4.3969686527582512;
-        b_l2_norm = 0.95470326978246278;
+        A_frobenius_norm =  4.3969686527582512
+        b_l2_norm = 0.95470326978246278
+
+        # Assemble A and b separately
+        parameters["num_threads"] = 0
         self.assertAlmostEqual(assemble(a).norm("frobenius"), A_frobenius_norm, 10)
         self.assertAlmostEqual(assemble(L).norm("l2"), b_l2_norm, 10)
 
+        # Assemble system
         A, b = assemble_system(a, L)
         self.assertAlmostEqual(A.norm("frobenius"), A_frobenius_norm, 10)
         self.assertAlmostEqual(b.norm("l2"), b_l2_norm, 10)
 
+        # Assemble A and b separately (multi-threaded)
+        parameters["num_threads"] = 4
+        self.assertAlmostEqual(assemble(a).norm("frobenius"), A_frobenius_norm, 10)
+        self.assertAlmostEqual(assemble(L).norm("l2"), b_l2_norm, 10)
+
+    def test_nonsquare_assembly(self):
+        """Test assembly of a rectangular matrix"""
+
+        mesh = UnitSquare(16, 16)
+
+        V = VectorFunctionSpace(mesh, "CG", 2)
+        Q = FunctionSpace(mesh, "CG", 1)
+        W = V*Q
+
+        (v, q) = TestFunctions(W)
+        (u, p) = TrialFunctions(W)
+
+        a = div(v)*p*dx
+        A_frobenius_norm = 9.6420303878382718e-01
+
+        parameters["num_threads"] = 0
+        self.assertAlmostEqual(assemble(a).norm("frobenius"), A_frobenius_norm, 10)
+
+        parameters["num_threads"] = 4
+        self.assertAlmostEqual(assemble(a).norm("frobenius"), A_frobenius_norm, 10)
+
 
 class DofMap(unittest.TestCase):
 
     def test_sub_dofmap(self):
-        """ Test extraction of sub- and sub-sub-dofmaps."""
+
+        if MPI.num_processes() > 1:
+            print "FIXME: This unit test does not work in parallel, skipping"
+            return
+
         mesh = UnitSquare(1, 1)
         V = FunctionSpace(mesh, "CG", 1)
         Q = VectorFunctionSpace(mesh, "CG", 1)
