@@ -1,7 +1,7 @@
 """Unit tests for the function library"""
 
 __author__ = "Anders Logg (logg@simula.no)"
-__date__ = "2007-05-24 -- 2009-12-16"
+__date__ = "2007-05-24 -- 2011-01-28"
 __copyright__ = "Copyright (C) 2007 Anders Logg"
 __license__  = "GNU LGPL Version 2.1"
 
@@ -11,7 +11,9 @@ from math   import sin
 from numpy  import array, zeros
 
 mesh = UnitCube(8, 8, 8)
-V = FunctionSpace(mesh,'CG',1)
+V = FunctionSpace(mesh, 'CG', 1)
+
+W = VectorFunctionSpace(mesh, 'CG', 1)
 
 class Eval(unittest.TestCase):
 
@@ -20,7 +22,7 @@ class Eval(unittest.TestCase):
                def eval(self, values, x):
                     values[0] = sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])
 
-          f0 = F0(V)
+          f0 = F0()
           f1 = Expression("sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])", degree=2)
           f2, f3 = Expressions("sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])",
                               "1.0 + 3.0*x[0] + 4.0*x[1] + 0.5*x[2]", degree=2)
@@ -63,11 +65,13 @@ class Eval(unittest.TestCase):
           if not has_cgal():
                return
 
-          V2 = FunctionSpace(mesh, 'CG', 2)
-          g = project(f3, V2)
-          u3 = f3(x)
-          u4 = g(x)
-          self.assertAlmostEqual(u3,u4, places=5)
+          # FIXME: eval does not work in parallel yet
+          if MPI.num_processes() == 0:
+              V2 = FunctionSpace(mesh, 'CG', 2)
+              g = project(f3, V2)
+              u3 = f3(x)
+              u4 = g(x)
+              self.assertAlmostEqual(u3, u4, places=5)
 
      def testOverLoadAndCallBack(self):
           class F0(Expression):
@@ -75,12 +79,14 @@ class Eval(unittest.TestCase):
                     values[0] = sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])
 
           class F1(Expression):
-               def eval_data(self, values, data):
-                    x = data.x()
+               def __init__(self, mesh, *arg, **kwargs):
+                    self.mesh = mesh
+               def eval_cell(self, values, x, cell):
+                    c = Cell(self.mesh, cell.index)
                     values[0] = sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])
 
-          e0 = F0(V, degree=2)
-          e1 = F1(V, degree=2)
+          e0 = F0(degree=2)
+          e1 = F1(mesh, degree=2)
           e2 = Expression("sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])", degree=2)
 
           s0 = norm(interpolate(e0, V))
@@ -96,7 +102,7 @@ class Eval(unittest.TestCase):
                def eval(self, values, x):
                     values[0] = sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])
 
-          f0 = F0(V)
+          f0 = F0()
           f1 = Expression("sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])", degree=2)
           f2, f3 = Expressions("sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])",
                                "1.0 + 3.0*x[0] + 4.0*x[1] + 0.5*x[2]", degree=2)
@@ -104,38 +110,14 @@ class Eval(unittest.TestCase):
           for f in [f0,f1,f2,f3]:
                self.assertRaises(TypeError, f, "s")
                self.assertRaises(TypeError, f, [])
-               #self.assertRaises(TypeError, f, zeros(2))
                self.assertRaises(TypeError, f, 0.5, 0.5, 0.5, values = zeros(3,'i'))
-               #self.assertRaises(TypeError, f, zeros(4))
-               self.assertRaises(ValueError,f, [0.3, 0.2, []])
+               self.assertRaises(TypeError, f, [0.3, 0.2, []])
                self.assertRaises(TypeError, f, 0.3, 0.2, {})
-               #self.assertRaises(TypeError, f, 0.3, 0.2)
-               #self.assertRaises(TypeError, f, [0.5, 0.2, 0.1, 0.2])
                self.assertRaises(TypeError, f, zeros(3), values = zeros(4))
                self.assertRaises(TypeError, f, zeros(4), values = zeros(3))
 
 class Instantiation(unittest.TestCase):
 
-     def testSameBases(self):
-
-          # FIXME: Hake: What should this unit test do?
-
-          return
-
-          f0 = Expression("2", degree=2)
-
-          class MyConstant(Expression):
-               cpparg = "2"
-
-          f1 = MyConstant(V)
-
-          self.assertNotEqual(type(f0).__bases__,type(f1).__bases__)
-
-          #f2, f3 = Expressions("2", V, "3", V)
-          f4, f5 = Expressions("2", "3", V = V)
-          self.assertEqual(type(f2).__bases__, type(f4).__bases__)
-          self.assertEqual(type(f3).__bases__, type(f5).__bases__)
-
      def testWrongSubClassing(self):
 
           def noAttributes():
@@ -148,7 +130,7 @@ class Instantiation(unittest.TestCase):
 
           def wrongEvalDataAttribute():
                class WrongEvalDataAttribute(Expression):
-                    def eval_data(values, data):
+                    def eval_cell(values, data):
                          pass
 
           def noEvalAttribute():
@@ -156,27 +138,36 @@ class Instantiation(unittest.TestCase):
                     def evaluate(self, values, data):
                          pass
 
-          def bothCompileAndPythonSubClassing0():
-               class bothCompileAndPythonSubClassing(Expression):
-                    def eval(self, values, x):pass
-                    cpparg = "2"
+          def wrongArgs():
+               class WrongArgs(Expression):
+                    def eval(self, values, x):
+                         pass
+               e = WrongArgs(V)
 
-          def bothCompileAndPythonSubClassing1():
-               class bothCompileAndPythonSubClassing(Expression):
-                    def eval_data(self, values, data):pass
-                    cpparg = "2"
+          def wrongElement():
+               class WrongElement(Expression):
+                    def eval(self, values, x):
+                         pass
+                    def value_shape(self):
+                         return (2,)
+               e = WrongElement(element=V.ufl_element())
 
-          def wrongCppargType():
-               class WrongCppargType(Expression):
-                    cpparg = 2
+          def deprecationWarning():
+               class Deprecated(Expression):
+                    def eval(self, values, x):
+                         pass
+                    def dim(self):
+                         return 2
+               e = Deprecated()
+
+          self.assertRaises(TypeError, noAttributes)
+          self.assertRaises(TypeError, noEvalAttribute)
+          self.assertRaises(TypeError, wrongEvalAttribute)
+          self.assertRaises(TypeError, wrongEvalDataAttribute)
+          self.assertRaises(TypeError, wrongArgs)
+          #self.assertRaises(ValueError, wrongElement)
+          self.assertRaises(DeprecationWarning, deprecationWarning)
 
-          self.assertRaises(TypeError,noAttributes)
-          self.assertRaises(TypeError,noEvalAttribute)
-          self.assertRaises(TypeError,wrongEvalAttribute)
-          self.assertRaises(TypeError,wrongEvalDataAttribute)
-          #self.assertRaises(TypeError,bothCompileAndPythonSubClassing0)
-          #self.assertRaises(TypeError,bothCompileAndPythonSubClassing1)
-          self.assertRaises(TypeError,wrongCppargType)
 
 class Interpolate(unittest.TestCase):
 
@@ -188,21 +179,23 @@ class Interpolate(unittest.TestCase):
             def eval(self, values, x):
                 values[0] = 1.0
                 values[1] = 1.0
-            def dim(self):
-                return 2
+            def value_shape(self):
+                return (2,)
 
-        # Scalar interpolation
-        f0 = F0()
-        f = Function(V)
-        f.interpolate(f0)
-        self.assertAlmostEqual(f.vector().norm("l1"), mesh.num_vertices())
+        # Interpolation not working in parallel yet
+        if MPI.num_processes() == 0:
+            # Scalar interpolation
+            f0 = F0()
+            f = Function(V)
+            f.interpolate(f0)
+            self.assertAlmostEqual(f.vector().norm("l1"), mesh.num_vertices())
 
-        # Vector interpolation
-        f1 = F1()
-        W = V * V
-        f = Function(W)
-        f.interpolate(f1)
-        self.assertAlmostEqual(f.vector().norm("l1"), 2*mesh.num_vertices())
+            # Vector interpolation
+            f1 = F1()
+            W = V * V
+            f = Function(W)
+            f.interpolate(f1)
+            self.assertAlmostEqual(f.vector().norm("l1"), 2*mesh.num_vertices())
 
 if __name__ == "__main__":
     unittest.main()
